/*
Data definitions for intelligible reading and writing OpenType fonts.
Copyright (C) 2013-2015      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include "bitcodes.h"

/* f(bit, short, long, start, end) */
#define ulranges(f) \
    f(0, "ASCII",      "Basic Latin",                      0000, 0x007F), \
    f(1, "Latin-1",    "Latin-1 Supplement",             0x0080, 0x00FF), \
    f(2, "Latin-A",    "Latin Extended-A",               0x0100, 0x017F), \
    f(3, "Latin-B",    "Latin Extended-B",               0x0180, 0x024F), \
    f(4, "IPA",        "IPA Extensions",                 0x0250, 0x02AF), \
    f(4, "IPA",        "Phonetic Extensions",            0x1D00, 0x1D7F), \
    f(4, "IPA",        "Phonetic Extensions Supplement", 0x1D80, 0x1DBF), \
    f(5, "Modifiers",  "Spacing Modifier Letters",       0x02B0, 0x02FF), \
    f(5, "Modifiers",  "Modifier Tone Letters",          0xA700, 0xA71F), \
    f(6, "Diacritics", "Combining Diacritical Marks",    0x0300, 0x036F), \
    f(6, "Diacritics", "Combining Diacritical Marks Supplement", \
                                                         0x1DC0, 0x1DFF), \
    f(7, "Greek",      "Greek and Coptic",               0x0370, 0x03FF), \
    f(8, "Coptic",     "Coptic",                         0x2C80, 0x2CFF), \
    f(9, "Cyrillic",   "Cyrillic",                       0x0400, 0x04FF), \
    f(9, "Cyrillic",   "Cyrillic Supplement",            0x0500, 0x052F), \
    f(9, "Cyrillic",   "Cyrillic Extended-A",            0x2DE0, 0x2DFF), \
    f(9, "Cyrillic",   "Cyrillic Extended-B",            0xA640, 0xA69F), \
    f(10, "Armenian",  "Armenian",                       0x0530, 0x058F), \
    f(11, "Hebrew",    "Hebrew",                         0x0590, 0x05FF), \
    f(12, "Vai",       "Vai",                            0xA500, 0xA63F), \
    f(13, "Arabic",    "Arabic",                         0x0600, 0x06FF), \
    f(13, "Arabic",    "Arabic Supplement",              0x0750, 0x077F), \
    f(14, "Nko",       "NKo",                            0x07C0, 0x07FF), \
    f(15, "Devanagari", "Devanagari",                    0x0900, 0x097F), \
    f(16, "Bengali",   "Bengali",                        0x0980, 0x09FF),\
    f(17, "Gurmukhi",  "Gurmukhi",                       0x0A00, 0x0A7F), \
    f(18, "Gujarati",  "Gujarati",                       0x0A80, 0x0AFF), \
    f(19, "Oriya",     "Oriya",                          0x0B00, 0x0B7F), \
    f(20, "Tamil",     "Tamil",                          0x0B80, 0x0BFF), \
    f(21, "Telugu",    "Telugu",                         0x0C00, 0x0C7F), \
    f(22, "Kannada",   "Kannada",                        0x0C80, 0x0CFF), \
    f(23, "Malayalam", "Malayalam",                      0x0D00, 0x0D7F), \
    f(24, "Thai",      "Thai",                           0x0E00, 0x0E7F), \
    f(25, "Lao",       "Lao",                            0x0E80, 0x0EFF), \
    f(26, "Georgian",  "Georgian",                       0x10A0, 0x10FF), \
    f(26, "Georgian",  "Georgian Supplement",            0x2D00, 0x2D2F), \
    f(27, "Balinese",  "Balinese",                       0x1B00, 0x1B7F), \
    f(28, "Jamo",      "Hangul Jamo",                    0x1100, 0x11FF), \
    f(29, "Latin-ext", "Latin Extended Additional",      0x1E00, 0x1EFF), \
    f(29, "Latin-ext", "Latin Extended-C",               0x2C60, 0x2C7F), \
    f(29, "Latin-ext", "Latin Extended-D",               0xA720, 0xA7FF), \
    f(30, "Greek-ext", "Greek Extended",                 0x1F00, 0x1FFF), \
    f(31, "Punctuation", "General Punctuation",          0x2000, 0x206F), \
    f(31, "Punctuation", "Supplemental Punctuation",     0x2E00, 0x2E7F), \
    f(32, "Superscripts", "Superscripts And Subscripts", 0x2070, 0x209F), \
    f(33, "Currency",   "Currency Symbols",              0x20A0, 0x20CF), \
    f(34, "Symbol-diacritics", "Combining Diacritical Marks For Symbols", \
                                                         0x20D0, 0x20FF), \
    f(35, "Letterlike", "Letterlike Symbols",            0x2100, 0x214F), \
    f(36, "Number-forms", "Number Forms",                0x2150, 0x218F), \
    f(37, "Arrows",    "Arrows",                         0x2190, 0x21FF), \
    f(37, "Arrows",    "Supplemental Arrows-A",          0x27F0, 0x27FF), \
    f(37, "Arrows",    "Supplemental Arrows-B",          0x2900, 0x297F), \
    f(37, "Arrows",  "Miscellaneous Symbols and Arrows", 0x2B00, 0x2BFF), \
    f(38, "Operators", "Mathematical Operators",         0x2200, 0x22FF), \
    f(38, "Operators", "Supplemental Mathematical Operators", 0x2A00, 0x2AFF), \
    f(38, "Operators", "Miscellaneous Mathematical Symbols-A", \
                                                         0x27C0, 0x27EF), \
    f(38, "Operators", "Miscellaneous Mathematical Symbols-B", 0x2980, 0x29FF),\
    f(39, "Technical", "Miscellaneous Technical",        0x2300, 0x23FF), \
    f(40, "Controls",  "Control Pictures",               0x2400, 0x243F), \
    f(41, "OCR",       "Optical Character Recognition",  0x2440, 0x245F), \
    f(42, "Enclosed",  "Enclosed Alphanumerics",         0x2460, 0x24FF), \
    f(43, "Box",       "Box Drawing",                    0x2500, 0x257F), \
    f(44, "Block",     "Block Elements",                 0x2580, 0x259F), \
    f(45, "Geometric", "Geometric Shapes",               0x25A0, 0x25FF), \
    f(46, "Misc",      "Miscellaneous Symbols",          0x2600, 0x26FF), \
    f(47, "Dingbats",  "Dingbats",                       0x2700, 0x27BF), \
    f(48, "CJK-symbols", "CJK Symbols And Punctuation",  0x3000, 0x303F), \
    f(49, "Hiragana",  "Hiragana",                       0x3040, 0x309F), \
    f(50, "Katakana",  "Katakana",                       0x30A0, 0x30FF), \
    f(50, "Katakana",  "Katakana Phonetic Extensions",   0x31F0, 0x31FF), \
    f(51, "Bopomofo",  "Bopomofo",                       0x3100, 0x312F), \
    f(51, "Bopomofo",  "Bopomofo Extended",              0x31A0, 0x31BF), \
    f(52, "Compat-jamo", "Hangul Compatibility Jamo",    0x3130, 0x318F), \
    f(53, "Phags-pa",  "Phags-pa",                       0xA840, 0xA87F), \
    f(54, "CJK-enclosed", "Enclosed CJK Letters And Months", 0x3200, 0x32FF), \
    f(55, "CJK-compat", "CJK Compatibility",             0x3300, 0x33FF), \
    f(56, "Hangul",     "Hangul Syllables",              0xAC00, 0xD7AF), \
    f(57, "Supplementary", "Non-Plane 0",                0xD800, 0xDFFF), \
    f(58, "Phoenician", "Phoenician",                    0x10900, 0x1091F), \
    f(59, "CJK",        "CJK Unified Ideographs",        0x4E00, 0x9FFF), \
    f(59, "CJK",        "CJK Radicals Supplement",       0x2E80, 0x2EFF), \
    f(59, "CJK",        "Kangxi Radicals",               0x2F00, 0x2FDF), \
    f(59, "CJK",   "Ideographic Description Characters", 0x2FF0, 0x2FFF), \
    f(59, "CJK",   "CJK Unified Ideographs Extension A", 0x3400, 0x4DBF), \
    f(59, "CJK",   "CJK Unified Ideographs Extension B", 0x20000, 0x2A6DF), \
    f(59, "CJK",        "Kanbun",                        0x3190, 0x319F), \
    f(60, "PUA",         "Private Use Area (plane 0)",   0xE000, 0xF8FF), \
    f(61, "CJK-misc",    "CJK Strokes",                  0x31C0, 0x31EF), \
    f(61, "CJK-misc",    "CJK Compatibility Ideographs", 0xF900, 0xFAFF), \
    f(61, "CJK-misc",    "CJK Compatibility Ideographs Supplement", \
                                                         0x2F800, 0x2FA1F), \
    f(62, "Forms",       "Alphabetic Presentation Forms", 0xFB00, 0xFB4F), \
    f(63, "Forms-Arabic", "Arabic Presentation Forms-A", 0xFB50, 0xFDFF), \
    f(64, "Half-marks",   "Combining Half Marks",        0xFE20, 0xFE2F), \
    f(65, "CJK-forms",    "Vertical Forms",              0xFE10, 0xFE1F), \
    f(65, "CJK-forms",    "CJK Compatibility Forms",     0xFE30, 0xFE4F), \
    f(66, "CJK-forms",    "Small Form Variants",         0xFE50, 0xFE6F), \
    f(67, "Forms-Arabic-B", "Arabic Presentation Forms-B", 0xFE70, 0xFEFF), \
    f(68, "Width-forms",  "Halfwidth And Fullwidth Forms", 0xFF00, 0xFFEF), \
    f(69, "Specials",     "Specials",                    0xFFF0, 0xFFFF), \
    f(70, "Tibetan",      "Tibetan",                     0x0F00, 0x0FFF), \
    f(71, "Syriac",       "Syriac",                      0x0700, 0x074F), \
    f(72, "Thaana",       "Thaana",                      0x0780, 0x07BF), \
    f(73, "Sinhala",      "Sinhala",                     0x0D80, 0x0DFF), \
    f(74, "Myanmar",      "Myanmar",                     0x1000, 0x109F), \
    f(75, "Ethiopic",     "Ethiopic",                    0x1200, 0x137F), \
    f(75, "Ethiopic",     "Ethiopic Supplement",         0x1380, 0x139F), \
    f(75, "Ethiopic",     "Ethiopic Extended",           0x2D80, 0x2DDF), \
    f(76, "Cherokee",     "Cherokee",                    0x13A0, 0x13FF), \
    f(77, "UCAS", "Unified Canadian Aboriginal Syllabics", 0x1400, 0x167F), \
    f(78, "Ogham",        "Ogham",                       0x1680, 0x169F), \
    f(79, "Runic",        "Runic",                       0x16A0, 0x16FF), \
    f(80, "Khmer",        "Khmer",                       0x1780, 0x17FF), \
    f(80, "Khmer",        "Khmer Symbols",               0x19E0, 0x19FF), \
    f(81, "Mongolian",    "Mongolian",                   0x1800, 0x18AF), \
    f(82, "Braille",      "Braille Patterns",            0x2800, 0x28FF), \
    f(83, "Yi",           "Yi Syllables",                0xA000, 0xA48F), \
    f(83, "Yi",           "Yi Radicals",                 0xA490, 0xA4CF), \
    f(84, "Phillipine",   "Tagalog",                     0x1700, 0x171F) ,\
    f(84, "Phillipine",   "Hanunoo",                     0x1720, 0x173F), \
    f(84, "Phillipine",   "Buhid",                       0x1740, 0x175F), \
    f(84, "Phillipine",   "Tagbanwa",                    0x1760, 0x177F), \
    f(85, "Old-Italic",   "Old Italic",                  0x10300, 0x1032F), \
    f(86, "Gothic",       "Gothic",                      0x10330, 0x1034F), \
    f(87, "Deseret",      "Deseret",                     0x10400, 0x1044F), \
    f(88, "Musical",      "Byzantine Musical Symbols",   0x1D000, 0x1D0FF), \
    f(88, "Musical",      "Musical Symbols",             0x1D100, 0x1D1FF), \
    f(88, "Musical",   "Ancient Greek Musical Notation", 0x1D200, 0x1D24F), \
    f(89, "Mathematical", "Mathematical Alphanumeric Symbols", \
                                                         0x1D400, 0x1D7FF), \
    f(90, "Supp-PUA",     "Private Use (plane 15)",      0xFF000, 0xFFFFD), \
    f(90, "Supp-PUA",     "Private Use (plane 16)",      0x100000, 0x10FFFD), \
    f(91, "VS",           "Variation Selectors",         0xFE00, 0xFE0F), \
    f(91, "VS",        "Variation Selectors Supplement", 0xE0100, 0xE01EF), \
    f(92, "Tags",         "Tags",                        0xE0000, 0xE007F), \
    f(93, "Limbu",        "Limbu",                       0x1900, 0x194F), \
    f(94, "Tai-Le",       "Tai Le",                      0x1950, 0x197F), \
    f(95, "New-Tai-Lue",  "New Tai Lue",                 0x1980, 0x19DF), \
    f(96, "Buginese",     "Buginese",                    0x1A00, 0x1A1F), \
    f(97, "Glagolitic",   "Glagolitic",                  0x2C00, 0x2C5F), \
    f(98, "Tifinagh",     "Tifinagh",                    0x2D30, 0x2D7F), \
    f(99, "Hexagram",     "Yijing Hexagram Symbols",     0x4DC0, 0x4DFF), \
    f(100, "Syloti",      "Syloti Nagri",                0xA800, 0xA82F), \
    f(101, "Linear-B",    "Linear B Syllabary",          0x10000, 0x1007F), \
    f(101, "Linear-B",    "Linear B Ideograms",          0x10080, 0x100FF), \
    f(101, "Linear-B",    "Aegean Numbers",              0x10100, 0x1013F), \
    f(102, "Greek-numbers", "Ancient Greek Numbers",     0x10140, 0x1018F), \
    f(103, "Ugaritic",    "Ugaritic",                    0x10380, 0x1039F), \
    f(104, "Old-Persian", "Old Persian",                 0x103A0, 0x103DF), \
    f(105, "Shavian",     "Shavian",                     0x10450, 0x1047F), \
    f(106, "Osmanya",     "Osmanya",                     0x10480, 0x104AF), \
    f(107, "Cypriot",     "Cypriot Syllabary",           0x10800, 0x1083F), \
    f(108, "Kharoshthi",  "Kharoshthi",                  0x10A00, 0x10A5F), \
    f(109, "Tai-Xuan-Jing-Symbols", "Tai Xuan Jing Symbols", \
                                                         0x1D300, 0x1D35F), \
    f(110, "Cuneiform",   "Cuneiform",                   0x12000, 0x123FF), \
    f(110, "Cuneiform",   "Cuneiform Numbers and Punctuation", \
                                                         0x12400, 0x1247F), \
    f(111, "Counting-Rod", "Counting Rod Numerals",      0x1D360, 0x1D37F), \
    f(112, "Sundanese",   "Sundanese",                   0x1B80, 0x1BBF), \
    f(113, "Lepcha",      "Lepcha",                      0x1C00, 0x1C4F), \
    f(114, "Ol-Chiki",    "Ol Chiki",                    0x1C50, 0x1C7F), \
    f(115, "Saurashtra",  "Saurashtra",                  0xA880, 0xA8DF), \
    f(116, "Kayah-Li",    "Kayah Li",                    0xA900, 0xA92F), \
    f(117, "Rejang",      "Rejang",                      0xA930, 0xA95F), \
    f(118, "Cham",        "Cham",                        0xAA00, 0xAA5F), \
    f(119, "Ancient-Symbols", "Ancient Symbols",         0x10190, 0x101CF), \
    f(120, "Phaistos",    "Phaistos Disc",               0x101D0, 0x101FF), \
    f(121, "Anatolian",   "Carian",                      0x102A0, 0x102DF), \
    f(121, "Anatolian",   "Lycian",                      0x10280, 0x1029F), \
    f(121, "Anatolian",   "Lydian",                      0x10920, 0x1093F), \
    f(122, "Tiles",       "Domino Tiles",                0x1F030, 0x1F09F), \
    f(122, "Tiles",       "Mahjong Tiles",               0x1F000, 0x1F02F)
const int is_supplementary = 57;

#define eldef(bit, short, long, start, end) {short, long, bit, start, end}
const struct st_entry script_table[] = {ulranges(eldef)};
#undef eldef

const int n_script_table = sizeof script_table / sizeof * script_table;

const struct code_page_table code_page_table[] = {
	  {0, "CP1252", "Latin-1"},
	  {1, "CP1250", "Latin-2"},
	  {2, "CP1251", "Cyrillic"},
	  {3, "CP1253", "Greek"},
	  {4, "CP1254", "Turkish"},
	  {5, "CP1255", "Hebrew"},
	  {6, "CP1256", "Arabic"},
	  {7, "CP1257", "Windows Baltic"},
	  {8, "CP1258", "Vietnamese"},
	  {16, "CP874", "Thai"},
	  {17, "CP932", "JIS"},
	  {18, "CP936", "simplified Chinese"},
	  {18, "CP936", "Simplified Chinese"},
	  {19, "CP949", "Korean Wansung"},
	  {20, "CP950", "traditional Chinese"},
	  {20, "CP950", "Traditional Chinese"},
	  {21, "CP1361", "Korean Johab"},
	  {29, "MacRoman", "US Roman"},
	  {30, "OEM", "OEM Character Set"},
	  {31, "Symbol Character Set", "Symbol"},
	  {48, "CP869", "IBM Greek"},
	  {49, "CP866", "MS-DOS Russian"},
	  {50, "CP865", "MS-DOS Nordic"},
	  {51, "CP864", "Arabic"},
	  {52, "CP863", "MS-DOS Canadian French"},
	  {52, "CP863", "FrCanadian"},
	  {53, "CP862", "Hebrew"},
	  {54, "CP861", "MS-DOS Icelandic"},
	  {55, "CP860", "MS-DOS Portuguese"},
	  {56, "CP857", "IBM Turkish"},
	  {57, "CP855", "IBM Cyrillic"},
	  {58, "CP852", "CP852"},  /* Latin 2 */
	  {59, "CP775", "MS-DOS Baltic"},
	  {60, "CP737", "CP737"},  /* Greek; former 437 G */
	  {61, "CP708", "Arabic"}, /* ASMO 708 */
	  {62, "CP850", "CP850"},  /* WE/Latin 1 */
	  {63, "CP437", "US"}};
const int n_code_page_table = sizeof code_page_table / sizeof * code_page_table;

const char *ordered_mac_name[] = {
#if 0 /* old version, no longer believed */
    "",
    "",
    "",
    "space",
    "exclam",
    "quotedbl",
    "numbersign",
    "dollar",
    "percent",
    "ampersand",
    "quotesingle",
    "parenleft",
    "parenright",
    "asterisk",
    "plus",
    "comma",
    "hyphen",
    "period",
    "slash",
    "zero",
    "one",
    "two",
    "three",
    "four",
    "five",
    "six",
    "seven",
    "eight",
    "nine",
    "colon",
    "semicolon",
    "less",
    "equal",
    "greater",
    "question",
    "at",
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "H",
    "I",
    "J",
    "K",
    "L",
    "M",
    "N",
    "O",
    "P",
    "Q",
    "R",
    "S",
    "T",
    "U",
    "V",
    "W",
    "X",
    "Y",
    "Z",
    "bracketleft",
    "backslash",
    "bracketright",
    "asciicircum",
    "underscore",
    "grave",
    "a",
    "b",
    "c",
    "d",
    "e",
    "f",
    "g",
    "h",
    "i",
    "j",
    "k",
    "l",
    "m",
    "n",
    "o",
    "p",
    "q",
    "r",
    "s",
    "t",
    "u",
    "v",
    "w",
    "x",
    "y",
    "z",
    "braceleft",
    "bar",
    "braceright",
    "asciitilde",
    "Adieresis",
    "Aring",
    "Ccedilla",
    "Eacute",
    "Ntilde",
    "multiply",
    "Udieresis",
    "aacute",
    "agrave",
    "acircumflex",
    "adieresis",
    "atilde",
    "aring",
    "ccedilla",
    "eacute",
    "egrave",
    "ecircumflex",
    "edieresis",
    "iacute",
    "igrave",
    "icircumflex",
    "eth",
    "ntilde",
    "oacute",
    "ograve",
    "ocircumflex",
    "odieresis",
    "otilde",
    "uacute",
    "ugrave",
    "ucircumflex",
    "udieresis",
    "dagger",
    "degree",
    "cent",
    "sterling",
    "section",
    "bullet",
    "paragraph",
    "germandbls",
    "registered",
    "copyright",
    "trademark",
    "acute",
    "dieresis",
    "notequal",
    "AE",
    "Oslash",
    "infinity",
    "plusminus",
    "lessequal",
    "greaterequal",
    "brokenbar",
    "mu",
    "partialdiff",
    "summation",
    "product",
    "pi",
    "integral",
    "ordfeminine",
    "ordmasculine",
    "Omega",
    "ae",
    "oslash",
    "questiondown",
    "exclamdown",
    "logicalnot",
    "radical",
    "florin",
    "approxequal",
    "Delta",
    "guillemotleft",
    "guillemotright",
    "ellipsis",
    "space",
    "Agrave",
    "Atilde",
    "Otilde",
    "OE",
    "oe",
    "endash",
    "emdash",
    "quotedblleft",
    "quotedblright",
    "quoteleft",
    "quoteright",
    "divide",
    "lozenge",
    "ydieresis",
    "Ydieresis",
    "fraction",
    "currency",
    "guilsinglleft",
    "guilsinglright",
    "fi",
    "fl",
    "daggerdbl",
    "periodcentered",
    "quotereversed",
    "quotedblbase",
    "perthousand",
    "Acircumflex",
    "Ecircumflex",
    "Aacute",
    "Edieresis",
    "Egrave",
    "Iacute",
    "Icircumflex",
    "Eth",
    "Igrave",
    "Oacute",
    "Ocircumflex",
    "",
    "Ograve",
    "Uacute",
    "Ucircumflex",
    "Ugrave",
    "dotlessi",
    "circumflex",
    "",
    "macron",
    "breve",
    "dotaccent",
    "ring",
    "cedilla",
    "hungarumlaut",
    "ogonek",
    "caron",
    "Lslash",
    "lslash",
    "Scaron",
    "scaron",
    "Zcaron",
    "zcaron",
    "",
    "",
    "",
    "Yacute",
    "yacute",
    "Thorn",
    "thorn",
    "minus",
    "",
    "onesuperior",
    "twosuperior",
    "threesuperior",
    "onehalf",
    "onequarter",
    "threequarters",
    "franc",
    "Gbreve",
    "gbreve",
    "Idotaccent",
    "Scedilla",
    "scedilla",
    "Cacute",
    "cacute",
    "Ccaron",
    "ccaron",
    "dcroat",
#else
    ".notdef",
    "null",
    "nonmarkingreturn", /* First 3 from old TT spec. */
    "space",
    "exclam",
    "quotedbl",
    "numbersign",
    "dollar",
    "percent",
    "ampersand",
    "quotesingle",
    "parenleft",
    "parenright",
    "asterisk",
    "plus",
    "comma",
    "hyphen",
    "period",
    "slash",
    "zero",
    "one",
    "two",
    "three",
    "four",
    "five",
    "six",
    "seven",
    "eight",
    "nine",
    "colon",
    "semicolon",
    "less",
    "equal",
    "greater",
    "question",
    "at",
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "H",
    "I",
    "J",
    "K",
    "L",
    "M",
    "N",
    "O",
    "P",
    "Q",
    "R",
    "S",
    "T",
    "U",
    "V",
    "W",
    "X",
    "Y",
    "Z",
    "bracketleft",
    "backslash",
    "bracketright",
    "asciicircum",
    "underscore",
    "grave",
    "a",
    "b",
    "c",
    "d",
    "e",
    "f",
    "g",
    "h",
    "i",
    "j",
    "k",
    "l",
    "m",
    "n",
    "o",
    "p",
    "q",
    "r",
    "s",
    "t",
    "u",
    "v",
    "w",
    "x",
    "y",
    "z",
    "braceleft",
    "bar",
    "braceright",
    "asciitilde",
    "Adieresis",
    "Aring",
    "Ccedilla",
    "Eacute",
    "Ntilde",
    "Odieresis",
    "Udieresis",
    "aacute",
    "agrave",
    "acircumflex",
    "adieresis",
    "atilde",
    "aring",
    "ccedilla",
    "eacute",
    "egrave",
    "ecircumflex",
    "edieresis",
    "iacute",
    "igrave",
    "icircumflex",
    "idieresis",
    "ntilde",
    "oacute",
    "ograve",
    "ocircumflex",
    "odieresis",
    "otilde",
    "uacute",
    "ugrave",
    "ucircumflex",
    "udieresis",
    "dagger",
    "degree",
    "cent",
    "sterling",
    "section",
    "bullet",
    "paragraph",
    "germandbls",
    "registered",
    "copyright",
    "trademark",
    "acute",
    "dieresis",
    "notequal",
    "AE",
    "Oslash",
    "infinity",
    "plusminus",
    "lessequal",
    "greaterequal",
    "yen",
    "mu",
    "partialdiff",
    "summation",
    "product",
    "pi",
    "integral",
    "ordfeminine",
    "ordmasculine",
    "Omega",
    "ae",
    "oslash",
    "questiondown",
    "exclamdown",
    "logicalnot",
    "radical",
    "florin",
    "approxequal",
    "Delta",
    "guillemotleft",
    "guillemotright",
    "ellipsis",
    "nonbreakingspace", /* From TTX and an old TT spec */
    "Agrave",
    "Atilde",
    "Otilde",
    "OE",
    "oe",
    "endash",
    "emdash",
    "quotedblleft",
    "quotedblright",
    "quoteleft",
    "quoteright",
    "divide",
    "lozenge",
    "ydieresis",
    "Ydieresis",
    "fraction",
    "currency",
    "guilsinglleft",
    "guilsinglright",
    "fi",
    "fl",
    "daggerdbl",
    "periodcentered",
    "quotesinglbase",
    "quotedblbase",
    "perthousand",
    "Acircumflex",
    "Ecircumflex",
    "Aacute",
    "Edieresis",
    "Egrave",
    "Iacute",
    "Icircumflex",
    "Idieresis",
    "Igrave",
    "Oacute",
    "Ocircumflex",
    "apple",
    "Ograve",
    "Uacute",
    "Ucircumflex",
    "Ugrave",
    "dotlessi",
    "circumflex",
    "tilde",
    "macron",
    "breve",
    "dotaccent",
    "ring",
    "cedilla",
    "hungarumlaut",
    "ogonek",
    "caron",
    "Lslash",
    "lslash",
    "Scaron",
    "scaron",
    "Zcaron",
    "zcaron",
    "brokenbar",
    "Eth",
    "eth",
    "Yacute",
    "yacute",
    "Thorn",
    "thorn",
    "minus",
    "multiply",
    "onesuperior",
    "twosuperior",
    "threesuperior",
    "onehalf",
    "onequarter",
    "threequarters",
    "franc",
    "Gbreve",
    "gbreve",
    "Idotaccent",
    "Scedilla",
    "scedilla",
    "Cacute",
    "cacute",
    "Ccaron",
    "ccaron",
    "dcroat",
#endif
    "extra"
};
