/*
Program to insert checksums in a TrueType file
Copyright (C) 2018      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#define WHAT                            \
"OpenType checksum applier.  Copyright (C) 2020 Richard Wordingham\n"

#define WARRANTY \
"This program comes with ABSOLUTELY NO WARRANTY; for details\n" \
"run with option '-w'."

#define COPYING \
"This is free software, and you are welcome\n" \
"to redistribute it under certain conditions; for details \n" \
"run with option '-c'."

#define WELCOME WHAT WARRANTY "  " COPYING "\n"
#define WO WHAT WARRANTY "\n\n"
#define CO WHAT COPYING "\n\n"

#include <stdio.h>
#include <string.h>
#include "wordplot.h"
#include "opcodes.h"

int sum_UW(unsigned short int *arr, int n);
void* element(void *base, int index);
int advance_coords(int force_advance);

#include "oft_io_globals.h"

#include "manip_structure.h"
#include "oft.h"
#include "manip_befile.h"
#include "manip_get.h"
#include "oft.h"
#include "manip_put.h"
#include "oft.h"
#include "manip_print.h"
#include "manip_display.h"
#include "oft.h"
#include "manip_free.h"
#include "oft.h"

int advance_coords(int force_advance) {
  while (glyph.n_to_go > 0) {
#define need_next\
    if (!glyph.flag_ptr->_next) {\
      fprintf(stderr, "%s", "advance_coords: Mismatch between "\
	      "glyph flags and coordinates.\n");\
      exit(1);\
    }
    if (force_advance) {
      need_next;
      glyph.flag_ptr = glyph.flag_ptr->_next;
    }
    force_advance = 1;
    glyph.flag_is_count = 0;
    glyph.flags = glyph.flag_ptr->val;
    glyph.repeat_count = 1;
    if ((glyph.flags & glyph.short_xy) || !(glyph.flags & glyph.same)) {
      return glyph.n_to_go;
    }
    if (glyph.flags & 8) {
      need_next;
      glyph.flag_ptr = glyph.flag_ptr->_next;
      glyph.repeat_count = 1 + glyph.flag_ptr->val;
    } else {
      glyph.repeat_count = 1;
    }
    glyph.n_to_go -= glyph.repeat_count;
  }
  return glyph.n_to_go;
}
/* Globals for extra communication of array bounds and the like. */
int hhea_numberofHMetrics;
unsigned short int VALUE_FORMAT, VALUE_FORMAT1, VALUE_FORMAT2;
int CVT_SIZE, LONG_OFFSETS, PREP_SIZE, FPGM_SIZE, N_GLYPHS;
int mark_lig_class_count, class_2_count, base_record_class_count;
int class_count;

int sum_UW(unsigned short int *arr, int n) {
  int total = 0;
  for (; n; n--) total += *arr++;
  return total;
}

int max_UW(unsigned short int *arr, int n) {
  int the_max, i;

  the_max = arr[0];
  for (i=1; i<n; i++) {
    if (arr[i] > the_max) the_max = arr[i];
  }
  return the_max;
}

#define min(x, y) ((x)<(y)?(x):(y))
#define max(x, y) ((x)>(y)?(x):(y))
struct glyph_globals glyph;

/* Byte swapping flag */
int swap_order;

/* Array access function */
void* element(void *base, int index) {
  void *loc;
  for (loc = base; index; index--) loc = *(void**) loc;
  return loc;
}

int byte_dump_count;

/* Get and display head table */
void display_head(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *head_table;
  int pos, junk;
  struct font_header fontHeader;

  for (head_table = top.table_directory; head_table;
       head_table = head_table->_next) {
    if (!strncmp(head_table->tag, "head", 4)) break;
  }
  if (head_table) {
    pos = head_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking font header table");
      exit(1);
    }
    get_bine_font_header(fp, &fontHeader, &junk, &pos);
    /*    print_font_header(stdout, &fontHeader, &junk, "head", 0); */
  }
}

int good_long(int longun) {
  if (swap_order) {
    return flip4(longun);
  } else {
    return (longun);
  }
}

static void synopsis(FILE *dst) {
  fputs("Usage:\n\tfix_checksum [-s] font\nor\n\tfix_checksum -[scw]\n"
        "Options:\n"
        "\tc\tTell restrictions on copying.\n"
        "\ts\tSuppress welcome message.\n"
        "\tw\tDisplay warranty.\n", dst);
}

int main(int argc, char *argv[]) {
  FILE *fp;
  int pos, junk, itab, have_font, iarg, badargs;
  char *source = 0;
  char *flagptr;
  struct offset_table_header top2;
  struct offset_table top;
  struct maximum_profile maximumProfile;
  struct directory_entry *table;
  struct font_header fontHeader;
  int *whole_table, table_size, n_got, total_sum, sum, k, n_tables;
  int head_pos = 0;
  int tell_c, tell_w, suppress, gonbad;

  tell_c = tell_w = suppress = gonbad = 0;
  for (iarg = 1; iarg < argc; iarg++) {
    if ('-' == *argv[iarg]) {
      const char *optch;
      for (optch = 1 + argv[iarg]; *optch; optch++) {
        switch (*optch) {
        case 'c':
          tell_c = 1;
          break;
        case 's':
          suppress = 1;
          break;
        case 'w':
          tell_w = 1;
          break;
        default:
          fprintf(stderr, "Option \"%c\" not recognised.\n", *optch);
          gonbad = 1;
        }
      }
    } else if (source) {
      fprintf(stderr, "Too many arguments.\n");
      gonbad = 1;
    } else {
      source = argv[iarg];
    }
  }
  if (tell_c || tell_w) {
    if (suppress) {
      /* Totally suppress the welcome message. */
    } else if (tell_c) {
      if (tell_w) {
        fputs(WHAT "\n", stdout);
      } else {
        fputs(WO, stdout);
      }
    } else if (tell_w) {
      fputs(CO, stdout);
    } else {
      fputs(WELCOME, stdout);
    }
    if (tell_w) {
          /* Clauses 15 & 16 of GNU GPL Version 3: */
      fprintf(stdout,
              "THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED\n"
              "BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING\n"
              "THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM\n"
              "“AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR\n"
              "IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES\n"
              "OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.\n"
              "THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE\n"
              "PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE,\n"
              "YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR\n"
              "OR CORRECTION.\n\n"
              
              "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO\n"
              "IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO\n"
              "MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE,\n"
              "BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,\n"
              "INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE\n"
              "OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n"
              "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES\n"
              "SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM\n"
              "TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER\n"
              "OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH\n"
              "DAMAGES.\n"
                  );
    }
    if (tell_c) {
      fprintf(stdout,
              "This program is licensed under Version 3 or later of the\n"
              "GNU General Public Licence as published by the\n"
              "Free Software Foundation.\n\n"

              "You may convey verbatim copies of the Program's source code\n"
              "as you receive it, in any medium, provided that you\n"
              "conspicuously and appropriately publish on each copy\n"
              "an appropriate copyright notice; keep intact all notices\n"
              "stating that the License and any non-permissive terms added\n"
              "in accord with section 7 apply to the code; keep intact\n"
              "all notices of the absence of any warranty; and give all\n"
              "recipients a copy of this License along with the Program.\n"
              );
    }
    if (gonbad || source) {
      synopsis(stderr);
      return 1;
    } else {
      return 0;
    }
  }
  if (!suppress) fputs(WELCOME, stdout);
  if (!source) {
    fprintf(stderr, "Insufficient arguments.\n");
    gonbad = 1;
  }
  if (gonbad) {
    synopsis(stderr);
    return 1;
  }
  fp = fopen(source, "rb+");
  if (!fp) perror(source);
  if (!fp) return 1;
  swap_order = 0;
  get_bine_offset_table_header(fp, &top2, &junk, &pos);
  if ((top2.entrySelector & 0xff) == 0 &&
      (top2.entrySelector & 0xff00) != 0) {
    /* Wrong way round! */
    swap_order = 1;
    fseek(fp, 0, SEEK_SET);
    if (1 == iarg) printf("Fixing endianity and re-reading.\n");
  } else if ((top2.entrySelector & 0xff) != 0 &&
	     (top2.entrySelector & 0xff00) == 0) {
    if (1 == iarg) printf("File is in machine's natural order!\n");
  } else {
    printf("Entry selector appears to be corrupt!");
    return 1;
  }
  fseek(fp, 0, SEEK_SET);
  get_bine_offset_table(fp, &top, &junk, &pos);
  if (0x4f54544f == top.sfnt_version) { /* OTTO */
    /* No problem */
  } else if (top.sfnt_version != 0x00010000) {
    printf("Format is %8.8X, not 1.0\n", top.sfnt_version);
    return 1;
  }
  total_sum = 0;
  n_tables = 0;
  /* Calculate check sums for the tables */
  for (table = top.table_directory; table;
       table = table->_next) {
    n_tables++;
    table_size = (table->length + 3) / 4;
    whole_table = malloc(4 * table_size);
    if (whole_table) {
      if (fseek(fp, table->offset, SEEK_SET)) {
	fprintf(stderr, "%.4s: ", table->tag);
	perror("Seeking a font table");
      }
      n_got = fread(whole_table, 4, table_size, fp);
      if (n_got != table_size) {
	fprintf(stdout, "Only got %d (instead of %d) longwords for %.4s\n",
		n_got, table_size, table->tag);
	return 1;
      } else {
	sum = 0;
	for (k = 0; k < table_size; k++) sum += good_long(whole_table[k]);
	if (!strncmp(table->tag, "head", 4)) {
	  if (fseek(fp, table->offset, SEEK_SET)) {
	    perror("Seeking font header table");
	    exit(1);
	  }
	  head_pos = table->offset;
	  get_bine_font_header(fp, &fontHeader, &junk, &pos);
	  sum -= fontHeader.checkSumAdjustment;
	}
	table->checkSum = sum;
      }
      total_sum += sum;
      free(whole_table);
    } else {
      fprintf(stderr, "Malloc failed for %.4s\n", table->tag);
    }
  }
  if (!head_pos) {
    fprintf(stderr, "No header table found!\n");
    exit(1);
  }
  /* Update the index tables */
  if (fseek(fp, 0, SEEK_SET)) {
    perror("Rewinding font.");
    exit(1);
  }
  pos = 0;
  /*  print_offset_table(stdout, &top, &junk, "offset", 0); */
  put_bine_offset_table(fp, &top, &junk, &pos);
  fflush(fp);
  /* Read them into contiguous memory and calculate grand checksum */
  table_size = 3 + 4 * n_tables;
  whole_table = malloc(4 * table_size);
  if (fseek(fp, 0, SEEK_SET)) {
    perror("Seeking index tables");
    exit(1);
  }
  n_got = fread(whole_table, 4, table_size, fp);
  if (n_got != table_size) {
    fprintf(stdout, "Only got %d (instead of %d) longwords for "
	    "index tables\n",
	    n_got, table_size);
    exit(1);
  }
  sum = 0;
  for (k = 0; k < table_size; k++) sum += good_long(whole_table[k]);
  total_sum += sum;
  free(whole_table);
  /* Update header table. */
  fontHeader.checkSumAdjustment = 0xb1b0afba - total_sum;
  if (fseek(fp, head_pos, SEEK_SET)) {
    perror("Rewriting header table");
    exit(1);
  }
  pos = head_pos;
  put_bine_font_header(fp, &fontHeader, &junk, &pos);
  fclose(fp);
  return 0;
}
