/*
Merge sort in place using working space.
Copyright (C) 2001-2005      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*//*

----------------------------------------------------------------------
   History as IASORT:
   Version 1.0 : Creator J.R.Wordingham  Date 22 January 2001

   Modification History :

   Ver. Ref.     Date       By      Comments
    ?   None  05-Nov-05  JRW  Converted to C.
   1.0    -   13-Jun-20  JRW  GNU GPL 2.0
   1.1    -   13-Jun-20  JRW  GNU GPL 3.0
   1.2    -   13-Jun-20  JRW  Really GNU GPL 3.0
----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/* N.B. DTYPE must be defined by some macro! */
void ipsort(int cmp(DTYPE, DTYPE), DTYPE list[], int n) {
  int ii, j, k, e1, e2, i1, i2, pos, nss, e0;
  DTYPE tmp;
  DTYPE *work;
  int swapt;
  /* Get storage for a copy of list[] */
  if (!(work = malloc(n * sizeof(DTYPE)))) {
    fprintf(stderr, "IASORT: Memory exhausted.\n");
    exit(1);
  }
#define NBS 4

/* Bubble sort each group of NBS elements. */
  for (k = 0; k < n; k+=NBS) {
/* Determine upper bound over which preliminary sort will be performed. */
    e0 = k + NBS;
    if (e0 > n) e0 = n;
/* Bubble sort from K to E0 */
    for (j = e0 - 2; j >= k; j--) {
      swapt = 0;
      for (ii = k; ii <= j; ii++) {
	if (cmp(list[ii],list[ii+1]) > 0) {
	  tmp = list[ii+1];
	  list[ii+1] = list[ii];
	  list[ii] = tmp;
	  swapt = 1;
	}
      }
      if (!swapt) break; /* In order! */
    }
  }
/* Now successively sort pairs of sorted sequences. */
  nss = NBS;
  while (nss < n) {
    memcpy(work, list, n * sizeof(list[0]));
    for (k = 0; k < n; k += 2 * nss) {
/* Calculate end of first input */
      e1 = k + nss;
/* No merging if no partner */
      if (e1 >= n) break;
/* Calculate end of second input */
      e2 = e1 + nss;
      if (e2 >= n) e2 = n;
/* Set up starts of lists to be compared */
      i1 = k;
      i2 = e1;
/* Merge the lists */
      for (pos = k; pos < e2; pos++) {
	if (i1 >= e1) {
	  list[pos] = work[i2++];
	} else if (i2 >= e2) {
	  list[pos] = work[i1++];
	} else if (cmp(work[i1], work[i2]) <= 0) {
	  list[pos] = work[i1++];
	} else {
	  list[pos] = work[i2++];
	}
      }
    }
    nss *= 2;
  }
  free(work);
}
