/*
Load Unicode properties from defining files,
Copyright (C) 2007-2014      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "map.h"
#include "properties.h"
#define UNIDATA "UnicodeData.txt"
#include "unidata.h"

space unicode_names;
space unicode_dirs;

/* Performs simple substitutions */
static void convert(const char *source, const char *rules[], char *dest) {
  char *input, *output, *temp, *start;
  char buffer[128];
  int  offset;

  input = strcpy(dest, source);
  output = buffer;
  for (;*rules;rules+=2) {
    offset = 0;
    while(start = strstr(input + offset, *rules)) {
      *start = 0;
      strcpy(output, input);
      strcat(output, *(rules+1));
      strcat(output, start + strlen(*rules));
      offset = start - input + strlen(*(rules+1));
      temp = output;
      output = input;
      input = temp;
    }
  }
  if (input != dest) strcpy(dest, input);
}

void load_properties() {
  FILE *fp;
  int onch, repl, cpt;
#define MAXLEN 500
#define N_FIELDS 15
  int max_length, field_no, max_fields, codepoint, nval, result;
  char *field_end, *name, *curpos;
  void *dirn;
  int direction;
  char field[MAXLEN], modif[MAXLEN];
  char *source_name, *env_value;
  struct map dir_names;
  const char *changes1[] = {
    " DOUBLE GRAVE",                 " 2GRAVE",
    " DOUBLE ACUTE",                 " 2ACUTE",
    " WITH SMALL ",                  "_SMALL ",
    " WITH TAIL",                    "_TAILED",
    " WITH ",                        "_",
    " AND ",                         "_",
    "CAPITAL LETTER AFRICAN ",       "AFRICAN CAPITAL LETTER ",
    "SMALL LETTER AFRICAN ",         "AFRICAN SMALL LETTER ",
    "CAPITAL LETTER OPEN ",          "OPEN CAPITAL LETTER ",
    "CAPITAL LETTER REVERSED OPEN ", "REVERSED OPEN CAPITAL LETTER ",
    "CAPITAL LETTER CLOSED REVERSED OPEN ",
                                     "CLOSED REVERSED OPEN CAPITAL LETTER ",
    "SMALL LETTER OPEN ",            "OPEN SMALL LETTER ",
    "SMALL LETTER REVERSED OPEN ",   "REVERSED OPEN SMALL LETTER ",
    "SMALL LETTER CLOSED REVERSED OPEN ",
                                     "CLOSED REVERSED OPEN SMALL LETTER ",
    "INVERTED BREVE",                "INVBREVE",
    "SMALL CAPITAL",                 "SMALLCAP",
    "CAPITAL LETTER TURNED ",        "TURNED CAPITAL LETTER ",
    "SMALL LETTER TURNED ",          "TURNED SMALL LETTER ",
    "MODIFIER LETTER SMALL TURNED ", "MODIFIER TURNED LETTER SMALL ",
    "CAPITAL LETTER ABKHASIAN ",     "ABKHASIAN CAPITAL LETTER ",
    "SMALL LETTER ABKHASIAN ",       "ABKHASIAN SMALL LETTER ",
    "CAPITAL LETTER KOMI ",          "KOMI CAPITAL LETTER ",
    "SMALL LETTER KOMI ",            "KOMI SMALL LETTER ",
    "CAPITAL LETTER OLD COPTIC ",    "OLD COPTIC CAPITAL LETTER ",
    "SMALL LETTER OLD COPTIC ",      "OLD COPTIC SMALL LETTER ",
    "CAPITAL LETTER OLD NUBIAN ",    "OLD NUBIAN CAPITAL LETTER ",
    "SMALL LETTER OLD NUBIAN ",      "OLD NUBIAN SMALL LETTER ",
    "CAPITAL LETTER LONG ",          "LONG CAPITAL LETTER ",
    "SMALL LETTER LONG ",            "LONG SMALL LETTER ",
    "CAPITAL LETTER SHORT ",         "SHORT CAPITAL LETTER ",
    "SMALL LETTER SHORT ",           "SHORT SMALL LETTER ",
    "PALATAL HOOK",                  "PALATAL_HOOK",
    "RETROFLEX HOOK",                "RETROFLEX_HOOK",
    "DOT ABOVE",                     "DOTABOVE",
    "DOT BELOW",                     "DOTBELOW",
    "LINE BELOW",                    "LINE_BELOW",
    "LONG LEG",                      "LONG_LEG",
    "COMMA BELOW",                   "COMMA_BELOW",
    "SMALL DOTLESS",                 "DOTLESS SMALL",
    "THETA SYMBOL",                  "SMALL THETA SYMBOL",
    "LETTER WYNN",                   "SMALL LETTER WYNN",
    "LETTER GLOTTAL STOP",           "SMALL LETTER GLOTTAL STOP",
    "CAPITAL SMALL ",                "CAPITAL ",
    "DOTLESS I",                     "I DOTLESS",
    "DOTLESS J",                     "J DOTLESS",
    "BIG YUS",                       "BIG_YUS",
    "LITTLE YUS",                    "LITTLE_YUS",
    "STRAIGHT U",                    "STRAIGHT_U",
    "GLOTTAL STOP",                  "GLOTTAL_STOP",
    "SMALL REVERSED GLOTTAL_STOP",   "SMALL_REVERSED GLOTTAL_STOP",
    0, 0};
  const char *changes2[] = {
    "SMALL THETA",       "theta", /* Often insufficient cue! */
    "SMALL LETTER ",     "",
    "LETTER SMALL ",     "",
    "CAPITAL LETTER ",   "",
    "SMALL LIGATURE ",   "",
    "CAPITAL LIGATURE ", "",
    "GEORGIAN LETTER",   "GEORGIAN_LETTER", /* Sort out Georgian case! */
    " LETTER ",          " ",
    "BOLD SMALL",        "BOLD",
    "BOLD CAPITAL",      "BOLD",
    "ITALIC SMALL",      "ITALIC",
    "ITALIC CAPITAL",    "ITALIC",
    "SCRIPT SMALL",      "SCRIPT",
    "SCRIPT CAPITAL",    "SCRIPT",
    "FRAKTUR SMALL",     "FRAKTUR",
    "FRAKTUR CAPITAL",   "FRAKTUR",
    "STRUCK SMALL",      "STRUCK",
    "STRUCK CAPITAL",    "STRUCK",
    "SERIF SMALL",       "SERIF",
    "SERIF CAPITAL",     "SERIF",
    "MONOSPACE SMALL",   "MONOSPACE",
    "MONOSPACE CAPITAL", "MONOSPACE",
    " DIGIT ",           " ",
    " ONE",              " 1",
    " TWO",              " 2",
    " THREE",            " 3",
    " FOUR",             " 4",
    " FIVE",             " 5",
    " SIX",              " 6",
    " SEVEN",            " 7",
    " EIGHT",            " 8",
    " NINE",             " 9",
    " ZERO",             " 0",
    " INITIAL FORM",     " INITIAL",
    " MEDIAL FORM",      " MEDIAL",
    " FINAL FORM",       " FINAL",
    " ISOLATED FORM",    " ISOLATED",
    " LETTER ",          " ",
    " CHARACTER ",       " ",
    " LIGATURE ",        " ",
    " SARA ",            " ",
    /*    "SEONG FILLER",      "SEONG_FILLER",*/
    /*    " CHOSEONG ",        " ",*/
    /*    " JUNGSEONG ",       " ",*/
    " SYLLABLE ",        " ",
    " SYLLABICS ",       " ",
    "FUNCTIONAL SYMBOL", "",
    "PATTERN ",          " ",
    "YI RADICAL",        "YI_RADICAL",
    " RADICAL ",         " ",
    " BYELORUSSIAN UKRAINIAN ", " UKRAINIAN ",
    "CYRILLIC UKRAINIAN", "UKRAINIAN",
    "CYRILLIC ABKHASIAN", "ABKHASIAN",
    "SYRIAC PERSIAN",     "SYRIAC",
    "SYRIAC SOGDIAN",     "SOGDIAN",
    "CANADIAN BLACKFOOT", "BLACKFOOT",
    "CANADIAN SAYISI",    "SAYISI",
    "CANADIAN CARRIER",   "CARRIER",
    "MONGOLIAN MANCHU",   "MANCHU",
    "COPTIC OLD COPTIC",  "OLD COPTIC",
    "COPTIC OLD NUBIAN",  "OLD NUBIAN",
    "MUSICAL SYMBOL",     "MUSICAL",
    0, 0};

  memset(&unicode_names, 0, sizeof(unicode_names));
  memset(&unicode_dirs,  0, sizeof(unicode_dirs));
  env_value = getenv("UNIDATA");
  source_name = env_value ? env_value : UNIDATA; 
  fp = fopen(source_name, "r");
  if (!fp) {
    if (!env_value) {
      fprintf(stderr,
	      "You may set up environmental variable UNIDATA to designate "
	      "local copy of UnicodeData.txt.\n");
    } else {
      fprintf(stderr, "The following filename is defiend by environment "
	      "variable UNIDATA.\n");
    }
    perror(source_name);
    exit(1);
  }
/* Bidi data is for Unicode 4.1. */
  create_map(&dir_names);
  insert_in_map(&dir_names, "L",   (void*)K_DIR_L);
  insert_in_map(&dir_names, "AL",  (void*)K_DIR_AL);
  insert_in_map(&dir_names, "R",   (void*)K_DIR_R);
  insert_in_map(&dir_names, "NSM", (void*)K_DIR_NSM);
  insert_in_map(&dir_names, "ON",  (void*)K_DIR_ON);
  insert_in_map(&dir_names, "EN",  (void*)K_DIR_EN);
  insert_in_map(&dir_names, "ES",  (void*)K_DIR_ES);
  insert_in_map(&dir_names, "ET",  (void*)K_DIR_ET);
  insert_in_map(&dir_names, "AN",  (void*)K_DIR_AN);
  insert_in_map(&dir_names, "CS",  (void*)K_DIR_CS);
  insert_in_map(&dir_names, "BN",  (void*)K_DIR_BN);
  insert_in_map(&dir_names, "B",   (void*)K_DIR_B);
  insert_in_map(&dir_names, "S",   (void*)K_DIR_S);
  insert_in_map(&dir_names, "LRI", (void*)K_DIR_LRI);
  insert_in_map(&dir_names, "RLI", (void*)K_DIR_RLI);
  insert_in_map(&dir_names, "FSI", (void*)K_DIR_FSI);
  insert_in_map(&dir_names, "PDI", (void*)K_DIR_PDI);
#if 1 /* Set to zero to check internal check! */
  insert_in_map(&dir_names, "WS",  (void*)K_DIR_WS);
#endif
  insert_in_map(&dir_names, "LRE", (void*)K_DIR_LRE);
  insert_in_map(&dir_names, "LRO", (void*)K_DIR_LRO);
  insert_in_map(&dir_names, "RLE", (void*)K_DIR_RLE);
  insert_in_map(&dir_names, "RLO", (void*)K_DIR_RLO);
  insert_in_map(&dir_names, "PDF", (void*)K_DIR_PDF);
/* Set up the R to L block defaults */
/* The unassigned characters that default to R are:
   Hebrew, Cypriot_Syllabary, Kharoshthi, and the ranges \u07C0-\u08FF
   \uFB1D-\uFB4F \U00010840-\U000109FF \U00010A60-\U00010FFF
*/
/* Hebrew: */
  set_property(&unicode_dirs, 0x05d0, (void*)K_DIR_R, (void*)K_DIR_R);
/* But not Armenian! */
  for (cpt = 0x0580; onch < 0x0590; onch++) {
    set_property(&unicode_dirs, cpt, (void*)K_DIR_L, (void*)K_DIR_L);
  }
  /* Cypriot and U+10840 to U+109ff*/
  set_property(&unicode_dirs, 0x10800, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10880, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10900, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10980, (void*)K_DIR_R, (void*)K_DIR_R);
  /* Kharoshthi and U+10A60 to U+10FFF */
  set_property(&unicode_dirs, 0x10a00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10a80, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10b00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10b80, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10c00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10c80, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10d00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10d80, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10e00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10e80, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10f00, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x10f80, (void*)K_DIR_R, (void*)K_DIR_R);
  /* U+07C0 to U+08FF, and do Thaana - which is ** AL ** */
  for (cpt = 0x07c0; cpt < 0x0800; cpt++) {
    set_property(&unicode_dirs, cpt, (void*)K_DIR_R, (void*)K_DIR_AL);
  }
  set_property(&unicode_dirs, 0x0800, (void*)K_DIR_R, (void*)K_DIR_R);
  set_property(&unicode_dirs, 0x0880, (void*)K_DIR_R, (void*)K_DIR_R);
  for (cpt = 0xfb1d; cpt <= 0xfb4f; cpt++) {
    set_property(&unicode_dirs, cpt, (void*)K_DIR_R, (void*)K_DIR_R);
  }
/* The unassigned characters that default to AL are:
   Arabic, Syriac, Arabic_Supplement, Thaana, Arabic_Presentation_Forms_A,
   Arabic_Presentation_Forms_B, minus the Noncharacter_Code_Points */
/* Arabic, Syriac, Arabic supplement */
  set_property(&unicode_dirs, 0x0600, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0x0680, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0x0700, (void*)K_DIR_AL, (void*)K_DIR_AL);
/* Arabic presentation forms A - preceded by left to right! */
  for (cpt = 0xfb50; cpt <= 0xfb7f; cpt++) {
    set_property(&unicode_dirs, cpt, (void*)K_DIR_AL, (void*)K_DIR_L);
  }
  set_property(&unicode_dirs, 0xfb80, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0xfc00, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0xfc80, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0xfd00, (void*)K_DIR_AL, (void*)K_DIR_AL);
  set_property(&unicode_dirs, 0xfd80, (void*)K_DIR_AL, (void*)K_DIR_AL);
/* Arabic presentation forms-B - preceded by left to right! */
/* U+0FE75 is only undefined codepoint of this block in U+FE70 to U+FE7F. */
  set_property(&unicode_dirs, 0xfe75, (void*)K_DIR_AL, (void*)K_DIR_L);
  set_property(&unicode_dirs, 0xfe80, (void*)K_DIR_AL, (void*)K_DIR_AL);
/* Exclude non-characters - left-to-right! */
  for (cpt = 0xfdd0; cpt <= 0xfdef; cpt++) {
    set_property(&unicode_dirs, cpt, (void*)K_DIR_L, (void*)K_DIR_AL);
  }
/* Set up the general default */
  set_property_default(&unicode_dirs, (void*) K_DIR_L);

  max_fields = 0;
  field[0] = 0;
  field_end = field;
  field_no = 0;
  while (EOF != (onch = fgetc(fp))) {
    if (';' == onch || '\n' == onch) {
      *field_end = 0;
      switch (field_no) {
      case 0:
	nval = sscanf(field, "%x", &codepoint);
	if (1 != nval) {
	  fprintf(stdout, "Cannot interpret codepoint value \"%s\".\n", field);
	  codepoint = 0;
	}
	break;
      case 1:
	if (codepoint && field[0] && field[0] != '<') {
	  char *sl_pos;
	  const char *ls[] = {"LETTER SMALL ",   "SMALL LETTER ",
			      "SMALL LIGATURE ", "BOLD SMALL ",
			      "ITALIC SMALL ",   "SCRIPT SMALL ",
			      "FRAKTUR SMALL ",  "STRUCK SMALL ",
			      "SERIF SMALL " ,   "MONOSPACE SMALL ", 0};
	  int k;

	  /* Simplify the names case sensitively. */
	  convert(field, changes1, modif);
	  for (k = 0; ; k++) {
	    if (!ls[k]) break;
	    sl_pos = strstr(modif, ls[k]);
	    if (sl_pos) {
	      if (0) fprintf(stdout, "Modifying %s\n", modif);
	      sl_pos += strlen(ls[k]);
	      for (; *sl_pos && ' ' != *sl_pos; sl_pos++) {
		if ('A' <= *sl_pos && *sl_pos <= 'Z') *sl_pos += ('a' - 'A');
	      }
	    }
	  }
	  convert(modif, changes2, field);
	  if (name = malloc(strlen(field)+1)) {
	    strcpy(name, field);
	    if (0) fprintf(stdout, "Adding %4.4x => %s.\n", codepoint, name);
	    set_property(&unicode_names, codepoint, name, 0);
	  } else {
	    fprintf(stderr, "Memory exhausted when processing "
		    "codepoint U+%4.4X.\n", codepoint);
	  }
	}
	break;
      case 4:
	if (lookup_in_map(dir_names, field, &dirn)) {
	  set_property(&unicode_dirs, codepoint, dirn, (void*)K_DIR_L);
	} else {
	  fprintf(stderr, "Codepoint U+%4.4X has unintelligible "
		  "direction %s.\n", codepoint, field);
	}
	break;
#if 0
      case 9:
	if (!strcmp("Y", field)) {
	  fprintf(stdout, "Codepoint U+%4.4X is mirrored.\n", codepoint);
	} else if (strcmp("N", field)) {
	  fprintf(stderr, "Codepoint U+%4.4X has unexpected mirroring "
		  "property value \"%s\"", codepoint, field);
	}
	break;
#endif
      default:
	break;
      }
      field_no++;
      field[0] = 0;
      field_end = field;
      if ('\n' == onch) field_no = 0;
    } else {
      *field_end++ = onch;
    }
  }
#if 0
  for (codepoint = 0x0e00; codepoint <= 0x0e5f; codepoint++) {
    result = get_property(names, codepoint, (void**) &name);
    if (FOUND != result) {
      fprintf(stdout, "Search for %4.4x failed with error code %d.\n",
	      codepoint, result);
    } else {
      fprintf(stdout, "U+%4.4x => %s.\n", codepoint, name);
    }
  }
  for (codepoint = 0x10000; codepoint <= 0x10020; codepoint++) {
    result = get_property(names, codepoint, (void**) &name);
    if (FOUND != result) {
      fprintf(stdout, "Search for %4.4x failed with error code %d.\n",
	      codepoint, result);
    } else {
      fprintf(stdout, "U+%4.4x => %s.\n", codepoint, name);
    }
  }
#endif
}
