/*
Input routines built from type definition and atomic operations.
Copyright (C) 2005-2017      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include <stdlib.h>
#include <string.h>
#ifdef DATA_UB
  #include "manip_undef.h"
#endif

#define STRUCTURE(type) void MANIP_XGLUE(GET_, type) (GET_SOURCE* fp,\
                                     struct type *here, int*follow, int*pos){\
unsigned char      ub_temp; signed char  sb_temp;\
unsigned short int uw_temp; signed short int sw_temp;\
unsigned int       ul_temp; signed int   sl_temp;\
int nover, nskip, i; signed long long int sq_temp;\
*follow = 0; memset(here, 0, sizeof(*here));
#ifndef MANIP_XGLUE
  #define MANIP_XGLUE(a,b) MANIP_GLUE(a,b)
  #define MANIP_GLUE(a, b) a ## b
#endif
#define END_STRUCTURE(type) here->_next = 0;}
#define ALIGN(n)\
  if (nover = *pos%n) {nskip = n - nover; GET0(fp, nskip, pos)}

#define DATA_CH(name) GETB(fp, here->name, pos);
#define DATA_UB(name) GETB(fp, here->name, pos);
#define DATA_SB(name) GETB(fp, here->name, pos);
#define DATA_UW(name) GETW(fp, here->name, pos);
#define DATA_SW(name) GETW(fp, here->name, pos);
#define DATA_UL(name) GETL(fp, here->name, pos);
#define DATA_SL(name) GETL(fp, here->name, pos);
#define DATA_SQ(name) GETQ(fp, here->name, pos);

#define DATA_N_CH(name,n) for (i=0;i<n;i++) {GETB(fp, here->name[i], pos)}
#define DATA_N_UB(name,n) for (i=0;i<n;i++) {GETB(fp, here->name[i], pos)}
#define DATA_N_SB(name,n) for (i=0;i<n;i++) {GETB(fp, here->name[i], pos)}
#define DATA_N_UW(name,n) for (i=0;i<n;i++) {GETW(fp, here->name[i], pos)}
#define DATA_N_SW(name,n) for (i=0;i<n;i++) {GETW(fp, here->name[i], pos)}
#define DATA_N_UL(name,n) for (i=0;i<n;i++) {GETL(fp, here->name[i], pos)}
#define DATA_N_SL(name,n) for (i=0;i<n;i++) {GETL(fp, here->name[i], pos)}
#define DATA_N_SQ(name,n) for (i=0;i<n;i++) {GETQ(fp, here->name[i], pos)}

#ifdef DATA_VL_TEMPLATE
  #undef DATA_VL_TEMPLATE
#endif
#define DATA_VL_TEMPLATE(name, count, size, func)\
  if ((here->count) > 0) {\
    here->name = malloc(size here->count);\
    memset(here->name, 0, size here->count);\
    for (i = 0; i < here->count; i++) {func(fp, here->name[i], pos)}\
  } else {\
    here->name = 0;\
  }
#define DATA_VL_CH(name, count) DATA_VL_TEMPLATE(name, count, 1*, GETB)
#define DATA_VL_UB(name, count) DATA_VL_TEMPLATE(name, count, 1*, GETB)
#define DATA_VL_SB(name, count) DATA_VL_TEMPLATE(name, count, 1*, GETB)
#define DATA_VL_UW(name, count) DATA_VL_TEMPLATE(name, count, 2*, GETW)
#define DATA_VL_SW(name, count) DATA_VL_TEMPLATE(name, count, 2*, GETW)
#define DATA_VL_UL(name, count) DATA_VL_TEMPLATE(name, count, 4*, GETL)
#define DATA_VL_SL(name, count) DATA_VL_TEMPLATE(name, count, 4*, GETL)
#define DATA_VL_SQ(name, count) DATA_VL_TEMPLATE(name, count, 8*, GETQ)

#define DATA_VR_CH(name, count)\
  here->_n_##name = (count); DATA_VL_CH(name, _n_##name);
#define DATA_VR_UB(name, count)\
  here->_n_##name = (count); DATA_VL_UB(name, _n_##name);
#define DATA_VR_SB(name, count)\
  here->_n_##name = (count); DATA_VL_SB(name, _n_##name);
#define DATA_VR_UW(name, count)\
  here->_n_##name = (count); DATA_VL_UW(name, _n_##name);
#define DATA_VR_SW(name, count)\
  here->_n_##name = (count); DATA_VL_SW(name, _n_##name);
#define DATA_VR_UL(name, count)\
  here->_n_##name = (count); DATA_VL_UL(name, _n_##name);
#define DATA_VR_SL(name, count)\
  here->_n_##name = (count); DATA_VL_SL(name, _n_##name);
#define DATA_VR_SQ(name, count)\
  here->_n_##name = (count); DATA_VL_SQ(name, _n_##name);

#define ALT_UB(name) GETB(fp, ub_temp, pos); here->name = ub_temp;
#define ALT_SB(name) GETB(fp, sb_temp, pos); here->name = sb_temp;
#define ALT_UW(name) GETW(fp, uw_temp, pos); here->name = uw_temp;
#define ALT_SW(name) GETW(fp, sw_temp, pos); here->name = sw_temp;
#define ALT_UL(name) GETL(fp, ul_temp, pos); here->name = ul_temp;
#define ALT_SL(name) GETL(fp, sl_temp, pos); here->name = sl_temp;
#define ALT_SQ(name) GETQ(fp, sq_temp, pos); here->name = sq_temp;

#define ZERO(name) here->name = 0;
#define DATA_STR(field, eltype)\
  MANIP_XGLUE(GET_, eltype)(fp, &here->field, &nover, pos);
#define ARRAY_VC(field, n, eltype) {struct eltype **ptr2; int report;\
  for (i=(n), ptr2=&here->field; i>0;i--) {\
    *ptr2 = (struct eltype*) malloc(sizeof(struct eltype));\
    memset(*ptr2, 0, sizeof(struct eltype));\
    (*ptr2)->_next = 0;\
    MANIP_XGLUE(GET_, eltype)(fp, *ptr2, &report, pos);\
    ptr2 = &(*ptr2)->_next;\
  }}
#define ARRAY_TC(field, eltype) {struct eltype **ptr2; int report = 2;\
  ptr2 = &here->field;\
  while (2 == report) {\
    *ptr2 = (struct eltype*) malloc(sizeof(struct eltype));\
    memset(*ptr2, 0, sizeof(struct eltype));\
    (*ptr2)->_next = 0;\
    MANIP_XGLUE(GET_, eltype)(fp, *ptr2, &report, pos);\
    ptr2 = &(*ptr2)->_next;\
  }\
  if (report != 1) {\
      fprintf(stderr, "Final/non-final not specified when processing "\
	      "an element of" #field ",\n of type " #eltype ".\n");\
      exit(1);}\
  }
#define IF(condition) if (condition) {
#define ELSE } else {
#define ELSEIF(condition) } else if (condition) {
#define ENDIF }
#define LAST_ONE *follow = 1;
#define NOT_LAST_ONE *follow = 2;
