/*
Printing routines built from type definitions.
Copyright (C) 2010      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#ifdef DISPLAY_DEST
  #undef DISPLAY_DEST

  #undef DISPLAY_
  #undef DISPLAY_SB
  #undef DISPLAY_UB
  #undef DISPLAY_SW
  #undef DISPLAY_UW
  #undef DISPLAY_SL
  #undef DISPLAY_SQ

#endif
#define DISPLAY_ print_
#define DISPLAY_DEST FILE
#define DISPLAY_NAME(name, level) fprintf(fp, "%2d %s:\n", level, name);
#define DISPLAY_CH(fp, field, name, level) DISPLAY_UB(fp, field, name, level)
#define DISPLAY_UB(fp, field, name, level)\
  fprintf(fp, "%2d %s: %3hu %2.2hX\n", level, name, field, field & 0xff); 
#define DISPLAY_SB(fp, field, name, level)\
  fprintf(fp, "%2d %s: %4hd %2.2hX\n", level, name, field, field & 0xff); 
#define DISPLAY_UW(fp, field, name, level)\
  fprintf(fp, "%2d %s: %5hu %4.4hX\n", level, name, field, field & 0xffff); 
#define DISPLAY_SW(fp, field, name, level)\
  fprintf(fp, "%2d %s: %6hd %4.4hX\n", level, name, field, field & 0xffff); 
#define DISPLAY_UL(fp, field, name, level)\
  fprintf(fp, "%2d %s: %10u %8.8X\n", level, name, field, field); 
#define DISPLAY_SL(fp, field, name, level)\
  fprintf(fp, "%2d %s: %11d %8.8X\n", level, name, field, field); 
#define DISPLAY_SQ(fp, field, name, level)\
  fprintf(fp, "%2d %s: %lld %16.16llX\n", level, name, field, field);

/* The formatting below assumes roughly 80 columns */
#define DISPLAY_N_CH(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name); if ((n) > 8) fprintf(fp, "\n");\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=16) {\
    int j = i; char strbit[17] = ""; char *x = strbit;\
    if ((n) > 8) fprintf(fp, "%2d [%2.2d-]", level+1, j);\
    for (; j < i+16; j++) {\
      if (j < (n)) {\
        fprintf(fp, " %2.2X", field[j] & 0xff);\
        *x++ = (32 <= field[j] && field[j] <= 126) ? (char)field[j] : '.';\
      } else {\
        fputs("   ", fp);\
      }\
    }\
    *x = 0;\
    fprintf(fp, " \"%s\"\n", strbit);\
  }
#define DISPLAY_N_UB(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name); if ((n) > 5) fprintf(fp, "\n");\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=8) {\
    int j = i; char strbit[11] = ""; char *x = strbit;\
    if ((n) > 5) fprintf(fp, "%2d [%2.2d-]", level+1, j);\
    for (; (j < (n)) && (j < i+8); j++) {\
      fprintf(fp, "  %3hu %2.2hX", field[j], field[j]);\
      *x++ = (32 <= field[j] && field[j] <= 126) ? (char)field[j] : '.';\
    }\
    *x = 0;\
    fprintf(fp, " \"%s\"\n", strbit);\
  }
#define DISPLAY_N_SB(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name); if ((n) > 5) fprintf(fp, "\n");\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=8) {\
    int j = i; char strbit[11] = ""; char *x = strbit;\
    if ((n) > 5) fprintf(fp, "%2d [%2.2d-]", level+1, j);\
    for (; (j < (n)) && (j < i+8); j++) {\
      fprintf(fp, " %4hd %2.2hX", field[j], field[j] & 0xff);\
      if (32 <= field[j] && field[j] <= 126) *x++ = (char)field[j];\
    }\
    *x = 0;\
    fprintf(fp, " \"%s\"\n", strbit);\
  }
#define DISPLAY_N_UW(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name); if ((n) > 5) fprintf(fp, "\n");\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=5) {\
    int j = i;\
    if ((n) > 5) fprintf(fp, "%2d [%2.2d-]", level+1, j);\
    for (; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, "  %5hu %4.4hX", field[j], field[j]);\
    }\
    fprintf(fp, "\n");\
  }
#define DISPLAY_N_SW(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name); if ((n) > 5) fprintf(fp, "\n");\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=5) {\
    int j = i;\
    if ((n) > 5) fprintf(fp, "%2d [%2.2d-]", level+1, j);\
    for (; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, "  %6hd %4.4hX", field[j], field[j] & 0xffff);\
    }\
    fprintf(fp, "\n");\
  }
#define DISPLAY_N_UL(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name);\
  if ((n) > 5) fputc('\n', fp);\
  for (i = 0; i < (n); i+=5) {\
    char tag[30] = "", *x;\
    int j = i;\
    if ((n) > 5) sprintf(tag, "%2d [%2.2d-]", level+1, j);\
    fputs(tag, fp);\
    for (; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, " %10u", field[j]);\
    }\
    for (x=tag; *x; *x++ = ' ');\
    fprintf(fp, "\n%s", tag);\
    for (j = i; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, "   %8.8X", field[j]);\
    }\
    fprintf(fp, "\n");\
  }
#define DISPLAY_N_SL(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name);\
  if ((n) > 5) fputc('\n', fp);\
  for (i = 0; i < (n); i+=5) {\
    char tag[30] = "", *x;\
    int j = i;\
    if ((n) > 5) sprintf(tag, "%2d [%2.2d-]", level+1, j);\
    fputs(tag, fp);\
    for (; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, " %11d", field[j]);\
    }\
    for (x=tag; *x; *x++ = ' ');\
    fprintf(fp, "\n%s", tag);\
    for (j = i; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, "    %8.8X", field[j]);\
    }\
    fprintf(fp, "\n");\
  }
#define DISPLAY_N_SQ(fp, field, n, name, level)\
  fprintf(fp, "%2d %s:", level, name);\
  if (!n) fputc('\n', fp);\
  for (i = 0; i < (n); i+=5) {\
    char tag[30], *x;\
    int j = i;\
    sprintf(tag, "%2d [%2.2d-]", level+1, j);\
    fputs(tag, fp);\
    for (; (j < (n)) && (j < i+5); j++) {\
      fprintf(fp, " %20lld", field[j]);\
    }\
    for (x=tag; *x; *x++ = ' ');\
    fprintf(fp, "\n%s", tag);\
    for (j = i; (j < (n)) && (j < i+5); j++) {\
      sprintf(tag, "%16.16llX", field[j]);\
      fprintf(fp, "  %4.4s %4.4s %4.4s %4.4s", tag, tag+4, tag+8, tag+12);\
    }\
    fprintf(fp, "\n");\
  }
#define DISPLAY_NONE fputc('\n', fp);
