/*
Output routine from data definition and atomic operations.
Utility to examine and decompile OpenType fonts.
Copyright (C) 2005-2020      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include <stdlib.h>
#include <string.h>
#ifdef DATA_UB
  #include "manip_undef.h"
#endif

#define STRUCTURE(type) void MANIP_XGLUE(PUT_, type) (PUT_SOURCE* fp,\
                                     struct type *here, int*follow, int*pos){\
unsigned char      ub_temp; signed char  sb_temp;\
unsigned short int uw_temp; signed short int sw_temp;\
unsigned int       ul_temp; signed int   sl_temp;\
int nover, nskip, i; signed long long int sq_temp;\
*follow = 0;
#ifndef MANIP_XGLUE
  #define MANIP_XGLUE(a,b) MANIP_GLUE(a,b)
  #define MANIP_GLUE(a, b) a ## b
#endif
#define END_STRUCTURE(type) }
#define ALIGN(n)\
  if (nover = *pos%n) {nskip = n - nover; PUT0(fp, nskip, pos)}

#define DATA_CH(name) PUTB(fp, here->name, pos);
#define DATA_UB(name) PUTB(fp, here->name, pos);
#define DATA_SB(name) PUTB(fp, here->name, pos);
#define DATA_UW(name) PUTW(fp, here->name, pos);
#define DATA_SW(name) PUTW(fp, here->name, pos);
#define DATA_UL(name) PUTL(fp, here->name, pos);
#define DATA_SL(name) PUTL(fp, here->name, pos);
#define DATA_SQ(name) PUTQ(fp, here->name, pos);

#define DATA_N_CH(name,n) for (i=0;i<n;i++) {PUTB(fp, here->name[i], pos)}
#define DATA_N_UB(name,n) for (i=0;i<n;i++) {PUTB(fp, here->name[i], pos)}
#define DATA_N_SB(name,n) for (i=0;i<n;i++) {PUTB(fp, here->name[i], pos)}
#define DATA_N_UW(name,n) for (i=0;i<n;i++) {PUTW(fp, here->name[i], pos)}
#define DATA_N_SW(name,n) for (i=0;i<n;i++) {PUTW(fp, here->name[i], pos)}
#define DATA_N_UL(name,n) for (i=0;i<n;i++) {PUTL(fp, here->name[i], pos)}
#define DATA_N_SL(name,n) for (i=0;i<n;i++) {PUTL(fp, here->name[i], pos)}
#define DATA_N_SQ(name,n) for (i=0;i<n;i++) {PUTQ(fp, here->name[i], pos)}

#ifdef DATA_VL_TEMPLATE
  #undef DATA_VL_TEMPLATE
#endif
#define DATA_VL_TEMPLATE(name, count, size, func)\
  for (i = 0; i < here->count; i++) {func(fp, here->name[i], pos)}
#define DATA_VL_CH(name, count) DATA_VL_TEMPLATE(name, count, 1*, PUTB)
#define DATA_VL_UB(name, count) DATA_VL_TEMPLATE(name, count, 1*, PUTB)
#define DATA_VL_SB(name, count) DATA_VL_TEMPLATE(name, count, 1*, PUTB)
#define DATA_VL_UW(name, count) DATA_VL_TEMPLATE(name, count, 2*, PUTW)
#define DATA_VL_SW(name, count) DATA_VL_TEMPLATE(name, count, 2*, PUTW)
#define DATA_VL_UL(name, count) DATA_VL_TEMPLATE(name, count, 4*, PUTL)
#define DATA_VL_SL(name, count) DATA_VL_TEMPLATE(name, count, 4*, PUTL)
#define DATA_VL_SQ(name, count) DATA_VL_TEMPLATE(name, count, 8*, PUTQ)

#define DATA_VR_CH(name, count)\
  here->_n_##name = (count); DATA_VL_CH(name, _n_##name);
#define DATA_VR_UB(name, count)\
  here->_n_##name = (count); DATA_VL_UB(name, _n_##name);
#define DATA_VR_SB(name, count)\
  here->_n_##name = (count); DATA_VL_SB(name, _n_##name);
#define DATA_VR_UW(name, count)\
  here->_n_##name = (count); DATA_VL_UW(name, _n_##name);
#define DATA_VR_SW(name, count)\
  here->_n_##name = (count); DATA_VL_SW(name, _n_##name);
#define DATA_VR_UL(name, count)\
  here->_n_##name = (count); DATA_VL_UL(name, _n_##name);
#define DATA_VR_SL(name, count)\
  here->_n_##name = (count); DATA_VL_SL(name, _n_##name);
#define DATA_VR_SQ(name, count)\
  here->_n_##name = (count); DATA_VL_SQ(name, _n_##name);

#define ALT_UB(name) ub_temp = here->name; PUTB(fp, ub_temp, pos);
#define ALT_SB(name) sb_temp = here->name; PUTB(fp, sb_temp, pos);
#define ALT_UW(name) uw_temp = here->name; PUTW(fp, uw_temp, pos);
#define ALT_SW(name) sw_temp = here->name; PUTW(fp, sw_temp, pos);
#define ALT_UL(name) ul_temp = here->name; PUTL(fp, ul_temp, pos);
#define ALT_SL(name) sl_temp = here->name; PUTL(fp, sl_temp, pos);
#define ALT_SQ(name) sq_temp = here->name; PUTQ(fp, sq_temp, pos);

#define ZERO(name) here->name = 0;
#define DATA_STR(field, eltype)\
  MANIP_XGLUE(PUT_, eltype)(fp, &here->field, &nover, pos);
#define ARRAY_VC(field, n, eltype) {struct eltype *ptr; int report;\
  for (i=0, ptr=here->field; i<(n);i++) {\
    MANIP_XGLUE(PUT_, eltype)(fp, ptr, &report, pos);\
    ptr = ptr->_next;\
  }}
#define ARRAY_TC(field, eltype) {struct eltype *ptr; int report = 2;\
  ptr = here->field;\
  while (2 == report) {\
    MANIP_XGLUE(PUT_, eltype)(fp, ptr, &report, pos);\
    ptr = ptr->_next;\
  }\
  if (report != 1) {\
      fprintf(stderr, "Final/non-final not specified when processing "\
	      "an element of" #field ",\n of type " #eltype ".\n");\
      exit(1);}\
  }
#define IF(condition) if (condition) {
#define ELSE } else {
#define ELSEIF(condition) } else if (condition) {
#define ENDIF }
#define LAST_ONE *follow = 1;
#define NOT_LAST_ONE *follow = 2;
