/*
Interfaces for maps from strings to pointers.
Copyright (C) 2007      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#ifndef MAP__H
#define MAP__H 1
struct map_entry {
  char *key;
  void *datum;
  char status;
};

struct map {
  struct map_entry *x;
  int capacity;
  int usage;
};

void create_map(struct map *the_map);
void delete_map(struct map *the_map, int free_value);
int lookup_in_map(struct map the_map, const char *key, void **val);
/* Returns 1 if found, 0 if not. */
/* The map takes ownership of *val as the result of a successful call.
   It makes its own copy of *key      */
void replace_in_map(struct map *the_map, const char *key, void *val,
		    int free_value);
void insert_in_map(struct map *the_map, const char *key, void *val);
int delete_from_map(struct map *the_map, const char *key, int free_value);
/* Returns 1 if found, 0 if not. */

/* sort_map() generates a list of entries in alphabeitc order by key.      */
/* *entries belongs to caller - the entries pointed to belong to the map,  */
/* and may change or be deallocated as the map is updated.                 */
void sort_map(struct map the_map, struct map_entry ***entries, int *n_entries);
#endif /* MAP__H */
