/*
Utility to read and write OpenType fonts.
Copyright (C) 2004-2020      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#ifndef OFT__H
#include <string.h>
struct glyph_globals {
  struct repeatable_flag *flag_ptr;
  int n_points, n_to_go, repeat_count;
  int flag_is_count;
  short int we_have_instr;
  char flags, short_xy, same;
};
extern struct glyph_globals glyph;
#define OFT__H
extern int max_UW(unsigned short int *arr, int);
extern int PREP_SIZE;
#endif
STRUCTURE(directory_entry)
     DATA_N_CH(tag, 4)
     DATA_UL(checkSum)
     DATA_UL(offset)
     DATA_UL(length)
     IF (!strncmp((char*)here->tag, "prep", 4) && (PREP_SIZE = here->length))
     ENDIF
     IF (!strncmp((char*)here->tag, "cvt ", 4) && (CVT_SIZE = here->length/2))
     ENDIF
     IF (!strncmp((char*)here->tag, "fpgm", 4) && (FPGM_SIZE = here->length))
     ENDIF
END_STRUCTURE(directory_entry)
STRUCTURE(offset_table)
     DATA_SL(sfnt_version)
     DATA_UW(numTables)
     DATA_UW(searchRange)
     DATA_UW(entrySelector)
     DATA_UW(rangeShift)
     ARRAY_VC(table_directory, here->numTables, directory_entry)
END_STRUCTURE(offset_table)
/* For initial probing only */
STRUCTURE(offset_table_header)
     DATA_SL(sfnt_version)
     DATA_UW(numTables)
     DATA_UW(searchRange)
     DATA_UW(entrySelector)
     DATA_UW(rangeShift)
/*     ARRAY_VC(table_directory, here->numTables, directory_entry) */
END_STRUCTURE(offset_table_header)
STRUCTURE(SB_item)
     DATA_SB(val)
END_STRUCTURE(SB_item)
STRUCTURE(SW_item)
     DATA_SW(val)
END_STRUCTURE(SW_item)
STRUCTURE(UW_item)
     DATA_UW(val)
END_STRUCTURE(UW_item)
STRUCTURE(UL_item)
     DATA_UL(val)
END_STRUCTURE(UL_item)
STRUCTURE(UW_items)
    DATA_VR_UW(val, N_ITEMS)
END_STRUCTURE(UW_items)
STRUCTURE(UL_items)
    DATA_VR_UL(val, N_ITEMS)
END_STRUCTURE(UL_items)

     /* Next table is for a True Type Collection (.ttc) */
STRUCTURE(TTC_header)
     DATA_N_CH(TTCTag, 4) /* Must be 'ttcf' */
     DATA_UL(version)
     DATA_UL(numFonts)
     ARRAY_VC(offsetTable, here->numFonts, UL_item)
     IF (here->version == 0x00020000)
       DATA_UL(ulDsigTag)
       DATA_UL(ulDsigLength)
       DATA_UL(ulDsigOffset)
     ENDIF
END_STRUCTURE(TTC_header)
     /* Character to glyph mapping - cmap */
STRUCTURE(encoding_record)
     DATA_UW(platformID)
     DATA_UW(encodingID)
     DATA_UL(offset)
END_STRUCTURE(encoding_record)
STRUCTURE(cmap_header)
     DATA_UW(version)
     DATA_UW(numTables)
     ARRAY_VC(encodingRecord, here->numTables, encoding_record)
END_STRUCTURE(cmap_header)
STRUCTURE(subHeader)
     DATA_UW(firstCode)
     DATA_UW(entryCount)
     DATA_SW(idDelta)
     DATA_UW(idRangeOffset)
     ALIGN(4)
END_STRUCTURE(subHeader)
STRUCTURE(format_8_group)
     DATA_UL(startCharCode)
     DATA_UL(endCharCode)
     DATA_UL(startGlyphID)
END_STRUCTURE(format_8_group)
STRUCTURE(variation_selector)
    DATA_N_UB(varSelector, 3)
    DATA_UL(defaultUVSOffset)
    DATA_UL(nonDefaultUVSOffset)
END_STRUCTURE(variation_selector)
STRUCTURE(unicode_range)
    DATA_N_UB(startUnicodeValue, 3)
    DATA_UB(additionalCount)
END_STRUCTURE(unicode_range)
STRUCTURE(default_UVS_table)
    DATA_UL(numUnicodeValueRanges)
    ARRAY_VC(ranges, (here->numUnicodeValueRanges), unicode_range)
END_STRUCTURE(default_UVS_table)
STRUCTURE(UVS_mapping)
    DATA_N_UB(unicodeValue, 3)
    DATA_UW(glyphID)
END_STRUCTURE(UVS_mapping)
STRUCTURE(non_default_UVS_table)
    DATA_UL(numUVSMappings)
    ARRAY_VC(uvsMappings, here->numUVSMappings, UVS_mapping)
END_STRUCTURE(non_default_UVS_table)
STRUCTURE(encoding)
     DATA_UW(format)
     IF (here->format < 8)
         ALT_UW(length)
         ALT_UW(language)
     ELSEIF (14 == here->format)
         ALT_UL(length)
     ELSE
         DATA_UW(reserved)
         DATA_UL(length)
         DATA_UL(language)
     ENDIF
     IF (0 == here->format)
         DATA_N_UB(glyphIdArray_b, 256)
     ELSEIF (2 == here->format)
         DATA_N_UW(subHeaderKeys, 256)
         DATA_N_SQ(subHeaders, 256) /* May be fewer! */
     /*  DATA_N_UW(glyphIndexArray, ??) *//* Think about calculations */
     ELSEIF (4 == here->format)
         DATA_UW(segCountX2)
         DATA_UW(searchRange)
         DATA_UW(entrySelector)
         DATA_UW(rangeShift)
         DATA_VR_UW(endCount, (here->segCountX2/2))
         DATA_UW(reservedPad)
         DATA_VR_UW(startCount, (here->segCountX2/2))
         DATA_VR_SW(idDelta, (here->segCountX2/2))
         DATA_VR_UW(idRangeOffset, (here->segCountX2/2))
     ELSEIF (6 == here->format)
         DATA_UW(firstCode)
         DATA_UW(entryCount)
     ELSEIF (8 == here->format || 12 == here->format)
         IF (8 == here->format)
             DATA_N_UB(is32, 8192)
         ENDIF
         DATA_UL(nGroups)
         ARRAY_VC(groups, here->nGroups, format_8_group)
     ELSEIF (10 == here->format)
         DATA_UL(startCharCode)
         DATA_UL(numChars)
         DATA_VL_UW(glyphs, numChars)
     ELSEIF (14 == here->format)
         DATA_UL(numVarSelectorRecords)
         ARRAY_VC(varSelector, (here->numVarSelectorRecords),
                  variation_selector)
     ENDIF
     IF (4 == here->format || 6 == here->format)
     /* For format 4, index into array for character c in segment i is:

	i + (idRangeOffset[1] - segCountX2)/2 + c - StartCount[i].

	(The logic is to find the location of idRangeOffset[i], and then
	go idRangeOffset[i] bytes beyond it.)  idDelta[i] then has to be added
	to yield glyph number! */

         DATA_VR_UW(glyphIdArray,
		    (4 == here->format)
		    ? (here->length/2 - 8 - 2 * here->segCountX2)
		    : here->entryCount)
     ENDIF
     ALIGN(4)
END_STRUCTURE(encoding)
     /* Font header - head */
extern int LONG_OFFSETS;
STRUCTURE(font_header)
     DATA_SL(tableVersionNumber)
     DATA_SL(fontRevision)
     DATA_UL(checkSumAdjustment)
     DATA_UL(magicNumber) /* 0x5F0F3CF5 */
     DATA_UW(flags)
     DATA_UW(unitsPerEm)
     DATA_SQ(created)
     DATA_SQ(modified)
     DATA_SW(xMin)
     DATA_SW(yMin)
     DATA_SW(xMax)
     DATA_SW(yMax)
     DATA_UW(macStyle)
     DATA_UW(lowestRecPPEM)
     DATA_SW(fontDirectionHint)
     DATA_SW(indexToLocFormat)
     IF (LONG_OFFSETS = here->indexToLocFormat) ENDIF
     DATA_SW(glyphDataFormat)
END_STRUCTURE(font_header)
     /* Horizontal header - hhea */
extern int hhea_numberofHMetrics;
STRUCTURE(horizontal_header)
     DATA_UL(tableVersionNumber)
     DATA_SW(ascender)
     DATA_SW(descender)
     DATA_SW(lineGap)
     DATA_UW(advanceWidthMax)
     DATA_SW(minLeftSideBearing)
     DATA_SW(minRightSideBearing)
     DATA_SW(xMaxExtent)
     DATA_SW(caretSlopeRise)
     DATA_SW(caretSlopeRun)
     DATA_SW(caretOffset)
     DATA_SW(res_0)
     DATA_SW(res_1)
     DATA_SW(res_2)
     DATA_SW(res_3)
     DATA_SW(metricDataFormat)
     DATA_UW(numberofHMetrics)
     IF (hhea_numberofHMetrics = here->numberofHMetrics) ENDIF
END_STRUCTURE(horizontal_header)
     /* Horizontal metrics - hmtx */
STRUCTURE(longHorMetric)
     DATA_UW(advanceWidth)
     DATA_SW(lsb)
END_STRUCTURE(longHorMetric)
STRUCTURE(horizontal_metrics)
     ARRAY_VC(hMetrics, hhea_numberofHMetrics, longHorMetric)
     DATA_VR_SW(leftSideBearing, N_GLYPHS - hhea_numberofHMetrics)
END_STRUCTURE(horizontal_metrics)
     /* Maximum profile - maxp */
extern int N_GLYPHS;
STRUCTURE(maximum_profile)
     DATA_UL(tableVersionNumber)
     DATA_UW(numGlyphs)
     IF (N_GLYPHS = here->numGlyphs) ENDIF
     IF (here->tableVersionNumber != 0x00005000)
       DATA_UW(maxPoints)
       DATA_UW(maxContours)
       DATA_UW(maxCompositePoints)
       DATA_UW(maxCompositeContours)
       DATA_UW(maxZones)
       DATA_UW(maxTwilightPoints)
       DATA_UW(maxStorage)
       DATA_UW(maxFunctionDefs)
       DATA_UW(maxInstructionDefs)
       DATA_UW(maxStackElements)
       DATA_UW(maxSizeOfInstructions)
       DATA_UW(maxComponentElements)
       DATA_UW(maxComponentDepth)
     ENDIF
END_STRUCTURE(maximum_profile)
     /* Naming table - name */
STRUCTURE(name_record)
     DATA_UW(platformID)
     DATA_UW(encodingID)
     DATA_UW(languageID)
     DATA_UW(nameID)
     DATA_UW(length)
     DATA_UW(offset)
END_STRUCTURE(name_record)
STRUCTURE(name_table)
     DATA_UW(format)
     DATA_UW(count)
     DATA_UW(stringOffset)
     ARRAY_VC(nameRecord, here->count, name_record)
     /* STORAGE FOR STRINGS - TBC */
END_STRUCTURE(name_table)
     /* OS/2 and Windows specific metrics - OS/2 */
STRUCTURE (OS2_metrics)
     DATA_UW(version)
     DATA_SW(xAvgCharWidth)
     DATA_UW(usWeightClass)
     DATA_UW(usWidthClass)
     DATA_UW(fsType)
     DATA_SW(ySubscriptXSize)
     DATA_SW(ySubscriptYSize)
     DATA_SW(ySubscriptXOffset)
     DATA_SW(ySubscriptYOffset)
     DATA_SW(ySuperscriptXSize)
     DATA_SW(ySuperscriptYSize)
     DATA_SW(ySuperscriptXOffset)
     DATA_SW(ySuperscriptYOffset)
     DATA_SW(yStrikeoutSize)
     DATA_SW(yStrikeoutPosition)
     DATA_UW(sFamilyClass) /* signed makes no sense! */
     DATA_N_UB(panose, 10)
     DATA_UL(ulUnicodeRange1)
     DATA_UL(ulUnicodeRange2)
     DATA_UL(ulUnicodeRange3)
     DATA_UL(ulUnicodeRange4)
     DATA_N_CH(achVendID, 4)
     DATA_UW(fsSelection)
     DATA_UW(usFirstCharIndex)
     DATA_UW(usLastCharIndex)
     DATA_SW(sTypoAscender)
     DATA_SW(sTypoDescender)
     DATA_SW(sTypoLineGap)
     DATA_UW(usWinAscent)
     DATA_UW(usWinDescent)
     IF (here->version >= 1)
         DATA_UL(ulCodePageRange1)
         DATA_UL(ulCodePageRange2)
     ENDIF
     IF (here->version >= 2)
         DATA_SW(sxHeight)
         DATA_SW(sCapHeight)
         DATA_UW(usDefaultChar)
         DATA_UW(usBreakChar)
         DATA_UW(usMaxContext)
     ENDIF
END_STRUCTURE(OS2_metrics)
     /* Postscript information - post */
STRUCTURE(ASCIC)
     DATA_UB(count)
     DATA_VL_CH(body, count)
END_STRUCTURE(ASCIC)
STRUCTURE(postscript_information)
/* WARNING: Also update postscript_information_nonames */
     DATA_UL(version)
     DATA_SL(italicAngle)
     DATA_SW(underlinePosition)
     DATA_SW(underlineThickness)
     DATA_UL(isFixedPitch)
     DATA_UL(minMemType42)
     DATA_UL(maxMemType42)
     DATA_UL(minMemType1)
     DATA_UL(maxMemType1)
     IF (here->version == 0x00020000)
       DATA_UW(numberOfGlyphs)
       DATA_VL_UW(glyphNameIndex, numberOfGlyphs)
       ARRAY_VC(names,
		/* Max *subscript* is max index less 258, so: */
                max_UW(here->glyphNameIndex, here->numberOfGlyphs) - 258 + 1,
	        ASCIC)
     ELSEIF (here->version == 0x00025000)
       ALT_UW(numberOfGlyphs)
       DATA_VL_SB(offset, numberOfGlyphs)
     ENDIF
END_STRUCTURE(postscript_information)
STRUCTURE(postscript_information_nonames)
     DATA_UL(version)
     DATA_SL(italicAngle)
     DATA_SW(underlinePosition)
     DATA_SW(underlineThickness)
     DATA_UL(isFixedPitch)
     DATA_UL(minMemType42)
     DATA_UL(maxMemType42)
     DATA_UL(minMemType1)
     DATA_UL(maxMemType1)
     IF (here->version == 0x00020000)
       DATA_UW(numberOfGlyphs)
       DATA_VL_UW(glyphNameIndex, numberOfGlyphs)
/* The names field that should go here has to be handled manually! */
     ELSEIF (here->version == 0x00025000)
       ALT_UW(numberOfGlyphs)
       DATA_VL_SB(offset, numberOfGlyphs)
     ENDIF
END_STRUCTURE(postscript_information_nonames)
/* Truetype Outlines only: */
     /* Control Value Table - CVT */
     extern int CVT_SIZE;
STRUCTURE(control_value_table)
     DATA_VR_SW(datum, CVT_SIZE)
END_STRUCTURE(control_value_table)
     /* Font program - fpgm */
     extern int FPGM_SIZE;
STRUCTURE(font_program)
     DATA_VR_UB(datum, FPGM_SIZE)
END_STRUCTURE(font_program)
     /* Glyph data - glyf */
STRUCTURE(coordinate)
     IF (0)
     /* Hidden data to clarify sign.  A bit more jiggery pokery can make 
	field flipped appear when appropriate. */
         DATA_UB(loaded)
         DATA_UB(flipped)
     ELSE
         ZERO(loaded)
         ZERO(flipped)
     ENDIF
     IF (glyph.flags & glyph.short_xy)
         ALT_UB(pos)
         IF (!(glyph.flags & glyph.same) && !here->loaded &&
             (here->flipped = 1))
         ENDIF
     ELSE
         DATA_SW(pos)
     ENDIF
     IF (here->loaded = 1) ENDIF
     IF (glyph.n_to_go--) ENDIF
     IF ((glyph.flags & 8) && !glyph.flag_is_count &&
	 (glyph.flag_ptr = glyph.flag_ptr->_next))
         IF ((glyph.flag_is_count = 1,
	      glyph.repeat_count = glyph.flag_ptr->val))
         ENDIF
         NOT_LAST_ONE
     ELSEIF ((glyph.flags & 8) && --glyph.repeat_count)
         NOT_LAST_ONE
     ELSEIF (advance_coords(1))
         NOT_LAST_ONE
     ELSE
         LAST_ONE
     ENDIF
END_STRUCTURE(coordinate)
STRUCTURE(component_entry)
     DATA_UW(flags)
#define ARG_1_AND_2_ARE_WORDS     (1<<0)
#define ARGS_ARE_XY_VALUES        (1<<1)
#define ROUND_XY_TO_GRID          (1<<2)
#define WE_HAVE_A_SCALE           (1<<3)
#define MORE_COMPONENTS           (1<<5)
#define WE_HAVE_AN_X_AND_Y_SCALE  (1<<6)
#define WE_HAVE_A_TWO_BY_TWO      (1<<7)
#define WE_HAVE_INSTRUCTIONS      (1<<8)
#define USE_MY_METRICS            (1<<9)
#define OVERLAP_COMPOUND          (1<<10)
#define SCALED_COMPONENT_OFFSET   (1<<11)
#define UNSCALED_COMPONENT_OFFSET (1<<12)
     DATA_SW(the_glyphIndex)
     IF (here->flags & ARG_1_AND_2_ARE_WORDS)
         DATA_SW(argument1)
         DATA_SW(argument2)
     ELSE
         IF (here->flags & ARGS_ARE_XY_VALUES)
/* Evidence of signedness can be found in the Trebuchet font, which
   composes double left guillemet (U+00AB) from single left guillemet
   (U+2039) and l cedilla (U+013C) from l (U+006C) and cedilla (U+00B8) */
             ALT_SB(argument1)
             ALT_SB(argument2)
         ELSE
             ALT_UB(argument1)
             ALT_UB(argument2)
         ENDIF
     ENDIF
     IF (here->flags & WE_HAVE_A_SCALE)
         DATA_SW(scale)
     ELSEIF (here->flags & WE_HAVE_AN_X_AND_Y_SCALE)
         DATA_SW(xscale)
         DATA_SW(yscale)
     ELSEIF (here->flags & WE_HAVE_A_TWO_BY_TWO)
         ALT_SW(xscale)
         DATA_SW(scale01)
         DATA_SW(scale10)
         ALT_SW(yscale)
     ENDIF
     IF (glyph.we_have_instr = here->flags & WE_HAVE_INSTRUCTIONS) ENDIF
     IF (here->flags & MORE_COMPONENTS)
         NOT_LAST_ONE
     ELSE
         LAST_ONE
     ENDIF
END_STRUCTURE(component_entry)
STRUCTURE(repeatable_flag)
     DATA_SB(val)
     IF (glyph.flag_is_count)
         IF (glyph.n_to_go -= here->val) ENDIF
         IF (glyph.flag_is_count = 0) ENDIF
     ELSE
         IF (glyph.n_to_go--) ENDIF
         IF (glyph.flag_is_count = here->val & 8) ENDIF
     ENDIF
     IF (glyph.n_to_go > 0)
         NOT_LAST_ONE
     ELSE
         LAST_ONE
     ENDIF
END_STRUCTURE(repeatable_flag)
STRUCTURE(glyph_entry)
     DATA_SW(numberOfContours)
     DATA_SW(xMin)
     DATA_SW(yMin)
     DATA_SW(xMax)
     DATA_SW(yMax)
     IF (here->numberOfContours >= 0)
       DATA_VL_UW(endPtsOfContours, numberOfContours)
       DATA_UW(instructionLength)
       DATA_VL_UB(instructions, instructionLength)
       IF (glyph.n_points = here->endPtsOfContours[here->numberOfContours-1]+1)
       ENDIF
       IF (glyph.n_to_go = glyph.n_points) ENDIF
       IF (glyph.flag_is_count = 0) ENDIF
       ARRAY_TC(flags, repeatable_flag)
       IF ((glyph.n_to_go = glyph.n_points,
	    glyph.flags = here->flags->val,
	    glyph.flag_ptr = here->flags,
	    glyph.flag_is_count = 0,
	    glyph.short_xy = 2,
	    glyph.same = 16))
       ENDIF
       IF (advance_coords(0))
           ARRAY_TC(xCoordinates, coordinate)
       ENDIF
       IF ((glyph.n_to_go = glyph.n_points,
	    glyph.flags = here->flags->val,
	    glyph.flag_ptr = here->flags,
	    glyph.flag_is_count = 0,
	    glyph.short_xy = 4,
	    glyph.same = 32))
       ENDIF
       IF (advance_coords(0))
           ARRAY_TC(yCoordinates, coordinate)
       ENDIF
     ELSE
       ARRAY_TC(components, component_entry)
       IF (glyph.we_have_instr)
         DATA_UW(numInstr)
         DATA_VL_UB(instr, numInstr)
       ELSE
         ZERO(numInstr)
         ZERO(instr)
       ENDIF
     ENDIF
     ALIGN(4) /* The recommendation for this is in the documentation of the
		 loca table. */
END_STRUCTURE(glyph_entry)
     /* partial_glyph_entry is for debugging problems reading glyph entries. */
STRUCTURE(partial_glyph_entry)
     DATA_SW(numberOfContours)
     DATA_SW(xMin)
     DATA_SW(yMin)
     DATA_SW(xMax)
     DATA_SW(yMax)
     IF (here->numberOfContours >= 0)
       DATA_VL_UW(endPtsOfContours, numberOfContours)
       DATA_UW(instructionLength)
       DATA_VL_UB(instructions, instructionLength)
       IF (glyph.n_points = here->endPtsOfContours[here->numberOfContours-1]+1)
       ENDIF
       IF (glyph.n_to_go = glyph.n_points) ENDIF
       IF (glyph.flag_is_count = 0) ENDIF
       ARRAY_TC(flags, repeatable_flag)
#if 0
       IF ((glyph.n_to_go = glyph.n_points,
	    glyph.flags = here->flags->val,
	    glyph.flag_ptr = here->flags,
	    glyph.flag_is_count = 0,
	    glyph.short_xy = 2,
	    glyph.same = 16))
       ENDIF
       ARRAY_TC(xCoordinates, coordinate)
       IF ((glyph.n_to_go = glyph.n_points,
	    glyph.flags = here->flags->val,
	    glyph.flag_ptr = here->flags,
	    glyph.flag_is_count = 0,
	    glyph.short_xy = 4,
	    glyph.same = 32))
       ENDIF
       ARRAY_TC(yCoordinates, coordinate)
#endif
     ELSE
       ARRAY_TC(components, component_entry)
       IF (glyph.we_have_instr)
         DATA_UW(numInstr)
         DATA_VL_UB(instr, numInstr)
       ELSE
         ZERO(numInstr)
         ZERO(instr)
       ENDIF
     ENDIF
END_STRUCTURE(partial_glyph_entry)
STRUCTURE(glyph_data)
     ARRAY_VC(datum, N_GLYPHS, glyph_entry)
END_STRUCTURE(glyph_data)
     /* Index to location - loca */
STRUCTURE(location_index)
     IF (LONG_OFFSETS)
         DATA_VR_UL(long_offsets, N_GLYPHS + 1)
     ELSE
         DATA_VR_UW(short_offsets, N_GLYPHS + 1)
     ENDIF
END_STRUCTURE(location_index)
STRUCTURE(one_location) /* A convenience for manipulation */
     IF (LONG_OFFSETS)
         DATA_UL(offset)
     ELSE
         ALT_UW(offset)
     ENDIF
END_STRUCTURE(one_location)
     /* CVT program - prep */
STRUCTURE(control_value_program)
     DATA_VR_UB(program, PREP_SIZE)
END_STRUCTURE(control_value_program)
/* PostScript Outlines only: */
     /* PostScript font program (compact font format) - CFF */
     /* Vertical Origin - VORG */
STRUCTURE(VORG_entry)
     DATA_UW(glyphIndex)
     DATA_SW(vertOriginY)
END_STRUCTURE(VORG_entry)
STRUCTURE(vertical_origin)
     DATA_UW(majorVersion)
     DATA_UW(minorVersion)
     DATA_SW(defaultVertOriginY)
     DATA_UW(numVertOriginYMetrics)
     ARRAY_VC(vertOriginYMetrics, here->numVertOriginYMetrics, VORG_entry)
END_STRUCTURE(vertical_origin)
/* Tables Related to Bitmap Glyphs */
     /* Embedded bitmap data - EBDT */

     /* Embedded bitmap location data - EBLC */
STRUCTURE(sbitLineMetrics)
     DATA_SB(ascender)
     DATA_SB(descender)
     DATA_UB(widthMax)
     DATA_SB(caretSlopeNumerator)
     DATA_SB(caretSlopeDenominator)
     DATA_SB(caretOffset)
     DATA_SB(minOriginSB)
     DATA_SB(minAdvanceSB)
     DATA_SB(maxBeforeBL)
     DATA_SB(minAfterBL)
     DATA_SB(pad1)
     DATA_SB(pad2)
END_STRUCTURE(sbitLineMetrics)
STRUCTURE(bitmap_size_table)
     DATA_UL(indexSubTableArrayOffset)
     DATA_UL(indexTablesSize)
     DATA_UL(numberOfIndexSubTables)
     DATA_UL(colorRef)
     DATA_STR(hori, sbitLineMetrics)
     DATA_STR(vert, sbitLineMetrics)
     DATA_UW(startGlyphIndex)
     DATA_UW(endGlyphIndex)
     DATA_SB(ppemX)
     DATA_SB(ppemY)
     DATA_SB(bitDepth)
     DATA_SB(flags)
END_STRUCTURE(bitmap_size_table)
STRUCTURE(eblc_table)
     DATA_SL(version)
     DATA_SL(numSizes)
     ARRAY_VC(bitmapSizeTable, here->numSizes, bitmap_size_table)
END_STRUCTURE(eblc_table)
     /* Resume at indexSubTableArray - the organisation of the data has stopped
        making sense. *

     /* Embedded bitmap scaling data - EBSC */
/* Advanced Typographic Tables */
     /* Baseline data - BASE */
STRUCTURE (baseline_data)
     DATA_SL(version)
     DATA_UW(horizAxis)
     DATA_UW(vertAxis)
END_STRUCTURE (baseline_data)
STRUCTURE (axis_table)
     DATA_UW(baseTagList)
     DATA_UW(baseScriptList)
END_STRUCTURE (axis_table)
STRUCTURE(tag_item)
     DATA_N_CH(name, 4)
END_STRUCTURE(tag_item)
STRUCTURE(base_tag_list)
     DATA_UW(baseTagCount)
     ARRAY_VC(tag, here->baseTagCount, tag_item)
END_STRUCTURE(base_tag_list)
STRUCTURE(base_script_record)
     DATA_N_CH(tag, 4)
     DATA_UW(baseScript)
END_STRUCTURE(base_script_record)
STRUCTURE(base_script_list)
     DATA_UW(baseScriptCount)
     ARRAY_VC(baseScriptRecord, here->baseScriptCount, base_script_record)
END_STRUCTURE(base_script_list)
STRUCTURE(base_lang_sys_record)
     DATA_N_CH(baseLangSysTag, 4)
     DATA_UW(minMax)
END_STRUCTURE(base_lang_sys_record)
STRUCTURE(base_script_table)
     DATA_UW(baseValues)
     DATA_UW(defaultMinMax)
     DATA_UW(baseLangSysCount)
     ARRAY_VC(baseLangSysRecord, here->baseLangSysCount, base_lang_sys_record)
END_STRUCTURE(base_script_table)
STRUCTURE(base_values_table)
     DATA_UW(defaultIndex)
     DATA_UW(baseCoordCount)
     ARRAY_VC(baseCoord, here->baseCoordCount, UW_item)
END_STRUCTURE(base_values_table)
STRUCTURE(feat_minmax_record)
     DATA_N_CH(featureTableTag, 4)
     DATA_UW(minCoord)
     DATA_UW(maxCoord)
END_STRUCTURE(feat_minmax_record)
STRUCTURE(mimmax_table)
     DATA_UW(minCoord)
     DATA_UW(maxCoord)
     DATA_UW(featMinMaxCount)
     ARRAY_VC(featMinMaxRecord, here->featMinMaxCount, feat_minmax_record)
END_STRUCTURE(mimmax_table)
STRUCTURE(base_coord_table)
     DATA_UW(baseCoordFormat)
     DATA_SW(coordinate)
     IF (here->baseCoordFormat == 1)
     ELSEIF (here->baseCoordFormat == 2)
         DATA_UW(referenceGlyph)
         DATA_UW(baseCoordPoint)
     ELSEIF(here->baseCoordFormat == 3)
         DATA_UW(deviceTable)
     ENDIF
END_STRUCTURE(base_coord_table)
     /* Glyph definition data - GDEF */
STRUCTURE(gdef_table)
     DATA_UL(version)
     DATA_UW(glyphClassDef)
     DATA_UW(attachList)
     DATA_UW(ligCaretList)
     DATA_UW(markAttachClassDef)
     IF (here->version >= 0x10002)
         DATA_UW(markGlyphSetsDef)
     ENDIF
     IF (here->version >= 0x10003)
         DATA_UL(itemVarStore)
     ENDIF
END_STRUCTURE(gdef_table)
STRUCTURE(class_range_record)
     DATA_UW(start)
     DATA_UW(end)
     DATA_UW(class)
END_STRUCTURE(class_range_record)
STRUCTURE(class_definition_table)
     DATA_UW(classFormat)
     IF (1 == here->classFormat)
         DATA_UW(startGlyph)
         DATA_UW(glyphCount)
         DATA_VL_UW(classValueArray, glyphCount)
     ELSEIF (2 == here->classFormat)
         DATA_UW(classRangeCount)
         ARRAY_VC(classRangeRecord, here->classRangeCount, class_range_record)
     ENDIF
END_STRUCTURE(class_definition_table)
STRUCTURE(attachment_list_table)
     DATA_UW(coverage)
     DATA_UW(glyphCount)
     ARRAY_VC(attachPoint, here->glyphCount, UW_item)
END_STRUCTURE(attachment_list_table)
STRUCTURE(attach_point_table)
     DATA_UW(pointCount)
     ARRAY_VC(pointIndex, here->pointCount, UW_item)
END_STRUCTURE(attach_point_table)
STRUCTURE(lig_caret_list_table)
     DATA_UW(coverage)
     DATA_UW(ligGlyphCount)
     DATA_VL_UW(ligGlyph, ligGlyphCount)
END_STRUCTURE(lig_caret_list_table)
STRUCTURE(ligature_glyph_table)
     DATA_UW(caretCount)
     ARRAY_VC(caretValue, here->caretCount, UW_item)
END_STRUCTURE(ligature_glyph_table)
STRUCTURE(caret_value_table)
     DATA_UW(caretValueFormat)
     IF (1 == here->caretValueFormat)
         DATA_SW(coordinate)
     ELSEIF(2 == here->caretValueFormat)
         DATA_UW(caretValuePoint)
     ELSEIF(3 == here->caretValueFormat)
         ALT_SW(coordinate)
         DATA_UW(deviceTable)
     ENDIF
END_STRUCTURE(caret_value_table)
STRUCTURE(mark_glyph_sets_table)
    DATA_UW(format)
    DATA_UW(count)
    DATA_VL_UL(offsets, count)
END_STRUCTURE(mark_glyph_sets_table)
STRUCTURE(range_record)
     DATA_UW(start)
     DATA_UW(end)
     DATA_UW(startCoverageIndex)
END_STRUCTURE(range_record)
STRUCTURE(coverage_table)
     DATA_UW(coverageFormat)
     IF (1 == here->coverageFormat)
         DATA_UW(glyphCount)
         DATA_VL_UW(glyphArray, glyphCount)
     ELSEIF (2 == here->coverageFormat)
         DATA_UW(rangeCount)
         ARRAY_VC(rangeRecord, here->rangeCount, range_record)
     ENDIF
END_STRUCTURE(coverage_table)
     /* Glyph positioning data - GPOS */
STRUCTURE(gpos_header)
     DATA_SL(version)
     DATA_UW(scriptList)
     DATA_UW(featureList)
     DATA_UW(lookupList)
END_STRUCTURE(gpos_header)
STRUCTURE(lookup_list)
     DATA_UW(lookupCount)
     DATA_VL_UW(lookup, lookupCount)
END_STRUCTURE(lookup_list)
STRUCTURE(lookup_table)
     DATA_UW(lookupType)
     DATA_UW(lookupFlag)
     DATA_UW(subTableCount)
     DATA_VL_UW(subTable, subTableCount)
     IF (here->lookupFlag & 0x0010)
     DATA_UW(markFilteringSet)
     ENDIF
END_STRUCTURE(lookup_table)
     extern unsigned short int VALUE_FORMAT;
STRUCTURE(ValueRecord)
     IF (VALUE_FORMAT & 1)
         DATA_SW(xPlacement)
     ENDIF
     IF (VALUE_FORMAT & 2)
         DATA_SW(yPlacement)
     ENDIF
     IF (VALUE_FORMAT & 4)
         DATA_SW(xAdvance)
     ENDIF
     IF (VALUE_FORMAT & 8)
         DATA_SW(yAdvance)
     ENDIF
     IF (VALUE_FORMAT & 16)
         DATA_UW(xPlaDevice)
     ENDIF
     IF (VALUE_FORMAT & 32)
         DATA_UW(yPlaDevice)
     ENDIF
     IF (VALUE_FORMAT & 64)
         DATA_UW(xAdvDevice)
     ENDIF
     IF (VALUE_FORMAT & 128)
         DATA_UW(yAdvDevice)
     ENDIF
END_STRUCTURE(ValueRecord)
/* STRUCTURE(single_adjustment_positioning) */
STRUCTURE(sap)
     DATA_UW(posFormat)
     DATA_UW(coverage)
     DATA_UW(valueFormat)
     IF (VALUE_FORMAT = here->valueFormat) ENDIF
     IF (1 == here->posFormat)
         DATA_STR(value, ValueRecord)
     ELSEIF (2 == here->posFormat)
         DATA_UW(valueCount)
         ARRAY_VC(valueArr, here->valueCount, ValueRecord)
     ENDIF
END_STRUCTURE(sap)
/* END_STRUCTURE(single_adjustment_positioning) */
     extern unsigned short int VALUE_FORMAT1, VALUE_FORMAT2;
     extern int class_2_count;
STRUCTURE(Class2Record)
     IF (VALUE_FORMAT = VALUE_FORMAT1) ENDIF
     DATA_STR(value1, ValueRecord)
     IF (VALUE_FORMAT = VALUE_FORMAT2) ENDIF
     DATA_STR(value2, ValueRecord)
END_STRUCTURE(Class2Record)
STRUCTURE(Class1Record)
     ARRAY_VC(class2Record, class_2_count, Class2Record)
END_STRUCTURE(Class1Record)
STRUCTURE(pairpos_table)
     DATA_UW(posFormat)
     DATA_UW(coverage)
     DATA_UW(valueFormat1)
     IF (VALUE_FORMAT1 = here->valueFormat1) ENDIF
     DATA_UW(valueFormat2)
     IF (VALUE_FORMAT2 = here->valueFormat2) ENDIF
     IF (1 == here->posFormat)
         DATA_UW(pairSetCount)
         DATA_VL_UW(pairSetOffset, pairSetCount)
     ELSEIF (2 == here->posFormat)
         DATA_UW(classDef1)
         DATA_UW(classDef2)
         DATA_UW(class1Count)
         DATA_UW(class2Count)
         IF (class_2_count = here->class2Count) ENDIF
         ARRAY_VC(class1Record, here->class1Count, Class1Record)
     ENDIF
END_STRUCTURE(pairpos_table)
/* Must set VALUE_FORMAT1 and VALUE_FORMAT2 from pairpos_table. */
STRUCTURE(pair_value_record)
     DATA_UW(secondGlyph)
     IF (VALUE_FORMAT = VALUE_FORMAT1) ENDIF
     DATA_STR(value1, ValueRecord)
     IF (VALUE_FORMAT = VALUE_FORMAT2) ENDIF
     DATA_STR(value2, ValueRecord)
END_STRUCTURE(pair_value_record)
STRUCTURE(pairset_table)
     DATA_UW(pairValueCount)
     ARRAY_VC(pairValueRecord, here->pairValueCount, pair_value_record)
END_STRUCTURE(pairset_table)
STRUCTURE(entry_exit_record)
     DATA_UW(entryAnchor)
     DATA_UW(exitAnchor)
END_STRUCTURE(entry_exit_record)
STRUCTURE(cursive_positioning_table)
     DATA_UW(posFormat)
     DATA_UW(coverage)
     DATA_UW(entryExitCount)
     ARRAY_VC(entryExitRecord, here->entryExitCount, entry_exit_record)
END_STRUCTURE(cursive_positioning_table)
extern int class_count;
STRUCTURE(mark_basepos_table)
     DATA_UW(posFormat)
     DATA_UW(markCoverage)
     DATA_UW(baseCoverage)
     DATA_UW(classCount)
     IF (class_count = here->classCount) ENDIF
     DATA_UW(markArray)
     DATA_UW(baseArray)
END_STRUCTURE(mark_basepos_table)
STRUCTURE(base_record)
     DATA_VR_UW(baseAnchor, class_count)
END_STRUCTURE(base_record)
STRUCTURE(base_array_table)
     DATA_UW(baseCount)
     ARRAY_VC(baseRecord, here->baseCount, base_record)
END_STRUCTURE(base_array_table)
STRUCTURE(mark_lig_pos_table)
     DATA_UW(posFormat)
     DATA_UW(markCoverage)
     DATA_UW(ligCoverage)
     DATA_UW(classCount)
     IF (class_count = here->classCount) ENDIF
     DATA_UW(markArray)
     DATA_UW(ligatureArray)
END_STRUCTURE(mark_lig_pos_table)
/* Type for component record is the same as for a base record */
STRUCTURE(ligature_attach)
     DATA_UW(componentCount)
     ARRAY_VC(componentRecord, here->componentCount, base_record)
END_STRUCTURE(ligature_attach)
STRUCTURE(ligature_array_table)
     DATA_SW(ligatureCount)
     DATA_VL_UW(ligatureAttach, ligatureCount)
END_STRUCTURE(ligature_array_table)
STRUCTURE(mark_mark_pos_table)
     DATA_UW(posFormat)
     DATA_UW(mark1Coverage)
     DATA_UW(mark2Coverage)
     DATA_UW(classCount)
     IF (class_count = here->classCount) ENDIF
     DATA_UW(mark1Array)
     DATA_UW(mark2Array)
END_STRUCTURE(mark_mark_pos_table)
/* Type for mark2 record is the same as for a base record */
STRUCTURE(mark2_array)
     DATA_UW(mark2Count)
     ARRAY_VC(mark2Record, here->mark2Count, base_record)
END_STRUCTURE(mark2_array)
STRUCTURE(pos_rule_set)
     DATA_UW(posRuleCount)
     ARRAY_VC(posRule, here->posRuleCount, UW_item)
END_STRUCTURE(pos_rule_set)
STRUCTURE(pos_lookup_record)
     DATA_UW(sequenceIndex)
     DATA_UW(lookupListIndex)
END_STRUCTURE(pos_lookup_record)
STRUCTURE(context_pos_table)
     DATA_UW(posFormat)
     IF (1 == here->posFormat)
       DATA_UW(coverage)
       DATA_UW(posRuleSetCount)
       ARRAY_VC(posRuleSet, here->posRuleSetCount, pos_rule_set)
     ELSEIF (2 == here->posFormat)
       ALT_UW(coverage)
       DATA_UW(classDef)
       DATA_UW(posClassSetCnt)
       ARRAY_VC(posClassSet, here->posClassSetCnt, UW_item)
     ELSEIF (3 == here->posFormat)
       DATA_UW(glyphCount)
       DATA_UW(posCount)
/* Should the next item be unified with coverage above? */
       ARRAY_VC(coverage3, here->glyphCount, UW_item)
       ARRAY_VC(posLookupRecord, here->posCount, pos_lookup_record)
     ENDIF
END_STRUCTURE(context_pos_table)
STRUCTURE(pos_rule_table)
     DATA_UW(glyphCount)
     DATA_UW(posCount)
     ARRAY_VC(input, here->glyphCount-1, UW_item)
     ARRAY_VC(posLookupRecord, here->posCount, pos_lookup_record)
END_STRUCTURE(pos_rule_table)
STRUCTURE(pos_class_set_table)
     DATA_UW(posClassRuleCnt)
     ARRAY_VC(posClassRule, here->posClassRuleCnt, UW_item)
END_STRUCTURE(pos_class_set_table)
STRUCTURE(pos_class_rule_table)
     DATA_UW(glyphCount)
     DATA_UW(posCount)
     ARRAY_VC(class, here->glyphCount-1, UW_item)
     ARRAY_VC(posLookupRecord, here->posCount, pos_lookup_record)
END_STRUCTURE(pos_class_rule_table)
STRUCTURE(chain_pos_rule)
     DATA_UW(backtrackGlyphCount)
     DATA_VL_UW(backtrack, backtrackGlyphCount)
     DATA_UW(inputGlyphCount)
     DATA_VR_UW(input, here->inputGlyphCount-1)
     DATA_UW(lookaheadGlyphCount)
     DATA_VL_UW(lookahead, lookaheadGlyphCount)
     DATA_UW(posCount)
     ARRAY_VC(posLookupRecord, here->posCount, pos_lookup_record)
END_STRUCTURE(chain_pos_rule)
STRUCTURE(chain_pos_rule_set)
     DATA_UW(chainPosRuleCount)
     DATA_VL_UW(chainPosRule, chainPosRuleCount)
END_STRUCTURE(chain_pos_rule_set)
STRUCTURE(chain_pos_class_set)
     DATA_UW(chainPosClassRuleCnt)
     DATA_VL_UW(chainPosClassRule, chainPosClassRuleCnt)
     /* Use chain_pos_rule instead of chain_pos_class_rule! */
END_STRUCTURE(chain_pos_class_set)
STRUCTURE(chain_context_pos_table)
     DATA_UW(posFormat)
     IF (1 == here->posFormat)
       DATA_UW(coverage)
       DATA_UW(chainPosRuleSetCount)
       DATA_VL_UW(chainPosRuleSet, chainPosRuleSetCount)
     ELSEIF (2 == here->posFormat)
       ALT_UW(coverage)
       DATA_UW(backtrackClassDef)
       DATA_UW(inputClassDef)
       DATA_UW(lookaheadClassDef)
       DATA_UW(chainPosClassSetCnt)
       DATA_VL_UW(chainPosClassSet, chainPosClassSetCnt)
     ELSEIF (3 == here->posFormat)
       DATA_UW(backtrackGlyphCount)
       DATA_VL_UW(coverageb, backtrackGlyphCount)
       DATA_UW(inputGlyphCount)
       DATA_VL_UW(coveragei, inputGlyphCount)
       DATA_UW(lookaheadGlyphCount)
       DATA_VL_UW(coveragea, lookaheadGlyphCount)
       DATA_UW(posCount)
       ARRAY_VC(posLookupRecord, here->posCount, pos_lookup_record)
     ENDIF
END_STRUCTURE(chain_context_pos_table)
STRUCTURE(extension_pos_table)
     DATA_UW(posFormat)
     DATA_UW(extensionLookupType)
     DATA_UL(extensionOffset)
END_STRUCTURE(extension_pos_table)
STRUCTURE(anchor_table)
    DATA_UW(anchorFormat)
    DATA_SW(xCoordinate)
    DATA_SW(yCoordinate)
    IF (2 == here->anchorFormat)
      DATA_UW(anchorPoint)
    ELSEIF (3 == here->anchorFormat)
      DATA_UW(xDeviceTable)
      DATA_UW(yDeviceTable)
    ENDIF
END_STRUCTURE(anchor_table)
STRUCTURE(mark_record)
    DATA_UW(class)
    DATA_UW(markAnchor)
END_STRUCTURE(mark_record)
STRUCTURE (mark_array_table)
    DATA_UW(markCount)
    ARRAY_VC(markRecord, here->markCount, mark_record)
END_STRUCTURE (mark_array_table)
STRUCTURE(script_record)
     DATA_N_CH(tag, 4)
     DATA_UW(script)
END_STRUCTURE(script_record)
STRUCTURE(script_list)
     DATA_UW(scriptCount)
     ARRAY_VC(scriptRecord, here->scriptCount, script_record)
END_STRUCTURE(script_list)
STRUCTURE(lang_sys_record)
     DATA_N_CH(tag, 4)
     DATA_UW(langSys)
END_STRUCTURE(lang_sys_record)
STRUCTURE(script_table)
     DATA_UW(defaultLangSys)
     DATA_UW(langSysCount)
     ARRAY_VC(langSysRecord, here->langSysCount, lang_sys_record)
END_STRUCTURE(script_table)
STRUCTURE(lang_sys)
     DATA_UW(lookupOrder) /* MBZ */
     DATA_UW(reqFeatureIndex)
     DATA_UW(featureCount)
     DATA_VL_UW(featureIndex, featureCount)
END_STRUCTURE(lang_sys)
STRUCTURE(feature_record)
     DATA_N_CH(tag, 4)
     DATA_UW(feature)
END_STRUCTURE(feature_record)
STRUCTURE(feature_list)
     DATA_UW(featureCount)
     ARRAY_VC(featureRecord, here->featureCount, feature_record)
END_STRUCTURE(feature_list)
STRUCTURE(feature_table)
     DATA_UW(featureParams) /* MBZ */
     DATA_UW(lookupCount)
     DATA_VL_UW(lookupListIndex, lookupCount)
END_STRUCTURE(feature_table)
     /* Glyph substitution data - GSUB */
STRUCTURE(gsub_header)
     DATA_UL(version)
     DATA_UW(scriptList)
     DATA_UW(featureList)
     DATA_UW(lookupList)
END_STRUCTURE(gsub_header)
STRUCTURE(single_subst)
     DATA_UW(substFormat)
     DATA_UW(coverage)
     IF (1 == here->substFormat)
         DATA_SW(deltaGlyphID)
     ELSEIF (2 == here->substFormat)
         DATA_UW(glyphCount)
         DATA_VL_UW(substitute, glyphCount)
     ENDIF
END_STRUCTURE(single_subst)
STRUCTURE(multiple_subst)
     DATA_UW(substFormat)
     DATA_UW(coverage)
     DATA_UW(sequenceCount)
     DATA_VL_UW(sequence, sequenceCount)
END_STRUCTURE(multiple_subst)
STRUCTURE(sequence_table)
     DATA_UW(glyphCount)
     DATA_VL_UW(substitute, glyphCount)
END_STRUCTURE(sequence_table)
STRUCTURE(alternate_subst)
     DATA_UW(substFormat)
     DATA_UW(coverage)
     DATA_UW(alternateSetCount)
     DATA_VL_UW(alternateSet, alternateSetCount)
END_STRUCTURE(alternate_subst)
STRUCTURE(alternate_set_table)
     DATA_UW(glyphCount)
     DATA_VL_UW(alternate, glyphCount)
END_STRUCTURE(alternate_set_table)
STRUCTURE(ligature_subst)
     DATA_UW(substFormat)
     DATA_UW(coverage)
     DATA_UW(ligSetCount)
     DATA_VL_UW(ligatureSet, ligSetCount)
END_STRUCTURE(ligature_subst)
STRUCTURE(ligature_set)
     DATA_UW(ligatureCount)
     DATA_VL_UW(ligature, ligatureCount)
END_STRUCTURE(ligature_set)
STRUCTURE(subst_ligature_table) /* As opposed to pos */
     DATA_UW(ligGlyph)
     DATA_UW(compCount)
     DATA_VR_UW(component, here->compCount - 1)
END_STRUCTURE(subst_ligature_table)
STRUCTURE(subst_lookup_record)
     DATA_UW(sequenceIndex)
     DATA_UW(lookupListIndex)
END_STRUCTURE(subst_lookup_record)
STRUCTURE(context_subst)
     DATA_UW(substFormat)
     IF (1 == here->substFormat)
         DATA_UW(coverage)
         DATA_UW(subRuleSetCount)
         DATA_VL_UW(subRuleSet, subRuleSetCount)
     ELSEIF (2 == here->substFormat)
         ALT_UW(coverage)
         DATA_UW(classDef)
         DATA_UW(subClassSetCnt)
         DATA_VL_UW(subClassSet, subClassSetCnt)
     ELSEIF (3 == here->substFormat)
         DATA_UW(glyphCount)
         DATA_UW(substCount)
         DATA_VL_UW(coverage3, glyphCount)
         ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record) 
     ENDIF
END_STRUCTURE(context_subst)
STRUCTURE(sub_class_set)
     DATA_UW(subClassRuleCnt)
     DATA_VL_UW(subClassRule, subClassRuleCnt)
END_STRUCTURE(sub_class_set)
STRUCTURE(sub_class_rule)
     DATA_UW(glyphCount)
     DATA_UW(substCount)
     DATA_VR_UW(class, here->glyphCount-1)
     ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record)
END_STRUCTURE(sub_class_rule)
STRUCTURE(sub_rule_set)
     DATA_UW(subRuleCount)
     DATA_VL_UW(subRule, subRuleCount)
END_STRUCTURE(sub_rule_set)
STRUCTURE(sub_rule)
     DATA_UW(glyphCount)
     DATA_UW(substCount)
     DATA_VR_UW(input, here->glyphCount - 1)
     ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record)
END_STRUCTURE(sub_rule)
STRUCTURE(chain_context_subst)
     DATA_UW(substFormat)
     IF (1 == here->substFormat)
	 DATA_UW(coverage)
         DATA_UW(chainSubRuleSetCount)
	 DATA_VL_UW(chainSubRuleSet, chainSubRuleSetCount)
     ELSEIF (2 == here->substFormat)
	 ALT_UW(coverage)
	 DATA_UW(backtrackClassDef)
	 DATA_UW(inputClassDef)
         DATA_UW(lookaheadClassDef)
         DATA_UW(chainSubClassSetCnt)
         DATA_VL_UW(chainSubClassSet, chainSubClassSetCnt)
     ELSEIF (3 == here->substFormat)
         DATA_UW(backtrackGlyphCount)
         DATA_VL_UW(coverageb, backtrackGlyphCount)
	 DATA_UW(inputGlyphCount)
         DATA_VL_UW(coveragei, inputGlyphCount)
         DATA_UW(lookaheadGlyphCount)
         DATA_VL_UW(coveragea, lookaheadGlyphCount)
         DATA_UW(substCount)
         ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record)
     ENDIF
END_STRUCTURE(chain_context_subst)
STRUCTURE(chain_sub_rule_set)
     DATA_UW(chainSubRuleCount)
     DATA_VL_UW(chainSubRule, chainSubRuleCount)
END_STRUCTURE(chain_sub_rule_set)
STRUCTURE(chain_sub_rule)
     DATA_UW(backtrackGlyphCount)
     DATA_VL_UW(backtrack, backtrackGlyphCount)
     DATA_UW(inputGlyphCount)
     DATA_VR_UW(input, here->inputGlyphCount - 1)
     DATA_UW(lookaheadGlyphCount)
     DATA_VL_UW(lookahead, lookaheadGlyphCount)
     DATA_UW(substCount)
     ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record)
END_STRUCTURE(chain_sub_rule)
STRUCTURE(chain_sub_class_set)
     DATA_UW(chainSubClassRuleCnt)
     DATA_VL_UW(chainSubClassRule, chainSubClassRuleCnt)
END_STRUCTURE(chain_sub_class_set)
STRUCTURE(chain_sub_class_rule)
     DATA_UW(backtrackGlyphCount)
     DATA_VL_UW(backtrack, backtrackGlyphCount)
     DATA_UW(inputGlyphCount)
     DATA_VR_UW(input, here->inputGlyphCount - 1)
     DATA_UW(lookaheadGlyphCount)
     DATA_VL_UW(lookahead, lookaheadGlyphCount)
     DATA_UW(substCount)
     ARRAY_VC(substLookupRecord, here->substCount, subst_lookup_record)
END_STRUCTURE(chain_sub_class_rule)
STRUCTURE(reverse_chain_single_subst)
     DATA_UW(substFormat)
     DATA_UW(coverage1)
     DATA_UW(backtrackGlyphCount)
     DATA_VL_UW(coverage2, backtrackGlyphCount)
     DATA_UW(lookaheadGlyphCount)
     DATA_VL_UW(coverage3, lookaheadGlyphCount)
     DATA_UW(glyphCount)
     DATA_VL_UW(substitute, glyphCount)
END_STRUCTURE(reverse_chain_single_subst)

STRUCTURE(PCL_table)
     DATA_UL(version)
     DATA_UL(fontNumber)
     DATA_UW(pitch)
     DATA_UW(xHeight)
     DATA_UW(style)
     DATA_UW(typeFamily)
     DATA_UW(capHeight)
     DATA_UW(symbolSet)
     DATA_N_CH(typeface, 16)
     DATA_N_CH(characterComplement, 8)
     DATA_N_CH(fileName, 6)
     DATA_SB(strokeWeight)
     DATA_SB(widthType)
     DATA_UB(serifStyle)
     DATA_UB(pad)
END_STRUCTURE(PCL_table)
/* Resume at structures for lookup type 9. */
     /* Glyph substitution data - GSUB */

STRUCTURE(kern_pair)
    DATA_UW(left)
    DATA_UW(right)
    DATA_SW(value)
END_STRUCTURE(kern_pair)
STRUCTURE(kern_subtable)
    DATA_UW(version)
    DATA_UW(length)
    DATA_UW(coverage)
    IF (0 == here->version)
        DATA_UW(nPairs)
        DATA_UW(searchRange)
        DATA_UW(entrySelector)
        DATA_UW(rangeShift)
        ARRAY_VC(pairs, here->nPairs, kern_pair)
    ELSEIF (2 == here->version)
        DATA_UW(rowWidth)
        DATA_UW(leftClassTable)
        DATA_UW(rightClassTable)
        DATA_UW(array)
    ELSE
    ENDIF
END_STRUCTURE(kern_subtable)
STRUCTURE(kern_table)
    DATA_UW(version)
    DATA_UW(nTables)
    ARRAY_VC(sub, here->nTables, kern_subtable)
END_STRUCTURE(kern_table)
extern unsigned short int N_COLORS, N_PALETTE, N_PALETTE_ENTRIES;
STRUCTURE(cpal_header)
    DATA_UW(version)
    DATA_UW(numPalettesEntries)
    IF(N_PALETTE_ENTRIES = here->numPalettesEntries) ENDIF
    DATA_UW(numPalette)
    IF (N_PALETTE = here->numPalette) ENDIF
    DATA_UW(numColorRecords)
    IF (N_COLORS = here->numColorRecords) ENDIF
    DATA_UL(offsetFirstColorRecord)
    DATA_VL_UW(colorRecordIndices, numPalette)
    IF (1 == here->version)
        DATA_UL(offsetPaletteTypeArray)
        DATA_UL(offsetPaletteLabelArray)
        DATA_UL(offsetPaletteEntryLabelArray)
    ENDIF
END_STRUCTURE(cpal_header)
STRUCTURE(color_record)
    DATA_UB(blue)
    DATA_UB(green)
    DATA_UB(red)
    DATA_UB(alpha)
END_STRUCTURE(color_record)
STRUCTURE(cra)
    ARRAY_VC(x, N_COLORS, color_record)
END_STRUCTURE(cra)
STRUCTURE(pta)
    DATA_VR_UL(paletteType, N_PALETTE)
END_STRUCTURE(pta)
STRUCTURE(pla)
    DATA_VR_UW(paletteLabel, N_PALETTE)
END_STRUCTURE(pla)
STRUCTURE(pela)
    DATA_VR_UW(paletteEntryLabel, N_PALETTE_ENTRIES)
END_STRUCTURE(pela)
extern unsigned short int N_COLR_BGR, N_COLR_LAYER;
STRUCTURE(base_glyph_record)
    DATA_UW(GID)
    DATA_UW(firstLayerIndex)
    DATA_UW(numLayers)
END_STRUCTURE(base_glyph_record)
STRUCTURE(layer_record)
    DATA_UW(GID)
    DATA_UW(paletteIndex)
END_STRUCTURE(layer_record)
STRUCTURE(colr_header)
    DATA_UW(version)
    DATA_UW(numBaseGlyphRecords)
    IF (N_COLR_BGR = here->numBaseGlyphRecords) ENDIF
    DATA_UL(offsetBaseGlyphRecord)
    DATA_UL(offsetLayerRecord)
    DATA_UW(numLayerRecords)
    IF (N_COLR_LAYER = here->numLayerRecords) ENDIF
    IF (0) /* Conveniences for generation */
        ARRAY_VC(baseGlyphRecord, here->numBaseGlyphRecords, base_glyph_record)
        ARRAY_VC(layerRecord, here->numLayerRecords, layer_record)
        DATA_UL(maxBaseGlyphRecords)
        DATA_UL(maxLayerRecords)
    ENDIF
END_STRUCTURE(colr_header)
STRUCTURE(base_glyph_records)
    ARRAY_VC(base, N_COLR_BGR, base_glyph_record)
END_STRUCTURE(base_glyph_records)
STRUCTURE(layer_records)
    ARRAY_VC(layers, N_COLR_LAYER, layer_record)
END_STRUCTURE(layer_records)
/* gasp */
STRUCTURE(gasp_range)
    DATA_UW(maxPPEM)
    DATA_UW(behaviour)
END_STRUCTURE(gasp_range)
STRUCTURE(gasp_header)
    DATA_UW(version)
    DATA_UW(numRanges)
    ARRAY_VC(gaspRanges, here->numRanges, gasp_range)
END_STRUCTURE(gasp_header)
/* Justification data - JSTF */
/* Other OpenType Tables */
     /* Digital signature - DSIG */
     /* Grid-fitting/Scan-conversion - gasp */
     /* Horizontal device metrics - hdmx */
     /* Linear threshold data - LTSH */
     /* PCL 5 data - PCLT */
     /* Vertical device metrics - VDMX */
     /* Vertical Metrics header - vhea */
     /* Vertical Metrics - vmtx */

     extern int byte_dump_count;
STRUCTURE(UB_256_dump)
     IF (byte_dump_count <= 16)
         DATA_N_UB(data4, 16)
     ELSEIF (byte_dump_count <= 32)
         DATA_N_UB(data5, 32)
     ELSEIF (byte_dump_count <= 64)
         DATA_N_UB(data6, 64)
     ELSEIF (byte_dump_count <= 128)
         DATA_N_UB(data7, 128)
     ELSEIF (byte_dump_count <= 256) 
         DATA_N_UB(data8, 256)
     ELSEIF (byte_dump_count <= 512) 
         DATA_N_UB(data9, 512)
     ELSEIF (byte_dump_count <= 1024) 
         DATA_N_UB(dataa, 1024)
     ELSE 
         DATA_N_UB(dataah, 1600)
     ENDIF
END_STRUCTURE(UB_256_dump)
STRUCTURE(string_dump)
     DATA_VR_CH(chars, byte_dump_count)
END_STRUCTURE(string_dump)
STRUCTURE(one_value)
     IF (0)
         DATA_SB(size)
     ENDIF
     IF (1 == here->size)
         ALT_UB(val)
     ELSEIF (2 == here->size)
         ALT_UW(val)
     ELSEIF (4 == here->size)
         ALT_UL(val)
     ELSEIF (8 == here->size)
         DATA_SQ(val)
     ENDIF
END_STRUCTURE(one_value)
STRUCTURE(tag)
     DATA_N_CH(x, 4)
END_STRUCTURE(tag)
extern int Feat_version;
STRUCTURE(FeatureDefn)
    IF (Feat_version >= 0x00020000)
        DATA_UL(id)
    ELSE
        ALT_UW(id)
    ENDIF
    DATA_UW(numSettings)
    IF (Feat_version >= 0x00020000)
        DATA_UW(reserved)
    ENDIF
    DATA_UL(offset)
    DATA_UW(flags)
    DATA_UW(label)
END_STRUCTURE(FeatureDefn)
STRUCTURE(FeatureSettingDefn)
    DATA_SW(value)
    DATA_UW(label)
END_STRUCTURE(FeatureSettingDefn)
STRUCTURE(Feat)
    DATA_UL(version)
    IF (Feat_version = here->version) ENDIF
    DATA_UW(numFeat)
    DATA_UW(reserved1)
    DATA_UL(reserved2)
    ARRAY_VC(Features, here->numFeat, FeatureDefn)
    ARRAY_VC(featSettings, 0, FeatureSettingDefn)
END_STRUCTURE(Feat)
STRUCTURE(pseudo_map)
     IF (silf_version >= 0x00020000)
         DATA_UL(uniCode)
     ELSE
         ALT_UW(uniCode)
     ENDIF
     DATA_UW(nPseudo)
END_STRUCTURE(pseudo_map)
STRUCTURE(lookup_pair)
     DATA_UW(glyphid)
     DATA_UW(index)
END_STRUCTURE(lookup_pair)
STRUCTURE(lookup_class)
     DATA_UW(numIDs)
     DATA_UW(searchRange)
     DATA_UW(entrySelector)
     DATA_UW(rangeShift)
     ARRAY_VC(lookups, here->numIDs, lookup_pair)
END_STRUCTURE(lookup_class)
STRUCTURE(class_map)
     DATA_UW(numClass)
     DATA_UW(numLinear)
     DATA_VR_UW(oClass, here->numClass + 1)
/* Number of elements in glyphs is not documented.  The 'linear'
   class definitions are lists of glyphs. */
     DATA_VR_UW(glyphs,
		((here->oClass[here->numLinear] - here->oClass[0])/2))
     ARRAY_VC(lookups, here->numClass - here->numLinear, lookup_class)
END_STRUCTURE(class_map)
STRUCTURE(pass_range)
     DATA_UW(firstId)
     DATA_UW(lastId)
     DATA_UW(colId)
END_STRUCTURE(pass_range)
STRUCTURE(sil_pass)
     DATA_UB(flags)
     DATA_UB(maxRuleLoop)
     DATA_UB(maxRuleContext)
     DATA_UB(maxBackup)
     DATA_UW(numRules)
     IF (silf_version >= 0x00020000)
         DATA_UW(fsmOffset) /* Only meaningful for format 3.0 onwards */
         DATA_UL(pcCode)
     ENDIF
     DATA_UL(oCode) /* rcCode in Version 2 */
     DATA_UL(aCode)
     DATA_UL(oDebug)
     DATA_UW(numRows)
     DATA_UW(numTransitional)
     DATA_UW(numSuccess)
     DATA_UW(numColumns)
     DATA_UW(numRange)
     DATA_UW(searchRange)
     DATA_UW(entrySelector)
     DATA_UW(rangeShift)
     /* Rev 140, 31 March 2003 is wrong - numRange entries, not numColumns */
     /* ARRAY_VC(ranges, here->numColumns, pass_range) */
     ARRAY_VC(ranges, here->numRange, pass_range)
     DATA_VR_UW(oRuleMap, (here->numSuccess+1)) /* +1 deduced from examples */
     /* grLanna Rule 4 (1-origin) indicates garbage can be inserted in the ruleMap,
	e.g.:
 5 oRuleMap:
 6 [00-]      0 0000      1 0001      3 0003      4 0004      5 0005
 6 [05-]      6 0006
 5 ruleMap:      4 0004      0 0000      1 0001      1 0001      2 0002
 5 minRulePreContext:   0 00
 5 maxRulePreContext:   3 03

The last two bytes above clearly belong to ruleMap.

Based on example of constraints, size of ruleMap arrray is only determined by
the last entry.
     */
     /* DATA_VL_UW(ruleMap, numSuccess) */
     DATA_VR_UW(ruleMap, here->oRuleMap[here->numSuccess])
     DATA_UB(minRulePreContext)
     DATA_UB(maxRulePreContext)
     DATA_VR_UW(startStates,
		(here->maxRulePreContext - here->minRulePreContext + 1))
     DATA_VL_UW(ruleSortKeys, numRules)
     DATA_VL_UB(rulePreContext, numRules)
     IF (silf_version >= 0x00020000)
         DATA_UB(reserved_2)
         DATA_UW(pConstraint)
     ELSE
     /* ALIGN(2) */ /* A possibility! */
     ENDIF
     DATA_VR_UW(oConstraints, (here->numRules + 1))
     DATA_VR_UW(oActions, (here->numRules + 1))
     DATA_VR_SW(stateTrans, (here->numTransitional * here->numColumns))
     /* Constraint offset is index into constraints array - size is determined by
	the last offset, not the difference between last and first. */
     IF (silf_version >= 0x00020000)
         DATA_UB(reserved_3)
         DATA_VL_UB(passConstraints, pConstraint)
     ENDIF
     DATA_VR_SB(constraints, here->oConstraints[here->numRules])
     /* Assume same for actions. */
     DATA_VR_SB(actions, here->oActions[here->numRules])
     /* ALIGN(2) */ /* A possibility! */
     IF (0) /* Needs further investigation! */
         DATA_VL_UW(dActions, numRules)
         DATA_VR_UW(dStates, (here->numRows - here->numRules))
         DATA_VL_UW(dCols, numRows) /* Or numcolumns? */
     ENDIF
END_STRUCTURE(sil_pass)
extern int silf_version;
extern int max_graphite_glyph_ID;
STRUCTURE(justification_level)
    DATA_UB(attrStretch)
    DATA_UB(attrShrink)
    DATA_UB(attrStep)
    DATA_UB(attrWeight)
    DATA_UB(runto)
    DATA_UB(reserved_1)
    DATA_UB(reserved_2)
    DATA_UB(reserved_3)
END_STRUCTURE(justification_level)
STRUCTURE(sil_sub)
     IF (silf_version >= 0x00030000)
         DATA_UL(ruleVersion)
         DATA_UW(passOffset)
         DATA_UW(pseudosOffset)
     ENDIF
     DATA_UW(maxGlyphID)
     IF (max_graphite_glyph_ID = here->maxGlyphID) ENDIF
     DATA_SW(extraAscent)
     DATA_SW(extraDescent)
     DATA_UB(numPasses)
     DATA_UB(iSubst)
     DATA_UB(iPos)
     DATA_UB(iJust)
     DATA_UB(iBidi)
     IF (silf_version >= 0x00030000)
         DATA_UB(flags)
     ELSE
         DATA_UB(lineBreakFlag)
     ENDIF
     DATA_UB(maxPreContext)
     DATA_UB(maxPostcontext)
     DATA_UB(attrPseudo)
     DATA_UB(attrBreakWeight)
     DATA_UB(attrBidi)
     IF (silf_version >= 0x00020000)
/* In sil_sub2, not sil_sub */
         DATA_UB(reserved_4)
         DATA_UB(reserved_5)
         DATA_UB(numJLevels)
         ARRAY_VC(jLevels, here->numJLevels, justification_level)
/* End sil_sub2 additions */
     ENDIF
     DATA_UW(numLigComp)
     DATA_UB(numUserDefn)
     DATA_UB(maxCompPerLig)
     DATA_UB(direction)
     DATA_UB(reserved_1)
     DATA_UB(reserved_2)
     DATA_UB(reserved_3)
     IF (silf_version >= 0x00020000)
/* In sil_sub2, not sil_sub */
         DATA_UB(reserved_6)
         DATA_UB(numCritFeatures)
         DATA_VL_UW(critFeatures, numCritFeatures)
         DATA_UB(reserved_7)
/* End sil_sub2 additions */
     ENDIF
     DATA_UB(numScriptTag)
     ARRAY_VC(scriptTag, here->numScriptTag, tag)
     DATA_UW(lbGID)
     DATA_VR_UL(oPasses, here->numPasses+1)
     DATA_UW(numPseudo)
     DATA_UW(searchPseudo)
     DATA_UW(pseudoSelector)
     DATA_UW(pseudoShift)
     ARRAY_VC(pMaps, here->numPseudo, pseudo_map)
     DATA_STR(classes, class_map)
     ARRAY_VC(passes, here->numPasses, sil_pass)
END_STRUCTURE(sil_sub)
STRUCTURE (silf)
     DATA_UL(version)
     IF ((silf_version = here->version) >= 0x00030000)
         DATA_UL(compilerVersion)
     ENDIF
     DATA_UW(num)
     IF (0x00010000 != (silf_version = here->version))
         DATA_UW(reserved)
     ENDIF
     DATA_VL_UL(offset, num)
     IF (here->version < 0x00040000)
         ARRAY_VC(tables, here->num, sil_sub)
     ENDIF
END_STRUCTURE(silf)
STRUCTURE(sile)
    DATA_UL(version) /* Coded for 00010000 */
    DATA_UL(checksum)
    DATA_SQ(created)
    DATA_SQ(modified)
    DATA_UW(fontNameLength)
    DATA_UW(fontName)
    DATA_UW(fontFileLength)
    DATA_UW(baseFile)
END_STRUCTURE(sile)
STRUCTURE(gloc)
    DATA_UL(version) /* coded for 00010000 */
    DATA_UW(flags)
    DATA_UW(numAttribs)
    IF (1 & here->flags)
        DATA_VR_UL(locations4, 2 + max_graphite_glyph_ID)
    ELSE
        DATA_VR_UW(locations2, 2 + max_graphite_glyph_ID)
    ENDIF
    IF (2 & here->flags)
        DATA_VL_UW(AttribIds, numAttribs)
    ENDIF
END_STRUCTURE(gloc)
STRUCTURE(glat_entry)
    DATA_UB(att_num)
    DATA_UB(num)
    DATA_VL_SW(attributes, num)
END_STRUCTURE(glat_entry)
STRUCTURE(glat_header)
    DATA_UL(version)
#if 0 /* Need special code! */
        ARRAY_VC(entries, too complicated, glat_entry)
#endif
END_STRUCTURE(glat_header)
/* This sill description is for Version 3 */
extern int n_sill_settings;
STRUCTURE(LanguageEntry)
    DATA_N_CH(langcode, 4)
    DATA_UW(numSettings)
    DATA_UW(offset)
#if 0 /* If trust calculation - untested! */
    IF (n_sill_settings = (here->offset - 12) / 8)
    ENDIF
#endif
END_STRUCTURE(LanguageEntry)
STRUCTURE(LangFeatureSetting)
    DATA_UL(featureId)
    DATA_SW(value)
    DATA_UW(reserved)
END_STRUCTURE(LangFeatureSetting)
STRUCTURE(sill)
    DATA_UL(version)
    DATA_UW(numLangs)
    DATA_UW(searchRange)
    DATA_UW(entrySelector)
    DATA_UW(rangeShift)
    ARRAY_VC(entries, here->numLangs + 1, LanguageEntry)
#if 0 /* Easier to use special logic handling it language by language */
    ARRAY_VC(settings, n_sill_settings, LangFeatureSetting)
#endif
END_STRUCTURE(sill)

extern int offSize;
#define MULTIPLE_OFFSET_ARRAYS 0
STRUCTURE(cff_offset)
    IF (1 == offSize)
        ZERO(x_hi)
        ALT_UB(x_lo)
    ELSEIF (2 == offSize)
        ZERO(x_hi)
        ALT_UW(x_lo)
    ELSEIF (3 == offSize)
        ALT_UB(x_hi)
        ALT_UW(x_lo)
    ELSE
        DATA_UW(x_hi)
        DATA_UW(x_lo)
    ENDIF
END_STRUCTURE(cff_offset)
STRUCTURE(cff_index)
    DATA_UW(count)
    IF (here->count)
        DATA_UB(offSize)
        IF (offSize = here->offSize) ENDIF
        ARRAY_VC(offset, here->count + 1, cff_offset)
        DATA_VR_UB(data, 
(((struct cff_offset*) element(here->offset, here->count))->x_hi << 16) + 
 ((struct cff_offset*) element(here->offset, here->count))->x_lo - 1
)
    ENDIF
END_STRUCTURE(cff_index)
STRUCTURE(cff_base)
     /* header */
    DATA_UB(major)
    DATA_UB(minor)
    DATA_UB(hdrSize)
    DATA_UB(offSize)
    DATA_VR_UB(header_extensions, here->hdrSize - 4)
    DATA_STR(name, cff_index)
    DATA_STR(top_dict, cff_index)
    DATA_STR(strings, cff_index)
    DATA_STR(global_subr, cff_index)
END_STRUCTURE(cff_base)
STRUCTURE(code_range)
    DATA_UB(first)
    DATA_UB(nLeft)
END_STRUCTURE(code_range)
STRUCTURE(encoding_supplement)
   DATA_UB(code)
   DATA_UW(glyph) /* SID */
END_STRUCTURE(encoding_supplement)
STRUCTURE(cff_encoding)
    DATA_UB(format)
    IF (0 == here->format & 0x7f)
        DATA_UB(nCodes)
        DATA_VL_UB(code, nCodes)
    ELSEIF (1 == here->format & 0x7f)
        DATA_UB(nRanges)
        ARRAY_VC(range, here->nRanges, code_range)
    ENDIF
    IF (0x80 & here->format)
        DATA_UB(nSubs)
        ARRAY_VC(Supplement, here->nSubs, encoding_supplement)
    ENDIF
END_STRUCTURE(cff_encoding)
extern int charset_n_to_go;
STRUCTURE(charset_range1)
    DATA_UW(first)
    DATA_UB(nLeft)
    IF ((charset_n_to_go -= (1 + here->nLeft)) > 0)
        NOT_LAST_ONE
    ELSE
        LAST_ONE
    ENDIF
END_STRUCTURE(charset_range1)
STRUCTURE(charset_range2)
    DATA_UW(first)
    DATA_UW(nLeft)
    IF ((charset_n_to_go -= (1 + here->nLeft)) > 0)
        NOT_LAST_ONE
    ELSE
        LAST_ONE
    ENDIF
END_STRUCTURE(charset_range2)
STRUCTURE(charset)
    DATA_UB(format)
    IF (charset_n_to_go = N_GLYPHS - 1) ENDIF
    IF (0 == here->format)
        DATA_VR_UW(glyph, N_GLYPHS - 1)
    ELSEIF (1 == here->format && charset_n_to_go > 0)
        ARRAY_TC(Range1, charset_range1)
    ELSEIF (2 == here->format && charset_n_to_go > 0)
        ARRAY_TC(Range2, charset_range2)
    ENDIF
END_STRUCTURE(charset)
  /* Extended metamorphosis table (morx) */
STRUCTURE(feature_entry)
    DATA_UW(featureType)
    DATA_UW(featureSetting)
    DATA_UL(enableFlags)
    DATA_UL(disableFlags)
END_STRUCTURE(feature_entry)
STRUCTURE(StateHeader)
    DATA_UW(stateSize)
    DATA_UW(classTable)
    DATA_UW(stateArray)
    DATA_UW(entryTable)
END_STRUCTURE(StateHeader)
extern int N_AAT_CLASSES, N_AAT_STATES, N_AAT_ACTIONS, AAT_TABLE;
STRUCTURE(STXHeader)
    DATA_UL(nClasses)
    IF (N_AAT_CLASSES = here->nClasses) ENDIF
    DATA_UL(classTableOffset)
    DATA_UL(stateArrayOffset)
    DATA_UL(entryTableOffset)
/* Clever, but timing does not work.  Must set manually. */
/*  IF ((N_AAT_STATES = 0,
         (N_AAT_CLASSES > 0) &&
         (N_AAT_STATES =
 	    (here->entryTableOffset - here->stateArrayOffset) / N_AAT_CLASSES)))
    ENDIF */
END_STRUCTURE(STXHeader)
STRUCTURE(response_array)
    DATA_VR_UW(action, N_AAT_CLASSES)
END_STRUCTURE(response_array)
STRUCTURE(state_array)
    ARRAY_VC(responses, N_AAT_STATES, response_array)
END_STRUCTURE(state_array)
STRUCTURE(action)
    DATA_UW(newState)
    DATA_UW(flags)
    IF (MORX_CONTEXTUAL == AAT_TABLE)
        DATA_UW(markIndex)
        DATA_UW(currentIndex)
    ENDIF
    IF (MORX_LIGATURE == AAT_TABLE)
        DATA_UW(ligActionIndex)
    ENDIF
END_STRUCTURE(action)
STRUCTURE(actions)
    ARRAY_VC(action, N_AAT_ACTIONS, action)
END_STRUCTURE(actions)
STRUCTURE(metamorphosis_table)
    DATA_UL(length)
    DATA_UL(coverage)
    DATA_UL(subFeatureFlags)
#define MORX_COMLEN 12 /* Offset to here */
#define MORX_INDIC 0  /* Indic rearrangement */
#define MORX_INDIC_HDR_SIZE (MORX_COMLEN + 16)
    IF (MORX_INDIC == (0xFF & here->coverage))
        DATA_STR(indic, STXHeader)
        IF (here->header_size = MORX_INDIC_HDR_SIZE) ENDIF
#define MORX_CONTEXTUAL 1 /* contextual subtable */
#define MORX_CONTEXTUAL_HDR_SIZE (MORX_COMLEN + 16 + 4)
    ELSEIF (MORX_CONTEXTUAL == (0xFF & here->coverage))
	DATA_STR(context, STXHeader)
        DATA_UL(substitutionTable)
	IF (here->header_size = MORX_CONTEXTUAL_HDR_SIZE) ENDIF
#define MORX_LIGATURE 2  /* ligature subtable */
#define MORX_LIGATURE_HDR_SIZE (MORX_COMLEN + 16 + 12)
    ELSEIF (MORX_LIGATURE == (0xFF & here->coverage))
        DATA_STR(ligature, STXHeader)
        DATA_UL(ligActionOffset)
        DATA_UL(componentOffset)
        DATA_UL(ligatureOffset)
	IF (here->header_size = MORX_LIGATURE_HDR_SIZE) ENDIF
#define MORX_OTHER_HDR_SIZE (MORX_COMLEN)
    ELSE
	IF (here->header_size = MORX_OTHER_HDR_SIZE) ENDIF
    ENDIF
#if 1
    IF (!here->loaded++)
#else
    IF (1)
#endif
        DATA_VR_UB(data, here->length - here->header_size)
    ENDIF
    IF (0)
        DATA_UL(header_size)
        DATA_UB(loaded)
    ENDIF
END_STRUCTURE(metamorphosis_table)
STRUCTURE(morx2_chain)
    DATA_UL(defaultFlags)
    DATA_UL(chainLength)
    DATA_UL(nFeatureEntries)
    DATA_UL(nSubtables)
    ARRAY_VC(feature, here->nFeatureEntries, feature_entry)
    ARRAY_VC(metamorphosis, here->nSubtables, metamorphosis_table)
END_STRUCTURE(morx2_chain)
STRUCTURE(morx3_chain)
    DATA_STR(c, morx2_chain)
    DATA_VL_UL(subtableOffsets, c.nSubtables)
    DATA_VR_UL(coverageBitfields, ((N_GLYPHS + 31) / 32) * here->c.nSubtables)
END_STRUCTURE(morx3_chain)
STRUCTURE(morx)
    DATA_UW(version)
    DATA_UW(unused)
    DATA_UL(nChains)
    IF (2 == here->version)
        ARRAY_VC(chains2, here->nChains, morx2_chain)
    ELSEIF (3 == here->version)
        ARRAY_VC(chains3, here->nChains, morx3_chain)
    ENDIF
END_STRUCTURE(morx)
STRUCTURE(BinSrchHeader)
    DATA_UW(unitSize)
    DATA_UW(nUnits)
    DATA_UW(searchRange)
    DATA_UW(entrySelector)
    DATA_UW(rangeShift)
END_STRUCTURE(BinSrchHeader)
STRUCTURE(lookup_segment_2) /* For lookup of 2-byte value */
    DATA_UW(lastGlyph)
    DATA_UW(firstGlyph)
    DATA_UW(value)
END_STRUCTURE(lookup_segment_2)
STRUCTURE(lookup_single_2)
    DATA_UW(glyph)
    DATA_UW(value)
END_STRUCTURE(lookup_single_2)
STRUCTURE(lookup_2) /* Look-up of 2-byte value, indexed by glyph number */
    DATA_UW(format)
    IF (2 == here->format || 4 == here->format || 6 == here->format)
        DATA_STR(binSrchHeader, BinSrchHeader)
    ENDIF
    IF (0 == here->format)
        DATA_VR_UW(datum, N_GLYPHS) /* Beware! Array size is an asumption. */
    ELSEIF (2 == here->format || 4 == here->format) /* Semantics differ */
        ARRAY_VC(segment, here->binSrchHeader.nUnits+1, lookup_segment_2)
    ELSEIF (6 == here->format)
        ARRAY_VC(entry, here->binSrchHeader.nUnits+1, lookup_single_2)
    ELSEIF (8 == here->format)
        DATA_UW(firstGlyph)
        DATA_UW(glyphCount)
        DATA_VL_UW(valueArray, glyphCount)
    ENDIF
END_STRUCTURE(lookup_2)
  /* AAT feature table (feat) */
STRUCTURE(feature_name)
    DATA_UW(feature)
    DATA_UW(nSettings)
    DATA_UL(settingTable)
    DATA_UW(featureFlags)
    DATA_UW(nameIndex)
END_STRUCTURE(feature_name)
STRUCTURE(setting_name)
    DATA_UW(setting)
    DATA_UW(name_index)
END_STRUCTURE(setting_name)
STRUCTURE(feat)
    DATA_UL(version) /* coded for 00010000 */
    DATA_UW(featureNameCount)
    DATA_UW(res1)
    DATA_UL(res2)
    ARRAY_VC(names, here->featureNameCount, feature_name)
    ARRAY_VC(settings, 0, setting_name)
END_STRUCTURE(feat)
