/*
Opcodes for the TrueType hinting language.
Copyright (C) 2005 Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*//*
Opcode      Mnemonic

0x00..0x01  SVTCA
0x02..0x03  SPVTCA
0x04..0x05  SFVTCA
0x06..0x07  SPVTL
0x08..0x09  SFVTL
0x0a        SPVFS
0x0b        SFVFS
0x0c        GPV
0x0d        GFV
0x0e        SFVTPV
0x0f        ISECT
0x10        SRP0
0x11        SRP1
0x12        SRP2
0x13        SZP0
0x14        SZP1
0x15        SZP2
0x16        SZPS
0x17        SLOOP
0x18        RTG
0x19        RTHG
0x1a        SMD
0x1b        ELSE
0x1c        JMPR
0x1d        SCVTCI
0x1e        SSWCI
0x1f        SSW
0x20        DUP
0x21        POP
0x22        CLEAR
0x23        SWAP
0x24        DEPTH
0x25        CINDEX
0x26        MINDEX
0x27        ALIGNPTS
0x29        UTP
0x2a        LOOPCALL
0x2b        CALL
0x2c        FDEF
0x2d        ENDF
0x2e..0x2f  MDAP
0x30..0x31  IUP
0x32..0x33  SHP
0x34..0x35  SHC
0x36..0x37  SHZ
0x38        SHPIX
0x39        IP
0x3a..0x3b  MSIRP
0x3c        ALIGNRP
0x3d        RTDG
0x3e..0x3f  MIAP
0x40        NPUSHB     n b1 .. bn (1 byte wide)
0x41        NPUSHW     n w1 .. wn (2 bytes wide - bigendian)
0x42        WS
0x43        RS
0x44        WCVTP      (pixels)
0x45        RCVT
0x46..0x47  GC
0x48        SCFS
0x49..0x4a  MD
0x4b        MPPEM
0x4c        MPS
0x4d        FLIPON
0x4e        FLIPOFF
0x4f        DEBUG
0x50        LT
0x51        LTEQ
0x52        GT
0x53        GTEQ
0x54        EQ
0x55        NEQ
0x56        ODD
0x57        EVEN
0x58        IF
0x59        EIF
0x5a        AND
0x5b        OR
0x5c        NOT
0x5d        DELTAP1
0x5e        SDB
0x5f        SDS
0x60        ADD
0x61        SUB
0x62        DIV
0x63        MUL
0x64        ABS
0x65        NEG
0x66        FLOOR
0x67        CEILING
0x68..0x6b  ROUND
0x6c..0x6f  NROUND
0x70        WCVTF      (FUnits)
0x71        DELTAP2
0x72        DELTAP3
0x73        DELTAC1
0x74        DELTAC2
0x75        DELTAC3
0x76        SROUND
0x77        SROUND45
0x78        JROT
0x79        JROF
0x7a        ROFF
0x7c        RUTG
0x7d        RDTG
0x7e        SANGW
0x80        FLIPPT
0x81        FLIPRGON
0x82        FLIPRGOFF
0x85        SCANCTRL
0x86..0x87  SDPVTL
0x88        GETINFO
0x89        IDEF
0x8a        ROLL
0x8b        MAX
0x8c        MIN
0x8d        SCANTYPE
0x8e        INSTCTRL (CHECK MNEMONIC)
0xb0..0xb7  PUSHB      abc + 1 bytes
0xb8..0xbf  PUSHW      abc + 1 words
0xc0..0xdf  MDRP
0xe0..0xff  MIRP

*/
const char *mnemonic[144] = {
  "SVTCA[0]",  "SVTCA[1]",  "SPVTCA[0]", "SPVTCA[1]",
  "SFVTCA[0]", "SFVTCA[1]", "SPVTL[0]",  "SPVTL[1]",
  "SFVTL[0]",  "SFVTL[1]",  "SPVFS",     "SFVFS",
  "GPV",       "GFV",       "SFVTPV",    "ISECT",
  "SRP0",      "SRP1",      "SRP2",      "SZP0",
  "SZP1",      "SZP2",      "SZPS",      "SLOOP",
  "RTG",       "RTHG",      "SMD",       "ELSE",
  "JMPR",      "SCVTCI",    "SSWCI",     "SSW",
  "DUP",       "POP",       "CLEAR",     "SWAP",
  "DEPTH",     "CINDEX",    "MINDEX",    "ALIGNPTS",
  "undefined_28", "UTP",       "LOOPCALL",  "CALL",
  "FDEF",      "ENDF",      "MDAP[0]",   "MDAP[1]",
  "IUP[0]",    "IUP[1]",    "SHP[0]",    "SHP[1]",
  "SHC[0]",    "SHC[1]",    "SHZ[0]",    "SHZ[1]",
  "SHPIX",     "IP",        "MSIRP[0]",  "MSIRP[1]",
  "ALIGNRP",   "RTDG",      "MIAP[0]",   "MIAP[1]",
  "NPUSHB",    "NPUSHW",    "WS",        "RS",
  "WCVTP",     "RCVT",      "GC[0]",     "GC[1]",
  "SCFS",      "MD[0]",     "MD[1]",     "MPPEM",
  "MPS",       "FLIPON",    "FLIPOFF",   "DEBUG",
  "LT",        "LTEQ",      "GT",        "GTEQ",
  "EQ",        "NEQ",       "ODD",       "EVEN",
  "IF",        "EIF",       "AND",       "OR",
  "NOT",       "DELTAP1",   "SDB",       "SDS",
  "ADD",       "SUB",       "DIV",       "MUL",
  "ABS",       "NEG",       "FLOOR",     "CEILING",
  "ROUND[0]",  "ROUND[1]",  "ROUND[2]",  "ROUND[3]",
  "NROUND[0]", "NROUND[1]", "NROUND[2]", "NROUND[3]",
  "WCVTF",     "DELTAP2",   "DELTAP3",   "DELTAC1",
  "DELTAC2",   "DELTAC3",   "SROUND",    "SROUND45",
  "JROT",      "JROF",      "ROFF",      "undefined_7b",
  "RUTG",      "RDTG",      "SANGW",     "undefined_7f",
  "FLIPPT",    "FLIPRGON",  "FLIPRGOFF", "undefined_83",
  "undefined_84", "SCANCTRL",  "SDPVTL[0]", "SDPVTL[1]",
  "GETINFO",   "IDEF",      "ROLL",      "MAX",
  "MIN",       "SCANTYPE",  "INSTCTRL",  "undefined_8f"
};
/* 0x9* and 0xa* are unassigned!"
0xb0..0xb7  "PUSHB"      abc + 1 bytes
0xb8..0xbf  "PUSHW"      abc + 1 words
0xc0..0xdf  "MDRP"
0xe0..0xff  "MIRP"
*/
