/*
Interface between OTL and other parts of OpenType compiler
Copyright (C) 2007-2020      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
struct glyph_group {
  union {
    unsigned int number; /* SOLO */
    struct class_data *class_data; /* CLASS */
    struct coverage_data *coverage_data; /* COVERAGE */
  } x;
  enum grouping {SOLO, COVERAGE, CLASS} grouping;
};
extern int last_glyph;

enum lookup_type {
  GDEF_TGT, /* Table identifier for GDEF */
  GSUB_SINGLE, GSUB_MULTIPLE, GSUB_ALTERNATE,
  GSUB_LIGATURE, GSUB_CONTEXT, GSUB_CHAINED,
  GSUB_EXTENSION, GSUB_REVERSE,
  GPOS_SINGLE, GPOS_PAIR, GPOS_CURSIVE,
  GPOS_MARK2BASE, GPOS_MARK2LIGATURE, GPOS_MARK2MARK,
  GPOS_CONTEXT, GPOS_CHAINED, GPOS_EXTENSION};

enum storedness {PENDING, DEFERRED, WRITTEN, EXTENDED};
struct ptr_spec {
  unsigned int offset_loc; /* Position in table to store offset */ 
  unsigned int offset_ref; /* Position to which offset is relative */
  enum lookup_type table;  /* Table in which to store - implicit in type */
  char offset_size;        /* Size of offset in bytes - 2 or 4 */
};
struct locations {
  struct ptr_spec *loc;
  int nloc, mloc; /* Number of entries and capacity of field loc */
};

/* Encapsulate access to look-ups of features */ 
#define core_encapsulate_map(OBJECT)		\
struct OBJECT##_map {/* Treat fields as private! */\
  struct map x;\
};\
static void create_##OBJECT##_map(struct OBJECT##_map *ft) {\
  create_map(&ft->x);\
}

core_encapsulate_map(gclass)

struct classification_data {
  struct gclass_map classes;
  struct locations lox;
  int n_classes;
};
struct class_data {
  int_set members;
  struct classification_data *classification;
  int number;
};

struct coverage_data {
  int_set members;
  struct locations lox;
};

extern FILE *gsub, *gpos, *outfp;
extern int gsub_pos, gpos_pos, *out_pos;
extern int have_GSUB, have_GPOS;

core_encapsulate_map(lookup)
extern struct lookup_map lookups;
core_encapsulate_map(subtable)
extern struct subtable_map subtables;
core_encapsulate_map(gclassification)
extern struct gclassification_map classifications;
extern struct map glyph_names;
extern struct classification_data *attachment_classes;

const char* get_glyph_name(unsigned short int gi);
void process_gsubpos();
void store_classification(const char *name,       /* For error messages */
			  enum lookup_type table, /* Table in which to store */
			  struct classification_data * cldat);
struct classification_data *create_classification(const char *name);
void add_location(enum lookup_type the_type,
		  unsigned int ref_pos, unsigned int store_pos,
		  unsigned int size,
		  struct locations *lox);
void align_file(FILE *dest, int *size, int el_size);

void init_glyph_sets();
int store_glyph_sets();
void free_glyph_sets();
