/*
Routines to dump glyph shapes in PostScript
Copyright (C) 2011      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include <math.h>
#include <string.h>
#include "psplot.h"

#define csize 9
/* Thoughts on achieving line drawing */
FILE *StartPSFile(const char *name);
void EndPSFile(FILE *fp);
void NewPSGraph(FILE *fp, float scale, float x0, float y0);
void EndPSGraph(FILE *fp);
void DrawPSSeg(FILE *fp, float x1, float y1, float x2, float y2,
	     const char *options);
/* Options: A = Arrow, B = Bold, P = dotted, D = dashed */

FILE *StartPSFile(const char* name) {
  FILE *fp;
  if (!(fp = fopen(name, "w"))) return (FILE*)0;
  fprintf(fp, "%%!PS\n");
  fprintf(fp, "%%%% BoundingBox: 0 0 595 841\n");
  fprintf(fp, "(Times-Roman) findfont %d scalefont setfont\n", csize);
  fprintf(fp,
	  "%% Define 'quadratic spline':  xc yc xe ye quadto -\n"
	  "/quadto {\n"
	  "	currentpoint\n"
	  "	exch 5 index 2 mul add 3 div\n"
	  "	exch 4 index 2 mul add 3 div\n"
	  "	6 4 roll\n"
	  "	2 mul 4 index add 3 div\n"
	  "	exch 2 mul 5 index add 3 div exch\n"
	  "	6 4 roll curveto\n"
	  "} def\n"
	  "\n");
  fprintf(fp,
	  "%% Render a TrueType outline.  Input data is array of\n"
	  "%% trios of [x y on/off].\n"
	  "\n"
	  "/on true def\n"
	  "/off false def\n"
	  "/ttcurve {\n"
	  "    0 exch %% stack = 0 array\n"
	  "%% State is coded:\n"
	  "%% 0 => First point awaited\n"
	  "%% 1 => Prev point on curve\n"
	  "%% 2 => Prev point off curve\n"
	  "%% 3 => Prev point was first point, and was off the curve\n"
	  "    {\n"
	  "	exch dup 0 eq\n"
	  "	{ %% First point awaited\n"
	  "	    pop %% stack = point\n"
	  "	    {} forall\n"
	  "	    { %% point on curve\n"
	  "		2 copy moveto\n"
	  "		false 1 %% stack = x1 y1 false 1\n"
	  "	    }\n"
	  "	    { %% point off curve\n"
	  "		true 3 %% stack = x1 y1 true 3\n"
	  "	    } ifelse\n"
	  "	}\n"
	  "	{dup 1 eq\n"
	  "	{ %% Prev point on curve\n"
	  "	    pop %% stack = ... 1st-off point\n"
	  "	    {} forall\n"
	  "	    { %% point on curve\n"
	  "		lineto 1\n"
	  "	    }\n"
	  "	    { %% point off curve\n"
	  "		2\n"
	  "	    } ifelse\n"
	  "	}\n"
	  "	{dup 2 eq\n"
	  "	{ %% Prev point off curve\n"
	  "	    pop %% stack = ... 1st-off x1 y1 point\n"
	  "	    {} forall\n"
	  "	    { %% point on curve\n"
	  "		quadto 1\n"
	  "	    }\n"
	  "	    { %% point off curve, stack = ... 1st-off x1 y1 x2 y2\n"
	  "		2 copy 6 2 roll\n"
	  "		2 index add 2 div "
                           "%% stack = ... 1-o x2 y2 x1 y1 x2 (y2 + y1)/2\n"
	  "		exch\n"
	  "		3 index add 2 div exch "
                           "%% stack = ... 1-o x2 y2 x1 y1 (x1+x2)/2 (y1+y2)/2\n"
	  "		quadto 2\n"
	  "	    } ifelse\n"
	  "	}\n"
	  "	{ %% Prev point was first point, and was off the curve\n"
	  "	    pop %% stack = x1 y1 true point\n"
	  "	    {} forall\n"
	  "	    { %% point on curve\n"
	  "		2 copy moveto\n"
	  "		3 2 roll 1 %% stack = x1 y1 x2 y2 true 1\n"
	  "	    }\n"
	  "	    { %% point off curve, stack = x1 y1 true x2 y2\n"
	  "		3 2 roll pop %% stack = x1 y1 x2 y2\n"
	  "		1 index 4 index add 2 div "
                          "%% stack = x1 y1 x2 y2 (x1+x2)/2\n"
	  "		1 index 4 index add 2 div "
                          "%% stack = x1 y1 x2 y2 (x1+x2)/2 (y1+y2)/2\n"
	  "                2 copy moveto\n"
	  "		4 2 roll\n"
	  "		true 3 1 roll 2 %% stack = x1 y1 xm ym true x2 y2 2\n"
	  "	    } ifelse\n"
	  "	} ifelse } ifelse } ifelse\n"
	  "    } forall\n"
	  "%% Stack now contains one of\n"
	  "%% 0\n"
	  "%% xf yf true 3\n"
	  "%% xs ys false 1\n"
	  "%% xs ys false xg yg 2\n"
	  "%% xf xf xs ys true 1\n"
	  "%% xf yf xs ys true xg yg 2\n"
	  "%% xs, ys is path start, (xf, yf) & (xg, yf) "
              "are off-path points not yet\n"
	  "%% (fully) used in drawing the path\n"
	  "    dup 0 eq\n"
	  "    {%% No points!\n"
	  "	pop\n"
	  "    }		\n"
	  "    {dup 1 eq\n"
	  "    {%% Previous point on curve\n"
	  "	pop\n"
	  "	{%% First point off curve\n"
	  "	    quadto\n"
	  "	    closepath %% Zero length segment\n"
	  "	}\n"
	  "	{%% First point on curve\n"
	  "	    pop pop closepath\n"
	  "	} ifelse\n"
	  "    }\n"
	  "    {2 eq\n"
	  "    {%% Previous point off curve\n"
	  "	3 2 roll\n"
	  "	{%% First point off curve - stack = xf yf xs ys xg yg\n"
	  "	    1 index 6 index add 2 div\n"
	  "	    1 index 6 index add 2 div\n"
	  "	    quadto quadto closepath\n"
	  "	}\n"
	  "	{%% First point on curve - stack = xs ys xg yg\n"
	  "	    4 2 roll quadto closepath\n"
	  "	} ifelse\n"
	  "    }\n"
	  "    {%% One point - so no path\n"
	  "	pop pop pop\n"
	  "    } ifelse } ifelse } ifelse\n"
	  "} def\n");

  fflush(0);
  return fp;
}

void EndPSFile(FILE *fp) {
  fclose(fp);
}

static float g_scale = 10.0;
static float g_x0 =  1000.0;
static float g_y0 = -10.0e3;

/* Input in twips - must convert to points */
#define twip_to_point 0.05
void NewPSGraph(FILE *fp, float scale, float x0, float y0) {
  double local_scale;

  g_scale = scale;
  g_x0 = x0;
  g_y0 = y0;
  fprintf(fp, "gsave\n");
  fprintf(fp, "%% scale=%g x0=%.1f y0=%.1f\n", scale, x0, y0);
  fprintf(fp, "%d %d translate\n", (int) (twip_to_point * x0 + 0.501),
	  (int) (twip_to_point * y0 + 0.501));
  local_scale = twip_to_point * scale;
  fprintf(fp, "%e %e scale\n", local_scale, local_scale);
  fprintf(fp, "1 setlinecap\n");
  fprintf(fp, "0 0 0 setrgbcolor\n");
  fprintf(fp, "4 setlinewidth\n");
  fflush(0);
}

void EndPSGraph(FILE *fp) {
  fprintf(fp, "grestore\n");
}

void EndPSPage(FILE *fp) {
  fprintf(fp, "showpage\n");
}

void PSText(FILE *fp, float x0, float y0, char *str, int cpr) {
  int len, ic, ibeg, iend, ires;
  int ix, iy;

  ix = (int) (twip_to_point * x0 + 0.501);
  iy = (int) (twip_to_point * y0 + 0.501);
  len = strlen(str);
  ires = 0;
  while (ires < len) {
    ibeg = ires;
    if (ibeg + cpr > len) {
      iend = len - 1;
      ires = len;
    } else {
      iend = ibeg + cpr - 1;
      while (' ' != str[iend+1] && iend >= ibeg) iend--;
      if (iend < ibeg) iend = ibeg + cpr - 1; /* Must just chop */
      ires = iend + 1;
      while (str[ires] == ' ') ires++;
    }
    fprintf(fp, "%d %d moveto (", ix, iy);
    for (ic = ibeg; ic <= iend; ic++) {
      if ('(' == str[ic] || ')' == str[ic] || '\\' == str[ic]) putc('\\', fp);
      putc(str[ic], fp);
    }
    fprintf(fp, ") show\n");
    iy = iy - csize;
  }
}

static void stroke(FILE *fp, const char *options);

void DrawPSSeg(FILE *fp, float x1, float y1, float x2, float y2,
	       const char *options) {
  /* Options: A = Arrow, B = Bold,  P = dotted, D = dashed */
  /*          b = blue,  g = green, r = red */
  int gx1, gy1, gx2, gy2;
  int fliph, flipv;

  fprintf(fp, "gsave\nnewpath\n");
  fprintf(fp, "%.1f %.1f moveto\n", x1, y1);
  fprintf(fp, "%.1f %.1f lineto\n", x2, y2);
  stroke(fp, options);
#if 0
  fprintf(stdout, "%f %f %f %f to %d %d %d %d\n",
	  x1, y1, x2, y2, gx1, gx2, gy1, gy2);
#endif
};

static void stroke(FILE *fp, const char *options) {
  for (; *options; options++) {
    switch (*options) {
#if 0
    case 'A':
      fprintf(fp, "{\\sp{\\sn lineEndArrowhead}{\\sv 1}}\n");
      break;
#endif
    case 'B':
      fprintf(fp, "2 setlinewidth\n");
      break;
    case 'D': /* long dashes */
      fprintf(fp, "[46 26] 0 setdash\n");
      break;
    case 'P': /* short dashes */
      fprintf(fp, "[22 26] 0 setdash\n");
      break;
    case 'b':
      fprintf(fp, "0 0 1 setrgbcolor\n");
      break;
    case 'g':
      fprintf(fp, "0 1 0 setrgbcolor\n");
      break;
    case 'r':
      fprintf(fp, "1 0 0 setrgbcolor\n");
      break;
    default:
      fprintf(stderr, "DrawSeg: Option %c not supported.\n", *options);
    }
  }
  fprintf(fp, "stroke\ngrestore\n");
}

void DrawPSQuad(FILE *fp, float x1, float y1, float x2, float y2,
		float x3, float y3, const char *options) {
  /* Options: A = Arrow, B = Bold,  P = dotted, D = dashed */
  /*          b = blue,  g = green, r = red */
  int gx1, gy1, gx2, gy2;
  int fliph, flipv;

  fprintf(fp, "gsave\nnewpath\n");
  fprintf(fp, "%.1f %.1f moveto\n", x1, y1);
  fprintf(fp, "%.1f %.1f %.1f %.1f quadto\n", x2, y2, x3, y3);
  stroke(fp, options);
#if 0
  fprintf(stdout, "%f %f %f %f to %d %d %d %d\n",
	  x1, y1, x2, y2, gx1, gx2, gy1, gy2);
#endif
};

void StartPSContour(FILE *fp) {
  fprintf(fp, "gsave newpath [\n");
}

void PSContourPoint(FILE *fp, int x, int y, int ison) {
  fprintf(fp, "[%d %d %s]\n", x, y, ison?"on":"off");
}

void EndPSContour(FILE *fp, const char *options) {
  fprintf(fp, "] ttcurve\n");
  stroke(fp, options);
}
