/*
Program to move tables between OpenType files
Copyright (C) 2007-2018, 2020 Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#define WHAT \
"Check sum applier.  Copyright (C) 2020 Richard Wordingham\n"

#define WARRANTY \
"This program comes with ABSOLUTELY NO WARRANTY; for details\n" \
"run with option '-w'."

#define COPYING \
"This is free software, and you are welcome\n" \
"to redistribute it under certain conditions; for details \n" \
"run with option '-c'."

#define WELCOME WHAT WARRANTY "  " COPYING "\n"
#define WO WHAT WARRANTY "\n\n"
#define CO WHAT COPYING "\n\n"

#include <stdio.h>
#include <string.h>
int sum_UW(unsigned short int *arr, int n);
void* element(void *base, int index);
int advance_coords(int force_advance);

#include "oft_io_globals.h"

#include "manip_structure.h"
#include "oft.h"
#include "manip_befile.h"
#include "manip_get.h"
#include "oft.h"
#include "manip_put.h"
#include "oft.h"

#define DTYPE struct directory_entry*
#define ipsort entry_sort
#include "ipsort.h"
#undef ipsort
#undef DTYPE
int compare_entry(struct directory_entry *left,
		  struct directory_entry *right) {
  return (int) left->offset - (int) right->offset;
}

static void synopsis(FILE *dst) {
  fputs("Synopsis:\n"
        "\treplace_tables [-s] base_font tables subst_font new_font\n"
        "or\n"
        "\treplace_tables -[chsw]\nOptions:\n"
        "\tc\tGive rules on copying the program.\n"
        "\th\tPrint this synopsis.\n"
        "\ts\tSuppress the identifying message at the start of program\n"
        "\t\texecution.\n"
        "\tw\tDisplay the program's warranty.\n\n", dst);
}

int main(int argc, char *argv[]) {
  int base_arg, table_arg, repl_arg, dst_arg;
  int badargs;
  int iarg;
  FILE *base, *subst, *dst;
  struct offset_table_header base_top2;
  struct offset_table base_top, subst_top, dst_top;
  int junk, pos;
  struct directory_entry *table, **old_table;
#define MAXTAB 255
  struct directory_entry *tables[MAXTAB];
  int itab;
  char *tag, *data;
  char full_tag[5];
  int tag_len, k, two_to_k, total_sum, outpos, data_alloc, at_head;
  struct font_header fontHeader;
  int suppress, tell_c, tell_w, dohelp;


  base_arg  = 0;
  table_arg = 0;
  repl_arg  = 0;
  dst_arg   = 0;

  badargs = 0;
  suppress = tell_c =  tell_w = dohelp = 0;

  for (iarg = 1; iarg < argc; iarg++) {
    if ('-' == *argv[iarg]) {
      const char *optch;
      for (optch = 1 + argv[iarg]; *optch; optch++) {
        switch (*optch) {
        case 'c':
          tell_c = 1;
          break;
        case 'h':
          dohelp = 1;
          break;
        case 's':
          suppress = 1;
          break;
        case 'w':
          tell_w = 1;
          break;
        default:
          fprintf(stderr, "Option \"%c\" not recognised.\n", *optch);
          badargs = 1;
        }
      }
    } else if (dst_arg) {
      badargs = 1;
    } else if (repl_arg) {
      dst_arg = iarg;
    } else if (table_arg) {
      repl_arg = iarg;
    } else if (base_arg) {
      table_arg = iarg;
    } else {
      base_arg = iarg;
    }
  }
  if (dohelp || tell_c || tell_w) {
    if (suppress) {
      /* Totally suppress the welcome message. */
    } else if (tell_c) {
      if (tell_w) {
        fputs(WHAT "\n", stdout);
      } else {
        fputs(WO, stdout);
      }
    } else if (tell_w) {
      fputs(CO, stdout);
    } else {
      fputs(WELCOME, stdout);
    }
    if (tell_w) {
      /* Clauses 15 & 16 of GNU GPL Version 3: */
      fprintf(stdout,
              "THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED\n"
              "BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING\n"
              "THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM\n"
              "“AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR\n"
              "IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES\n"
              "OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.\n"
              "THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE\n"
              "PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE,\n"
              "YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR\n"
              "OR CORRECTION.\n\n"
              
              "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO\n"
              "IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO\n"
              "MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE,\n"
              "BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,\n"
              "INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE\n"
              "OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n"
              "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES\n"
              "SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM\n"
              "TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER\n"
              "OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH\n"
              "DAMAGES.\n"
              );
    }
    if (tell_c) {
      fprintf(stdout,
              "This program is licensed under Version 3 or later of the\n"
              "GNU General Public Licence as published by the\n"
              "Free Software Foundation.\n\n"

              "You may convey verbatim copies of the Program's source code\n"
              "as you receive it, in any medium, provided that you\n"
              "conspicuously and appropriately publish on each copy\n"
              "an appropriate copyright notice; keep intact all notices\n"
              "stating that the License and any non-permissive terms added\n"
              "in accord with section 7 apply to the code; keep intact\n"
              "all notices of the absence of any warranty; and give all\n"
              "recipients a copy of this License along with the Program.\n"
              );
    }
    if (dohelp) synopsis(stdout);
    return 0;
  } else {
    if (!dst_arg) {
      fprintf(stderr, "Insufficent arguments.\n");
      badargs = 1;
    }
    if (badargs) {
      synopsis(stderr);
      return 1;
    }
  }
  if (!suppress) fputs(WELCOME, stdout);
  base = fopen(argv[base_arg], "rb");
  if (!base) {
    perror(argv[base_arg]);
    exit(1);
  }
  subst = fopen(argv[repl_arg], "rb");
  if (!subst) {
    perror(argv[repl_arg]);
    exit(1);
  }
  dst = fopen(argv[dst_arg], "wb");
  if (!dst) {
    perror(argv[dst_arg]);
    exit(1);
  }

  swap_order = 0;
  get_bine_offset_table_header(base, &base_top2, &junk, &pos);
  if ((base_top2.entrySelector & 0xff) == 0 &&
      (base_top2.entrySelector & 0xff00) != 0) {
    /* Wrong way round! */
    swap_order = 1;
    fseek(base, 0, SEEK_SET);
    printf("-- Fixing endianity and re-reading.\n");
  } else if ((base_top2.entrySelector & 0xff) != 0 &&
	     (base_top2.entrySelector & 0xff00) == 0) {
    printf("-- File is in machine's natural order!\n");
  } else {
    printf("Entry selector of %s appears to be corrupt!", argv[base_arg]);
    return 1;
  }
  fseek(base, 0, SEEK_SET);
  get_bine_offset_table(base, &base_top, &junk, &pos);
  if (0x4f54544f == base_top.sfnt_version) { /* OTTO */
    /* No problems */
  } else if (base_top.sfnt_version != 0x00010000) {
    printf("Format is %8.8X, not 1.0\n", base_top.sfnt_version);
    return 1;
  }
/* Reread as simplest way of initialising the destination table. */
  fseek(base, 0, SEEK_SET);
  get_bine_offset_table(base, &dst_top, &junk, &pos);
  /* Encode table source in offset field:
         +ve => base, -ve => subst
  */
  for (table = dst_top.table_directory, itab = 0; table;
       table = table->_next, itab++) {
    if (table->_next && strncmp(table->tag, table->_next->tag, 4) > 0) {
      fprintf(stderr, "Base tables out of order in directory - "
	      "%.4s precedes %.4s.\n", table->tag, table->_next->tag);
      exit(1);
    }
    tables[itab] = table;
  }
/* Sort the tables by offset */
  entry_sort(compare_entry, tables, dst_top.numTables);
#if 0
  fprintf(stdout, "Base font's tables are stored in the order:\n");
  for (itab = 0; itab < dst_top.numTables; itab++) {
    fprintf(stdout, "%.4s %8.8X\n", tables[itab]->tag, tables[itab]->offset);
  }
#endif
/* Investigate the table of potential replacements */
  get_bine_offset_table(subst, &subst_top, &junk, &pos);
  for (table = subst_top.table_directory; table; table = table->_next) {
    if (table->_next && strncmp(table->tag, table->_next->tag, 4) > 0) {
      fprintf(stderr, "Substitute tables out of order in directory - "
	      "%4s precedes %4s.\n", table->tag, table->_next->tag);
      exit(1);
    }
  }
/* Update table with substitutions and additions. */
  for (tag = argv[table_arg]; *tag; tag += tag_len) {
    tag_len = strcspn(tag, ",");
    if (tag_len <= 0) {
      tag_len = 1;
    } else if (tag_len > 4) {
      fprintf(stderr, "\"%.*s\" is an invalid tag.\n", tag_len, tag);
    } else {
      memset(full_tag, ' ', 4);
      full_tag[4] = 0;
      strncpy(full_tag, tag, tag_len);
      for (table = subst_top.table_directory; table; table = table->_next) {
	if (strncmp(full_tag, table->tag, 4) <= 0) break;
      }
      if (table && !strncmp(full_tag, table->tag, 4)) {
	for (old_table = &dst_top.table_directory; *old_table;
	     old_table = &(*old_table)->_next) {
	  if (strncmp(full_tag, (*old_table)->tag, 4) <= 0) break;
	}
	if (!*old_table || strncmp(full_tag, (*old_table)->tag, 4)) {
	  struct directory_entry *nextun;

	  nextun = *old_table;
	  if (!(*old_table = malloc(sizeof(**old_table)))) {
	    fprintf(stderr, "Memory exhausted.\n");
	    return 1;
	  }
	  tables[dst_top.numTables++] = *old_table;
	  for (k = 15; k >= 0; k--) {
	    two_to_k = 1 << k;
	    if (two_to_k <= dst_top.numTables) {
	      dst_top.searchRange = 16 * two_to_k;
	      dst_top.entrySelector = k;
	      dst_top.rangeShift = 16 * (dst_top.numTables - two_to_k);
	      break;
	    }
	  }
	  (*old_table)->_next = nextun;
	  strncpy((*old_table)->tag, full_tag, 4);
	}
	(*old_table)->checkSum = table->checkSum;
	(*old_table)->offset = - table->offset;
	(*old_table)->length   = table->length;
      } else {
	fprintf(stderr, "%s does not contain table with tag \"%s\".\n",
		argv[repl_arg], full_tag);
      }
    }
  }
#define FIX_CHECKSUM 0
/* Update the globalchecksum. */
  for (k = 0, total_sum = 0; k < dst_top.numTables; k++) {
    total_sum += tables[k]->checkSum;
  }
  /* Logic INCOMPLETE!  The index tables should also be included. */
/* Generate the output file */
  outpos = 0; data = 0;
  put_bine_offset_table(dst, &dst_top, &junk, &outpos);
  for (k = 0; k < dst_top.numTables; k++) {
    at_head = !strncmp("head", tables[k]->tag, 4);
    data_alloc = (tables[k]->length + 3) / 4 * 4;
    if (!(data = realloc(data, data_alloc))) {
      fprintf(stderr, "Memory exhausted.\n");
      return 1;
    }
    if (data_alloc) memset(data + data_alloc - 4, 0, 4);
    if ((int) tables[k]->offset >= 0) {
      if (fseek(base, tables[k]->offset, SEEK_SET)) {
	perror(argv[base_arg]);
	return 1;
      }
      if (at_head && FIX_CHECKSUM) {
	get_bine_font_header(base, &fontHeader, &junk, &pos);
      } else {
	fread(data, 1, tables[k]->length, base);
      }
    } else {
      if (fseek(subst, -tables[k]->offset, SEEK_SET)) {
	perror(argv[repl_arg]);
	return 1;
      }
      if (at_head && FIX_CHECKSUM) {
	get_bine_font_header(subst, &fontHeader, &junk, &pos);
      } else {
	fread(data, 1, tables[k]->length, subst);
      }
    }
    tables[k]->offset = outpos;
    if (fseek(dst, outpos, SEEK_SET)) {
      perror(argv[dst_arg]);
      return 1;
    }
    if (at_head && FIX_CHECKSUM) {
/* Update the font checksum */
      fontHeader.checkSumAdjustment = 0xb1b0afba - total_sum;
      put_bine_font_header(dst, &fontHeader, &junk, &outpos);
    } else {
      fwrite(data, 1, data_alloc, dst);
      outpos += data_alloc;
    }
  }
  fclose(base); fclose(subst);
  if (fseek(dst, outpos = 0, SEEK_SET)) {
    perror(argv[dst_arg]);
    return 1;
  }
  put_bine_offset_table(dst, &dst_top, &junk, &outpos);
  fclose(dst);
  if (!FIX_CHECKSUM) {
    fprintf(stderr, "The checksum probably needs to be corrected.\n");
  }
  return 0;
}
