/*
Routines for storing and accessing Unicode properties.
Utility to examine and decompile OpenType fonts.
Copyright (C) 2005 Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "unidata.h"

/* Set up some database access */
/* Organise Unicode space as 17 by 512 by 128. */

int get_property(space db, int onch, datatype *value) {
  int plane_no, hb_no, pt_no;
  plane_no = onch >> 16;
  hb_no = (onch >> 7) & 511;
  pt_no = onch & 127;
  if (db.plane_ptr[plane_no]) {
    if (db.plane_ptr[plane_no]->items[hb_no]) {
      *value = (*db.plane_ptr[plane_no]->items[hb_no])[pt_no];
      return FOUND;
    } else if (db.plane_ptr[plane_no]->hblock_defaults[hb_no>>5] &
	       (1 << (hb_no&31))) {
      *value = db.plane_ptr[plane_no]->hblock_default[hb_no];
      return BLOCK_DEFAULT;
    } else if (db.plane_defaults[plane_no]) {
      *value = db.plane_default[plane_no];
      return PLANE_DEFAULT;
    } else if (db.space_defaults) {
      *value = db.space_default;
      return SPACE_DEFAULT;
    } else {
      return NOT_FOUND;
    }
  } else if (db.plane_defaults[plane_no]) {
    *value = db.plane_default[plane_no];
    return PLANE_DEFAULT;
  } else if (db.space_defaults) {
    *value = db.space_default;
    return SPACE_DEFAULT;
  } else {
    return NOT_FOUND;
  }
}

void set_property(space *db, int onch, datatype value, datatype def_value) {
  int plane_no, hb_no, pt_no, k, result;
  datatype local_def_value;
  plane_no = onch >> 16;
  hb_no = (onch >> 7) & 511;
  pt_no = onch & 127;
  if (!db->plane_ptr[plane_no]) {
    if (db->plane_ptr[plane_no] = malloc(sizeof(plane))) {
      memset(db->plane_ptr[plane_no], 0, sizeof(plane));
    } else {
      fprintf(stderr, "set_property: Memory exhausted.\n");
      exit(1);
    }
  }
  if (!db->plane_ptr[plane_no]->items[hb_no]) {
    result = get_property(*db, onch, &local_def_value);
    if (NOT_FOUND == result) local_def_value = def_value;
    if (db->plane_ptr[plane_no]->items[hb_no]= malloc(128*sizeof(datatype))){
      for (k = 0; k < 128; k++) {
	(*db->plane_ptr[plane_no]->items[hb_no])[k] = local_def_value;
      }
    } else {
      fprintf(stderr, "set_property: Memnory exhusted.\n");
    }
  }
  (*db->plane_ptr[plane_no]->items[hb_no])[pt_no] = value;
}

void set_property_default(space *db, datatype value) {
  db->space_defaults = 1;
  db->space_default = value;
}
