/*
Routines for writing glyph shapes into Word file.
Copyright (C) 2005      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/
#include "wordplot.h"

static int g_id;
FILE *StartWordFile(const char* name) {
  FILE *fp;
  fp = fopen(name, "w");
  if (!fp) return 0;
  fprintf(fp, "{\\rtf1\\ansi\\deflang2057\n");
  fprintf(fp, "{\\fonttbl{\\f0\\fswiss Arial}{\\f1\\fmodern Courier New}}\n");
  fprintf(fp, "{\\stylesheet\n");
# if 0
  fprintf(fp, "{\\colortbl;\n");
  fprintf(fp, "\\red0\\green0\\blue0;\\red0\\green0\\blue255;\n");
  fprintf(fp, "\\red0\\green255\\blue255;\n");
  fprintf(fp, "\\red0\\green255\\blue0;\\red255\\green0\\blue255;\n");
  fprintf(fp, "\\red255\\green0\\blue0;\\red255\\green255\\blue0;\n");
  fprintf(fp, "\\red255\\green255\\blue255;\\red0\\green0\\blue128;\n");
  fprintf(fp, "\\red0\\green128\\blue128;\\red0\\green128\\blue0;\n");
  fprintf(fp, "\\red128\\green0\\blue128;\\red128\\green0\\blue0;\n");
  fprintf(fp, "\\red128\\green128\\blue0;\\red128\\green128\\blue128;\n");
  fprintf(fp, "\\red192\\green192\\blue192;}\n");
#endif
  fprintf(fp, "{\\s0\\f0\\qj\\fs24\\sa120 Normal;}\n");
  fprintf(fp, "{\\s5\\f0\\gj\\fi-1077\\li1077\\b1\\keep\\keepn\\tx1080\n");
  fprintf(fp, "\\fs24\\sa120\\sb120 Heading 2;}\n");
  fprintf(fp, "\\paperw11907\\paperh16840\n");
  fprintf(fp, "\\pard}\n");
  g_id = 1;
  return fp;
}

void EndWordFile(FILE *fp) {
  fprintf(fp, "}");
  fclose(fp);
}

static float g_scale = 10.0;
static float g_x0 =  1000.0;
static float g_y0 = -10.0e3;

void NewGraph(FILE *fp, float scale, float x0, float y0) {
  fprintf(fp, "\\par\n");
  g_scale = scale;
  g_x0 = x0;
  g_y0 = y0;
}

void EndGraph(FILE *fp) {
  fprintf(fp, "\\par\\page\n");
}

void DrawSeg(FILE *fp, float x1, float y1, float x2, float y2, const char *options) {
  /* Options: A = Arrow, B = Bold,  P = dotted, D = dashed */
  /*          b = blue,  g = green, r = red */
  int gx1, gy1, gx2, gy2;
  int fliph, flipv;
    
  gx1 = g_x0 + g_scale * x1;
  gx2 = g_x0 + g_scale * x2;
  gy1 = -(g_y0 + g_scale * y1);
  gy2 = -(g_y0 + g_scale * y2);
#if 0
  fprintf(stdout, "%f %f %f %f to %d %d %d %d\n",
	  x1, y1, x2, y2, gx1, gx2, gy1, gy2);
#endif
/*
  Setting nosort to 0 works on Windows XP (2002?), but not OpenOffice.org 3.2
  or LibreOffice 3.3.

  However, setting nosort to 1 works with both Windows 2010 and LibeOffice 3.2.
*/
#define nosort 1
  fprintf(fp, "{\\shp{\\*\\shpinst");
  if (nosort || gx1 <= gx2) {
    fprintf(fp, "\\shpleft%1d\\shpright%1d", gx1, gx2);
    fliph = 0;
  } else {
    fprintf(fp, "\\shpleft%1d\\shpright%1d", gx2, gx1);
    fliph = 1;
  }
  if (nosort || gy1 <= gy2) {
    fprintf(fp, "\\shptop%1d\\shpbottom%1d\n", gy1, gy2);
    flipv = 0;
  } else {
    fprintf(fp, "\\shptop%1d\\shpbottom%1d\n", gy2, gy1);
    flipv = 1;
  }
  fprintf(fp, "\\shpfhdr0\\shpbxcolumn\\shpbypara\\shpz0\\shplid%d\\shpwr1", g_id++);
  fprintf(fp, "{\\sp{\\sn shapeType}{\\sv 20}}\n");
  fprintf(fp, "{\\sp{\\sn fFlipH}{\\sv %d}}\n", fliph);
  fprintf(fp, "{\\sp{\\sn fFlipV}{\\sv %d}}\n", flipv);
  for (; *options; options++) {
    switch (*options) {
    case 'A':
      fprintf(fp, "{\\sp{\\sn lineEndArrowhead}{\\sv 1}}\n");
      break;
    case 'B':
      fprintf(fp, "{\\sp{\\sn lineWidth}{\\sv 36000}}\n");
      break;
    case 'D':
      fprintf(fp, "{\\sp{\\sn lineDashing}{\\sv 7}}\n");
      break;
    case 'P':
      fprintf(fp, "{\\sp{\\sn lineDashing}{\\sv 6}}\n");
      break;
    case 'b':
      fprintf(fp, "{\\sp{\\sn lineColor}{\\sv %d}}\n", 0xff0000);
      break;
    case 'g':
      fprintf(fp, "{\\sp{\\sn lineColor}{\\sv %d}}\n", 0x00ff00);
      break;
    case 'r':
      fprintf(fp, "{\\sp{\\sn lineColor}{\\sv %d}}\n", 0x0000ff);
      break;
    default:
      fprintf(stderr, "DrawSeg: Option %c not supported.\n", *options);
    }
  }
  fprintf(fp, "}}\n");
}
