/*
OpenType font compiler
Copyright (C) 2005-2020      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#define WHAT \
"OpenType Font Table generator.  "      \
      "Copyright (C) 2020 Richard Wordingham\n"

#define WARRANTY \
"This program comes with ABSOLUTELY NO WARRANTY; for details\n" \
"run with option '-w'."

#define COPYING \
"This is free software, and you are welcome\n" \
"to redistribute it under certain conditions; for details \n" \
"run with option '-c'."

#define WELCOME WHAT WARRANTY "  " COPYING "\n"
#define WO WHAT WARRANTY "\n\n"
#define CO WHAT COPYING "\n\n"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "bitcodes.h"
#include "compile_instr.h"
#include "dates1904.h"
#include "int_set.h"
#include "properties.h"
#include "tokeniser.h"
#include "map.h"
#include "unidata.h"
#include "parse.h"
void* element(void *base, int index);
int advance_coords(int force_advance);
void load_properties();
/* Next inclusion should precede most inclusions of oft.h */
#include "oft_io_globals.h"
/* Do not disturb the sequence of the next two inclusions. */
#include "manip_structure.h"
#include "oft.h"
/* Do not disturb the sequence of the next five inclusions. */
#include "manip_befile.h"
#include "manip_put.h"
#include "oft.h"
#include "manip_get.h"
#include "oft.h"
#if 0
/* Do not disturb the sequence of the next three inclusions. */
#include "manip_print.h"
#include "manip_display.h"
#include "oft.h"
#endif
/* Do not disturb the sequence of the next two inclusions. */
#include "manip_free.h"
#include "oft.h"

/* Desperate debugging aids */
#define DOFREE 1

/* Declarations from process_gsubpos.c */
FILE *gsub, *gpos, *outfp;
int gsub_pos, gpos_pos, *out_pos;
int have_GSUB, have_GPOS;
struct lookup_map lookups;
struct subtable_map subtables;
struct gclassification_map classifications;
void locate_class(const char *name, struct classification_data *cd,
		  struct class_data **cldat);
void process_subtable();
void process_classification();
void process_coverage();
void store_something();
void store_all_lookups();
void check_all_subtables();
void store_all_used_coverages();
void store_all_used_classifications();

/* From other files */
void make_cmap_subtable4(
      int n, int cpt[], unsigned short int gid[], struct encoding *subtable);
void make_cmap_subtable12(
      int n, int cpt[], unsigned short int gid[], struct encoding *subtable);


/* Internal declarations */
static void do_parsing(void);
static void build_font(FILE*);
static char *endianity(void);
static void resolve();
int bad_syntax;
int have_COLR, have_CPAL;

static void synopsis(FILE *dst) {
  fputs("The command\n"
        "\tparse [-s] SOURCE FONT\n"
        "generates font file FONT from the definition in SOURCE.  It may also\n"
        "be invoked, without parsing or generating anything, as\n"
        "\tparse -[schw]\n"
        "The options are:\n"
        "\tc\tGive rules on copying the program.\n"
        "\th\tPrint this synopsis.\n"
        "\ts\tSuppress the identifying message at the start of program\n"
        "\t\texecution.\n"
        "\tw\tDisplay the program's warranty.\n\n", dst);
}

int main(int argc, char *argv[]) {
  FILE *font;
  char *host_endianity;
  const char *src = 0, *dst = 0, *optch;
  int iarg, gonbad;
  int suppress, about_c, about_w, dohelp;

/* Fonts are big-endian, regardless of host! */
  host_endianity = endianity();
  if (!strcmp("big", host_endianity)) {
    swap_order = 0;
  } else if (!strcmp("little", host_endianity)) {
    swap_order = 1;
  } else {
    fprintf(stderr, "Host has %s endianity.  Font file may be corrupt.\n",
	    host_endianity);
    swap_order = 0;
  }
  suppress = about_c = about_w = dohelp = gonbad = 0;
  for (iarg = 1; iarg < argc; iarg++) {
    if ('-' == *argv[iarg]) {
      for (optch = 1 + argv[iarg]; *optch; optch++) {
        switch (*optch) {
        case 'c':
          about_c = 1;
          break;
        case 'h':
          dohelp = 1;
          break;
        case 's':
          suppress = 1;
          break;
        case 'w':
          about_w = 1;
          break;
        default:
          fprintf(stderr, "Option \"%c\" not recognised.\n", *optch);
          gonbad = 1;
        }
      } 
    } else if (!src) {
      src = argv[iarg];
    } else if (!dst) {
      dst = argv[iarg];
    } else {
      fprintf(stderr, "Too many arguments.\n");
      gonbad = 1;
    }
  }
  if (about_c || about_w || dohelp) {
    if (suppress) {
      /* Totally suppress the welcome message. */
    } else if (about_c) {
      if (about_w) {
        fputs(WHAT "\n", stdout);
      } else {
        fputs(WO, stdout);
      }
    } else if (about_w) {
      fputs(CO, stdout);
    } else {
      fputs(WELCOME, stdout);
    }
    if (about_w) {
          /* Clauses 15 & 16 of GNU GPL Version 3: */
      fprintf(stdout,
              "THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED\n"
              "BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING\n"
              "THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM\n"
              "“AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR\n"
              "IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES\n"
              "OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.\n"
              "THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE\n"
              "PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE,\n"
              "YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR\n"
              "OR CORRECTION.\n\n"
              
              "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO\n"
              "IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO\n"
              "MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE,\n"
              "BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,\n"
              "INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE\n"
              "OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n"
              "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES\n"
              "SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM\n"
              "TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER\n"
              "OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH\n"
              "DAMAGES.\n"
                  );
    }
    if (about_c) {
      fprintf(stdout,
              "This program is licensed under Version 3 or later of the\n"
              "GNU General Public Licence as published by the\n"
              "Free Software Foundation.\n\n"

              "You may convey verbatim copies of the Program's source code\n"
              "as you receive it, in any medium, provided that you\n"
              "conspicuously and appropriately publish on each copy\n"
              "an appropriate copyright notice; keep intact all notices\n"
              "stating that the License and any non-permissive terms added\n"
              "in accord with section 7 apply to the code; keep intact\n"
              "all notices of the absence of any warranty; and give all\n"
              "recipients a copy of this License along with the Program.\n"
              );
    }
    if (dohelp) synopsis(gonbad ? stderr : stdout);
  } else {
    if (!dst) {
      fprintf(stderr, "Insufficient arguments.\n");
      gonbad = 1;
    }
    if (gonbad) {
      synopsis(stderr);
      return 1;
    }
    if (!suppress) fputs(WELCOME, stdout);
    fprintf(stdout, "Analysing %s.\n", src);
    open_source(src);
    load_properties();
    bad_syntax = 0;
    init_glyph_sets();
    do_parsing();
    if (!bad_syntax) resolve();
    if (!bad_syntax) {
      font = fopen(dst, "wb");
      if (font) {
        build_font(font);
      } else {
        perror("Creating font");
      }
    }
  }
  return 0;
}

/* Globals for analysis */
struct unresolved_component {
  char *glyph_name;
  unsigned int for_glyph; /* 0x10000 is added for colour component */
  unsigned int component_index; /* zero based; base record number for
                                   coloured component. */
};

struct glyph_etc {
  unsigned int location;
  unsigned short int advanceWidth;
  unsigned short int leftSideBearing;
  unsigned char category; /* Mark, base etc. */
#define CAT_DEFAULT   0  /* Explicitly record as such */
#define CAT_BASE      1
#define CAT_LIGATURE  2
#define CAT_MARK      3
#define CAT_COMPONENT 4
#define NO_CATEGORY 0xFF /* Do not record in GDEF */
  unsigned char attachment_class; /* As in GDEF */
#define NO_MAC 0xFF /* No mark attachment class to be recorded */
};
int cap_glyphs = 0;
struct glyph_etc *glyph_spp = 0;
struct map category_names; /* Names of glyph categories.  -ve for fakes. */
#define CAT_DEFAULTMARK (-1)
#define CAT_PSEUDOMARK (-2) /* Dummy for loading of mark attachment classes */

int last_glyph;
struct map glyph_names, name_names;
struct map nameID_codes, platID_codes, unicode_encodings, ms_encodings;
struct map mac_languages, ms_languages, mac_encodings, month_names;
struct map colours;
int p0c0; /* Colour entry doubling as generic first colour in palette */
struct color_record *colval;
int mcolval;
unsigned short int *pe_label;
int npe_label, mpe_label;
unsigned int *pal_type;
unsigned short int *pal_label;
int mpal; /* Count is palette.numPalette */
struct cpal_header palette;
struct colr_header colr;
space charmap;
struct {int vs; space *map;} vscharmap[] = {
  { 0x180B, NULL}, { 0x180C, NULL}, { 0x180D, NULL},
  { 0xFE00, NULL}, { 0xFE01, NULL}, { 0xFE02, NULL}, { 0xFE03, NULL},
  { 0xFE04, NULL}, { 0xFE05, NULL}, { 0xFE06, NULL}, { 0xFE07, NULL},
  { 0xFE08, NULL}, { 0xFE09, NULL}, { 0xFE0A, NULL}, { 0xFE0B, NULL},
  { 0xFE0C, NULL}, { 0xFE0D, NULL}, { 0xFE0E, NULL}, { 0xFE0F, NULL},
  {0xE0100, NULL}, {0xE0101, NULL}, {0xE0102, NULL}, {0xE0103, NULL},
  {0xE0104, NULL}, {0xE0105, NULL}, {0xE0106, NULL}, {0xE0107, NULL},
  {0xE0108, NULL}, {0xE0109, NULL}, {0xE010A, NULL}, {0xE010B, NULL},
  {0xE010C, NULL}, {0xE010D, NULL}, {0xE010E, NULL}, {0xE010F, NULL},
  {0xE0110, NULL}, {0xE0111, NULL}, {0xE0112, NULL}, {0xE0113, NULL},
  {0xE0114, NULL}, {0xE0115, NULL}, {0xE0116, NULL}, {0xE0117, NULL},
  {0xE0118, NULL}, {0xE0119, NULL}, {0xE011A, NULL}, {0xE011B, NULL},
  {0xE011C, NULL}, {0xE011D, NULL}, {0xE011E, NULL}, {0xE011F, NULL},
  {0xE0120, NULL}, {0xE0121, NULL}, {0xE0122, NULL}, {0xE0123, NULL},
  {0xE0124, NULL}, {0xE0125, NULL}, {0xE0126, NULL}, {0xE0127, NULL},
  {0xE0128, NULL}, {0xE0129, NULL}, {0xE012A, NULL}, {0xE012B, NULL},
  {0xE012C, NULL}, {0xE012D, NULL}, {0xE012E, NULL}, {0xE012F, NULL},
  {0xE0130, NULL}, {0xE0131, NULL}, {0xE0132, NULL}, {0xE0133, NULL},
  {0xE0134, NULL}, {0xE0135, NULL}, {0xE0136, NULL}, {0xE0137, NULL},
  {0xE0138, NULL}, {0xE0139, NULL}, {0xE013A, NULL}, {0xE013B, NULL},
  {0xE013C, NULL}, {0xE013D, NULL}, {0xE013E, NULL}, {0xE013F, NULL},
  {0xE0140, NULL}, {0xE0141, NULL}, {0xE0142, NULL}, {0xE0143, NULL},
  {0xE0144, NULL}, {0xE0145, NULL}, {0xE0146, NULL}, {0xE0147, NULL},
  {0xE0148, NULL}, {0xE0149, NULL}, {0xE014A, NULL}, {0xE014B, NULL},
  {0xE014C, NULL}, {0xE014D, NULL}, {0xE014E, NULL}, {0xE014F, NULL},
  {0xE0150, NULL}, {0xE0151, NULL}, {0xE0152, NULL}, {0xE0153, NULL},
  {0xE0154, NULL}, {0xE0155, NULL}, {0xE0156, NULL}, {0xE0157, NULL},
  {0xE0158, NULL}, {0xE0159, NULL}, {0xE015A, NULL}, {0xE015B, NULL},
  {0xE015C, NULL}, {0xE015D, NULL}, {0xE015E, NULL}, {0xE015F, NULL},
  {0xE0160, NULL}, {0xE0161, NULL}, {0xE0162, NULL}, {0xE0163, NULL},
  {0xE0164, NULL}, {0xE0165, NULL}, {0xE0166, NULL}, {0xE0167, NULL},
  {0xE0168, NULL}, {0xE0169, NULL}, {0xE016A, NULL}, {0xE016B, NULL},
  {0xE016C, NULL}, {0xE016D, NULL}, {0xE016E, NULL}, {0xE016F, NULL},
  {0xE0170, NULL}, {0xE0171, NULL}, {0xE0172, NULL}, {0xE0173, NULL},
  {0xE0174, NULL}, {0xE0175, NULL}, {0xE0176, NULL}, {0xE0177, NULL},
  {0xE0178, NULL}, {0xE0179, NULL}, {0xE017A, NULL}, {0xE017B, NULL},
  {0xE017C, NULL}, {0xE017D, NULL}, {0xE017E, NULL}, {0xE017F, NULL},
  {0xE0180, NULL}, {0xE0181, NULL}, {0xE0182, NULL}, {0xE0183, NULL},
  {0xE0184, NULL}, {0xE0185, NULL}, {0xE0186, NULL}, {0xE0187, NULL},
  {0xE0188, NULL}, {0xE0189, NULL}, {0xE018A, NULL}, {0xE018B, NULL},
  {0xE018C, NULL}, {0xE018D, NULL}, {0xE018E, NULL}, {0xE018F, NULL},
  {0xE0190, NULL}, {0xE0191, NULL}, {0xE0192, NULL}, {0xE0193, NULL},
  {0xE0194, NULL}, {0xE0195, NULL}, {0xE0196, NULL}, {0xE0197, NULL},
  {0xE0198, NULL}, {0xE0199, NULL}, {0xE019A, NULL}, {0xE019B, NULL},
  {0xE019C, NULL}, {0xE019D, NULL}, {0xE019E, NULL}, {0xE019F, NULL},
  {0xE01A0, NULL}, {0xE01A1, NULL}, {0xE01A2, NULL}, {0xE01A3, NULL},
  {0xE01A4, NULL}, {0xE01A5, NULL}, {0xE01A6, NULL}, {0xE01A7, NULL},
  {0xE01A8, NULL}, {0xE01A9, NULL}, {0xE01AA, NULL}, {0xE01AB, NULL},
  {0xE01AC, NULL}, {0xE01AD, NULL}, {0xE01AE, NULL}, {0xE01AF, NULL},
  {0xE01B0, NULL}, {0xE01B1, NULL}, {0xE01B2, NULL}, {0xE01B3, NULL},
  {0xE01B4, NULL}, {0xE01B5, NULL}, {0xE01B6, NULL}, {0xE01B7, NULL},
  {0xE01B8, NULL}, {0xE01B9, NULL}, {0xE01BA, NULL}, {0xE01BB, NULL},
  {0xE01BC, NULL}, {0xE01BD, NULL}, {0xE01BE, NULL}, {0xE01BF, NULL},
  {0xE01C0, NULL}, {0xE01C1, NULL}, {0xE01C2, NULL}, {0xE01C3, NULL},
  {0xE01C4, NULL}, {0xE01C5, NULL}, {0xE01C6, NULL}, {0xE01C7, NULL},
  {0xE01C8, NULL}, {0xE01C9, NULL}, {0xE01CA, NULL}, {0xE01CB, NULL},
  {0xE01CC, NULL}, {0xE01CD, NULL}, {0xE01CE, NULL}, {0xE01CF, NULL},
  {0xE01D0, NULL}, {0xE01D1, NULL}, {0xE01D2, NULL}, {0xE01D3, NULL},
  {0xE01D4, NULL}, {0xE01D5, NULL}, {0xE01D6, NULL}, {0xE01D7, NULL},
  {0xE01D8, NULL}, {0xE01D9, NULL}, {0xE01DA, NULL}, {0xE01DB, NULL},
  {0xE01DC, NULL}, {0xE01DD, NULL}, {0xE01DE, NULL}, {0xE01DF, NULL},
  {0xE01E0, NULL}, {0xE01E1, NULL}, {0xE01E2, NULL}, {0xE01E3, NULL},
  {0xE01E4, NULL}, {0xE01E5, NULL}, {0xE01E6, NULL}, {0xE01E7, NULL},
  {0xE01E8, NULL}, {0xE01E9, NULL}, {0xE01EA, NULL}, {0xE01EB, NULL},
  {0xE01EC, NULL}, {0xE01ED, NULL}, {0xE01EE, NULL}, {0xE01EF, NULL}};
char **postscript_name;
int max_postscript_name;
struct map opcode_def;
struct map program_def;
FILE *glyphs; /* Scratch file for glyphs */
int glyphs_size; /* Size of the file */
struct unresolved_component *unresolved_components = 0;
int cap_unresolved_components = 0;
int n_unresolved_components = 0;
unsigned short int default_aw = 0;
int_set untracked;

struct calculable_item {
  int forced_value;
  int calculated_value;
  short int causerID;
  char is_forced;
  enum {NO_CAUSER, BY_GLYPH, BY_POINT, ALL_GLYPHS} causer_type;
};

/* Calculable items for the hhea table */
struct calculable_item advanceWidthMax, xMaxExtent;
struct calculable_item minLeftSideBearing, minRightSideBearing;
/* Non-calculable items fot the hhea table. */
short int ascender, descender, lineGap;
short int caretSlopeRise, caretSlopeRun, caretOffset;
/* Calculable items for the head table */
struct calculable_item head_xMin, head_yMin, head_xMax, head_yMax;
struct calculable_item fontDirectionHint;       /* forceable, not calculable */
struct calculable_item fDH_l2r, fDH_r2l, fDH_0; /* calculable, not forceable */
/* fDH_r2l also affects flags!  Let it be forceable for ** flags ** bit 9 */
/* Calculable items for bits 1, 2, 4 of flags in head table. */
struct calculable_item lsbx0, ins_use_PPEM, aw_by_PPEM;
/* Non-calculable items for the head table. */
int fontRevision;
unsigned short int unitsPerEm, lowestRecPPEM, head_flags, macStyle;
unsigned long long int created, modified;
short int indexToLocFormat;
int indexToLocFormat_forced;
/* Calculable items for the OS/2 table. */
struct calculable_item xAvgCharWidth;
int totCharWidth, nCharWidth;
struct calculable_item ulUnicodeRange[4*32];
struct calculable_item usFirstCharIndex;
struct calculable_item usLastCharIndex;
struct calculable_item usWinAscent;
struct calculable_item usWinDescent;
struct calculable_item sxHeight;
struct calculable_item sCapHeight;
struct calculable_item usMaxContext;
/* Non-calculable items for the OS/2 table. */
unsigned short int usWeightClass;
unsigned short int usWidthClass;
unsigned short int fsType;
short int ySubscriptXSize;
short int ySubscriptYSize;
short int ySubscriptXOffset;
short int ySubscriptYOffset;
short int ySuperscriptXSize;
short int ySuperscriptYSize;
short int ySuperscriptXOffset;
short int ySuperscriptYOffset;
short int yStrikeoutSize;
short int yStrikeoutPosition;
unsigned short int sFamilyClass; /* Signed makes no sense */
unsigned char panose[10];
char achVendID[5];
unsigned short int fsSelection;
short int sTypoAscender ;
short int sTypoDescender;
short int sTypoLineGap  ;
unsigned int ulCodePageRange[2];
unsigned short int usDefaultChar; /* Default to 0 */
unsigned short int usBreakChar; /* Default to U+0020 SPACE */
/* Calculable items for the post table */
struct calculable_item isFixedPitch;
/* Non-calculable items for the post table */
double italicAngle;
short int underlinePosition;
short int underlineThickness;
unsigned int minMemType42;
unsigned int maxMemType42;
unsigned int minMemType1;
unsigned int maxMemType1;
/* Font and CVT table programs */
int have_fpgm, have_prep, have_GDEF;
int n_fpgm_instr, n_prep_instr;
unsigned char *fpgm_instr, *prep_instr;
int have_cvt, n_cvt, m_cvt;
short int *p_cvt = 0;
/* Calculable items for maxp */
struct calculable_item maxPoints;
struct calculable_item maxContours;
struct calculable_item maxCompositePoints;
struct calculable_item maxCompositeContours;
struct calculable_item maxZones;
struct calculable_item maxTwilightPoints;
struct calculable_item maxStorage;
struct calculable_item maxFunctionDefs;
struct calculable_item maxInstructionDefs;
struct calculable_item maxStackElements;
struct calculable_item maxSizeOfInstructions;
struct calculable_item maxComponentElements;
struct calculable_item maxComponentDepth;

struct classification_data  *attachment_classes, *attachment_classes2;

/* Data for gasp table */
struct gasp_range *v_gasp;
int n_gasp;

void init_calculable_item(struct calculable_item *x) {
  x->is_forced = 0;
  x->causer_type = NO_CAUSER;
}

void init_analysis() {
  struct tm time_now;
  time_t    t_now;
  unsigned long long int day_number;
  int i;
  struct class_data *cldat;
  struct glyph_group *gg;
  /* Set up dynamic variables */
  last_glyph = -1;
  create_map(&glyph_names);
  create_map(&name_names);
  max_postscript_name = 0;
  postscript_name = 0;
  /* Set up the data for the glyph file. */
  glyphs = tmpfile();
  if (!glyphs) {
    perror(0);
    fprintf(stderr, "- so cannot create scratch file for glyphs.\n");
    exit(1);
  }
  glyphs_size = 0;
  untracked = create_int_set();
  /* Cannot re-initialise: charmap */
  /* Once-only initialisations */
  create_map(&nameID_codes);
  insert_in_map(&nameID_codes, "copyright",    (void*) 0);
  insert_in_map(&nameID_codes, "family",       (void*) 1);
  insert_in_map(&nameID_codes, "subfamily",    (void*) 2);
  insert_in_map(&nameID_codes, "identifier",   (void*) 3);
  insert_in_map(&nameID_codes, "full_name",    (void*) 4);
  insert_in_map(&nameID_codes, "version",      (void*) 5);
  insert_in_map(&nameID_codes, "postscript_name",      (void*) 6);
  insert_in_map(&nameID_codes, "PostScript_name",      (void*) 6);
  insert_in_map(&nameID_codes, "trademark",    (void*) 7);
  insert_in_map(&nameID_codes, "manufacturer", (void*) 8);
  insert_in_map(&nameID_codes, "designer",     (void*) 9);
  insert_in_map(&nameID_codes, "description",  (void*) 10);
  insert_in_map(&nameID_codes, "vendor_URL",   (void*) 11);
  insert_in_map(&nameID_codes, "designer_URL", (void*) 12);
  insert_in_map(&nameID_codes, "licence",      (void*) 13);
  insert_in_map(&nameID_codes, "license",      (void*) 13);
  insert_in_map(&nameID_codes, "licence_URL",  (void*) 14);
  insert_in_map(&nameID_codes, "preferred_family",     (void*) 16);
  insert_in_map(&nameID_codes, "preferred_subfamily",  (void*) 17);
  insert_in_map(&nameID_codes, "compatible_full_name", (void*) 18);
  insert_in_map(&nameID_codes, "sample",       (void*) 19);
  insert_in_map(&nameID_codes, "sample_text",  (void*) 19);
  insert_in_map(&nameID_codes, "postscript_CID_name",  (void*) 20);
  insert_in_map(&nameID_codes, "PostScript_CID_name",  (void*) 20);
  create_map(&platID_codes);
#define platID_Unicode   0
#define platID_Macintosh 1
#define platID_Microsoft 3
#define platID_Custom    4
  insert_in_map(&platID_codes, "Unicode",   (void*) platID_Unicode);
  insert_in_map(&platID_codes, "Macintosh", (void*) platID_Macintosh);
  insert_in_map(&platID_codes, "Microsoft", (void*) platID_Microsoft);
  insert_in_map(&platID_codes, "Custom",    (void*) platID_Custom);
  insert_in_map(&platID_codes, "Mac",       (void*) platID_Macintosh);
  insert_in_map(&platID_codes, "MS",        (void*) platID_Microsoft);
  create_map(&unicode_encodings);
#define uc_enc_UCS_2  3
#define uc_enc_UCS_4  4
#define uc_enc_VS     5
#define uc_enc_UCS_fb 6
  insert_in_map(&unicode_encodings, "v1.0",     (void*) 0);
  insert_in_map(&unicode_encodings, "v1.1",     (void*) 1);
  insert_in_map(&unicode_encodings, "ISO10646", (void*) 2);
  insert_in_map(&unicode_encodings, "BMP",      (void*) uc_enc_UCS_2);
  insert_in_map(&unicode_encodings, "UCS-2",    (void*) uc_enc_UCS_2);
  insert_in_map(&unicode_encodings, "UCS-4",    (void*) uc_enc_UCS_4);
  insert_in_map(&unicode_encodings, "VS",       (void*) uc_enc_VS);
  insert_in_map(&unicode_encodings, "fallback", (void*) uc_enc_UCS_fb);
  create_map(&ms_encodings);
#define ms_enc_UCS_2 1
#define ms_enc_UCS_4 10
  insert_in_map(&ms_encodings, "Symbol",       (void*) 0);
  insert_in_map(&ms_encodings, "symbol",       (void*) 0);
  insert_in_map(&ms_encodings, "BMP",          (void*) ms_enc_UCS_2);
  insert_in_map(&ms_encodings, "Unicode_BMP",  (void*) ms_enc_UCS_2);
  insert_in_map(&ms_encodings, "UCS-2",        (void*) ms_enc_UCS_2);
  insert_in_map(&ms_encodings, "ShiftJIS",     (void*) 2);
  insert_in_map(&ms_encodings, "PRC",          (void*) 3);
  insert_in_map(&ms_encodings, "Big5",         (void*) 4);
  insert_in_map(&ms_encodings, "Wansung",      (void*) 5);
  insert_in_map(&ms_encodings, "Johab",        (void*) 6);
  insert_in_map(&ms_encodings, "Unicode_full", (void*) ms_enc_UCS_4);
  insert_in_map(&ms_encodings, "UCS-4",        (void*) ms_enc_UCS_4);
  create_map(&mac_encodings);
  insert_in_map(&mac_encodings, "Roman",      (void*) 0);
  insert_in_map(&mac_encodings, "Japanese",   (void*) 1);
  insert_in_map(&mac_encodings, "Chinese_Traditional", (void*) 2);
  insert_in_map(&mac_encodings, "Korean",     (void*) 3);
  insert_in_map(&mac_encodings, "Arabic",     (void*) 4);
  insert_in_map(&mac_encodings, "Hebrew",     (void*) 5);
  insert_in_map(&mac_encodings, "Greek",      (void*) 6);
  insert_in_map(&mac_encodings, "Russian",    (void*) 7);
  insert_in_map(&mac_encodings, "RSymbol",    (void*) 8);
  insert_in_map(&mac_encodings, "Devanagari", (void*) 9);
  insert_in_map(&mac_encodings, "Gurmukhi",   (void*) 10);
  insert_in_map(&mac_encodings, "Gujarati",   (void*) 11);
  insert_in_map(&mac_encodings, "Oriya",      (void*) 12);
  insert_in_map(&mac_encodings, "Bengali",    (void*) 13);
  insert_in_map(&mac_encodings, "Tamil",      (void*) 14);
  insert_in_map(&mac_encodings, "Telugu",     (void*) 15);
  insert_in_map(&mac_encodings, "Kannada",    (void*) 16);
  insert_in_map(&mac_encodings, "Malayalam",  (void*) 17);
  insert_in_map(&mac_encodings, "Sinhalese",  (void*) 18);
  insert_in_map(&mac_encodings, "Burmese",    (void*) 19);
  insert_in_map(&mac_encodings, "Khmer",      (void*) 20);
  insert_in_map(&mac_encodings, "Thai",       (void*) 21);
  insert_in_map(&mac_encodings, "Laotian",    (void*) 22);
  insert_in_map(&mac_encodings, "Georgian",   (void*) 23);
  insert_in_map(&mac_encodings, "Armenian",   (void*) 24);
  insert_in_map(&mac_encodings, "Chinese_Simplified",  (void*) 25);
  insert_in_map(&mac_encodings, "Tibetan",    (void*) 26);
  insert_in_map(&mac_encodings, "Mongolian",  (void*) 27);
  insert_in_map(&mac_encodings, "Geez",       (void*) 28);
  insert_in_map(&mac_encodings, "Slavic",     (void*) 29);
  insert_in_map(&mac_encodings, "Vietnamese", (void*) 30);
  insert_in_map(&mac_encodings, "Sindhi",     (void*) 31);
  insert_in_map(&mac_encodings, "Uninterpreted",       (void*) 32);
  create_map(&ms_languages);
  /* http://www.microsoft.com/globaldev/reference/lcid-all.mspx */
  insert_in_map(&ms_languages, "en_US", (void*) 0x0409);
  insert_in_map(&ms_languages, "en_UK", (void*) 0x0809);
  insert_in_map(&ms_languages, "th",    (void*) 0x041e);
  insert_in_map(&ms_languages, "Thai",  (void*) 0x041e);
  create_map(&mac_languages);
  insert_in_map(&mac_languages, "Not_specific", (void*) 0xffff);
  insert_in_map(&mac_languages, "None",    (void*) 0xffff);
  insert_in_map(&mac_languages, "English", (void*) 0);
  insert_in_map(&mac_languages, "French",  (void*) 1);
  insert_in_map(&mac_languages, "Thai",    (void*) 22);
  insert_in_map(&mac_languages, "Khmer",   (void*) 78);
  insert_in_map(&mac_languages, "Lao",     (void*) 79);
  create_map(&month_names);
  insert_in_map(&month_names, "January",   (void*) 0);
  insert_in_map(&month_names, "February",  (void*) 1);
  insert_in_map(&month_names, "March",     (void*) 2);
  insert_in_map(&month_names, "April",     (void*) 3);
  insert_in_map(&month_names, "May",       (void*) 4);
  insert_in_map(&month_names, "June",      (void*) 5);
  insert_in_map(&month_names, "July",      (void*) 6);
  insert_in_map(&month_names, "August",    (void*) 7);
  insert_in_map(&month_names, "September", (void*) 8);
  insert_in_map(&month_names, "October",   (void*) 9);
  insert_in_map(&month_names, "November",  (void*) 10);
  insert_in_map(&month_names, "December",  (void*) 11);
  insert_in_map(&month_names, "Jan", (void*) 0);
  insert_in_map(&month_names, "Feb", (void*) 1);
  insert_in_map(&month_names, "Mar", (void*) 2);
  insert_in_map(&month_names, "Apr", (void*) 3);
  insert_in_map(&month_names, "Jun", (void*) 5);
  insert_in_map(&month_names, "Jul", (void*) 6);
  insert_in_map(&month_names, "Aug", (void*) 7);
  insert_in_map(&month_names, "Sep", (void*) 8);
  insert_in_map(&month_names, "Oct", (void*) 9);
  insert_in_map(&month_names, "Nov", (void*) 10);
  insert_in_map(&month_names, "Dec", (void*) 11);
  insert_in_map(&month_names, "JAN", (void*) 0);
  insert_in_map(&month_names, "FEB", (void*) 1);
  insert_in_map(&month_names, "MAR", (void*) 2);
  insert_in_map(&month_names, "APR", (void*) 3);
  insert_in_map(&month_names, "MAY", (void*) 4);
  insert_in_map(&month_names, "JUN", (void*) 5);
  insert_in_map(&month_names, "JUL", (void*) 6);
  insert_in_map(&month_names, "AUG", (void*) 7);
  insert_in_map(&month_names, "SEP", (void*) 8);
  insert_in_map(&month_names, "OCT", (void*) 9);
  insert_in_map(&month_names, "NOV", (void*) 10);
  insert_in_map(&month_names, "DEC", (void*) 11);
  create_map(&colours);

  p0c0 = -1; /* Special value */
  colval = NULL;
  mcolval = 0;
  pe_label = NULL;
  mpe_label = 0;
  mpal = 0;
  have_COLR = have_CPAL = 0;
  memset(&palette, 0, sizeof palette);
  memset(&colr, 0, sizeof colr);
  define_opcodes(&opcode_def);
  create_map(&program_def);
  hhea_numberofHMetrics = 0;
  init_calculable_item(&advanceWidthMax);
  init_calculable_item(&xMaxExtent);
  init_calculable_item(&minLeftSideBearing);
  init_calculable_item(&minRightSideBearing);
  ascender  = 0;
  descender = 0;
  lineGap   = 0;
  caretSlopeRise = 1;
  caretSlopeRun  = 0;
  caretOffset    = 0;
  /* Initialise non-constants for the head table */
/* Calculable items for the head table */
  init_calculable_item(&head_xMin);
  init_calculable_item(&head_yMin);
  init_calculable_item(&head_xMax);
  init_calculable_item(&head_yMax);
  init_calculable_item(&fontDirectionHint); /* forceable, not calculable */
  init_calculable_item(&fDH_l2r);           /* calculable, not forceable */
  fDH_l2r.calculated_value = 0;
  fDH_l2r.causer_type = ALL_GLYPHS;
  init_calculable_item(&fDH_r2l); /* Forcing affects bit 9 of flags */
  fDH_r2l.calculated_value = 0;
  fDH_r2l.causer_type = ALL_GLYPHS;
  init_calculable_item(&fDH_0);
  fDH_0.calculated_value = 0;
  fDH_0.causer_type = ALL_GLYPHS;
/* fDH_r2l also affects flags!  Let it be forceable for ** flags ** bit 9 */
/* Calculable items for bits 1, 2, 4 of flags in head table. */
  init_calculable_item(&lsbx0);
  lsbx0.calculated_value = 0;
  lsbx0.causer_type = ALL_GLYPHS;
  init_calculable_item(&ins_use_PPEM);
  ins_use_PPEM.calculated_value = 0;
  ins_use_PPEM.causer_type = ALL_GLYPHS;
  init_calculable_item(&aw_by_PPEM);
  aw_by_PPEM.calculated_value = 0;
  aw_by_PPEM.causer_type = ALL_GLYPHS;
/* Non-calculable items for the head table. */
  fontRevision = 0x00010000; /* 1.0 */
  unitsPerEm = 2048;
  lowestRecPPEM = 7;
  head_flags = 0; /* But some calculable! */
  macStyle = 0;
  (void) time(&t_now);
#ifdef __MINGW32__
  time_now = *localtime(&t_now);
#else
  (void) localtime_r(&t_now, &time_now);
#endif
  date1904_to_seconds(1900+time_now.tm_year, time_now.tm_mon, time_now.tm_mday,
		      time_now.tm_hour, time_now.tm_min, time_now.tm_sec,
		      &day_number, &created);
  modified = created;
  indexToLocFormat = 1;
  indexToLocFormat_forced = 0;
/* Calculable items for the OS/2 table. */
  init_calculable_item(&xAvgCharWidth);
  totCharWidth = nCharWidth = 0;
  for (i = 0; i < 4*32; i++) {
    init_calculable_item(&ulUnicodeRange[i]);
    ulUnicodeRange[i].causer_type = ALL_GLYPHS;
  }
  init_calculable_item(&usFirstCharIndex);
  usFirstCharIndex.calculated_value = 0xFFFF;
  init_calculable_item(&usLastCharIndex);
  init_calculable_item(&usWinAscent);
  init_calculable_item(&usWinDescent);
  init_calculable_item(&sxHeight);
  init_calculable_item(&sCapHeight);
  init_calculable_item(&usMaxContext);
/* Non-calculable items for the OS/2 table. */
  usWeightClass = 0x400; /* FW_MEDIUM */
  usWidthClass = 5; /* FWIDTH_NORMAL */
  fsType = 0; /* No restriction on embedding */
/* -1 implies a last resort calculation is needed. */
  ySubscriptXSize = -1;
  ySubscriptYSize = -1;
  ySubscriptXOffset = 0;
  ySubscriptYOffset = -1;
  ySuperscriptXSize = -1;
  ySuperscriptYSize = -1;
  ySuperscriptXOffset = 0;
  ySuperscriptYOffset = -1;
  yStrikeoutSize = -1;
  yStrikeoutPosition = -1;
  sFamilyClass = 0;
  memset(panose, 0, sizeof panose);
  strcpy(achVendID, "    ");
  fsSelection = 6; /* Regular */;
  sTypoAscender = -1;
  sTypoDescender = -1;
  sTypoLineGap = 0;
  memset(ulCodePageRange, 0, sizeof ulCodePageRange);
  usDefaultChar = 0; /* Use glyph 0.  Other values code by character */
  usBreakChar = ' '; /* Default to U+0020 SPACE */
/* Calculable items for the post table */
  init_calculable_item(&isFixedPitch);
  isFixedPitch.causerID = ALL_GLYPHS;
  isFixedPitch.calculated_value = -1;
/* Non-calculable items for the post table */
  italicAngle = 0.0;
  underlinePosition = 0;
  underlineThickness = -1;
  minMemType42 = 0;
  maxMemType42 = 0;
  minMemType1 = 0;
  maxMemType1 = 0;
/* Font and CVT table programs */
  have_fpgm = have_prep = 0;
/* OTL tables */
  have_GDEF = 0;
  have_cvt = 0;
  m_cvt = 0;
  p_cvt = 0;
/* Calculable items for maxp */
  init_calculable_item(&maxPoints);
  init_calculable_item(&maxContours);
  init_calculable_item(&maxCompositePoints);
  init_calculable_item(&maxCompositeContours);
  init_calculable_item(&maxZones);
  maxZones.calculated_value = 2;
  init_calculable_item(&maxTwilightPoints);
  init_calculable_item(&maxStorage);
  init_calculable_item(&maxFunctionDefs);
  init_calculable_item(&maxInstructionDefs);
  init_calculable_item(&maxStackElements);
  init_calculable_item(&maxSizeOfInstructions);
  init_calculable_item(&maxComponentElements);
  init_calculable_item(&maxComponentDepth);
/* Items for GDEF */
  create_map(&category_names);
  insert_in_map(&category_names, "default",     (void*) CAT_DEFAULT);
  insert_in_map(&category_names, "base",        (void*) CAT_BASE);
  insert_in_map(&category_names, "ligature",    (void*) CAT_LIGATURE);
  insert_in_map(&category_names, "mark",        (void*) CAT_MARK);
  insert_in_map(&category_names, "component",   (void*) CAT_COMPONENT);
  insert_in_map(&category_names, "defaultmark", (void*) CAT_DEFAULTMARK);
  insert_in_map(&category_names, "pseudomark",  (void*) CAT_PSEUDOMARK);
/* Items for GSUB and GPOS */
  gsub = gpos = 0;
  have_GSUB = have_GPOS = 0;
  create_lookup_map(&lookups);
  create_subtable_map(&subtables);
  create_gclassification_map(&classifications);
  attachment_classes = create_classification(" -MAC- ");
  attachment_classes2 = create_classification(" -MAC names- ");
  locate_class("none",  attachment_classes2, &cldat);
  assert(0 == cldat->number);
  locate_class("MAC 0", attachment_classes,  &cldat);
  lookup_in_map(glyph_names, "none", (void**)&gg);
  assert(CLASS == gg->grouping);
  gg->x.class_data = cldat;
  assert(0 == cldat->number);
/* Data for gasp table */
  v_gasp = 0;
  n_gasp = -1; /* Allow 0 for empty table! */
}

static char *endianity() {
/* Weirdly enough, this function is re-entrant! */
  static char result[10];
  volatile union eqv {
    char b[4];
    short int w[2];
    int lw;
  } td;

  td.b[0] = 1;
  td.b[1] = 3;
  td.b[2] = 5;
  td.b[3] = 7;
  if (0x07050301 == td.lw && 0x0301 == td.w[0]) {
    strcpy(result, "little");
  } else if (0x01030507 == td.lw && 0x0103 == td.w[0]) {
    strcpy(result, "big");
  } else {
    printf("Byte ordering or word size is not clear.  For the same memory:\n");
    printf("Bytes: %2.2x %2.2x %2.2x %2.2x\n",
	   td.b[0], td.b[1], td.b[2], td.b[3]);
    printf("Words: %4.4x %4.4x\n", td.w[0], td.w[1]);
    printf("Lw: %8.8x\n", td.lw);
    strcpy(result, "unknown");
  }
  return result;
}

void put_gid_ps(FILE *fp, int gid) {
  char *str = postscript_name[gid];
  if (str) {
    fprintf(fp, "%d (postscript name %s)", gid, str);
  } else {
    fprintf(fp, "%d", gid);
  }
}

void discrepancy(char *tag, struct calculable_item item) {
  if (NO_CAUSER == item.causer_type) return;
  if (!item.is_forced) return;
  if (item.forced_value == item.calculated_value) return;
  switch (item.causer_type) {
  case BY_GLYPH:
    fprintf(stderr, "%s forced to %d but calculated "
	    "as %d because of glyph ", tag,
	    item.forced_value, item.calculated_value);
    put_gid_ps(stderr, item.causerID);
    fprintf(stderr, ".\n");
    break;
  case BY_POINT:
    fprintf(stderr, "%s forced to %d but calculated "
	    "as %d because of point %d.\n", tag,
	    item.forced_value, item.calculated_value, item.causerID);
    break;
  case ALL_GLYPHS:
    fprintf(stderr, "%s forced to %d but calculated "
	    "as %d on the basis of all relevant glyphs.\n", tag,
	    item.forced_value, item.calculated_value);
    break;
  default:
    fprintf(stderr, "%s forced to %d but calculated as %d.\n", tag,
	    item.forced_value, item.calculated_value);
    fprintf(stderr, "Internal error: Causer type %d is not recognised.\n",
	    item.causer_type);
  }
}

void align_file(FILE *dest, int *size, int el_size) {
  int over, extra, i;
  char c = 0;

  over = *size % el_size;
  if (over) {
    extra = el_size - over;
    for (i = 0; i < extra; i++) fwrite(&c, 1, 1, dest);
    *size += extra;
  }
}

void process_glyph() {
/* Current token is glyph on input, and is immediately after definition on exit. */
  int oldno, this_glyph;
  struct glyph_group *glyph_def = 0;
  char *name;
  struct calculable_item xMin, xMax, yMin, yMax;
  int has_points = 0;
  int has_components = 0;
/* Data for points */
  struct coordinate *xpt = 0;
  struct coordinate *ypt = 0;
  struct /*un*/repeatable_flag *flg = 0;
  int nxpts = 0;
  int nypts = 0;
  int nfpts = 0;
  int maxpts = 0;
  int max_contours = 0;
  int n_contours = 0;
  unsigned short int *contour_ends = 0;
  int n_components = 0;
  int max_components = 0;
  struct component_entry *comp = 0;
  int hints_demanded = 0;
  int has_hints = 0;
  unsigned char *instr;
  int n_instr;
  struct glyph_entry the_glyph;
  int junk;
  int has_aw = 0;
  int has_lsb = 0;
  unsigned short int advanceWidth;
  short int lsb;
  int is_default_aw = 0;
  int rsb;
  const int nblocks = n_script_table;

  if (strcmp("glyph", current.sval)) {
    fprintf(stderr, "process_glyph: internal error - token is not 'glyph'.\n");
    exit(1);
  }
  last_glyph++;
#if 0
  fprintf(stderr, "Processing glyph %d.\n", last_glyph);
#endif
  if (cap_glyphs <= last_glyph) {
    glyph_spp = realloc(glyph_spp, (cap_glyphs+=256) * sizeof(*glyph_spp));
    if (!glyph_spp) {
      fprintf(stderr, "Memory exhausted!");
      exit(1);
    }
  }
  init_calculable_item(&xMin);
  init_calculable_item(&xMax);
  init_calculable_item(&yMin);
  init_calculable_item(&yMax);
  do {
    advance_token();
    if (AN_IDENT == current.form) {
      if (lookup_in_map(glyph_names, current.sval, (void**)&glyph_def)) {
	show_source();
	if (SOLO == glyph_def->grouping) {
	  fprintf(stderr, "Glyph name %s already applies to glyph ",
		  current.sval);
	  put_gid_ps(stderr, glyph_def->x.number);
	  fprintf(stderr, ".  Not superseded.\n");
	} else {
	  fprintf(stderr, "%s already denotes a group of glyphs.\n",
		  current.sval);
	}
      } else {
	glyph_def = malloc(sizeof(struct glyph_group));
	glyph_def->grouping = SOLO;
	glyph_def->x.number = last_glyph;
#if 0
	printf("process_glyph: Inserting entry for %s.\n", current.sval);
#endif
	insert_in_map(&glyph_names, current.sval, glyph_def);
      }
    } else {
      show_source();
      fprintf(stderr, "Expected an identifier.\n");
    }
    if (PUNCTUATION != current.form || ',' != current.ival) advance_token();
  } while (PUNCTUATION == current.form && ',' == current.ival);
  this_glyph = last_glyph;
  if (this_glyph >= max_postscript_name) {
    max_postscript_name += 100;
    postscript_name =
      realloc(postscript_name, (max_postscript_name + 1) * sizeof(char*));
  }
  postscript_name[this_glyph] = 0;
  glyph_spp[this_glyph].category = NO_CATEGORY;
  glyph_spp[this_glyph].attachment_class = NO_MAC;
  while (END_OF_FILE != current.form) {
    if (AN_IDENT != current.form) {
      show_source();
      fprintf(stderr, "Expected a keyword.\n");
      advance_token();
    } else {
      if (!strcmp("number", current.sval)) {
	advance_token();
	if (AN_INT != current.form) {
	} else if (current.ival != this_glyph) {
	  show_source();
	  fprintf(stderr, "Glyph definition is out of sequence - "
		  "assigned glyph number ");
	  put_gid_ps(stderr, this_glyph);
	  fprintf(stderr, ", not %d.\n", current.ival);
	}
	advance_token();
      } else if (!strcmp("character", current.sval)) {
	do {
          int basecp = -1;
	  advance_token();
	  if (AN_IDENT == current.form || A_STRING == current.form) {
	    show_source();
	    fprintf(stderr, "Sorry - Unicode names not supported.\n");
	  } else if (AN_INT == current.form) {
	    int i, loc;

            if (0  <= current.ival && current.ival <  0xD800 ||
	    0xE000 <= current.ival && current.ival <= 0x10ffff) {
	      datatype dirtype;

              basecp = current.ival;
	      get_property(unicode_dirs, current.ival, &dirtype);
	      switch ((int)dirtype) {
	      case K_DIR_L:
	      case K_DIR_LRE:
	      case K_DIR_LRO:
/* Calculable, not forceable */
		if (!fDH_l2r.calculated_value) {
		  fDH_l2r.calculated_value = 1;
		  fDH_l2r.causerID = this_glyph;
		  fDH_l2r.causer_type = BY_GLYPH;
		}
		break;
	      case K_DIR_AL:
	      case K_DIR_R:
	      case K_DIR_RLE:
	      case K_DIR_RLO:
/* Calculable, not forceable */
		if (!fDH_r2l.calculated_value) {
		  fDH_r2l.calculated_value = 1;
		  fDH_r2l.causerID = this_glyph;
		  fDH_r2l.causer_type = BY_GLYPH;
		}
		break;
	      case K_DIR_EN:
	      case K_DIR_AN:
		/* Weak - no need to record */
		break;
	      case K_DIR_PDF:
		break;
	      case K_DIR_ES:
	      case K_DIR_ET:
	      case K_DIR_CS:
	      case K_DIR_NSM:
	      case K_DIR_BN:
		/* Formally weak, so no recording */
		break;
	      case K_DIR_B:
	      case K_DIR_S:
	      case K_DIR_WS:
	      case K_DIR_ON:
	      case K_DIR_LRI: /* Classification of these is a little unclear */
	      case K_DIR_RLI:
	      case K_DIR_FSI:
	      case K_DIR_PDI:
/* Neutrals */
/* Calculable, not forceable */
		if (!fDH_0.calculated_value) {
		  fDH_0.calculated_value = 1;
		  fDH_0.causerID = this_glyph;
		  fDH_0.causer_type = BY_GLYPH;
		}
		break;
	      default:
		fprintf(stderr, "Unicode character U+%4.4x has undefined "
			"directionality %d.\n", current.ival, (int) dirtype);
	      }
	      loc = -1;
	      for (i = 0; i < nblocks; i++) {
		if (script_table[i].start <= current.ival &&
		    current.ival <= script_table[i].end) {
		  if (loc >= 0) {
		    fprintf(stderr, "U+%4.4X is defined to be in both "
			    "%s (U+%4.4X to U+%4.4X) and "
			    "%s (U+%4.4X to U+%4.4X).\n",
			    current.ival, script_table[loc].ln,
			    script_table[loc].start, script_table[loc].end,
			    script_table[i].ln,
			    script_table[i].start, script_table[i].end);
		  } else {
		    loc = i;
		  }
		}
	      }
	      if (loc >= 0) {
		ulUnicodeRange[script_table[loc].bit].calculated_value++;
		ulUnicodeRange[script_table[loc].bit].causerID = this_glyph;
		ulUnicodeRange[script_table[loc].bit].causer_type = BY_GLYPH;
	      } else if (1) {
		add_range(current.ival, current.ival, &untracked);
	      }
	      if (current.ival > 0xffff) {
		ulUnicodeRange[is_supplementary].calculated_value++;
		ulUnicodeRange[is_supplementary].causerID = this_glyph;
		ulUnicodeRange[is_supplementary].causer_type = BY_GLYPH;
	      }
/* Ignore mapping of 0x0000 and 0xFFFF to glyph 0 */
	      if (this_glyph || (current.ival && 0xFFFF != current.ival)) { 
		if (current.ival < usFirstCharIndex.calculated_value) {
		  usFirstCharIndex.causer_type = BY_GLYPH;
		  usFirstCharIndex.causerID = this_glyph;
		  usFirstCharIndex.calculated_value =
		    (current.ival <= 0xFFFF) ? current.ival : 0xFFFF;
		}
		if (current.ival >= usLastCharIndex.calculated_value) {
		  usLastCharIndex.causer_type = BY_GLYPH;
		  usLastCharIndex.causerID = this_glyph;
		  usLastCharIndex.calculated_value =
		    (current.ival <= 0xFFFF) ? current.ival : 0xFFFF;
		}
	      }
	    } else {
	      show_source();
	      fprintf(stderr, "Character code U+%4.4X is not legal.\n",
		      current.ival);
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected an identifier.\n");
	  }
	  if (PUNCTUATION != current.form || ',' != current.ival) {
	    advance_token();
	  }
          if (basecp < 0) {
            /* Bad codepoint; forget it. */
          } else if (AN_INT == current.form) { /* Variation selector */
            int i = 0, j = sizeof vscharmap / sizeof vscharmap[0] - 1;
            int mid = (i+j)/2;
            while (i <= j) {
              if (current.ival == vscharmap[mid].vs) break;
              if (current.ival < vscharmap[mid].vs) {
                j = mid - 1;
              } else {
                i = mid + 1;
              }
              mid = (i+j)/2;
            }
            if (i <= j) {
              if (!vscharmap[mid].map) {
                vscharmap[mid].map = malloc(sizeof * vscharmap[mid].map);
              }
              if (!vscharmap[mid].map) {
                fprintf(stderr, "Memory exhausted creating cmap for "
                        "VS = U+%4.4X.\n", current.ival);
                exit(1);
              }
              set_property(vscharmap[mid].map, basecp, (datatype) this_glyph,
                           (datatype) (-1));
              if (0) {
                fprintf(stderr, "Got VS U+%4.4X U+%4.4X.\n",
                        basecp, current.ival);
              }
            } else {
              show_source();
              fprintf(stderr, "U+%4.4X is not a variation selector.\n",
                      current.ival);
            }
            advance_token();
          } else {
            set_property(&charmap, basecp, (datatype) this_glyph,
                         (datatype) (-1));
          }
	} while (PUNCTUATION == current.form && ',' == current.ival);
      } else if (!strcmp("postname", current.sval)) {
	advance_token();
	if (AN_IDENT == current.form || A_STRING == current.form) {
	  int lenok = strlen(current.sval);
	  if (lenok > 255) {
	    show_source();
	    fprintf(stderr, "Truncated postscript name of glyph %d to %.255s",
		    this_glyph, current.sval);
	    lenok = 255;
	  }
	  if (postscript_name[this_glyph]) {
	    if (strncmp(postscript_name[this_glyph], current.sval, 255)) {
	      show_source();
	      fprintf(stderr, "Ignoring duplicate postscript name %s for glyph",
		      current.sval);
	      put_gid_ps(stderr, this_glyph);
	      fprintf(stderr, ".\n");
	    }
	  } else {
	    struct glyph_group *alt_glyph_def;
	    int altnum;

	    name = malloc(lenok+1);
	    strncpy(name, current.sval, lenok);
	    name[lenok] = 0;
	    postscript_name[this_glyph] = name;
/* Make postscript name an identifier for this glyph, and check its suitability 
   as a unique identifier. */
	    if (lookup_in_map(glyph_names, name, (void**)&alt_glyph_def)) {
	      if (SOLO != alt_glyph_def->grouping) {
		show_source();
		fprintf(stderr, "%s already denotes a group of glyphs.\n",
			current.sval);
	      } else {
		altnum = alt_glyph_def->x.number;
		if (altnum == this_glyph) {
		  /* Glyph name and postname the same - excellent! */
		} else if (!postscript_name[altnum] ||
			   strcmp(postscript_name[altnum], name)) {
		  show_source();
		  fprintf(stderr, "Postscript name %s of glyph %d already "
			  "designates glyph ", name, this_glyph);
		  put_gid_ps(stderr, altnum);
		  fprintf(stderr, ".\n");
		} else {
		  show_source();
		  fprintf(stderr, "Glyphs %d and %d have the same postscript "
			  "name, \"%s\".\n", altnum, this_glyph, name);
		}
	      }
	    } else {
	      insert_in_map(&glyph_names, name, glyph_def);
	    }
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected a name.\n");
	}
	advance_token();
#define getSW(name) \
	  advance_token();\
	  if (AN_INT != current.form) {\
	    show_source();\
	    fprintf(stderr, "Expected a dimension.");\
	  } else {\
	    if (current.ival > 0x7fff || current.ival < -0x8000) {\
              show_source(); \
	      fprintf(stderr, "Value of " #name " viz %d is not a " \
		      "16-bit signed number.\n", current.ival);\
	    } else {\
	      name.forced_value = current.ival;\
	      name.is_forced = 1;\
	    }\
	  }\
	  advance_token();
      } else if (!strcmp("xMin", current.sval)) {
	getSW(xMin);
      } else if (!strcmp("xMax", current.sval)) {
	getSW(xMax);
      } else if (!strcmp("yMin", current.sval)) {
	getSW(yMin);
      } else if (!strcmp("yMax", current.sval)) {
	getSW(yMax);
      } else if (!strcmp("points", current.sval)) {
        int x, y, old_x, old_y, count;
        unsigned char old_flag, new_flag;
        int diff, off_met, contour_end;
        int ipt;

        has_points = 1;
	advance_token();
	x = 0;
	y = 0;
	old_flag = 0xff; /* Unpleasant way of forcing new flag */
	ipt = -1;
#define ON_CURVE 1
#define X_BYTE   2
#define Y_BYTE   4
#define REPEAT   8
#define X_SAME  16
#define Y_SAME  32
	do {
	  new_flag = 0;
	  old_x = x;
	  old_y = y;
	  if (AN_IDENT == current.form || A_STRING == current.form) {
	    if (!strcmp("same", current.sval)) {
	      new_flag |= X_SAME;
	    } else if (!strcmp("word_to", current.sval)) {
	      advance_token();
	      if (AN_INT == current.form) {
		x = current.ival;
	      } else {
		show_source();
		fprintf(stderr, "Expected a number.\n");
	      }
#define extend_points {\
		    maxpts += 40;\
		    xpt = realloc(xpt, maxpts * sizeof(*xpt));\
		    ypt = realloc(ypt, maxpts * sizeof(*ypt));\
		    flg = realloc(flg, maxpts * sizeof(*flg));\
		    if (!xpt || !ypt || !flg) {\
		      fprintf(stderr,\
			      "Memory exhausted processing outline.\n");\
		      exit(1);\
		    }}
	      if (nxpts >= maxpts) extend_points;
	      xpt[nxpts].loaded  = 1;
	      xpt[nxpts].flipped = 0;
	      xpt[nxpts++].pos   = x - old_x;
	    } else if (!strcmp("byte_to", current.sval)) {
	      advance_token();
	      if (AN_INT == current.form) {
		x = current.ival;
	      } else {
		show_source();
		fprintf(stderr, "Expected a number.\n");
		got_what(stderr, current);
	      }
	      if (nxpts >= maxpts) extend_points;
	      xpt[nxpts].loaded = 1;
	      new_flag |= X_BYTE;
	      if (x >= old_x) {
		xpt[nxpts].flipped = 0;
		xpt[nxpts++].pos   = x - old_x;
		new_flag |= X_SAME;
	      } else {
		xpt[nxpts].flipped = 1;
		xpt[nxpts++].pos  = old_x - x;
	      }
	    } else {
	      show_source();
	      fprintf(stderr,
		      "Expected same, word_to, byte_to or a number.\n");
	      got_what(stderr, current);
	      new_flag |= X_SAME;
	    }
	  } else if (AN_INT == current.form) {
	    x = current.ival;
	    diff = x - old_x;
	    if (diff) {
	      if (nxpts >= maxpts) extend_points;
	      xpt[nxpts].loaded = 1;
	      xpt[nxpts].flipped = 0;
	      xpt[nxpts].pos = diff;
	      if (diff > 255 || diff < -255) {
		/* Do nothing */
	      } else if (diff >= 0) {
		new_flag |= X_SAME | X_BYTE;
	      } else {
		xpt[nxpts].flipped = 1;
		xpt[nxpts].pos = -diff;
		new_flag |= X_BYTE;
	      }
	      nxpts++;
	    } else {
	      new_flag |= X_SAME;
	    }
	  } else {
	    show_source();
	    fprintf(stderr,
		    "Expected a keyword or a number.\n");
	    new_flag |= X_SAME;
	  }
#if 0
	  fprintf(stderr, "Last part of x-coord is ");
	  got_what(stderr, current);
#endif
	  advance_token();
	  if (AN_IDENT == current.form || A_STRING == current.form) {
	    if (!strcmp("same", current.sval)) {
	      new_flag |= Y_SAME;
	    } else if (!strcmp("word_to", current.sval)) {
	      advance_token();
	      if (AN_INT == current.form) {
		y = current.ival;
	      } else {
		show_source();
		fprintf(stderr, "Expected a number.\n");
		got_what(stderr, current);
	      }
	      if (nypts >= maxpts) extend_points;
	      ypt[nypts].loaded  = 1;
	      ypt[nypts].flipped = 0;
	      ypt[nypts++].pos   = y - old_y;
	    } else if (!strcmp("byte_to", current.sval)) {
	      advance_token();
	      if (AN_INT == current.form) {
		y = current.ival;
	      } else {
		show_source();
		fprintf(stderr, "Expected a number.\n");
		got_what(stderr, current);
	      }
	      if (nypts >= maxpts) extend_points;
	      ypt[nypts].loaded = 1;
	      new_flag |= Y_BYTE;
	      if (y >= old_y) {
		ypt[nypts].flipped = 0;
		ypt[nypts++].pos   = y - old_y;
		new_flag |= Y_SAME;
	      } else {
		ypt[nypts].flipped = 1;
		ypt[nypts++].pos  = old_y - y;
	      }
	    } else {
	      show_source();
	      fprintf(stderr,
		      "Expected same, word_to, byte_to or a number.\n");
	      got_what(stderr, current);
	      new_flag |= Y_SAME;
	    }
	  } else if (AN_INT == current.form) {
	    y = current.ival;
	    diff = y - old_y;
	    if (diff) {
	      if (nypts >= maxpts) extend_points;
	      ypt[nypts].loaded = 1;
	      ypt[nypts].flipped = 0;
	      ypt[nypts].pos = diff;
	      if (diff > 255 || diff < -255) {
		/* Do nothing */
	      } else if (diff >= 0) {
		new_flag |= Y_SAME | Y_BYTE;
	      } else {
		ypt[nypts].flipped = 1;
		ypt[nypts].pos = -diff;
		new_flag |= Y_BYTE;
	      }
	      nypts++;
	    } else {
	      new_flag |= Y_SAME;
	    }
	  } else {
	    show_source();
	    fprintf(stderr,
		    "Expected a keyword or a number.\n");
	    got_what(stderr, current);
	    new_flag |= Y_SAME;
	  }
#if 0
	  fprintf(stderr, "Last part of y-coord is ");
	  got_what(stderr, current);
#endif
	  /* Now read in the flags */
	  off_met = 0;
	  contour_end = 0;
	  while (1) {
	    advance_token();
	    if (AN_IDENT == current.form || A_STRING == current.form) {
	      if (!strcmp("on", current.sval)) {
		new_flag |= ON_CURVE;
	      } else if (!strcmp("off", current.sval)) {
		off_met = 1;
	      } else if (!strcmp("end", current.sval)) {
		contour_end = 1;
	      } else {
		show_source();
		fprintf(stderr, "Flags are on, off and end.\n");
		got_what(stderr, current);
	      }
	    } else {
	      show_source();
	      fprintf(stderr, "Expected an identifier.\n");
	      got_what(stderr, current);
	    }
	    advance_token();
	    if (PUNCTUATION != current.form || ',' != current.ival) {
	      break;
	    }
#if 0
	    else {
	      fprintf(stderr, "Sequence of flags!  For joiner ");
	      got_what(stderr, current);
	    }
#endif
	  };
	  if (new_flag & ON_CURVE == off_met) {
	    show_source();
	    fprintf(stderr, "Exactly one of on or off should be present.  "
		    "Assuming on.\n");
	    new_flag |= ON_CURVE;
	  }
	  ipt++;
	  if (NO_CAUSER == xMin.causer_type) {
	    xMin.calculated_value = x;
	    xMin.causerID = ipt;
            xMin.causer_type = BY_POINT;
	    xMax.calculated_value = x;
	    xMax.causerID = ipt;
            xMax.causer_type = BY_POINT;
	    yMin.calculated_value = y;
	    yMin.causerID = ipt;
            yMin.causer_type = BY_POINT;
	    yMax.calculated_value = y;
	    yMax.causerID = ipt;
            yMax.causer_type = BY_POINT;
	  } else {
	    if (x < xMin.calculated_value) {
	      xMin.calculated_value = x;
	      xMin.causerID = ipt;
	      xMin.causer_type = BY_POINT;
	    }
	    if (x > xMax.calculated_value) {
	      xMax.calculated_value = x;
	      xMax.causerID = ipt;
	      xMax.causer_type = BY_POINT;
	    }
	    if (y < yMin.calculated_value) {
	      yMin.calculated_value = y;
	      yMin.causerID = ipt;
	      yMin.causer_type = BY_POINT;
	    }
	    if (y > yMax.calculated_value) {
	      yMax.calculated_value = y;
	      yMax.causerID = ipt;
	      yMax.causer_type = BY_POINT;
	    }
	  }
	  /* Just to get things going, don't generate repeat counts for flags. */ 
	  if (nfpts >= maxpts) extend_points;
	  flg[nfpts++].val = new_flag;
#define extend_contours {\
  max_contours += 32;\
  contour_ends = realloc(contour_ends, max_contours * sizeof(*contour_ends));\
  if (!contour_ends) {\
    fprintf(stderr, "Memory exhausted analysing glyph points.\n");\
    exit(1);\
  }}
	  if (contour_end) {
	    if (n_contours >= max_contours) extend_contours;
	    contour_ends[n_contours++] = ipt;
	  }
	} while (current.form == AN_INT ||
		 (current.form == A_STRING || current.form == AN_IDENT) &&
		 strcmp("end", current.sval));
	if (0 == n_contours || ipt != contour_ends[n_contours-1]) {
	  if (n_contours + 1 >= max_contours) extend_contours;
	  contour_ends[n_contours++] = ipt;
	  show_source();
	  fprintf(stderr, "Last point in glyph should have been marked "
		  "as end of contour!\n");
	}
	advance_token();
	if (AN_IDENT != current.form || strcmp("points", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected 'points'.\n");
	}
	advance_token();
      } else if (!strcmp("component", current.sval)) {
	struct component_entry *the_comp;
	int arg_use_got = 0;
#define FORCE_BYTE 1
#define FORCE_WORD 2
#define UNFORCED   0
	int arg_form = UNFORCED;
#define NO_GLYPH_YET (-1)
#define UNRESOLVED_GLYPH (-2)
#define BAD_GLYPH_ID (-3)
	int glyphID = NO_GLYPH_YET;
#define PLAIN_SCALE 1
#define HAVE_XSCALE 2
#define HAVE_YSCALE 4
#define HAVE_SCALE01 8
#define HAVE_SCALE10 16
	int scales_got = 0;
	int flags = 0;

	if (n_components >= max_components) {
	  max_components += 8;
	  comp = realloc(comp, max_components * sizeof(comp[0]));
	  if (!comp) {
	    show_source();
	    fprintf(stderr, "Memory exhausted.\n");
	    exit(1);
	  }
	}
	if (n_components) comp[n_components-1].flags |= MORE_COMPONENTS;
	the_comp = &comp[n_components++];
	advance_token();
	the_comp->argument1 = 0;
	the_comp->argument2 = 0;
	while (END_OF_FILE != current.form) {
	  if (AN_IDENT != current.form) {
	    show_source();
	    fprintf(stderr, "Expected a keyword.\n");
	    got_what(stderr, current);
	    advance_token();
	  } else if (!strcmp("use", current.sval)) {
	    advance_token();
	    if (AN_INT == current.form) {
	      glyphID = current.ival;
/* There ought to be some checks on the value! */
	    } else if (AN_IDENT == current.form) {
	      struct glyph_group *comp_def;
	      if (lookup_in_map(glyph_names, current.sval, (void**)&comp_def)){
		if (SOLO == comp_def->grouping) {
		  glyphID = comp_def->x.number;
		} else {
		  glyphID = BAD_GLYPH_ID;
		  fprintf(stderr, "%s cannot be a component; it denotes "
			  "a group of glyphs.", current.sval);
		}
	      } else {
/* Unresolved glyph name */
		struct unresolved_component *unres;

		if (n_unresolved_components >= cap_unresolved_components) {
		  cap_unresolved_components += 16;
		  unresolved_components =
		    realloc(unresolved_components,
			    cap_unresolved_components *
			    sizeof(unresolved_components[0]));
		  if (!unresolved_components) {
		    fprintf(stderr, "Memory exhausted.\n");
		    exit(1);
		  }
		}
		unres = &unresolved_components[n_unresolved_components++];
		unres->glyph_name = malloc(1+strlen(current.sval));
		strcpy(unres->glyph_name, current.sval);
		unres->for_glyph = this_glyph;
		unres->component_index = n_components - 1;
		glyphID = UNRESOLVED_GLYPH;
	      }
	    } else {
	      show_source();
	      fprintf(stderr, "Expected an identifier or number.\n");
	      glyphID = BAD_GLYPH_ID;
	    }
	    advance_token();
	  } else if (!strcmp("points", current.sval)) {
	    arg_form = UNFORCED;
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument1 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a point number.\n");
	      the_comp->argument1 = 0;
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument2 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a point number.\n");
	      the_comp->argument2 = 0;
	    }
	    if (the_comp->argument1 >=0 && the_comp->argument1 < 256 &&
		the_comp->argument2 >=0 && the_comp->argument2 < 256) {
	      flags &= ~ARG_1_AND_2_ARE_WORDS;
	    } else {
	      flags |= ARG_1_AND_2_ARE_WORDS;
	    }
	    advance_token();
	    arg_use_got = 1;
	  } else if (!strcmp("word_points", current.sval)) {
	    arg_form = FORCE_WORD;
	    flags |= ARG_1_AND_2_ARE_WORDS;
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument1 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a point number.\n");
	      the_comp->argument1 = 0;
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument2 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a point number.\n");
	      the_comp->argument2 = 0;
	    }
	    advance_token();
	    arg_use_got = 1;
	  } else if (!strcmp("xy", current.sval)) {
	    arg_form = UNFORCED;
	    flags |= ARGS_ARE_XY_VALUES;
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument1 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected an x co-ordinate.\n");
	      the_comp->argument1 = 0;
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument2 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a y co-ordinate.\n");
	      the_comp->argument2 = 0;
	    }
	    if (the_comp->argument1 >=0 && the_comp->argument1 < 128 &&
		the_comp->argument2 >=0 && the_comp->argument2 < 128) {
	      flags &= ~ARG_1_AND_2_ARE_WORDS;
	    } else {
	      flags |= ARG_1_AND_2_ARE_WORDS;
	    }
	    advance_token();
	    arg_use_got = 1;
	  } else if (!strcmp("word_xy", current.sval)) {
	    arg_form = FORCE_WORD;
	    flags |= ARGS_ARE_XY_VALUES | ARG_1_AND_2_ARE_WORDS;
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument1 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected an x co-ordinate.\n");
	      the_comp->argument1 = 0;
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument2 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a y co-ordinate.\n");
	      the_comp->argument2 = 0;
	    }
	    advance_token();
	    arg_use_got = 1;
	  } else if (!strcmp("byte_xy", current.sval)) {
	    arg_form = FORCE_BYTE;
	    flags |= ARGS_ARE_XY_VALUES;
	    flags &= ~ARG_1_AND_2_ARE_WORDS;
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument1 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected an x co-ordinate.\n");
	      the_comp->argument1 = 0;
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->argument2 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a y co-ordinate.\n");
	      the_comp->argument2 = 0;
	    }
/* Trebuchet double guillement and l cedilla show that xy_byte is signed */
#define xy_byte_is_signed 1
#if xy_byte_is_signed
	    if (the_comp->argument1 >= -128 && the_comp->argument1 < 128 &&
		the_comp->argument2 >= -128 && the_comp->argument2 < 128) {
#else
	    if (the_comp->argument1 >=0 && the_comp->argument1 < 128 &&
		the_comp->argument2 >=0 && the_comp->argument2 < 128) {
#endif
	      flags &= ~ARG_1_AND_2_ARE_WORDS;
	    } else {
	      show_source();
	      fprintf(stderr, "byte_xy specified but a value is not within "
#if xy_byte_is_signed
		      "the safe range for byte.\n");
#else
		      "the range for byte.\n");
#endif
	      flags |= ARG_1_AND_2_ARE_WORDS;
	    }
	    advance_token();
	    arg_use_got = 1;
	  } else if (!strcmp("round", current.sval)) {
	    flags |= ROUND_XY_TO_GRID;
	    advance_token();
	  } else if (!strcmp("has_instr", current.sval)) {
	    flags |= WE_HAVE_INSTRUCTIONS;
	    hints_demanded = 1;
	    advance_token();
	  } else if (!strcmp("use_my_metrics", current.sval)) {
	    flags |= USE_MY_METRICS;
	    advance_token();
	  } else if (!strcmp("overlap_compound", current.sval)) {
	    flags |= OVERLAP_COMPOUND;
	    advance_token();
	  } else if (!strcmp("scaled_offset", current.sval)) {
	    flags |= SCALED_COMPONENT_OFFSET;
	    advance_token();
	  } else if (!strcmp("unscaled_offset", current.sval)) {
	    flags |= UNSCALED_COMPONENT_OFFSET;
	    advance_token();
	  } else if (!strcmp("flag_bit", current.sval)) {
	    advance_token();
	    if (AN_INT == current.form) {
	      if (current.ival >= 0 && current.ival <= 15) {
		flags |= (1 << current.ival);
	      } else {
		show_source();
		fprintf(stderr, "%d is not a valid bit number.\n",
			current.ival);
	      }
	    } else {
	      fprintf(stderr, "Expected a bit number.\n");
	    }
	    advance_token();
	  } else if (!strcmp("scale", current.sval)) {
	    scales_got |= PLAIN_SCALE;
	    if (scales_got & ~PLAIN_SCALE) {
	      show_source();
	      fprintf(stderr, "Having a plain scale is inconsistent "
		      "with having other scales.\n");
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->scale = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a numeric scale.\n");
	    }
	    advance_token();
	  } else if (!strcmp("xscale", current.sval)) {
	    scales_got |= HAVE_XSCALE;
	    if (scales_got & PLAIN_SCALE) {
	      show_source();
	      fprintf(stderr, "Having a plain scale is inconsistent "
		      "with having an x-scale.\n");
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->xscale = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a numeric scale.\n");
	    }
	    advance_token();
	  } else if (!strcmp("yscale", current.sval)) {
	    scales_got |= HAVE_YSCALE;
	    if (scales_got & PLAIN_SCALE) {
	      show_source();
	      fprintf(stderr, "Having a plain scale is inconsistent "
		      "with having a y-scale.\n");
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->yscale = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a numeric scale.\n");
	    }
	    advance_token();
	  } else if (!strcmp("scale01", current.sval)) {
	    scales_got |= HAVE_SCALE01;
	    if (scales_got & PLAIN_SCALE) {
	      show_source();
	      fprintf(stderr, "Having a plain scale is inconsistent "
		      "with having a cross-scale.\n");
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->scale01 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a numeric scale.\n");
	    }
	    advance_token();
	  } else if (!strcmp("scale10", current.sval)) {
	    scales_got |= HAVE_SCALE10;
	    if (scales_got & PLAIN_SCALE) {
	      show_source();
	      fprintf(stderr, "Having a plain scale is inconsistent "
		      "with having a cross-scale.\n");
	    }
	    advance_token();
	    if (AN_INT == current.form) {
	      the_comp->scale10 = current.ival;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a numeric scale.\n");
	    }
	    advance_token();
	  } else if (!strcmp("end", current.sval)) {
	    if (NO_GLYPH_YET == glyphID) {
	      show_source();
	      fprintf(stderr, "Component glyph not specified.\n");
	      glyphID = BAD_GLYPH_ID;
	    }
	    the_comp->the_glyphIndex = glyphID;
	    if (scales_got & (HAVE_SCALE01 | HAVE_SCALE10)) {
	      if (!(scales_got & HAVE_XSCALE)) {
		show_source();
		fprintf(stderr, "X-scale is missing.\n");
		the_comp->xscale = 1<<14;
	      }
	      if (!(scales_got & HAVE_YSCALE)) {
		show_source();
		fprintf(stderr, "Y-scale is missing.\n");
		the_comp->yscale = 1<<14;
	      }
	      if (!(scales_got & HAVE_SCALE01)) {
		show_source();
		fprintf(stderr, "Scale01 is missing.\n");
		the_comp->scale01 = 0;
	      }
	      if (!(scales_got & HAVE_SCALE10)) {
		show_source();
		fprintf(stderr, "Scale10 is missing.\n");
		the_comp->scale10 = 0;
	      }
	      flags |= WE_HAVE_A_TWO_BY_TWO;
	    } else if (scales_got & (HAVE_XSCALE | HAVE_YSCALE)) {
	      if (!(scales_got & HAVE_XSCALE)) {
		show_source();
		fprintf(stderr, "X-scale is missing.\n");
		the_comp->xscale = 1<<14;
	      }
	      if (!(scales_got & HAVE_YSCALE)) {
		show_source();
		fprintf(stderr, "Y-scale is missing.\n");
		the_comp->yscale = 1<<14;
	      }
	      flags |= WE_HAVE_AN_X_AND_Y_SCALE;
	    } else if (scales_got & PLAIN_SCALE) {
	      flags |= WE_HAVE_A_SCALE;
	    }
	    the_comp->flags = flags;
	    advance_token();
	    break;
	  } else {
	    show_source();
	    fprintf(stderr, "Component field identifier not recognised.\n");
	    got_what(stderr, current);
	    advance_token();
	  }
	}
	if (AN_IDENT != current.form || strcmp("component", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected keyword 'component'.\n");
	  got_what(stderr, current);
	} else {
	  advance_token();
	}
      } else if (!strcmp("hints", current.sval)) {
	has_hints = 1;
	instr = 0;
	advance_token();
	compile_instr(&instr, &n_instr);
	if (strcmp("hints", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected keyword 'hints'.\n");
	  got_what(stderr, current);
	} else {
	  advance_token();
	}
      } else if (!strcmp("advanceWidth", current.sval)) {
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival < 0 || current.ival >= 1<<16) {
	    show_source();
	    fprintf(stderr, "Advance width %d is not valid, it must be an "
		    "unsigned short integer.\n", current.ival);
	  } else {
	    has_aw = 1;
	    advanceWidth = current.ival;
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected an integer.\n");
	}
	advance_token();
	if (PUNCTUATION == current.form && ',' == current.ival) {
	  advance_token();
	  if (AN_IDENT == current.form) {
	    if (!strcmp("default", current.sval)) {
	      is_default_aw = 1;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected 'default'.\n");
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected an identifier.\n");
	  }
	  advance_token();
	}
      } else if (!strcmp("leftSideBearing", current.sval)) {
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival < -(1<<15) || current.ival >= 1<<15) {
	    show_source();
	    fprintf(stderr, "Left side bearing %d is not valid, it must be an "
		    "signed short integer.\n", current.ival);
	  } else {
	    has_lsb = 1;
	    lsb = current.ival;
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected an integer.\n");
	}
	advance_token();
      } else if (!strcmp("category", current.sval)) {
	int cat_code;
#define CAT_ILLEGAL (-5)
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival < 0 || current.ival > 0xFFFF) {
	    show_source();
	    fprintf(stderr, "Category code %d is invalid and does not fit in "
		    "GDEF format.\n", current.ival);
	    cat_code = CAT_ILLEGAL;
	  } else if (current.ival >= 0xFF) {
	    show_source();
	    fprintf(stderr, "Category code %d is invalid and will be replaced "
		    "by equally invalid value 254.\n", current.ival);
	    cat_code = 0xFE;
	  } else {
	    if (current.ival > CAT_COMPONENT) {
	      show_source();
	      fprintf(stderr, "Category code %d is invalid but will be "
		      "honoured.  Compiler is only obeying orders.\n",
		      current.ival);
	    }
	    cat_code = current.ival;
	  }
	} else if (AN_IDENT == current.form) {
	  void* ptrval;

	  if (lookup_in_map(category_names, current.sval, &ptrval)) {
	    cat_code = (int) ptrval;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid category code.\n", current.sval);
	    cat_code = CAT_ILLEGAL;
	  }
	} else {
	  show_source();
	  fprintf(stdout, "Expected an identifier or a number.\n");
	  cat_code = CAT_ILLEGAL;
	}
	switch (cat_code) {
	case CAT_ILLEGAL:
	  /* Do nothing. */
	  break;
	case CAT_DEFAULTMARK:
	  glyph_spp[this_glyph].category = CAT_MARK;
	  cat_code = CAT_DEFAULT; /* Anything legal but CAT_MARK */
	  break;
	case CAT_PSEUDOMARK:
	  cat_code = CAT_MARK; /* To add mark attachment class. */
	  break;
	default:
	  glyph_spp[this_glyph].category = cat_code;
	}
	if (CAT_MARK == cat_code) {
	  advance_token();
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival > 0xFFFF) {
	      show_source();
	      fprintf(stderr, "%d is not a valid unsigned 16-bit integer.\n",
		      current.ival);
	    } else if (current.ival >= NO_MAC) {
	      fprintf(stderr, "Only %d mark attachment classes are "
		      "supported.  Forcing to class %d.\n",
		      NO_MAC, NO_MAC - 1);
	      glyph_spp[this_glyph].attachment_class = NO_MAC - 1;
	    } else {
	      glyph_spp[this_glyph].attachment_class = current.ival;
	    }
	  } else if (A_STRING == current.form || AN_IDENT == current.form) {
	    struct class_data *cldat, *cldat2;
	    struct glyph_group *gg;

	    locate_class(current.sval, attachment_classes2, &cldat2);
	    if (cldat2) {
	      char name[12];
	      if (cldat2->number >= NO_MAC) {
		fprintf(stderr, "Too many mark attachment class names.\n");
		cldat2->number = NO_MAC -1;
	      }
	      sprintf(name, "MAC %d", cldat2->number);
	      locate_class(name, attachment_classes, &cldat);
	    } else {
	      cldat = 0;
	    }
	    if (cldat) {
	      glyph_spp[this_glyph].attachment_class =
		cldat->number = cldat2->number;
	      lookup_in_map(glyph_names, current.sval, (void**)&gg);
	      assert(CLASS == gg->grouping);
	      gg->x.class_data = cldat;
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a number or an identifier.\n");
	  }
	}
	advance_token();
      } else if (!strcmp("colour", current.sval)) {
        if (colr.maxBaseGlyphRecords <= colr.numBaseGlyphRecords) {
          colr.maxBaseGlyphRecords += 10 + colr.maxBaseGlyphRecords / 2;
          colr.baseGlyphRecord =
            realloc(colr.baseGlyphRecord,
                    colr.maxBaseGlyphRecords * sizeof * colr.baseGlyphRecord);
          if (!colr.baseGlyphRecord) {
            fprintf(stderr, "Memory exhausted analysing coloured components "
                    "of glyph.\n");
            exit(1);
          }
        }
        colr.baseGlyphRecord[colr.numBaseGlyphRecords].GID = this_glyph;
        colr.baseGlyphRecord[colr.numBaseGlyphRecords].firstLayerIndex =
          colr.numLayerRecords;
        colr.baseGlyphRecord[colr.numBaseGlyphRecords].numLayers = 0;
        advance_token();
        while (AN_IDENT == current.form || A_STRING == current.form) {
          int glyphID = 0, col = 0xFFFF;
          struct glyph_group *comp_def;
          if (!strcmp("end", current.sval) && AN_IDENT == current.form) break;
          if (lookup_in_map(glyph_names, current.sval, (void**)&comp_def)){
            if (SOLO == comp_def->grouping) {
              glyphID = comp_def->x.number;
            } else {
              glyphID = 0;
              fprintf(stderr, "%s cannot be a component; it denotes "
                      "a group of glyphs.", current.sval);
            }
          } else {
/* Unresolved glyph name */
            struct unresolved_component *unres;

            if (n_unresolved_components >= cap_unresolved_components) {
              cap_unresolved_components += 16;
              unresolved_components =
                realloc(unresolved_components,
                        cap_unresolved_components *
                        sizeof(unresolved_components[0]));
              if (!unresolved_components) {
                fprintf(stderr, "Memory exhausted.\n");
                exit(1);
              }
            }
            unres = &unresolved_components[n_unresolved_components++];
            unres->glyph_name = malloc(1+strlen(current.sval));
            strcpy(unres->glyph_name, current.sval);
            unres->for_glyph = 0x10000 + this_glyph;
            unres->component_index = colr.numLayerRecords;
            glyphID = 0;
          }
          advance_token();
          if (AN_IDENT == current.form) {
            void *ptrval;
            if (p0c0 < 0) {
              show_source();
              fprintf(stderr, "Palette indices are not yet defined!\n");
            } else if (lookup_in_map(colours, current.sval, &ptrval)) {
              col = (int) ptrval - p0c0;
            } else {
              show_source();
              fprintf(stderr, "%s is not a palette index.\n", current.sval);
            }
            advance_token();
          } else if (AN_INT == current.form) {
            col = current.ival;
            advance_token;
          }
          if (colr.maxLayerRecords <= colr.numLayerRecords) {
            colr.maxLayerRecords += 10 + colr.maxLayerRecords / 2;
            colr.layerRecord =
              realloc(colr.layerRecord,
                      colr.maxLayerRecords * sizeof * colr.layerRecord);
            if (!colr.layerRecord) {
              fprintf(stderr, "Memory exhausted analysing coloured components "
                      "of glyph.\n");
              exit(1);
            }
          }
          colr.layerRecord[colr.numLayerRecords].GID = glyphID;
          colr.layerRecord[colr.numLayerRecords].paletteIndex = col;
          colr.numLayerRecords++;
          colr.baseGlyphRecord[colr.numBaseGlyphRecords].numLayers++;
        }
        colr.numBaseGlyphRecords++;
        have_COLR = 1;
        if (AN_IDENT != current.form || strcmp("end", current.sval)) {
          show_source();
          fprintf(stderr, "Expected keyword 'end'.\n");
          got_what(stderr, current);
        } else {
          advance_token();
          if (AN_IDENT != current.form || strcmp("colour", current.sval)) {
            show_source();
            fprintf(stderr, "Expected keyword 'colour'.\n");
            got_what(stderr, current);
          } else {
            advance_token();
          }
        }
      } else if (!strcmp("end", current.sval)) {
	break;
      } else {
	show_source();
	fprintf(stderr, "%s is not a glyph keyword.\n", current.sval);
	got_what(stderr, current);
	advance_token();
      }
    }
  }
  if (END_OF_FILE != current.form) {
    char item[300];
    int ipt;

    advance_token();
    glyph_spp[last_glyph].location = glyphs_size;
    if (AN_IDENT != current.form || strcmp("glyph", current.sval)) {
      show_source();
      fprintf(stderr, "Expected 'glyph'.\n");
    }
    advance_token();
    /* Write glyph to scratch file */
    if (n_contours > 0) {
      if (n_components > 0) {
	show_source();
	fprintf(stderr, "Glyph has both points and components.  "
		"Ignoring components.\n");
      }
      the_glyph.numberOfContours = n_contours;
#define establish(val) \
	if (val.is_forced) {\
	  if (postscript_name[last_glyph]) { \
	    sprintf(item, "Glyph %d (postscript name %s) " #val, \
		    last_glyph, postscript_name[last_glyph]);	 \
	  } else {					 \
	    sprintf(item, "Glyph %d " #val, last_glyph); \
	  }						 \
	  discrepancy(item, val);\
	  the_glyph.val = val.forced_value;\
	} else {\
	  the_glyph.val = val.calculated_value;\
	}
      establish(xMin);
      establish(xMax);
      establish(yMin);
      establish(yMax);
      the_glyph.endPtsOfContours = contour_ends;
      if (has_hints) {
	the_glyph.instructionLength = n_instr;
	the_glyph.instructions = instr;
      } else {
	the_glyph.instructionLength = 0;
	the_glyph.instructions = 0;
      }
      /* Worry about condensing flags later, if ever. */
      for (ipt = 0; ipt < nfpts - 1; ipt++) {
	flg[ipt]._next = &flg[ipt+1];
      }
      flg[nfpts-1]._next = 0;
      the_glyph.flags = flg;
      for (ipt = 0; ipt < nxpts - 1; ipt++) {
	xpt[ipt]._next = &xpt[ipt+1];
      }
      xpt[nxpts-1]._next = 0;
      the_glyph.xCoordinates = xpt;
      for (ipt = 0; ipt < nypts - 1; ipt++) {
	ypt[ipt]._next = &ypt[ipt+1];
      }
      ypt[nypts-1]._next = 0;
      the_glyph.yCoordinates = ypt;
      put_bine_glyph_entry(glyphs, &the_glyph, &junk, &glyphs_size);
    } else if (n_components > 0) {
      the_glyph.numberOfContours = -1;
      if (xMin.is_forced) {
	the_glyph.xMin = xMin.forced_value;
      } else {
	the_glyph.xMin = 0;
      }
      if (xMax.is_forced) {
	the_glyph.xMax = xMax.forced_value;
      } else {
	  the_glyph.xMax = 1000; /* Should use size of em square. */
      }
      if (yMin.is_forced) {
	the_glyph.yMin = yMin.forced_value;
      } else {
	the_glyph.yMin = 0;
      }
      if (yMax.is_forced) {
	the_glyph.yMax = yMax.forced_value;
      } else {
	the_glyph.yMax = 1000; /* Should use size of em square. */
      }
      for (ipt = 0; ipt < n_components - 1; ipt++) {
	comp[ipt]._next = &comp[ipt+1];
      }
      comp[n_components-1]._next = 0;
      the_glyph.components = comp;
      if (has_hints) {
	the_glyph.numInstr = n_instr;
	the_glyph.instr = instr;
	comp[n_components-1].flags |= WE_HAVE_INSTRUCTIONS;
      } else {
	if (hints_demanded) {
	  show_source();
	  fprintf(stderr, "A component definition says glyph ");
	  put_gid_ps(stderr, last_glyph);
	  fprintf(stderr, " has instructions, but none were provided.\n");
	} 
	the_glyph.numInstr = 0;
	the_glyph.instr = 0;
	comp[n_components-1].flags |= WE_HAVE_INSTRUCTIONS;
      }
#if 0
      print_glyph_entry(stderr, &the_glyph, &junk, "", 0);
#endif
      put_bine_glyph_entry(glyphs, &the_glyph, &junk, &glyphs_size);
    } else { /* Blank glyph */
      /* Nothing to do */
    }
    align_file(glyphs, &glyphs_size, 2);
    /* Resolve metrics */
    if (!has_lsb) lsb = 0;
    glyph_spp[last_glyph].leftSideBearing = lsb;
    if (has_aw) {
      if (is_default_aw) {
	hhea_numberofHMetrics = last_glyph + 1;
	default_aw = advanceWidth;
      } else if (hhea_numberofHMetrics) {
	hhea_numberofHMetrics = last_glyph + 1;
      }
      glyph_spp[last_glyph].advanceWidth = advanceWidth;
      rsb = advanceWidth - (lsb + (int)the_glyph.xMax - the_glyph.xMin);
    } else if (hhea_numberofHMetrics) {
      advanceWidth = default_aw;
      rsb = advanceWidth - (lsb + (int)the_glyph.xMax - the_glyph.xMin);
    } else {
      rsb = 0;
      advanceWidth = lsb + (int)the_glyph.xMax - the_glyph.xMin;
    }
    glyph_spp[last_glyph].advanceWidth = advanceWidth;
    if (n_contours || n_components) {
      const int xExtent = lsb + the_glyph.xMax - the_glyph.xMin;
      if (NO_CAUSER == advanceWidthMax.causer_type) {
	advanceWidthMax.calculated_value = advanceWidth;
	advanceWidthMax.causerID = last_glyph;
	advanceWidthMax.causer_type = BY_GLYPH;
	minLeftSideBearing.calculated_value = lsb;
	minLeftSideBearing.causerID = last_glyph;
	minLeftSideBearing.causer_type = BY_GLYPH;
	minRightSideBearing.calculated_value = rsb;
	minRightSideBearing.causerID = last_glyph;
	minRightSideBearing.causer_type = BY_GLYPH;
	xMaxExtent.calculated_value = xExtent;
	xMaxExtent.causerID = last_glyph;
	xMaxExtent.causer_type = BY_GLYPH;
	head_xMin.calculated_value = the_glyph.xMin;
	head_xMin.causerID = last_glyph;
	head_xMin.causer_type = BY_GLYPH;
	head_xMax.calculated_value = the_glyph.xMax;
	head_xMax.causerID = last_glyph;
	head_xMax.causer_type = BY_GLYPH;
	head_yMin.calculated_value = the_glyph.yMin;
	head_yMin.causerID = last_glyph;
	head_yMin.causer_type = BY_GLYPH;
	head_yMax.calculated_value = the_glyph.yMax;
	head_yMax.causerID = last_glyph;
	head_yMax.causer_type = BY_GLYPH;
      } else {
	if (advanceWidth > advanceWidthMax.calculated_value) {
	  advanceWidthMax.calculated_value = advanceWidth;
	  advanceWidthMax.causerID = last_glyph;
	  advanceWidthMax.causer_type = BY_GLYPH;
	}
	if (lsb < minLeftSideBearing.calculated_value) {
	  minLeftSideBearing.calculated_value = lsb;
	  minLeftSideBearing.causerID = last_glyph;
	  minLeftSideBearing.causer_type = BY_GLYPH;
	}
	if (rsb < minRightSideBearing.calculated_value) {
	  minRightSideBearing.calculated_value = rsb;
	  minRightSideBearing.causerID = last_glyph;
	  minRightSideBearing.causer_type = BY_GLYPH;
	}
	if (xExtent > xMaxExtent.calculated_value) {
	  xMaxExtent.calculated_value = xExtent;
	  xMaxExtent.causerID = last_glyph;
	  xMaxExtent.causer_type = BY_GLYPH;
	}
	if (the_glyph.xMin < head_xMin.calculated_value) {
	  head_xMin.calculated_value = the_glyph.xMin;
	  head_xMin.causerID = last_glyph;
	  head_xMin.causer_type = BY_GLYPH;
	}
	if (the_glyph.xMax > head_xMax.calculated_value) {
	  head_xMax.calculated_value = the_glyph.xMax;
	  head_xMax.causerID = last_glyph;
	  head_xMax.causer_type = BY_GLYPH;
	}
	if (the_glyph.yMin < head_yMin.calculated_value) {
	  head_yMin.calculated_value = the_glyph.yMin;
	  head_yMin.causerID = last_glyph;
	  head_yMin.causer_type = BY_GLYPH;
	}
	if (the_glyph.yMax > head_yMax.calculated_value) {
	  head_yMax.calculated_value = the_glyph.yMax;
	  head_yMax.causerID = last_glyph;
	  head_yMax.causer_type = BY_GLYPH;
	}
      }
      if (advanceWidth != 0) {
	totCharWidth += advanceWidth;
	nCharWidth++;
	if (isFixedPitch.calculated_value > 0) {
	  if (advanceWidth != isFixedPitch.calculated_value) {
	    isFixedPitch.calculated_value = 0;
	    isFixedPitch.causer_type = BY_GLYPH;
	    isFixedPitch.causerID = this_glyph;
	  }
	} else if (isFixedPitch.calculated_value < 0) {
	  if (advanceWidth > 0) {
	    isFixedPitch.calculated_value = advanceWidth;
	  } else {
	    isFixedPitch.calculated_value = 0;
	    isFixedPitch.causer_type = BY_GLYPH;
	    isFixedPitch.causerID = this_glyph;
	  }
	}
      }
      if (the_glyph.xMin < lsb) {
        const char *gname = get_glyph_name(this_glyph);
	show_source();
        if (strncmp(gname, "glyph ", 6)) {
          fputs(gname, stderr);
        } else {
          fprintf(stderr, "Glyph %d", this_glyph);
        }
	fprintf(stderr, " has xMin = %d but leftSideBearing = %d\n",
		the_glyph.xMin, lsb);
      }
    }
  }
  if (glyph_spp[this_glyph].attachment_class != NO_MAC) {
    int macno = glyph_spp[this_glyph].attachment_class;
    char name[20];
    struct class_data *cldat;

    sprintf(name, "MAC %d", macno);
    locate_class(name, attachment_classes, &cldat);
    if (cldat) {
      cldat->number = macno;
      add_range(this_glyph, this_glyph, &cldat->members);
    }
    have_GDEF = 1;
  } else if (glyph_spp[this_glyph].category != NO_CATEGORY) {
    have_GDEF = 1;
  }
}

const char *get_glyph_name(unsigned short int gi);
void resolve() {
  int iuc, glyphID, junk;
  int pos;
  struct glyph_group *comp_def;
  struct glyph_entry the_glyph; 
  struct component_entry *comp;

  /* Resolution of unresolved components. */
  
  fflush(glyphs);
  for (iuc = 0; iuc < n_unresolved_components; iuc++) {
    if (lookup_in_map(glyph_names, unresolved_components[iuc].glyph_name,
		      (void**)&comp_def)){
      if (SOLO == comp_def->grouping) {
	glyphID = comp_def->x.number;
        if (unresolved_components[iuc].for_glyph <= 0xFFFF) {
          memset(&the_glyph, 0, sizeof(the_glyph));
          pos = glyph_spp[unresolved_components[iuc].for_glyph].location;
          if (fseek(glyphs, pos, SEEK_SET)) {
            perror("Seeking failed in glyph scratch file");
            exit(1);
          }
          get_bine_glyph_entry(glyphs, &the_glyph, &junk, &pos);
          comp = element(the_glyph.components,
                         unresolved_components[iuc].component_index);
#if 0
          fprintf(stderr, "For glyph %d component %d, replacing component "
                  "glyph %d by %d.\n", unresolved_components[iuc].for_glyph,
                  unresolved_components[iuc].component_index,
                  comp->the_glyphIndex, glyphID);
#endif
          comp->the_glyphIndex = glyphID;
          pos = glyph_spp[unresolved_components[iuc].for_glyph].location;
          if (fseek(glyphs, pos, SEEK_SET)) {
            perror("Seeking failed in glyph scratch file");
            exit(1);
          }
          put_bine_glyph_entry(glyphs, &the_glyph, &junk, &pos);
          free_glyph_entry(&the_glyph);
        } else {
          colr.layerRecord[unresolved_components[iuc].component_index].GID =
            glyphID;
        }
      } else {
	glyphID = BAD_GLYPH_ID;
	fprintf(stderr, "%s cannot be a glyph component; it denotes "
		"a group of glyphs.", unresolved_components[iuc].glyph_name);
      }
    } else {
      fprintf(stderr, "Glyph component %s of %s is not defined.\n",
	      unresolved_components[iuc].glyph_name,
              get_glyph_name(unresolved_components[iuc].for_glyph % 0x10000));
    }
  }
  fflush(glyphs);
}

void process_name() {
/* Current token is name on input, and is immediately after definition on exit. */
  int nameID, platformID, encodingID, languageID;
  int form; /* 0 normal, 1 bad, 2 auto, 3 bad auto */
  void *encoded;
  char key[17], *value;

  if (strcmp("name", current.sval)) {
    fprintf(stderr, "process_name: internal error - token is not 'name'.\n");
    exit(1);
  }
  form = 0;
  /* Read nameID */
  advance_token();
  if (AN_IDENT == current.form) {
    if (lookup_in_map(nameID_codes, current.sval, &encoded)) {
      nameID = (int) encoded;
    } else {
      show_source();
      fprintf(stderr, "%s is not a valid name ID.\n", current.sval);
      form = 1;
    }
  } else if (AN_INT == current.form) {
    if (current.ival < 0 || current.ival > 0xffff) {
      show_source();
      fprintf(stderr, "%d is not a valid name ID.\n", current.ival);
      form = 1;
    } else {
      nameID = current.ival;
    }
  } else {
    show_source();
    fprintf(stderr, "Need number or special ID for name ID.\n");
    if (END_OF_FILE == current.form) return;
    form = 1;
  }
  /* Read platform ID */
  advance_token();
  if (AN_IDENT == current.form) {
    if (!strcmp("auto", current.sval)) {
      form += 2;
    } else if (lookup_in_map(platID_codes, current.sval, &encoded)) {
      platformID = (int) encoded;
    } else {
      show_source();
      fprintf(stderr, "%s is not a valid platform ID.\n", current.sval);
      form = 1;
    }
  } else if (AN_INT == current.form) {
    if (current.ival <0 || current.ival > 0xffff) {
      show_source();
      fprintf(stderr, "%4.4X is not a valid platform ID.\n", current.ival);
      form = 1;
    } else {
      platformID = current.ival;
    }
  } else {
    show_source();
    fprintf(stderr, "Need 'auto', number or special ID for platform ID.\n");
    if (END_OF_FILE == current.form) return;
    form = 1;
  }
  if (form < 2) {
    /* Read encoding */
    advance_token();
    if (AN_IDENT == current.form) {
      /* Ignore value if an error has occurred! */
      if (0 == form) {
	switch(platformID) {
	case 0:
	  if (lookup_in_map(unicode_encodings, current.sval, &encoded)) {
	    encodingID = (int)encoded;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid encoding ID,\n", current.sval);
	    form = 1;
	  }
	  break;
	case 1:
	  if (lookup_in_map(mac_encodings, current.sval, &encoded)) {
	    encodingID = (int)encoded;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid encoding ID,\n", current.sval);
	    form = 1;
	  }
	  break;
	case 3:
	  if (lookup_in_map(ms_encodings, current.sval, &encoded)) {
	    encodingID = (int)encoded;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid encoding ID,\n", current.sval);
	    form = 1;
	  }
	  break;
	default:
	  show_source();
	  fprintf(stderr, "Encoding %s cannot be interpreted, for there are "
		  "no named encodings for platformID %d.\n",
		  current.sval, platformID);
          form = 1;
          encodingID = 0; /* Minimise damage */
	  break;
	}
      }
    } else if (AN_INT == current.form) {
      if (current.ival <0 || current.ival > 0xffff) {
	show_source();
	fprintf(stderr, "%4.4X is not a valid encoding ID.\n", current.ival);
	form = 1;
        encodingID = 0; /* Minimise damage */
      } else {
	encodingID = current.ival;
      }
    } else {
      show_source();
      fprintf(stderr, "Need number or special ID for encoding ID.\n");
      if (END_OF_FILE == current.form) return;
      form = 1;
    }
    /* Read language */
    advance_token();
    if (AN_IDENT == current.form) {
      /* Ignore value if an error has occurred! */
      if (0 == form) {
	switch(platformID) {
	case 1:
	  if (lookup_in_map(mac_languages, current.sval, &encoded)) {
	    languageID = (int)encoded;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid encoding ID,\n", current.sval);
	    form = 1;
            languageID = 0; /* Minimise? damage */
	  }
	  break;
	case 3:
	  if (lookup_in_map(ms_languages, current.sval, &encoded)) {
	    languageID = (int)encoded;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid encoding ID,\n", current.sval);
	    form = 1;
            languageID = 0; /* Minimise? damage - or choose English?*/
	  }
	  break;
	default:
	  show_source();
	  fprintf(stderr, "Encoding %s cannot be interpreted, for there are "
		  "no named languages for platformID %d.\n",
		  current.sval, platformID);
          form = 1;
          languageID = 0;
	  break;
	}
      }
    } else if (AN_INT == current.form) {
      if (current.ival <-1 || current.ival > 0xffff) {
	show_source();
	fprintf(stderr, "%4.4X is not a valid language ID.\n", current.ival);
	form = 1;
      } else {
	languageID = current.ival & 0xffff;
      }
    } else {
      show_source();
      fprintf(stderr, "Need number or special ID for language ID.\n");
      if (END_OF_FILE == current.form) return;
      form = 1;
    }
  }
  advance_token();
  if (A_STRING == current.form) {
    if (0 == form) {
      sprintf(key, "%4x%4x%4x%4x", platformID, encodingID, languageID,
	      nameID);
      value = malloc(strlen(current.sval) + 9);
      if (!value) {
	fprintf(stderr, "Memory exhausted!\n");
      } else {
	strcpy(value, current.sval);
	replace_in_map(&name_names, key, value, 1);
      }
    } else if (2 == form) {
      sprintf(key, "%4x%4x%4x%4x", 1, 0, 0, nameID);
      value = malloc(strlen(current.sval) + 9);
      if (!value) {
	fprintf(stderr, "Memory exhausted!\n");
      } else {
	strcpy(value, current.sval);
	replace_in_map(&name_names, key, value, 1);
      }
      sprintf(key, "%4x%4x%4x%4x", 3, 1, 0x0409, nameID);
      value = malloc(strlen(current.sval) + 9);
      if (!value) {
	fprintf(stderr, "Memory exhausted!\n");
      } else {
	strcpy(value, current.sval);
	replace_in_map(&name_names, key, value, 1);
      }
    }
    advance_token();
#if 0
    fprintf(stderr, "process_name: Encoding incomplete!");
    exit(1);
#endif
  } else {
    form = 1;
    show_source();
    fprintf(stderr, "Expected string for name.\n");
  }
  /*
name nameID platformID encodingID languageID "text"

or 

name nameID auto "text"
  */
}
    
static void get_short(char *name, short int *value) {
/* At start current token is the name; on return, inmmediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  } else {
    if (current.ival > 0x7fff || current.ival < -0x8000) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "16-bit signed number.\n", name, current.ival);
    } else {
      *value = current.ival;
    }
  }
  advance_token();
}

static void get_ushort(char *name, unsigned short int *value) {
/* At start current token is the name; on return, inmmediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a number.");
  } else {
    if (current.ival > 0xffff || current.ival < 0) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "16-bit unsigned number.\n", name, current.ival);
    } else {
      *value = current.ival;
    }
  }
  advance_token();
}

static void get_long(char *name, int *value) {
/* At start current token is the name; on return, immediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  } else {
    *value = current.ival;
  }
  advance_token();
}

static void get_ulong(char *name, unsigned int *value) {
/* At start current token is the name; on return, inmmediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a number.");
  } else {
    if (current.ival > 0xffffffff || current.ival < 0) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "32-bit unsigned number.\n", name, current.ival);
    } else {
      *value = current.ival;
    }
  }
  advance_token();
}

static void get_double(char *name, double *value) {
/* At start current token is the name; on return, immediately after value. */
  advance_token();
  if (AN_INT == current.form) {
    *value = current.ival;
  } else if (A_REAL == current.form) {
    *value = current.rval;
  } else {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  }
  advance_token();
}

static void force_short(char *name, struct calculable_item *value) {
/* At start current token is the name; on return, immediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  } else {
    if (current.ival > 0x7fff || current.ival < -0x8000) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "16-bit signed number.\n", name, current.ival);
    } else {
      value->is_forced = 1;
      value->forced_value = current.ival;
    }
  }
  advance_token();
}

static void force_ushort(char *name, struct calculable_item *value) {
/* At start current token is the name; on return, immediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  } else {
    if (current.ival > 0xffff || current.ival < 0) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "16-bit unsigned number.\n", name, current.ival);
    } else {
      value->is_forced = 1;
      value->forced_value = current.ival;
    }
  }
  advance_token();
}

static void force_ulong(char *name, struct calculable_item *value) {
/* At start current token is the name; on return, immediately after value. */
  advance_token();
  if (AN_INT != current.form) {
    show_source();
    fprintf(stderr, "Expected a dimension.");
  } else {
    if (current.ival > 0xFFFFFFFF || current.ival < 0) {
      show_source(); 
      fprintf(stderr, "Value of %s viz %d is not a " 
		      "32-bit unsigned number.\n", name, current.ival);
    } else {
      value->is_forced = 1;
      value->forced_value = current.ival;
    }
  }
  advance_token();
}

static void do_parsing(void) {
  /* Set up analysis */
  init_analysis();
  
  /* Parse to end of file */
  while (END_OF_FILE != current.form) {
    if (AN_IDENT == current.form) {
/* For glyf & hmtx tables.  Incidentals for others. */
      if (!strcmp("glyph", current.sval)) {
	process_glyph();
/* For name table */
      } else if (!strcmp("name", current.sval)) {
	process_name();
/* For hhea table */
      } else if (!strcmp("ascender", current.sval)) {
	get_short("ascender", &ascender);
      } else if (!strcmp("descender", current.sval)) {
	get_short("descender", &descender);
      } else if (!strcmp("lineGap", current.sval)) {
	get_short("lineGap", &lineGap);
      } else if (!strcmp("advanceWidthMax", current.sval)) {
	force_short("advanceWidthMax", &advanceWidthMax);
      } else if (!strcmp("minLeftSideBearing", current.sval)) {
	force_short("minLeftSideBearing", &minLeftSideBearing);
      } else if (!strcmp("minRightSideBearing", current.sval)) {
	force_short("minRightSideBearing", &minRightSideBearing);
      } else if (!strcmp("xMaxExtent", current.sval)) {
	force_short("xMaxExtent", &xMaxExtent);
      } else if (!strcmp("caretSlopeRise", current.sval)) {
	get_short("caretSlopeRise", &caretSlopeRise);
      } else if (!strcmp("caretSlopeRun", current.sval)) {
	get_short("caretSlopeRun", &caretSlopeRun);
      } else if (!strcmp("caretOffset", current.sval)) {
	get_short("caretOffset", &caretOffset);
/* Calculable items for the head table */
      } else if (!strcmp("xMin", current.sval)) {
	force_short("xMin", &head_xMin);
      } else if (!strcmp("xMax", current.sval)) {
	force_short("xMax", &head_xMax);
      } else if (!strcmp("yMin", current.sval)) {
	force_short("yMin", &head_yMin);
      } else if (!strcmp("yMax", current.sval)) {
	force_short("yMax", &head_yMax);
      } else if (!strcmp("indexToLocFormat", current.sval)) {
	get_short("indexToLocFormat", &indexToLocFormat);
        indexToLocFormat_forced = 1;
/*  Partly calculable items for head table */
      } else if (!strcmp("fontRevision", current.sval)) {
	advance_token();
	if (AN_INT == current.form) {
	  fontRevision = current.ival;
	} else if (AN_IDENT == current.form && !strcmp(current.sval, "auto")) {
	  char v_name_key[17], *v_name;
	  sprintf(v_name_key, "%4x%4x%4x%4x", platID_Macintosh, 0, 0, 5);
	  if (lookup_in_map(name_names, v_name_key, (void**)&v_name)) {
	    /* So far, so good */
	  } else {
	    sprintf(v_name_key, "%4x%4x%4x%4x", platID_Microsoft, 0, 0, 5);
	    if (lookup_in_map(name_names, v_name_key, (void**)&v_name)) {
	      fprintf(stderr, "No Macintosh version 'name'.  Using Microsoft "
		      "version name.\n");
	    } else {
	      v_name = 0;
	      fprintf(stderr, "No Macintosh or Microsoft version name.\n");
	    }
	  }
	  if (v_name) {
	    char *loc = v_name;
	    float fvno;
	    
	    while (loc) {
	      loc = strpbrk(loc, "Vv");
	      if (loc) {
		if (strcasecmp(loc, "Version ")) {
		  if (1 == sscanf(loc+8, "%f", &fvno)) {
		    fontRevision = fvno * (1<<16);
		    break;
		  } else {
		    fprintf(stderr, "Bad version number in version "
			    "name <%s>.\n", v_name);
		    break;
		  }
		} else {
		  loc++; /* Look for a later 'v'. */
		}
	      } else {
		break; /* No version number */
	      }
	    }
	    if (!loc) {
	      fprintf(stderr, "No version number in version "
		      "name <%s>.\n", v_name);
	    }
	  }
	} else if (A_REAL == current.form) {
	  fontRevision = current.rval * (1<<16);
	} else {
	  show_source();
	  fprintf(stderr, "Expected a version number or 'auto'.");
	}
	advance_token();
      } else if (!strcmp("head_flags", current.sval)) {
	advance_token();
	while (1) {
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 16) {
	      show_source();
	      fprintf(stderr, "There is no bit %d in the 'flags' "
		      "of the head table.\n", current.ival);
	    } else {
	      switch (current.ival) {
	      case 1:
		lsbx0.forced_value = 1;
		lsbx0.is_forced = 1;
		break;
	      case 2:
		ins_use_PPEM.forced_value = 1;
		ins_use_PPEM.is_forced = 1;
		break;
	      case 4:
		aw_by_PPEM.forced_value = 1;
		aw_by_PPEM.is_forced = 1;
		break;
	      case 9:
		fDH_r2l.forced_value = 1;
		fDH_r2l.is_forced = 1;
		break;
	      default:
		head_flags |= (1 << current.ival); 
	      }
	    }
	  } else if (AN_IDENT == current.form) {
	    if (!strcmp("none", current.sval)) {
	      head_flags = 0;
	      lsbx0.forced_value = 0;
	      lsbx0.is_forced = 1;
	      ins_use_PPEM.forced_value = 0;
	      ins_use_PPEM.is_forced = 1;
	      aw_by_PPEM.forced_value = 0;
	      aw_by_PPEM.is_forced = 1;
	      fDH_r2l.forced_value = 0;
	      fDH_r2l.is_forced = 1;
	    } else if (!strcmp("auto", current.sval)) { /* Default */
/* Bits 1, 2, 4 & 9 are to be calculated. */
	      lsbx0.is_forced = 0;
	      ins_use_PPEM.is_forced = 0;
	      aw_by_PPEM.is_forced = 0;
	      fDH_r2l.is_forced = 0;
	    } else if (!strcmp("horizontal_baseline_0", current.sval)) {
/* Bit 0: Baseline for font at y=0; */
	      head_flags |= 1;
	    } else if (!strcmp("lsb_is_xmin", current.sval)) {
/* Bit 1: Left sidebearing point at x=0 */
	      lsbx0.forced_value = 1;
	      lsbx0.is_forced = 1;
	    } else if (!strcmp("PPEM_dependent", current.sval) ||
                       !strcmp("ins_use_PPEM", current.sval)) {
/* Bit 2: Instructions may depend on point size */
	      ins_use_PPEM.forced_value = 1;
	      ins_use_PPEM.is_forced = 1;
	    } else if (!strcmp("integer_PPEM", current.sval)) {
/* Bit 3: Force ppem to integer values for all internal scaler math;
          may use fractional ppem sizes if this bit is clear; */
	      head_flags |= (1<<3);
	    } else if (!strcmp("aw_by_PPEM", current.sval)) {
/* Bit 4: Instructions may alter advance width (the advance widths
          might not scale linearly) */ 
	      aw_by_PPEM.forced_value = 1;
	      aw_by_PPEM.is_forced = 1;
	    } else if (!strcmp("vertical_baseline_0", current.sval)) {
/* Bit 5: Vertical baseline for font is at x = 0 */
	      head_flags |= (1<<5);
	    } else if (!strcmp("needs_layout", current.sval)) {
/* Bit 7: Font needs layout for linguistically correct rendering */
	      head_flags |= (1<<7);
	    } else if (!strcmp("metamorphosis", current.sval)) {
/* Bit 8: GX font in which metamorphosis happens by default. */
	      head_flags |= (1<<8);
	    } else if (!strcmp("right2left", current.sval)) {
/* Bit 9: Right to left processing required */
	      fDH_r2l.forced_value = 1;
	      fDH_r2l.is_forced = 1;
	    } else if (!strcmp("Indic", current.sval)) {
/* Bit 10: Indic-style rearrangement */
	      head_flags |= (1<<10);
	    } else if (!strcmp("Agfa", current.sval)) {
/* Bit 11: Font data is 'lossless,' as a result of having been compressed
           and decompressed with the Agfa MicroType Express engine. */
	      fprintf(stderr, "Agfa compression bit presumptiously set.\n");
	      head_flags |= (1<<11);
	    } else if (!strcmp("converted", current.sval)) {
/* Bit 12: Font converted (produce compatible metrics) */
	      head_flags |= (1<<12);
	    } else if (!strcmp("ClearType", current.sval)) {
/* Bit 13: Font optimised for ClearType */
	      head_flags |= (1<<13);
	    } else {
	      show_source();
	      fprintf(stderr, "%s does not identify any of flags"
		      "in the head table.\n", current.sval);
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a bit identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	  if (END_OF_FILE == current.form) break;
	}
/*  Non-calculable items for head table */
      } else if (!strcmp("unitsPerEm", current.sval)) {
	get_ushort("unitsPerEm", &unitsPerEm);
      } else if (!strcmp("lowestRecPPEM", current.sval)) {
	get_ushort("lowestRecPPEM", &lowestRecPPEM);
      } else if (!strcmp("macStyle", current.sval)) {
	advance_token();
	while (1) {
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 16) {
	      show_source();
	      fprintf(stderr, "There is no bit %d macStyle.\n", current.ival);
	    } else {
	      head_flags |= (1 << current.ival); 
	    }
	  } else if (AN_IDENT == current.form) {
	    if (!strcmp("none", current.sval)) {
	      macStyle = 0;
	    } else if (!strcmp("bold", current.sval)) {
	      macStyle |= 1; /* Bit 0 */
	    } else if (!strcmp("italic", current.sval)) {
	      macStyle |= (1<<1); /* Bit 1 */
	    } else if (!strcmp("underline", current.sval)) {
	      macStyle |= (1<<2); /* Bit 2 */
	    } else if (!strcmp("outline", current.sval)) {
	      macStyle |= (1<<3); /* Bit 3 */
	    } else if (!strcmp("shadow", current.sval)) {
	      macStyle |= (1<<4); /* Bit 4 */
	    } else if (!strcmp("condensed", current.sval)) {
	      macStyle |= (1<<5); /* Bit 5 */
	    } else if (!strcmp("extended", current.sval)) {
	      macStyle |= (1<<6); /* Bit 6 */
	    } else {
	      show_source();
	      fprintf(stderr, "%s does not identify any of the macStyle bits\n",
		      current.sval);
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a bit identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	  if (END_OF_FILE == current.form) break;
	}
      } else if (!strcmp("fontDirectionHint", current.sval)) {
	force_short("fontDirectionHint", &fontDirectionHint);
      } else if (!strcmp("created",  current.sval) ||
		 !strcmp("modified", current.sval)) {
	int ok_so_far = 1;
	int day, month, year, hour, min, sec;
	void* mmm;
	unsigned long long int *the_time, numdays;

	if (!strcmp("created", current.sval)) {
	  the_time = &created;
	} else {
	  the_time = &modified;
	}
	advance_token();
	if (AN_INT == current.form) {
	  day = current.ival;
	  if (day < 1 || day > 31) {
	    show_source();
	    fprintf(stderr, "%d is not a valid day of the month.\n", day);
	    ok_so_far = 0;
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected day of the month.\n");
	  ok_so_far = 0;
	}
	if (ok_so_far) {
	  advance_token();
	  if (PUNCTUATION != current.form || '-' != current.ival) {
	    show_source();
	    fprintf(stderr, "Expected hyphen preceding month.\n");
	    ok_so_far = 0;
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (AN_IDENT == current.form) {
	    if (lookup_in_map(month_names, current.sval, &mmm)) {
	      month = (int) mmm;
	    } else {
	      show_source();
	      fprintf(stderr, "%s is not a recognised month name.\n",
		      current.sval);
	      ok_so_far = 0;
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a month name.\n");
	    ok_so_far = 0;
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (AN_INT == current.form) {
	    year = -current.ival;
	    if (current.ival >= 0) {
	      ok_so_far = 0;
	      show_source();
	      fprintf(stderr, "Expected hyphen before year number.\n");
	    } else if (current.ival > -1904) {
	      show_source();
	      ok_so_far = 0;
	      fprintf(stderr, "Truetype fonts had not been invented in %d.\n",
		      -current.ival);
	    }
	  }
	}
	hour = min = sec = 0;
	if (ok_so_far) {
	  advance_token();
	  if (AN_INT == current.form) {
	    hour = current.ival;
	    if (hour < 0 || hour >= 24) {
	      ok_so_far = 0;
	      show_source();
	      fprintf(stderr, "%d is not a valid hour.\n", hour);
	    }
	  } else {
	    ok_so_far = 0;
	    show_source();
	    fprintf(stderr, "Expected a time of day.\n");
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (PUNCTUATION != current.form || ':' != current.ival) {
	    ok_so_far = 0;
	    show_source();
	    fprintf(stderr, "Require colon between hours and minutes.\n");
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (AN_INT == current.form) {
	    min = current.ival;
	    if (min < 0 || min >= 60) {
	      ok_so_far = 0;
	      show_source();
	      fprintf(stderr, "%d is not a valid number of minutes past "
		      "the hour.\n", min);
	    }
	  } else {
	    ok_so_far = 0;
	    show_source();
	    fprintf(stderr, "Expected a number of minutes.\n");
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (PUNCTUATION != current.form || ':' != current.ival) {
	    ok_so_far = 0;
	    show_source();
	    fprintf(stderr, "Require colon between minutes and seconds.\n");
	  }
	}
	if (ok_so_far) {
	  advance_token();
	  if (AN_INT == current.form) {
	    sec = current.ival;
	    if (sec < 0 || sec > 60) {
	      ok_so_far = 0;
	      show_source();
	      fprintf(stderr, "%d is not a valid number of seconds past "
		      "the minute.\n", hour);
	    }
	  } else {
	    ok_so_far = 0;
	    show_source();
	    fprintf(stderr, "Expected a time of day.\n");
	  }
	}
	if (ok_so_far) {
	  date1904_to_seconds(year, month, day, hour, min, sec,
			      &numdays, the_time);
	  advance_token();
	}
/* Calculable items for the OS/2 table. */
      } else if (!strcmp("avgCharWidth", current.sval)) {
	force_short("avgCharWidth", &xAvgCharWidth);
      } else if (!strcmp("firstCharIndex", current.sval)) {
	force_ushort("firstCharIndex", &usFirstCharIndex); 
      } else if (!strcmp("lastCharIndex", current.sval)) {
	force_ushort("lastCharIndex", &usLastCharIndex); 
      } else if (!strcmp("winAscent", current.sval)) {
	force_ushort("winAscent", &usWinAscent);
      } else if (!strcmp("winDescent", current.sval)) {
	force_ushort("winDescent", &usWinDescent);
      } else if (!strcmp("xHeight", current.sval)) {
	force_short("xHeight", &sxHeight);
      } else if (!strcmp("capHeight", current.sval)) {
	force_short("capHeight", &sCapHeight);
      } else if (!strcmp("maxContext", current.sval)) {
	force_ushort("maxContext", &usMaxContext);
      } else if (!strcmp("unicodeRange", current.sval)) {
	advance_token();
	while (1) {
	  int negated = 0;
	  int i, val;

	  if (AN_IDENT == current.form && !strcmp("no", current.sval)) {
	    negated = 1;
	    advance_token();
	  }
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 128) {
	      show_source();
	      fprintf(stderr, "There is no bit %d in unicodeRange.\n",
		      current.ival);
	    } else {
	      ulUnicodeRange[current.ival].forced_value = !negated;
	      ulUnicodeRange[current.ival].is_forced = 1;
	    }
	  } else if (AN_IDENT == current.form || A_STRING == current.form) {
	    if (!strcmp("none", current.sval)) {
	      for (i = 0; i < 128; i++) {
		ulUnicodeRange[i].forced_value = negated;
		ulUnicodeRange[i].is_forced = 1;
	      }
	    } else if (!strcmp("all", current.sval)) {
	      for (i = 0; i < 128; i++) {
		ulUnicodeRange[i].forced_value = !negated;
		ulUnicodeRange[i].is_forced = 1;
	      }
	    } else {
	      int val = -1;
	      const int nvals = n_script_table;
	      for (i = 0; i < nvals; i++) {
		if (!strcmp(current.sval, script_table[i].sn)  ||
		    !strcmp(current.sval, script_table[i].ln)) {
		  if (val >= 0 &&
		      script_table[val].bit != script_table[i].bit) {
		    fprintf(stderr, "%s identifies both block range %4.4X to "
			    "%4.4X and %4.4X to %4.4X.\n", current.sval,
			    script_table[val].start, script_table[val].end,
			    script_table[i].start, script_table[i].end);
		  } else {
		    val = i;
		  }
		}
	      }
	      if (val >= 0) {
		ulUnicodeRange[script_table[val].bit].forced_value = !negated;
		ulUnicodeRange[script_table[val].bit].is_forced = 1;
	      } else {
		show_source();
		fprintf(stderr, "Unicode block not recognised.\n");
	      }
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a Unicode block identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	}	 
/* Non-calculable items for the OS/2 table. */
      } else if (!strcmp("weightClass", current.sval)) {
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival > 0xffff || current.ival < 0) {
	    show_source(); 
	    fprintf(stderr, "Value of weightClass viz %d is not a " 
		    "16-bit unsigned number.\n", current.ival);
	  } else {
	    usWeightClass = current.ival;
	  }
	} else if (AN_IDENT == current.form) {
	  int i, val = -1;
	  static const struct {int value; char *name;} values[] = {
	    {100, "thin"}, /* FW_THIN */
	    {200, "extra-light"}, 	/* FW_EXTRALIGHT */
	    {200, "extra_light"}, 	/* FW_EXTRALIGHT */
	    {200, "extralight"}, 	/* FW_EXTRALIGHT */
	    {200, "ultra-light"}, 	/* FW_EXTRALIGHT */
	    {200, "ultra_light"}, 	/* FW_EXTRALIGHT */
	    {200, "ultralight"}, 	/* FW_EXTRALIGHT */
	    {300, "light"},   /* FW_LIGHT */
	    {400, "normal"},  /* FW_NORMAL */
	    {400, "regular"}, /* FW_NORMAL */
	    {500, "medium"},  /* FW_MEDIUM */
	    {600, "semi-bold"}, /* FW_SEMIBOLD */
	    {600, "semi_bold"}, /* FW_SEMIBOLD */
	    {600, "semibold"}, 	/* FW_SEMIBOLD */
	    {600, "demi-bold"}, /* FW_SEMIBOLD */
	    {600, "demi_bold"}, /* FW_SEMIBOLD */
	    {600, "demibold"}, 	/* FW_SEMIBOLD */
	    {700, "bold"}, /* FW_BOLD */
	    {800, "extra-bold"}, /* FW_EXTRABOLD */
	    {800, "extra_bold"}, /* FW_EXTRABOLD */
	    {800, "extrabold"},  /* FW_EXTRABOLD */
	    {800, "ultra-bold"}, /* FW_EXTRABOLD */
	    {800, "ultra_bold"}, /* FW_EXTRABOLD */
	    {800, "ultrabold"},  /* FW_EXTRABOLD */
	    {900, "black"}, /* FW_BLACK */
	    {900, "heavy"} }; 	/* FW_BLACK */
	  int nvals = sizeof values / sizeof * values;
	  for (i = 0; i < nvals; i++) {
	    if (!strcmp(current.sval, values[i].name)) {
	      val = values[i].value;
	      break;
	    }
	  }
	  if (val > 0) {
	    usWeightClass = val;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid value for weightClass.\n",
		    current.sval);
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected a number or identifier.");
	}
	advance_token();
      } else if (!strcmp("widthClass", current.sval)) {
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival > 0xffff || current.ival < 0) {
	    show_source(); 
	    fprintf(stderr, "Value of widthClass viz %d is not a " 
		    "16-bit unsigned number.\n", current.ival);
	  } else {
	    usWidthClass = current.ival;
	  }
	} else if (AN_IDENT == current.form) {
	  int i, val = -1;
	  static const struct {int value; char *name;} values[] = {
	    {1, "ultra-condensed"}, /* FWIDTH_ULTRA_CONDENSED - 50% */
	    {1, "ultra_condensed"}, /* FWIDTH_ULTRA_CONDENSED - 50% */
	    {1, "ultracondensed"},  /* FWIDTH_ULTRA_CONDENSED - 50% */
	    {2, "extra-condensed"}, /* FWIDTH_EXTRA_CONDENSED - 62.5% */
	    {2, "extra_condensed"}, /* FWIDTH_EXTRA_CONDENSED - 62.5% */
	    {2, "extracondensed"},  /* FWIDTH_EXTRA_CONDENSED - 62.5% */
	    {3, "condensed"},       /* FWIDTH_CONDENSED - 75% */
	    {4, "semi-condensed"},  /* FWIDTH_SEMI_CONDENSED - 87.5% */
	    {4, "semi_condensed"},  /* FWIDTH_SEMI_CONDENSED - 87.5% */
	    {4, "semicondensed"},   /* FWIDTH_SEMI_CONDENSED - 87.5% */
	    {5, "medium"},          /* FWIDTH_NORMAL - 100% */
	    {5, "normal"},          /* FWIDTH_NORMAL - 100% */
	    {6, "semi-expanded"},   /* FWIDTH_SEMI_EXPANDED - 112.5% */
	    {6, "semi_expanded"},   /* FWIDTH_SEMI_EXPANDED - 112.5% */
	    {6, "semiexpanded"},    /* FWIDTH_SEMI_EXPANDED - 112.5% */
	    {7, "expanded"},        /* FWIDTH_EXPANDED - 125% */
	    {8, "extra-expanded"},  /* FWIDTH_EXTRA_EXPANDED - 150% */
	    {8, "extra_expanded"},  /* FWIDTH_EXTRA_EXPANDED - 150% */
	    {8, "extraexpanded"},   /* FWIDTH_EXTRA_EXPANDED - 150% */
	    {9, "ultra-expanded"},  /* FWIDTH_ULTRA_EXPANDED - 200% */
	    {9, "ultra_expanded"},  /* FWIDTH_ULTRA_EXPANDED - 200% */
	    {9, "ultraexpanded"}};   /* FWIDTH_ULTRA_EXPANDED - 200% */
	  int nvals = sizeof values / sizeof * values;
	  for (i = 0; i < nvals; i++) {
	    if (!strcmp(current.sval, values[i].name)) {
	      val = values[i].value;
	      break;
	    }
	  }
	  if (val >= 0) {
	    usWidthClass = val;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid value for widthClass.\n",
		    current.sval);
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected a number or identifier.");
	}
	advance_token();
      } else if (!strcmp("embedding", current.sval)) { /* fsType */
	fsType = 0;
	advance_token();
	while (1) {
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 16) {
	      show_source();
	      fprintf(stderr, "There is no bit %d in embedding.\n",
		      current.ival);
	    } else {
	      fsType |= (1 << current.ival); 
	    }
	  } else if (AN_IDENT == current.form) {
	    if (!strcmp("free", current.sval)) {
	      fsType = 0;
	    } else if (!strcmp("bit0", current.sval)) {
	      fsType |= 1; /* Bit 0 */
	    } else if (!strcmp("banned", current.sval)) {
	      fsType |= 2; /* Bit 1 */
	    } else if (!strcmp("print", current.sval)) {
	      fsType |= 4; /* Bit 2 */
	    } else if (!strcmp("edit", current.sval)) {
	      fsType |= 8; /* Bit 3 */
	    } else if (!strcmp("no-subsetting", current.sval) ||
		       !strcmp("no_subsetting", current.sval) ||
		       !strcmp("nosubsetting", current.sval) ||
		       !strcmp("noSubsetting", current.sval)) {
	      fsType |= 256; /* Bit 8 */
	    } else if (!strcmp("bitmap", current.sval)) {
	      fsType |= 512; /* Bit 9 */
	    } else {
	      show_source();
	      fprintf(stderr, "%s does not identify any of the "
		      "embedding bits\n", current.sval);
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a bit identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	}
	switch (fsType & 0x000F) {
	case 0:
	case 2:
	case 4:
	case 8:
	  /* Intelligible bit setting */
	  break;
	default:
	  fprintf(stderr, "Embedding (0x%4.4X) has incompatible or "
		  "meaningless permissions.\n",
		  fsType);
	  break;
	}
      } else if (!strcmp("subscriptXSize", current.sval)) {
	get_short("subscriptXSize", &ySubscriptXSize);
      } else if (!strcmp("subscriptYSize", current.sval)) {
	get_short("subscriptYSize", &ySubscriptYSize);
      } else if (!strcmp("subscriptXOffset", current.sval)) {
	get_short("subscriptXOffset", &ySubscriptXOffset);
      } else if (!strcmp("subscriptYOffset", current.sval)) {
	get_short("subscriptYOffset", &ySubscriptYOffset);
      } else if (!strcmp("superscriptXSize", current.sval)) {
	get_short("superscriptXSize", &ySuperscriptXSize);
      } else if (!strcmp("superscriptYSize", current.sval)) {
	get_short("superscriptYSize", &ySuperscriptYSize);
      } else if (!strcmp("superscriptXOffset", current.sval)) {
	get_short("superscriptXOffset", &ySuperscriptXOffset);
      } else if (!strcmp("superscriptYOffset", current.sval)) {
	get_short("superscriptYOffset", &ySuperscriptYOffset);
      } else if (!strcmp("strikeoutSize", current.sval)) {
	get_short("strikeoutSize", &yStrikeoutSize);
      } else if (!strcmp("strikeoutPosition", current.sval)) {
	get_short("strikeoutPosition", &yStrikeoutPosition);
      } else if (!strcmp("familyClass", current.sval)) {
	int class, subclass;
	advance_token();
	if (AN_INT == current.form) {
	  if (current.ival > 0xffff || current.ival < 0) {
	    show_source(); 
	    fprintf(stderr, "Value of familyClass viz %d is not a " 
		    "16-bit unsigned number.\n", current.ival);
	  } else {
	    sFamilyClass = current.ival;
	  }
	} else if (AN_IDENT == current.form || A_STRING == current.form) {
	  int i, val = -1;
	  static const struct {unsigned char class, subclass; char *name; }
	  values[] = {
	    {0, 0, "none"},
	    {1, 0, "oldstyle"},
	    {1, 1, "??"}, /* Oldstyle serifs - IBM rounded legibility */
	    {1, 2, "Garalde"},
	    {1, 3, "Venetian"},
	    {1, 4, "Modified Venetian"},
	    {1, 4, "modified Venetian"},
	    {1, 5, "Dutch Modern"},
	    {1, 6, "Dutch Traditional"},
	    {1, 7, "Contemporary"},
	    {1, 7, "contemporary"},
	    {1, 8, "Calligraphic"},
	    {1, 8, "calligraphic"},
	    {1, 15, "miscellaneous oldstyle"},
	    {2, 0, "transitional"},
	    {2, 1, "direct line"},
	    {2, 2, "transitional script"},
	    {2, 15, "miscellaneous transitional"},
	    {3, 0, "modern"},
	    {3, 1, "Italian"},
	    {3, 2, "modern script"},
	    {3, 15, "miscellaneous modern"},
	    {4, 0, "Clarendon general"},
	    {4, 1, "Clarendon specific"},
	    {4, 1, "Clarendon Clarendon"},
	    {4, 2, "Clarendon modern"},
	    {4, 3, "Clarendon traditional"},
	    {4, 4, "newspaper"},
	    {4, 5, "stub serif"},
	    {4, 6, "Clarendon monotone"},
	    {4, 7, "Clarendon typewriter"},
	    {4, 15, "Clarendon miscellaneous"},
	    {5, 0, "slab"},
	    {5, 1, "slab monotone"},
	    {5, 2, "slab humanist"},
	    {5, 3, "geometric"},
	    {5, 4, "swiss"},
	    {5, 5, "slab typewriter"},
	    {5, 15, "slab miscellaneous"},
	    {7, 0, "freeform"},
	    {7, 1, "freeform modern"},
	    {7, 15, "freeform miscellaneous"},
	    {8, 0, "sans serif"},
	    {8, 1, "IBM Gothic"},
	    {8, 2, "sans serif humanist"},
	    {8, 3, "low geometric"},
	    {8, 4, "high geometric"},
	    {8, 5, "Gothic"},
	    {8, 6, "modified Gothic"},
	    {8, 9, "typewriter Gothic"},
	    {8, 10, "matrix"},
	    {8, 15, "sans serif miscellaneous"},
	    {9, 0, "ornamental"},
	    {9, 1, "engraver"},
	    {9, 2, "black letter"},
	    {9, 3, "decorative"},
	    {9, 4, "three dimensional"},
	    {9, 4, "3D"},
	    {9, 15, "ornamental miscellaneous"},
	    {10, 0, "script"},
	    {10, 1, "uncial"},
	    {10, 2, "brush joined"},
	    {10, 3, "formal joined"},
	    {10, 4, "monotone joined"},
	    {10, 5, "script calligraphic"},
	    {10, 6, "brush unjoined"},
	    {10, 7, "formal unjoined"},
	    {10, 8, "monotone unjoined"},
	    {10, 15, "script miscellaneous"},
	    {12, 0, "symbolic"},
	    {12, 3, "mixed serif"},
	    {12, 6, "symbolic oldstyle"},
	    {12, 7, "symbolic sans serif"},
	    {12, 15, "symbolic miscellaneous"},
	  };
	  int nvals = sizeof values / sizeof * values;
	  for (i = 0; i < nvals; i++) {
	    if (!strcmp(current.sval, values[i].name)) {
	      val = (values[i].class << 8) | values[i].subclass;
	      break;
	    }
	  }
	  if (val >= 0) {
	    sFamilyClass = val;
	  } else {
	    show_source();
	    fprintf(stderr, "%s is not a valid value for widthClass.\n",
		    current.sval);
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected a number, identifier or string.");
	}
	advance_token();
      } else if (!strcmp("panose", current.sval)) {
	int i, val, iel;
	advance_token();
	if (AN_IDENT == current.form) {
	  iel = -1; /* No particular element to load */
	  if (!(strcmp("any", current.sval))) {
	    for (i = 0; i < 10; i++) panose[i] = 0;
	  } else if (!(strcmp("unmatched", current.sval))) {
	    for (i = 0; i < 10; i++) panose[i] = 1;
	  } else if (!(strcmp("all", current.sval))) {
	    for (i = 0; i < 10; i++) {
	      advance_token();
	      if (AN_INT == current.form) {
		if (0 <= current.ival && current.ival < 255) {
		  panose[i] = current.ival;
		} else {
		  show_source();
		  fprintf(stderr, "Value %d is out of range.\n", current.ival);
		  panose[i] = 0;
		}
	      } else {
		show_source();
		fprintf(stderr, "Expected a number.\n");
		panose[i] = 0;
	      }
	    }
	  } else if (!strcmp("familyType", current.sval) ||
		     !strcmp("family", current.sval)) {
	    iel = 0;
	  } else if (!strcmp("serifStyle", current.sval)) {
	    iel = 1;
	  } else if (!strcmp("weight", current.sval)) {
	    iel = 2;
	  } else if (!strcmp("proportion", current.sval)) {
	    iel = 3;
	  } else {
	    show_source();
	    fprintf(stderr, "Subkeyword for panose not recognised.\n");
	  }
	  if (iel >= 0) {
	    advance_token();
	    if (AN_INT == current.form) {
	      if (0 <= current.ival && current.ival < 255) {
		panose[i] = current.ival;
	      } else {
		show_source();
		fprintf(stderr, "Value %d is out of range.\n", current.ival);
		panose[i] = 0;
	      }
	    } else if (0 == iel && AN_IDENT == current.form &&
		       !strcmp("pictorial", current.sval)) {
	      panose[0] = 5;
	    } else {
	      show_source();
	      fprintf(stderr, "Expected a number.\n");
	      panose[i] = 0;
	    }
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected an identifier.\n");
	}
	advance_token();
      } else if (!strcmp("vendID", current.sval)) {
	advance_token();
	if (A_STRING == current.form || AN_IDENT == current.form) {
	  if (strlen(current.sval) <= 4) {
	    strcpy(achVendID, "    ");
	    strncpy(achVendID, current.sval, strlen(current.sval));
	  } else {
	    fprintf(stderr, "Value for VendID exceeds 4 characters.\n");
	  }
	} else {
	  show_source();
	  fprintf(stderr, "Expected an identifier or a string.\n");
	}
	advance_token();
      } else if (!strcmp("fsSelection", current.sval)) {
	fsSelection = 0;
	advance_token();
	while (1) {
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 16) {
	      show_source();
	      fprintf(stderr, "There is no bit %d in fsSelection.\n",
		      current.ival);
	    } else {
	      fsSelection |= (1 << current.ival); 
	    }
	  } else if (AN_IDENT == current.form || A_STRING == current.form) {
	    if (!strcmp("none", current.sval)) {
	      fsSelection = 0;
	    } else if (!strcmp("italic", current.sval)) {
	      fsSelection |= 1; /* Bit 0 */
	    } else if (!strcmp("underscore", current.sval)) {
	      fsSelection |= 2; /* Bit 1 */
	    } else if (!strcmp("negative", current.sval)) {
	      fsSelection |= 4; /* Bit 2 */
	    } else if (!strcmp("outlined", current.sval)) {
	      fsSelection |= 8; /* Bit 3 */
	    } else if (!strcmp("strikeout", current.sval)) {
	      fsSelection |= 16; /* Bit 4 */
	    } else if (!strcmp("bold", current.sval)) {
	      fsSelection |= 32; /* Bit 5 */
	    } else if (!strcmp("regular", current.sval)) {
	      fsSelection |= 64; /* Bit 6 */
	    } else if (!strcmp("use_typo_metrics", current.sval) ||
		       !strcmp("use typo metrics", current.sval) ||
		       !strcmp("usetypometrics", current.sval) ||
		       !strcmp("typo", current.sval)) {
	      fsSelection |= 128; /* Bit 7 */
	    } else if (!strcmp("wws", current.sval) ||
		       !strcmp("WWS", current.sval)) {
	      fsSelection |= 256; /* Bit 8 */
	    } else if (!strcmp("oblique", current.sval)) {
	      fsSelection |= 512; /* Bit 9 */
	    } else {
	      show_source();
	      fprintf(stderr, "%s does not identify any of the "
		      "embedding bits\n", current.sval);
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a bit identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	}
      } else if (!strcmp("typoAscender", current.sval)) {
	get_short("typoAscender", &sTypoAscender);
      } else if (!strcmp("typoDescender", current.sval)) {
	get_short("typoDescender", &sTypoDescender);
      } else if (!strcmp("typoLineGap", current.sval)) {
	get_short("typoLineGap", &sTypoLineGap);
      } else if (!strcmp("defaultChar", current.sval)) {
	get_ushort("defaultChar", &usDefaultChar);
      } else if (!strcmp("breakChar", current.sval)) {
	get_ushort("breakChar", &usBreakChar);
      } else if (!strcmp("codePage", current.sval)) {
	int i, val = -1;

	advance_token();
	if (AN_IDENT == current.form && !strcmp("extend", current.sval)) {
/* No wiping.  Get next token, skipping optional comma. */
	  advance_token(); 
	  if (PUNCTUATION == current.form && ',' == current.ival) {
	    advance_token();
	  }
	} else {
	  ulCodePageRange[1] = ulCodePageRange[0] = 0;
	}
	while (1) {
	  if (AN_INT == current.form) {
	    if (current.ival < 0 || current.ival >= 64) {
	      show_source();
	      fprintf(stderr, "There is no bit %d in codePage.\n",
		      current.ival);
	    } else if (current.ival < 32) {
	      ulCodePageRange[0] |= (1 << current.ival); 
	    } else {
	      ulCodePageRange[1] |= (1 << (current.ival - 32)); 
	    }
	  } else if (AN_IDENT == current.form || A_STRING == current.form) {
	    if (!strcmp("none", current.sval)) {
	      ulCodePageRange[1] = ulCodePageRange[0] = 0;
	    } else {
	      val = -1;
	      for (i = 0; i < n_code_page_table; i++) {
		if (!strcmp(current.sval, code_page_table[i].n1)  ||
		    !strcmp(current.sval, code_page_table[i].n2)) {
		  val = code_page_table[i].bit;
		  break;
		}
	      }
	      if (val >= 32) {
		ulCodePageRange[1] |= (1 << (val - 32));
	      } else if (val >= 0) {
		ulCodePageRange[0] |= (1 << val);
	      } else {
		show_source();
		fprintf(stderr, "Code page not recognised.\n");
	      }
	    }
	  } else {
	    show_source();
	    fprintf(stderr, "Expected a bit identifier.\n");
	  }
	  advance_token();
	  if (PUNCTUATION != current.form || ',' != current.ival) break;
	  advance_token();
	}
/* Calculable items for the post table */
      } else if (!strcmp("isFixedPitch", current.sval)) {
	force_ulong("isFixedPitch", &isFixedPitch);
/* Non-calculable items for the post table */
      } else if (!strcmp("italicAngle", current.sval)) {
	get_double("italicAngle", &italicAngle);
      } else if (!strcmp("underlinePosition", current.sval)) {
	get_short("underlinePosition", &underlinePosition);
      } else if (!strcmp("underlineThickness", current.sval)) {
	get_short("underlineThickness", &underlineThickness);
      } else if (!strcmp("minMemType42", current.sval)) {
	get_ulong("minMemType42", &minMemType42);
      } else if (!strcmp("maxMemType42", current.sval)) {
	get_ulong("maxMemType42", &maxMemType42);
      } else if (!strcmp("minMemType1", current.sval)) {
	get_ulong("minMemType1", &minMemType1);
      } else if (!strcmp("maxMemType1", current.sval)) {
	get_ulong("maxMemType1", &maxMemType1);
/* Font program */
      } else if (!strcmp("font_program", current.sval)) {
	have_fpgm = 1;
	fpgm_instr = 0;
	advance_token();
	compile_instr(&fpgm_instr, &n_fpgm_instr);
	if (strcmp("font_program", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected keyword 'font_program'.\n");
	  got_what(stderr, current);
	} else {
	  advance_token();
	}
/* CVT program */
      } else if (!strcmp("cvt_program", current.sval)) {
	have_prep = 1;
	prep_instr = 0;
	advance_token();
	compile_instr(&prep_instr, &n_prep_instr);
	if (strcmp("cvt_program", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected keyword 'cvt_program'.\n");
	  got_what(stderr, current);
	} else {
	  advance_token();
	}
/* control values */
      } else if (!strcmp("control_values", current.sval)) {
	have_cvt = 1;
	n_cvt = 0;
	advance_token();
	while (AN_INT == current.form) {
	  int newval;

	  if (current.ival > 0x7FFF || current.ival < -0x8000) {
	    show_source();
	    fprintf(stderr, "%d is not a valid value.\n", current.ival);
	    newval = 0;
	  } else {
	    newval = current.ival;
	  }
	  if (n_cvt >= m_cvt) {
	    m_cvt += 50;
	    p_cvt = realloc(p_cvt, m_cvt * sizeof * p_cvt);
	  }
	  p_cvt[n_cvt++] = newval;
	  advance_token();
	}
	if (AN_IDENT != current.form || strcmp("end", current.sval)) {
	  show_source();
	  fprintf(stderr, "Expected keyword 'end'.\n");
	  have_cvt = 0;
	} else {
	  advance_token();
	  if (AN_IDENT != current.form ||
	      strcmp("control_values", current.sval)) {
	    show_source();
	    fprintf(stderr, "Expected keyword 'control_values'.\n");
	    have_cvt = 0;
	  }
	}
	advance_token();
/* Calculable items for maxp */
      } else if (!strcmp("maxPoints", current.sval)) {
	force_ushort("maxPoints", &maxPoints);
      } else if (!strcmp("maxContours", current.sval)) {
	force_ushort("maxContours", &maxContours);
      } else if (!strcmp("maxCompositePoints", current.sval)) {
	force_ushort("maxCompositePoints", &maxCompositePoints);
      } else if (!strcmp("maxCompositeContours", current.sval)) {
	force_ushort("maxCompositeContours", &maxCompositeContours);
      } else if (!strcmp("maxZones", current.sval)) {
	force_ushort("maxZones", &maxZones);
      } else if (!strcmp("maxTwilightPoints", current.sval)) {
	force_ushort("maxTwilightPoints", &maxTwilightPoints);
      } else if (!strcmp("maxStorage", current.sval)) {
	force_ushort("maxStorage", &maxStorage);
      } else if (!strcmp("maxFunctionDefs", current.sval)) {
	force_ushort("maxFunctionDefs", &maxFunctionDefs);
      } else if (!strcmp("maxInstructionDefs", current.sval)) {
	force_ushort("maxInstructionDefs", &maxInstructionDefs);
      } else if (!strcmp("maxStackElements", current.sval)) {
	force_ushort("maxStackElements", &maxStackElements);
      } else if (!strcmp("maxSizeOfInstructions", current.sval)) {
	force_ushort("maxSizeOfInstructions", &maxSizeOfInstructions);
      } else if (!strcmp("maxComponentElements", current.sval)) {
	force_ushort("maxComponentElements", &maxComponentElements);
      } else if (!strcmp("maxComponentDepth", current.sval)) {
	force_ushort("maxComponentDepth", &maxComponentDepth);
/* Keywords for GSUB and GPOS */
      } else if (!strcmp("GSUB", current.sval)) {
	process_gsubpos();
      } else if (!strcmp("GPOS", current.sval)) {
	process_gsubpos();
      } else if (!strcmp("lookup", current.sval)) {
	process_subtable();
      } else if (!strcmp("classification", current.sval)) {
	process_classification();
      } else if (!strcmp("coverage", current.sval)) {
	process_coverage();
      } else if (!strcmp("store", current.sval)) {
	store_something();
      } else if (!strcmp("colours", current.sval)) {
        int ncolval = 0;
        advance_token();
        while (END_OF_FILE != current.form) {
          long int tval;
          char *unused;
          int tlen;
          if (mcolval <= ncolval) {
            mcolval += mcolval / 2 + 4;
            if (!(colval = realloc(colval, mcolval * sizeof * colval))) {
              fprintf(stderr, "Memory exhausted parsing colour list.\n");
              exit(1);
            }
          }
          while (AN_IDENT == current.form) {
            if (!strcmp("end", current.sval)) {
              break;
            } else if (AN_IDENT == current.form) {
              advance_token();
              if (PUNCTUATION == current.form && ':' == current.ival) {
                insert_in_map(&colours, previous.sval, (void*) ncolval);
                advance_token();
              } else {
                backstep_token();
                break;
              }
            }
          }
          if (AN_IDENT == current.form) {
            tlen = strlen(current.sval);
            if (!strcmp("end", current.sval)) break;
            if (8 == tlen || 6 == tlen) {
              tval = strtoll(current.sval, &unused, 16);
            } else {
              unused = "!";
            }
          } else if (AN_INT == current.form) {
            tval = current.ival;
            tlen = 8;
            unused = "";
          } else {
            unused = "!";
          }
          if (*unused) {
            show_source();
            fprintf(stderr, "Expected 8- (or 6-) digit hex number.\n");
            got_what(stderr, current);
            tval = 0x80808080;
            tlen = 8;
          }
          if (0) fprintf(stderr, "tval=%lx\n", tval);
          colval[ncolval].blue = tval & 0xff; tval >>= 8;
          colval[ncolval].green = tval & 0xff; tval >>= 8;
          colval[ncolval].red = tval & 0xff; tval >>= 8;
          colval[ncolval].alpha = tval & 0xff;
          if (6 == tlen) colval[ncolval].alpha = 0xff;
          advance_token();
          while (PUNCTUATION == current.form && ',' == current.ival) {
            int unexpected = 1;
            advance_token();
            if (AN_IDENT == current.form) {
              if (!strcmp("first", current.sval)) {
                unexpected = 0;
                if (p0c0 < 0) {
                  p0c0 = ncolval;
                } else {
                  show_source();
                  fprintf(stderr, "Generic first palette element already "
                          "defined - attempted redefinition ignored.\n");
                }
              } else if (!strcmp("last", current.sval)) {
                unexpected = 0;
                if (p0c0 < 0) {
                  show_source();
                  fprintf(stderr, "Generic first palette element not defined, "
                          "so definition of last element is not allowed.\n");
                } else {
                  palette.numPalettesEntries = ncolval - p0c0 + 1;
                }
              } else if (!strcmp("label", current.sval)) {
                int namid = -1;
                unexpected = 0;
                advance_token();
                if (AN_INT == current.form) {
                  if (current.ival < 0) {
                    show_source();
                    fprintf(stderr, "Invalid, negative name ID.\n");
                  } else {
                    namid = current.ival;
                  }
                } else if (AN_IDENT == current.form) {
                  void *ptrval;
                  if (lookup_in_map(nameID_codes, current.sval, &ptrval)) {
                    namid = (int) ptrval;
                    if (namid < 0) {
                      show_source();
                      fprintf(stderr, "Invalid, negative name ID.\n");
                    }
                  } else {
                    namid = -1;
                    show_source();
                    fprintf(stderr, "Name ID is not defined.\n");
                  }
                }
                if (namid > 0xffff) {
                  show_source();
                  fprintf(stderr, "Name ID cannot be expressed as "
                          "16-bit integer.");
                } else if (p0c0 < 0) {
                  show_source();
                  fprintf(stderr, "Generic palette entries are not defined "
                          "- use option 'first'.");
                } else if (palette.numPalettesEntries > 0 &&
                           ncolval >= p0c0 + palette.numPalettesEntries) {
                  show_source();
                  fprintf(stderr, "Entry is later than generic "
                          "palette entries.\n");
                } else if (namid >= 0) { /* Store the label */
                  int nnew = ncolval - p0c0 + 1;
                  int ipe;
                  if (mpe_label <= nnew) {
                    mpe_label = nnew + mpe_label/2;
                    if (!(pe_label = realloc(pe_label,
                                             mpe_label * sizeof * pe_label))) {
                      fprintf(stdout, "Memory exhausted recording "
                              "pallette entry label.\n");
                      exit(1);
                    }
                  }
                  for (ipe = npe_label; ipe < nnew - 1; ipe++) {
                    pe_label[ipe] = 0xffff;
                  }
                  pe_label[ipe] = namid;
                }
                if (unexpected) {
                  show_source();
                  fprintf(stderr, "Expected name ID.\n");
                  got_what(stderr, current);
                }
              } else {
                show_source();
                fprintf(stderr, "Unrecognised option on colour.\n");
              }
            } else {
              fprintf(stderr, "Expected an identifier as option.\n");
              got_what(stderr, current);
            }
            advance_token();
          }
          ncolval++;
        }
        if (p0c0 < 0) {
          p0c0 = 0;
          palette.numPalettesEntries = ncolval;
        }
        insert_in_map(&colours, "foreground", (void*) (p0c0 + 0xffFF));
        advance_token();
        if (AN_IDENT != current.form || strcmp("colours", current.sval)) {
          fprintf(stderr, "Expected keyword 'colours'.\n");
          got_what(stderr, current);
        }
        palette.numColorRecords = ncolval;
        if (0) fprintf(stderr, "Got %d colours.\n", ncolval);
        advance_token();
      } else if (!strcmp("palettes", current.sval)) {
        advance_token();
        while (END_OF_FILE != current.form) {
          int col = 0, name = 0xffff, nature = 0;
          int n = palette.numPalette;
          if (AN_IDENT != current.form) {
            show_source();
            fprintf(stderr, "Expected an identifier.\n");
            got_what(stderr, current);
          } else if (strcmp("end", current.sval)) {
            void *ptrval;
            if (lookup_in_map(colours, current.sval, &ptrval)) {
              col = (int) ptrval;
            } else {
              show_source();
              fprintf(stderr,
                      "Colour instance identifier is not recognised.\n");
              got_what(stderr, current);
            }
          } else {
            break;
          }
          advance_token();
          if (AN_IDENT == current.form) {
            void *ptrval;
            if (lookup_in_map(nameID_codes, current.sval, &ptrval)) {
              name = (int) ptrval;
              advance_token();
            } else {
              show_source();
              fprintf(stderr, "Palette label is not recognised.\n");
              got_what(stderr, current);
            }
          } else if (AN_INT == current.form) {
            if (current.ival < 0 || current.ival > 0xffff) {
              show_source();
              fprintf(stderr, "Numeric ID of label is not an unsigned 16-bit "
                      "number.\n");
              got_what(stderr, current);
            } else {
              name = current.ival;
            }
            advance_token();
          } else {
            show_source();
            fprintf(stderr, "Expected a number or an identifier.\n");
            got_what(stderr, current);
          }
          if (AN_IDENT != current.form && AN_INT != current.form) {
            show_source();
            fprintf(stderr, "Expected a bit field or bit number.\n");
            got_what(stderr, current);
          }
          while (AN_IDENT == current.form || AN_INT == current.form) {
            int doloop = 1;
            if (AN_INT == current.form) {
              if (2 <= current.ival && current.ival <= 31) {
                nature |= (1 << current.ival);
              } else {
                show_source();
                fprintf(stderr, "Expected a bit number.\n");
                got_what(stderr, current);
                doloop = 0;
              }
            } else if (!strcmp("none", current.sval)) {
                /* No effect */
            } else if (!strcmp("light", current.sval)) {
              nature |= 1;
            } else if (!strcmp("dark", current.sval)) {
              nature |= 2;
            } else if (!strcmp("end", current.sval)) {
              doloop = 0;
              break;
            } else {
              show_source();
              fprintf(stderr, "Expected the name of a bit field.\n");
              got_what(stderr, current);
            }
            if (doloop) {
              advance_token();
              if (PUNCTUATION == current.form && ',' == current.ival) {
                advance_token();
              } else {
                break;
              }
            } else {
              break;
            }
          }
          if (mpal <= palette.numPalette) {
            mpal += 4 + mpal/2;
            pal_type  = realloc(pal_type,  mpal * sizeof * pal_type);
            pal_label = realloc(pal_label, mpal * sizeof * pal_label);
            palette.colorRecordIndices =
              realloc(palette.colorRecordIndices, mpal * sizeof * pal_type);
          }
          palette.colorRecordIndices[n] = col;
          pal_label[n] = name;
          pal_type[n] = nature;
          palette.numPalette++;
        }
        advance_token();
        if (AN_IDENT != current.form || strcmp("palettes", current.sval)) {
          fprintf(stderr, "Expected keyword 'palettes'.\n");
          got_what(stderr, current);
        }
        if (0) fprintf(stderr, "Got %d palettes.\n", palette.numPalette);
        advance_token();
      } else if (!strcmp("gasp", current.sval)) {
        int cap_gasp = 0;
        free(v_gasp);
        v_gasp = 0;
        n_gasp = 0;
        advance_token();
        while (AN_INT == current.form) {
          int more_behaviour = 1;
          if (cap_gasp <= n_gasp) {
            cap_gasp += 7;
            if (!(v_gasp = realloc(v_gasp, cap_gasp * sizeof * v_gasp))) {
              show_source();
              fprintf(stderr, "Memory exhausted reading gash table.");
              exit(1);
            }
          }
          if (current.ival < 0 || current.ival > 0xFFFF) {
            show_source();
            fprintf(stderr, "Invalid max PPEM value.");
            v_gasp[n_gasp].maxPPEM = n_gasp ? v_gasp[n_gasp-1].maxPPEM : 0;
          } else {
            v_gasp[n_gasp].maxPPEM = current.ival;
            if (!n_gasp) {
              /* no constraint */
            } else if (v_gasp[n_gasp].maxPPEM < v_gasp[n_gasp-1].maxPPEM) {
              show_source();
              fprintf(stderr, "Max PPEM values are out of order.");
            }
          }
          v_gasp[n_gasp].behaviour = 0;
          while (more_behaviour) {
            advance_token();
            if (AN_IDENT == current.form | A_STRING == current.form) {
              if (!strcmp("none", current.sval)) {
                /* Eye-pleasing filler */
              } else if (!strcmp("gridfit", current.sval)) {
                v_gasp[n_gasp].behaviour |= 1;
              } else if (!strcmp("dogray", current.sval)) {
                v_gasp[n_gasp].behaviour |= 2;
              } else if (!strcmp("sym_gridfit", current.sval)) {
                v_gasp[n_gasp].behaviour |= 4;
              } else if (!strcmp("smooth", current.sval)) {
                v_gasp[n_gasp].behaviour |= 8;
              } else if (!strcmp("end", current.sval)) {
                show_source();
                fprintf(stderr, "gasp behaviour code missing.");
                more_behaviour = 0;
              } else {
                show_source();
                fprintf(stderr, "Undefined gasp code ignored.");
              }
            } else if (AN_INT == current.form) {
              if (current.ival < 0 || current.ival >= 16) {
                show_source();
                fprintf(stderr, "Invalid bit number for gaps code ignored.");
              } else {
                v_gasp[n_gasp].behaviour |= (1 << current.ival);
              }
            } else {
              show_source();
              fprintf(stderr, "Expected a number or an identifier.");
              got_what(stderr, current);
              more_behaviour = 0;
            }
            advance_token();
            if (!more_behaviour) {
              backstep_token();
            } else if (AN_IDENT == current.form &&
                       !strcmp("end", current.sval)) {
              more_behaviour = 0;
            } else if (AN_INT == current.form) {
              more_behaviour = 0;
            } else if (PUNCTUATION == current.form && ',' == current.ival) {
              /* Carry on */
            } else {
              show_source();
              fprintf(stderr, "Expected comma, \"end\" or a number.");
              got_what(stderr, current);
              more_behaviour = 0;
            }
          }
          n_gasp++;
        }
        if (AN_IDENT == current.form && !strcmp("end", current.sval)) {
          advance_token();
          if (AN_IDENT == current.form && !strcmp("gasp", current.sval)) {
            advance_token();
          } else {
            show_source();
            fprintf(stderr, "Expected \"gash\"");
            got_what(stderr, current);
          }
        } else {
          show_source();
          fprintf(stderr, "Expected \"end gash\"");
          got_what(stderr, current);
        }
        if (n_gasp > 0 && v_gasp[n_gasp-1].maxPPEM != 0xFFFF) {
          show_source();
          fprintf(stderr, "Last PPEM in gash table is %d, "
                  "not 0xFFFF as expected.\n", v_gasp[n_gasp-1].maxPPEM);
        }
/* Unrecognised keyword */
      } else {
	show_source();
	fprintf(stderr, "%s is not a keyword.\n", current.sval);
	advance_token();
      }
    } else {
      show_source();
      fprintf(stderr, "Expected a keyword.\n");
      advance_token();
    }
  }
  if (untracked) {
    struct int_range *irge;
    for (irge = untracked; irge; irge = irge->next) {
      if (irge->start == irge->end) {
	fprintf(stdout, "U+%4.4X is not in any tracked block.\n", irge->start);
      } else if (irge->start + 1 == irge->end) {
	fprintf(stdout, "U+%4.4X and U+%4.4X are not in any tracked block.\n",
		irge->start, irge->end);
      } else {
	fprintf(stdout, "U+%4.4X to U+%4.4X are not in any tracked block.\n",
		irge->start, irge->end);
      }
    }
  }
  fprintf(stdout, "Analysis complete.\n");
}

static void seti24(unsigned char var[3], int value) {
  var[0] = (value & 0xff0000) >> 16;
  var[1] = (value & 0x00ff00) >> 8;
  var[2] = (value & 0x0000ff);
}

static void build_font(FILE *font) {
  struct offset_table sfnt;
  struct directory_entry cmap_entry, maxp_entry, head_entry, loca_entry;
  struct directory_entry glyf_entry, name_entry, hmtx_entry, hhea_entry;
  struct directory_entry post_entry, OS2_entry,  fpgm_entry, prep_entry;
  struct directory_entry cvt_entry,  GDEF_entry, GSUB_entry, GPOS_entry;
  struct directory_entry COLR_entry, CPAL_entry, gasp_entry;
  struct encoding_record bmp_ptr, sup_ptr;
  struct encoding_record bmp_ptr0, sup_ptr0, vs_ptr; /* for platform 0 */
  struct encoding bmp_table, sup_table, vs_table;
  int follow, cmap_pos, vs_pos, old_pos, pos, head_pos, name_pos, string_pos;
  struct maximum_profile maxp;
  struct cmap_header cmap;
  struct font_header head;
  int *cpt, npt, maxpt, iglyph;
  unsigned short int *gid;
  int havesup = 0, havevs = 0;
  struct default_UVS_table *VS_default = 0;
  struct non_default_UVS_table *VS_nd =  0;
  struct one_location glyph_loc;
  char *whole_table;
  struct name_table name;
  struct map_entry **names;
  int iname, name_count;
  struct horizontal_header hhea;
  struct OS2_metrics OS2;
  struct postscript_information_nonames post;
  int have_cmap, have_maxp, have_head, have_loca;
  int have_glyf, have_name, have_hmtx, have_hhea;
  int have_post, have_OS2,  have_gasp;
  struct table_status {
    int *have; /* Enables globalisation of flag */
    struct directory_entry *entry;
    char tag[4];
  };
  int its;
  struct directory_entry **next_entry;

#if 0
#define entry_status(name) {&have_ ## name, &name ## _entry, #name}
  const struct table_status table_statuses[] = {
/* Note that directory entries have to be in ASCII alphabetical order. */
    entry_status(GDEF),
    entry_status(GPOS),
    entry_status(GSUB),
    entry_status(OS2), /* Missing '/' restored on copying to table entry. */
    entry_status(cmap),
    entry_status(cvt), /* Missing space similarly dealt with. */
    entry_status(fpgm),
    entry_status(gasp),
    entry_status(glyf),
    entry_status(head),
    entry_status(hhea),
    entry_status(hmtx),
    entry_status(loca),
    entry_status(maxp),
    entry_status(name),
    entry_status(post),
    entry_status(prep)
  };
#else
/* Note that tables have to be in ASCII alphabetical order. */
#define do_tables(f)  f(COLR), f(CPAL), f(GDEF), f(GPOS), f(GSUB), f(OS2), \
    f(cmap), f(cvt),  f(fpgm), f(gasp), f(glyf), f(head), f(hhea), f(hmtx), \
    f(loca), f(maxp), f(name), f(post), f(prep)
#define entry_status1
#define entry_status_1(name) {0, 0, #name}
#define entry_status_2(name) table_statuses[i].have  = &have_ ## name,\
                             table_statuses[i].entry = &name ## _entry, i++
   struct table_status table_statuses[] = {
     do_tables(entry_status_1)
   };

#endif
  have_cmap = 1;
  have_maxp = 1;
  have_head = 1;
  have_loca = 1;
  have_glyf = 1;
  have_name = 1;
  have_hmtx = 1;
  have_hhea = 1;
  have_post = 1;
  have_OS2  = 1;
  if (!have_GSUB && !have_GPOS) have_GDEF = 0;
#if 0
  fprintf(stderr, "Have %d palettes and %d colours.\n",
          palette.numPalette, palette.numColorRecords);
#endif
  have_CPAL = palette.numPalette > 0 && palette.numColorRecords > 0;
  if (0) fprintf(stderr, "have_CPAL = %d.\n", have_CPAL);
  have_gasp = n_gasp >= 0;
  {
    int i = 0;
    do_tables(entry_status_2);
  }
/* Set up first approximation of the table directory */
  sfnt.sfnt_version = 1 << 16;
  sfnt.numTables = 0;
/* Directory entries */
  next_entry = &sfnt.table_directory;
  for (its = 0; its < sizeof table_statuses / sizeof * table_statuses; its++) {
    if (!*table_statuses[its].have) continue;
    *next_entry = table_statuses[its].entry;
    next_entry = &table_statuses[its].entry->_next;
    sfnt.numTables++;
    if (!strcmp(table_statuses[its].tag, "OS2")) {
      strncpy(table_statuses[its].entry->tag, "OS/2", 4);
    } else if (!strcmp(table_statuses[its].tag, "cvt")) {
      strncpy(table_statuses[its].entry->tag, "cvt ", 4);
    } else {
      strncpy(table_statuses[its].entry->tag, table_statuses[its].tag, 4);
    }
    if (0) fprintf(stdout, "Added %.4s to list.\n", table_statuses[its].tag);
    table_statuses[its].entry->checkSum = 0;
  }
  *next_entry = 0;
  {
    int n, pwr;
    for (n = 0, pwr = 1; 2 * pwr <= sfnt.numTables; pwr *= 2, n++);
    sfnt.searchRange = pwr * 16;
    sfnt.entrySelector = n;
    sfnt.rangeShift = sfnt.numTables * 16 - sfnt.searchRange;
  }
#define establish2(val, table)			   \
  if (val.is_forced) {				   \
    discrepancy(#val " in " #table " table", val); \
    table.val = val.forced_value;		   \
  } else {					   \
    table.val = val.calculated_value;		   \
  }

/* To get locations for data, just start writing! */
  old_pos = pos = 0;
  put_bine_offset_table(font, &sfnt, &follow, &pos);
#if 0
  fprintf(stdout, "Writing offset table makes next position %d from start.\n",
	  pos);
#endif

/* Populate head table */
  align_file(font, &pos, 4);
  head_entry.offset = old_pos = pos;
  head.tableVersionNumber = 1 << 16; /* 1.0 */
  head.fontRevision = fontRevision;
  head.checkSumAdjustment = 0;
  head.magicNumber = 0x5F0F3CF5;
  head.flags = head_flags;
  head.unitsPerEm = unitsPerEm;
  head.created = created;
  head.modified = modified;
  head.macStyle = macStyle;
  head.lowestRecPPEM = lowestRecPPEM;
  #define establish3(tname, fname, item) \
	if (item.is_forced) {\
          discrepancy(#fname " in " #tname " table", item);\
	  tname.fname = item.forced_value;\
	} else {\
	  tname.fname = item.calculated_value;\
	}
  establish3(head, xMin, head_xMin);
  establish3(head, yMin, head_yMin);
  establish3(head, xMax, head_xMax);
  establish3(head, yMax, head_yMax);
  {
    int hint;
    if (fDH_l2r.calculated_value) {
      if (fDH_r2l.calculated_value) {
	hint = 0;
      } else if (fDH_0.calculated_value) {
	hint = 2;
      } else {
	hint = 1;
      }
    } else {
      if (fDH_r2l.calculated_value) {
	if (fDH_0.calculated_value) {
	  hint = -2;
	} else {
	  hint = -1;
	}
      } else {
	if (fDH_0.calculated_value) {
	  hint = 2; /* -2 also acceptable! */
	} else {
	  hint = 1; /* -1 also acceptable! */
	}
      }
    }
    if (fontDirectionHint.is_forced) {
      if (hint != fontDirectionHint.forced_value) {
	fprintf(stderr, "fontDirection Hint is forced to %d but is calculated "
		"as %d because\n", fontDirectionHint.forced_value, hint);
	if (ALL_GLYPHS == fDH_l2r.causer_type && !fDH_l2r.calculated_value) {
	  fprintf(stderr, "  Font lacks strongly left-to-right glyphs.\n");
	} else if (BY_GLYPH == fDH_l2r.causer_type &&
		   fDH_l2r.calculated_value) {
	  fprintf(stderr, "  Glyph ");
	  put_gid_ps(stderr, fDH_l2r.causerID);
	  fprintf(stderr, " is strongly left-to-right.\n");
	} else {
	  fprintf(stderr, "  Internal error: Causer code %d for presence of "
		  "strongly left-to-right glyphs "
		  "with ID %d and value %d is unexpected.\n",
		  fDH_l2r.causer_type, fDH_l2r.causerID,
		  fDH_l2r.calculated_value);
	}

	if (ALL_GLYPHS == fDH_r2l.causer_type && !fDH_r2l.calculated_value) {
	  fprintf(stderr, "  Font lacks strongly right-to-left glyphs.\n");
	} else if (BY_GLYPH == fDH_r2l.causer_type &&
		   fDH_r2l.calculated_value) {
	  fprintf(stderr, "  Glyph ");
	  put_gid_ps(stderr, fDH_r2l.causerID);
	  fprintf(stderr, " is strongly right-to-left.\n");
	} else {
	  fprintf(stderr, "  Internal error: Causer code %d for presence of "
		  "strongly right-to-left glyphs "
		  "with ID %d and value %d is unexpected.\n",
		  fDH_r2l.causer_type, fDH_r2l.causerID,
		  fDH_r2l.calculated_value);
	}

	if (ALL_GLYPHS == fDH_0.causer_type && !fDH_0.calculated_value) {
	  fprintf(stderr, "  Font lacks neutral glyphs.\n");
	} else if (BY_GLYPH == fDH_0.causer_type &&
		   fDH_0.calculated_value) {
	  fprintf(stderr, "  Glyph ");
	  put_gid_ps(stderr, fDH_0.causerID);
	  fprintf(stderr, " is neutral.\n");
	} else {
	  fprintf(stderr, "  Internal error: Causer code %d for presence of "
		  "neutral glyphs "
		  "with ID %d and value %d is unexpected.\n",
		  fDH_0.causer_type, fDH_0.causerID,
		  fDH_0.calculated_value);
	}

      }
      head.fontDirectionHint = fontDirectionHint.forced_value;
    } else {
      head.fontDirectionHint = hint;
    }
  }
  if (lsbx0.is_forced) {
    discrepancy("lsbx0 flag in head", lsbx0);
    head.flags |= (!!lsbx0.forced_value) << 1;
  } else {
    head.flags |= (!!lsbx0.calculated_value) << 1;
  }
  if (ins_use_PPEM.is_forced) {
    discrepancy("ins_use_PPEM flag in head", ins_use_PPEM);
    head.flags |= (!!ins_use_PPEM.forced_value) << 2;
  } else {
    head.flags |= (!!ins_use_PPEM.calculated_value) << 2;
  }
  if (aw_by_PPEM.is_forced) {
    discrepancy("aw_by_PPEM flag in head", aw_by_PPEM);
    head.flags |= (!!aw_by_PPEM.forced_value) << 4;
  } else {
    head.flags |= (!!aw_by_PPEM.calculated_value) << 4;
  }
  if (indexToLocFormat_forced) {
    if (glyph_spp[last_glyph].location > 0xffff && !indexToLocFormat) {
      fprintf(stderr, "Glyph location changed to long "
	      "to fit amount of data.\n");
      head.indexToLocFormat = 1;
    } else {
      head.indexToLocFormat = indexToLocFormat;
    }
  } else {
    head.indexToLocFormat = glyph_spp[last_glyph].location > 0xffff;
  }
  LONG_OFFSETS = !!head.indexToLocFormat;
#if 0
/* Calculable items for the head table */
Done: struct calculable_item head_xMin, head_yMin, head_xMax, head_yMax;
Done: struct calculable_item fontDirectionHint;       /* forceable, not calculable */
struct calculable_item fDH_l2r, fDH_r2l, fDH_0; /* calculable, not forceable */
Done: /* fDH_r2l also affects flags!  Let it be forceable for ** flags ** bit 9 */
/* Calculable items for bits 1, 2, 4 of flags in head table. */
Done: struct calculable_item lsbx0, ins_use_PPEM, aw_by_PPEM;
/* Non-calculable items for the head table. */
Done: int fontRevision;
Done: unsigned short int unitsPerEm, lowestRecPPEM, head_flags, macStyle;
Done: unsigned long long int created, modified;
Done: short int indexToLocFormat;
#endif
  head.glyphDataFormat = 0;
  put_bine_font_header(font, &head, &follow, &pos);
  head_entry.length = pos - old_pos;


  align_file(font, &pos, 4);
  maxp_entry.offset = old_pos = pos;
/* Populate maxp */
  maxp.tableVersionNumber = 0 ? (5 << 12) /* 0.5 */ : (1 << 16) /* 1.0 */; 
  maxp.numGlyphs = last_glyph + 1;

/* Calculable items for maxp */
  establish2(maxPoints, maxp);
  establish2(maxContours, maxp);
  establish2(maxCompositePoints, maxp);
  establish2(maxCompositeContours, maxp);
  establish2(maxZones, maxp);
  establish2(maxTwilightPoints, maxp);
  establish2(maxStorage, maxp);
  establish2(maxFunctionDefs, maxp);
  establish2(maxInstructionDefs, maxp);
  establish2(maxStackElements, maxp);
  establish2(maxSizeOfInstructions, maxp);
  establish2(maxComponentElements, maxp);
  establish2(maxComponentDepth, maxp);
  fprintf(stderr, "WARNING: Most of maxp depends on manual input.\n");

  put_bine_maximum_profile(font, &maxp, &follow, &pos);
  maxp_entry.length = pos - old_pos;
  align_file(font, &pos, 4);

  cmap_entry.offset = pos;
/* Populate cmap */
  maxpt = maxp.numGlyphs;
  cpt = 0;
  gid = 0;
#define extend_cpt {\
  gid = realloc(gid, maxpt * sizeof(*gid));\
  cpt = realloc(cpt, maxpt * sizeof(*cpt));\
  if (!cpt || !gid) {\
    fprintf(stderr, "Memory exhausted while generating cmap!\n");\
    exit(1);\
  }}
  extend_cpt;
/* The way of probing charmap is not good.  Dream up better access routines. */
  if (charmap.plane_ptr[0]) {
    int block, off;
    int glyphno;
    datatype *the_block;

    npt = 0;
    for (block = 0; block < 512; block++) {
      if (the_block = *charmap.plane_ptr[0]->items[block]) {
	for (off = 0; off < 128; off++) {
	  if ((glyphno = (int)the_block[off]) >= 0) {
	    if (npt >= maxpt) {
	      maxpt = 1 + 1.5 * maxpt;
	      extend_cpt;
	    }
	    cpt[npt] = block * 128 + off;
#if 0
	    fprintf(stdout, "Character U+%4.4X uses glyph %d.\n",
		    cpt[npt], glyphno);
#endif
	    gid[npt++] = glyphno;
	  }
	}
      }
    }
  } else {
    npt = 1;
    cpt[0] = 0xffff;
    gid[0] = 0;
  }
  (void) make_cmap_subtable4(npt, cpt, gid, &bmp_table);

  {
    int off;
    void* ptrval;

    for (off = 0x10000; off <= 0x10ffff; off++) {
      if (get_property(charmap, off, &ptrval) && (int) ptrval >= 0) {
	if (npt >= maxpt) {
	  maxpt = 1 + 1.5 * maxpt;
	  extend_cpt;
	}
	cpt[npt] = off;
	gid[npt++] = (int) ptrval;
	havesup = 1;
      }
    }
  }
  {
    int ivs, off;
    int curvs = -1;
    void *ptrval;
    int glyph;
    struct variation_selector **vsptr = &vs_table.varSelector;

    vs_table.format = 14;
    vs_table.length = 0;
    vs_table.numVarSelectorRecords = 0;
    vs_table.varSelector = 0;
    vs_table._next = 0;
    for (ivs = 0; ivs < sizeof vscharmap / sizeof * vscharmap; ivs++) {
      int cduv = -2;
      int glyph, def_glyph;
      int vsri;
      if (!vscharmap[ivs].map) continue;
      for (off = 0; off <= 0x10ffff; off++) {
        if (0xd800 <= off && off < 0xe000) continue;
        if (get_property(*vscharmap[ivs].map, off, &ptrval) &&
            (glyph = (int) ptrval) >= 0) {
          if (ivs > curvs) {
            vsri = vs_table.numVarSelectorRecords ++;
            vs_table.varSelector =
              realloc(vs_table.varSelector,
                      vs_table.numVarSelectorRecords * sizeof *
                      vs_table.varSelector);
            VS_default =
              realloc(VS_default,
                      vs_table.numVarSelectorRecords * sizeof * VS_default);
            VS_nd =
              realloc(VS_nd,
                      vs_table.numVarSelectorRecords * sizeof * VS_nd);
            if (!vs_table.varSelector || !VS_default || !VS_nd) {
              fprintf(stderr, "Memory exhausted creating variation "
                      "selector table.\n");
              exit(1);
            }
            seti24(vs_table.varSelector[vsri].varSelector, vscharmap[ivs].vs);
            vs_table.varSelector[vsri].defaultUVSOffset = 0;
            vs_table.varSelector[vsri].nonDefaultUVSOffset = 0;
            VS_default[vsri].numUnicodeValueRanges = 0;
            VS_default[vsri].ranges = 0;
            VS_default[vsri]._next = 0;
            VS_nd[vsri].numUVSMappings = 0;
            VS_nd[vsri].uvsMappings = 0;
            VS_nd[vsri]._next = 0;
            havevs = 1;
            curvs = ivs;
          }
          def_glyph = get_property(charmap, off, &ptrval) ? (int) ptrval : -1;
#define unwritten 23
          if (glyph == def_glyph) {
            int incr, last;
            if (cduv + 1 == off) {
              last = VS_default[vsri].numUnicodeValueRanges - 1;
              incr = 1;
            } else {
              last = VS_default[vsri].numUnicodeValueRanges++;
              if (!(VS_default[vsri].ranges =
                    realloc(VS_default[vsri].ranges,
                            VS_default[vsri].numUnicodeValueRanges * sizeof *
                            VS_default[vsri].ranges))) {
                fprintf(stderr, "Memory exhausted creating variation "
                        "selector table.\n");
                exit(1);
              }
              incr = 0;
              seti24(VS_default[vsri].ranges[last].startUnicodeValue, off);
              VS_default[vsri].ranges[last].additionalCount = 0;
            }
            if (255 ==
                (VS_default[vsri].ranges[last].additionalCount += incr)) {
              cduv = -2;
            } else {
              cduv = off;
            }
          } else {
            int last = VS_nd[vsri].numUVSMappings++;
            if (-1 == def_glyph) {
              fprintf(stderr, "There is a glyph for U+%4.4X U+%4.4X but "
                      "not for U+%4.4X!\n", off, vscharmap[ivs].vs, off);
            }
            if (!(VS_nd[vsri].uvsMappings =
                  realloc(VS_nd[vsri].uvsMappings,
                          (last + 1) * sizeof * VS_nd[vsri].uvsMappings))) {
              fprintf(stderr, "Memory exhausted creating variation "
                      "selector table.\n");
              exit(1);
            }
            seti24(VS_nd[vsri].uvsMappings[last].unicodeValue, off);
            VS_nd[vsri].uvsMappings[last].glyphID = glyph;
          }
        }
      }
      if (curvs == ivs) {
        int k;
        struct unicode_range **rangeptr = &VS_default[vsri].ranges;
        struct UVS_mapping **mapptr = &VS_nd[vsri].uvsMappings;
        for (k = 0; k < VS_default[vsri].numUnicodeValueRanges; k++) {
          *rangeptr = VS_default[vsri].ranges + k;
          rangeptr = &VS_default[vsri].ranges[k]._next;
        }
        *rangeptr = 0;
        for (k = 0; k < VS_nd[vsri].numUVSMappings; k++) {
          *mapptr = VS_nd[vsri].uvsMappings + k;
          mapptr = &VS_nd[vsri].uvsMappings[k]._next;
        }
        *mapptr = 0;
      }
    }
    for (ivs = 0; ivs < vs_table.numVarSelectorRecords; ivs++) {
      *vsptr = vs_table.varSelector + ivs;
      vsptr = &vs_table.varSelector[ivs]._next;
    }
    *vsptr = 0;
  }
  cmap.version = 0;
  if (havesup) {
    (void) make_cmap_subtable12(npt, cpt, gid, &sup_table);
    bmp_ptr._next = &sup_ptr;
    bmp_ptr0._next = &sup_ptr0;
    sup_ptr.platformID = platID_Microsoft;
    sup_ptr.encodingID = ms_enc_UCS_4;
    sup_ptr.offset = 0;
    sup_ptr._next = 0;
    sup_ptr0.platformID = platID_Unicode;
    sup_ptr0.encodingID = uc_enc_UCS_4;
    sup_ptr0.offset = 0;
    sup_ptr0._next = 0;
    cmap.numTables = 2;
  } else {
    bmp_ptr._next = 0;
    bmp_ptr0._next = 0;
    cmap.numTables = 1;
  }
  cmap.encodingRecord = &bmp_ptr;
  cmap._next = 0;
  bmp_ptr.platformID = platID_Microsoft;
  bmp_ptr.encodingID = ms_enc_UCS_2;
  bmp_ptr.offset = 0;
  bmp_ptr0.platformID = platID_Unicode;
  bmp_ptr0.encodingID = uc_enc_UCS_2;
  bmp_ptr0.offset = 0;
  if (havevs) {
    bmp_ptr0._next = havesup ? &sup_ptr0 : &vs_ptr;
    sup_ptr0._next = &vs_ptr;
    vs_ptr.platformID = platID_Unicode;
    vs_ptr.encodingID = uc_enc_VS;
    vs_ptr.offset = 0;
    vs_ptr._next = 0; /* Well, for Unicode only. */
  }
/* Include Unicode encodings, if appropriate. */    
  if (havevs) {
    cmap.encodingRecord = &bmp_ptr0;
    /* Stitch together */
    if (havevs) {
      vs_ptr._next = &bmp_ptr;
    } else if (havesup) {
      sup_ptr0._next = &bmp_ptr;
    } else {
      bmp_ptr0._next = &bmp_ptr;
    }
    cmap.numTables += 1 + !!havesup + !!havevs;
  } else { /* Just Microsoft encoding */
    cmap.encodingRecord = &bmp_ptr;
  }
  cmap_pos = pos;
  put_bine_cmap_header(font, &cmap, &follow, &pos);
#if 0
  fprintf(stdout, "Writing cmap header makes next position %d from start.\n",
	  pos);
#endif
  old_pos = pos;
  bmp_ptr0.offset = bmp_ptr.offset = old_pos - cmap_pos;
  put_bine_encoding(font, &bmp_table, &follow, &pos);
  if (havesup) {
    sup_ptr0.offset = sup_ptr.offset = pos - cmap_pos;
    put_bine_encoding(font, &sup_table, &follow, &pos);
  }
  if (havevs) {
    int ivs;

    vs_ptr.offset = (vs_pos = pos) - cmap_pos;
    put_bine_encoding(font, &vs_table, &follow, &pos);
    for (ivs = 0; ivs < vs_table.numVarSelectorRecords; ivs++) {
      if (VS_default[ivs].numUnicodeValueRanges) {
        vs_table.varSelector[ivs].defaultUVSOffset = pos - vs_pos;
        put_bine_default_UVS_table(font, &VS_default[ivs], &follow, &pos);
      } else {
        vs_table.varSelector[ivs].defaultUVSOffset = 0;
      }
      if (VS_nd[ivs].numUVSMappings) {
        vs_table.varSelector[ivs].nonDefaultUVSOffset = pos - vs_pos;
        put_bine_non_default_UVS_table(font, &VS_nd[ivs], &follow, &pos);
      } else {
        vs_table.varSelector[ivs].nonDefaultUVSOffset = 0;
      }
    }
    vs_table.length = pos - vs_pos;
  }
#if 0
  fprintf(stdout, "Writing a character map makes next position %d from start.\n",
	  pos);
#endif
/* Now calculate the offsets and lengths */
  cmap_entry.length = pos - cmap_pos;
  /* Write the updated entries to the file. */
  old_pos = pos;
  fseek(font, pos = cmap_pos, SEEK_SET);
  put_bine_cmap_header(font, &cmap, &follow, &pos);
  if (havevs) {
    fseek(font, pos = vs_pos, SEEK_SET);
    put_bine_encoding(font, &vs_table, &follow, &pos);
  }
  fseek(font, pos = old_pos, SEEK_SET);

/* Populate loca table */
  align_file(font, &pos, 4);
  loca_entry.offset = old_pos = pos;
  for (iglyph = 0; iglyph <= last_glyph; iglyph++) {
    glyph_loc.offset = glyph_spp[iglyph].location;
    if (!LONG_OFFSETS) glyph_loc.offset /= 2;
    put_bine_one_location(font, &glyph_loc, &follow, &pos);
  }
  glyph_loc.offset = glyphs_size;
  if (!LONG_OFFSETS) glyph_loc.offset /= 2;
  put_bine_one_location(font, &glyph_loc, &follow, &pos);
  loca_entry.length = pos - old_pos;

/* Populate glyf table */
  align_file(font, &pos, 4);
  glyf_entry.offset = old_pos = pos;
  if (!(whole_table = malloc(glyphs_size))) {
    fprintf(stderr, "Memory exhausted transferring glyf table.\n");
    exit(1);
  }
  if (fseek(glyphs, 0, SEEK_SET)) {
    perror("Seek failed on glyph scratch file");
    exit(1);
  }
  if (fread(whole_table, 1, glyphs_size, glyphs) != glyphs_size) {
    perror("Copying from glyph scratch file");
    exit(1);
  }
  if (fwrite(whole_table, 1, glyphs_size, font) != glyphs_size) {
    perror("Loading glyf table");
    exit(1);
  }
  free(whole_table);
  pos += glyphs_size;
  glyf_entry.length = pos - old_pos;

  align_file(font, &pos, 4);

/* Populate name table */
  name_entry.offset = name_pos = old_pos = pos;
  name.format = 0;
  name.stringOffset = 0;
  sort_map(name_names, &names, &name_count);
  name.count = name_count;
  name.nameRecord = malloc(name.count * sizeof(*name.nameRecord));
  if (name.nameRecord) {
    for (iname = 0; iname < name.count - 1; iname++) {
      name.nameRecord[iname]._next = &name.nameRecord[iname+1];
    }
  } else {
    fprintf(stderr, "Memory exhausted preparing name table.\n");
    exit(1);
  }
  /* Write draft name table */
  put_bine_name_table(font, &name, &follow, &pos);
  string_pos = pos;
  name.stringOffset = string_pos - old_pos;
  for (iname = 0; iname < name.count; iname++) {
    char *nexhar;
    int as_is, n_to_go;
    unsigned int code;
    enum {NONE, UNICODE, LITERAL_X, LITERAL_N, LITERAL_U, BACKSLASH} multiway;
    enum {extended_ASCII, as_ASCII, as_LATIN_1, as_UNICODE} praxis;
    struct one_value cu;

#if 0
    fprintf(stdout, "%s->%s\n", names[iname]->key, names[iname]->datum);
#endif
    sscanf(names[iname]->key, "%4hx%4hx%4hx%4hx",
	   &name.nameRecord[iname].platformID,
	   &name.nameRecord[iname].encodingID,
	   &name.nameRecord[iname].languageID,
	   &name.nameRecord[iname].nameID);
    switch (name.nameRecord[iname].platformID) {
    case 0: /* Unicode */
      praxis = as_UNICODE;
      break;
    case 1: /* Macintosh */
      if (0 == name.nameRecord[iname].encodingID) {
	praxis = as_LATIN_1;
      } else {
	praxis = extended_ASCII;
      }
      break;
    case 2: /* ISO */
      switch (name.nameRecord[iname].encodingID) {
      case 0:
	praxis = as_ASCII;
	break;
      case 1:
	praxis = as_UNICODE;
	break;
      case 2:
	praxis = as_LATIN_1;
	break;
      default:
	praxis = extended_ASCII; /* Success not guaranteed! */
      }
      break;
    case 3:
      switch (name.nameRecord[iname].encodingID) {
      case 0: /* Symbol */
	praxis = as_LATIN_1;
	break;
      case 1:  /* Unicode BMP only */
      case 10: /* Unicode full repertoire - assume UTF-16 */
	praxis = as_UNICODE;
	break;
      case 2: /* ShiftJIS */
      case 3: /* PRC */
      case 4: /* Big5 */
      case 5: /* Wansung */
      case 6: /* Johab */
	praxis = extended_ASCII;
	break;
      default:
	praxis = extended_ASCII; /* Wild guess */
      }
      break;
    default: /* Undefined platform */
      praxis = extended_ASCII; /* Wild guess */
    }
    name.nameRecord[iname].offset = pos - string_pos;
    old_pos = pos;
    nexhar = names[iname]->datum;
    n_to_go = 0;
    while (*nexhar) {
      if (!n_to_go) as_is = 0;
      if (n_to_go < 0) n_to_go = 0;
      if (!n_to_go) multiway = NONE;
      if (n_to_go) {
	switch (multiway) {
	case UNICODE:
	  if ((*nexhar & 0x0c0) == 0x80) {
	    code = (code << 6) + (*nexhar & 0x3f);
	    n_to_go--;
	  } else { /* Bad sequence */
	    code <<= (6 * n_to_go);
	    n_to_go = 0;
	    nexhar--; /* Reconsider current byte */
	  }
	  break;
	case LITERAL_X:
	case LITERAL_N:
	case LITERAL_U:
	  if ('0' <= *nexhar && *nexhar <= '9') {
	    code = (code << 4) + (*nexhar - '0');
	    n_to_go--;
	  } else if ('a' <= *nexhar && *nexhar <= 'f') {
	    code = (code << 4) + (*nexhar - 'a' + 10);
	    n_to_go--;
	  } else if ('A' <= *nexhar && *nexhar <= 'F') {
	    code = (code << 4) + (*nexhar - 'A' + 10);
	    n_to_go--;
	  } else { /* Premature end of sequence */
	    n_to_go = 0;
	    nexhar--; /* Reconsider current byte */
	  }
	  break;
	case BACKSLASH:
	  n_to_go = 0;
	  as_is = 0;
	  switch (*nexhar) {
	  case 'a':
	    code = 7; /* BEL */
	    break;
	  case 'b':
	    code = 8; /* BS */
	    break;
	  case 'f':
	    code = 12; /* FF */
	    break;
	  case 'n':
	    code = 10; /* NL */
	    break;
	  case 'N':
	    multiway = LITERAL_N;
	    as_is = 1;
	    n_to_go = 4;
	    code = 0;
	    break;
	  case 'r':
	    code = 13; /* CR */
	    break;
	  case 't':
	    code = 9; /* TAB */
	    break;
	  case 'v':
	    code = 11; /* VT */
	    break;
	  case 'x':
	  case 'X':
	    multiway = LITERAL_X;
	    as_is = 1;
	    n_to_go = 2;
	    code = 0;
	    break;
	  case 'u':
	    multiway = LITERAL_U;
	    as_is = 0;
	    n_to_go = 4;
	    code = 0;
	    break;
	  case 'U':
	    multiway = LITERAL_U;
	    as_is = 0;
	    n_to_go = 6;
	    code = 0;
	    break;
	  case '0':
	    code = 0; /* NUL */
	    break;
	  case '\\':
	    as_is = 0;
	    code = '\\';
	    break;
	  default:
	    /* Ignore backslash */
	    n_to_go = -1;
	    nexhar--;
	  }
	  break;
	default:
	  fprintf(stderr, "Internal error: multiway = %d.\n", (int) multiway);
	  exit(1);
	}
      } else if ('\\' == *nexhar) {
	multiway = BACKSLASH;
	n_to_go = 1;
      } else if (0 <= *nexhar && *nexhar <= 127) {
	code = *nexhar;
      } else if ((*nexhar & 0x0c0) == 0x80) {
	/* UTF-8 continuation-byte - treat as Latin-1 */
	code = *nexhar & 0xff;
      } else if ((*nexhar & 0x0e0) == 0xc0) {
	code = *nexhar & 0x1f;
	n_to_go = 1;
	multiway = UNICODE;
      } else if ((*nexhar & 0x0f0) == 0xe0) {
	code = *nexhar & 0x0f;
	n_to_go = 2;
	multiway = UNICODE;
      } else if ((*nexhar & 0x0f8) == 0xf0) {
	code = *nexhar & 0x07;
	n_to_go = 3;
	multiway = UNICODE;
      } else {
	/* Invalid for Unicode - no UTF-16 representation.
	   Treat as Latin-1 */
	code = *nexhar & 0xff;
      }
      nexhar++;
      if (!n_to_go) {
/*    enum {extended_ASCII, as_ASCII, as_LATIN_1, as_UNICODE} praxis; */
	switch (praxis) {
	case extended_ASCII:
	  cu.val = code;
	  switch (multiway) {
	  case NONE:
	  case UNICODE:
	  case LITERAL_U:
	    cu.size = 1;
	    if (code >= 128) {
	      fprintf(stderr, "Name %s has unflagged non-ASCII "
		      "character 0x%4.4x.\n", names[iname]->key, code);
	    }
	    break;
	  case LITERAL_X:
	    cu.size = 1;
	    break;
	  case LITERAL_N:
	    cu.size = 2;
	    break;
	  case BACKSLASH:
	  default:
	    fprintf(stderr, "Name undefined character specification "
		    "coding %d encountered at line %d.\n", praxis, __LINE__);
	    abort();
	  }
	  break;
	case as_ASCII:
	  cu.size = 1;
	  cu.val = code;
	  if (code >= 128) {
	    fprintf(stderr, "Name %s has non-ASCII character 0x%4.4x.\n",
		    names[iname]->key, code);
	  }
	  break;
	case as_LATIN_1:
	  cu.size = 1;
	  cu.val = code;
	  if (code >= 256) {
	    fprintf(stderr, "Name %s has non-Latin-1 character 0x%4.4x.\n",
		    names[iname]->key, code);
	  }
	  break;
	case as_UNICODE:
	  cu.size = 2;
	  if (code <= 0xffff) {
	    cu.val = code;
	  } else {
	    int high_surr, low_surr;
	    low_surr = 0xdc00 | (code & 0x3ff);
	    high_surr = 0xd800 | ((code - 0x10000) >> 10);
	    cu.val = high_surr;
	    put_bine_one_value(font, &cu, &follow, &pos);
	    cu.val = low_surr;
	  }
	  break;
	default:
	  fprintf(stderr, "Undefined strategy %d chosen for output of "
		  "name %s.\n", praxis, names[iname]->key);
	  exit(1);
	}
	put_bine_one_value(font, &cu, &follow, &pos);
      }
    }
    name.nameRecord[iname].length  = pos - old_pos;
  }
  name_entry.length = pos - name_pos;
  /* Write final name table */
  if (fseek(font, name_pos, SEEK_SET)) {
    perror("Writing name table");
    exit(1);
  }
  old_pos = pos;
  put_bine_name_table(font, &name, &follow, &pos);
  if (fseek(font, pos = old_pos, SEEK_SET)) {
    perror("Writing name table");
    exit(1);
  }
  align_file(font, &pos, 4);
/* Generate horizontal header */
  hhea_entry.offset = old_pos = pos;
  hhea.tableVersionNumber = 1 << 16; /* 1.0 */
  hhea.ascender = ascender;
  hhea.descender = descender;
  hhea.lineGap = lineGap;
  establish2(advanceWidthMax, hhea);
  establish2(minLeftSideBearing, hhea);
  establish2(minRightSideBearing, hhea);
  establish2(xMaxExtent, hhea);
  hhea.caretSlopeRise = caretSlopeRise;
  hhea.caretSlopeRun  = caretSlopeRun;
  hhea.caretOffset    = caretOffset;
  hhea.res_0 = 0;
  hhea.res_1 = 0;
  hhea.res_2 = 0;
  hhea.res_3 = 0;
  hhea.metricDataFormat = 0;
  hhea.numberofHMetrics = hhea_numberofHMetrics;
  put_bine_horizontal_header(font, &hhea, &follow, &pos);
  hhea_entry.length = pos - old_pos;
  align_file(font, &pos, 4);
/* Generate horizontal metrics */
  hmtx_entry.offset = old_pos = pos;
  for (iglyph = 0; iglyph < hhea_numberofHMetrics; iglyph++) {
    struct longHorMetric longhor;

    longhor.advanceWidth = glyph_spp[iglyph].advanceWidth;
    longhor.lsb          = glyph_spp[iglyph].leftSideBearing;
    put_bine_longHorMetric(font, &longhor, &follow, &pos);
  }
  for (iglyph = hhea_numberofHMetrics; iglyph <= last_glyph; iglyph++) {
    struct one_value the_lsb;

    the_lsb.size = 2;
    the_lsb.val  = (unsigned short int) glyph_spp[iglyph].leftSideBearing;
    put_bine_one_value(font, &the_lsb, &follow, &pos);
  }
  hmtx_entry.length = pos - old_pos;
  align_file(font, &pos, 4);

#define myround(x) (int) (((x) >= 0 ? 0.5 : -0.5) + (x))
  if (have_OS2) {
  /* Generate OS/2 table */
    int i;
    const int nvals = n_script_table;

    OS2_entry.offset = old_pos = pos;
    OS2.version = 4; /* Ideal */
    OS2.version = 3; /* Microsoft Font Validator objects to 4! */
    if (nCharWidth) {
      int avg;
      avg = myround((double) totCharWidth / nCharWidth);
      if (0 == avg) avg = 1; /* 0 is dangerous value */
      xAvgCharWidth.calculated_value = avg;
      xAvgCharWidth.causer_type = ALL_GLYPHS;
    } else {
      xAvgCharWidth.calculated_value = head.unitsPerEm;
      xAvgCharWidth.causer_type = BY_GLYPH;
    }
    xAvgCharWidth.causerID = 0;
    establish2(xAvgCharWidth, OS2);
    OS2.usWeightClass = usWeightClass;
    OS2.usWidthClass = usWidthClass;
    OS2.fsType = fsType;
    if (-1 == ySubscriptXSize) {
      OS2.ySubscriptXSize = 2 * head.unitsPerEm / 3;
    } else {
      OS2.ySubscriptXSize = ySubscriptXSize;
    }
    if (-1 == ySubscriptYSize) {
      OS2.ySubscriptYSize = 2 * head.unitsPerEm / 3;
    } else {
      OS2.ySubscriptYSize = ySubscriptYSize;
    }
    OS2.ySubscriptXOffset = ySubscriptXOffset;
    if (-1 == ySubscriptYOffset) {
      OS2.ySubscriptYOffset = -head.unitsPerEm / 3;
    } else {
      OS2.ySubscriptYOffset = ySubscriptYOffset;
    }
    if (-1 == ySuperscriptXSize) {
      OS2.ySuperscriptXSize = 2 * head.unitsPerEm / 3;
    } else {
      OS2.ySuperscriptXSize = ySuperscriptXSize;
    }
    if (-1 == ySuperscriptYSize) {
      ySuperscriptYSize = 2 * head.unitsPerEm / 3;
    } else {
      OS2.ySuperscriptYSize = ySuperscriptYSize;
    }
    OS2.ySuperscriptXOffset = ySuperscriptXOffset;
    if (-1 == ySuperscriptYOffset) {
      OS2.ySuperscriptYOffset = +head.unitsPerEm / 3;
    } else {
      OS2.ySuperscriptYOffset = ySuperscriptYOffset;
    }
    if (-1 == yStrikeoutSize) {
      OS2.yStrikeoutSize = head.unitsPerEm / 20;
    } else {
      OS2.yStrikeoutSize = yStrikeoutSize;
    }
    if (-1 == yStrikeoutPosition) {
      yStrikeoutPosition = head.unitsPerEm / 4;
    } else {
      OS2.yStrikeoutPosition = yStrikeoutPosition;
    }
    OS2.sFamilyClass = sFamilyClass;
    for (i = 0; i < 10; i++) OS2.panose[i]  = panose[i];
    OS2.ulUnicodeRange1 = 0;
    OS2.ulUnicodeRange2 = 0;
    OS2.ulUnicodeRange3 = 0;
    OS2.ulUnicodeRange4 = 0;
    for (i = 0; i < 4 * 32; i++) {
      int pos;
      char str[100];
      int bitval; 
      if (ulUnicodeRange[i].is_forced) {
	for (pos = 0; pos < nvals; pos++) {
	  if (script_table[pos].bit == i) {
	    sprintf(str, "%s bit in ulUnicodeRange in OS/2 table",
		    script_table[pos].sn);
	    break;
	  }
	}
	if (pos >= nvals) {
	  sprintf(str, "Bit %d of ulUnicodeRange in OS.2 table", i);
	}
	if (ulUnicodeRange[i].calculated_value) {
	  ulUnicodeRange[i].forced_value *= ulUnicodeRange[i].calculated_value;
	}
	discrepancy(str, ulUnicodeRange[i]);
	bitval = ulUnicodeRange[i].forced_value;
      } else {
	bitval = ulUnicodeRange[i].calculated_value;
      }
      if (!bitval) {
	/* Nothing to do. */
      } else if (i < 32) {
	OS2.ulUnicodeRange1 |= (1 << i);
      } else if (i < 2 * 32) {
	OS2.ulUnicodeRange2 |= (1 << (i - 32));
      } else if (i < 3 * 32) {
	OS2.ulUnicodeRange3 |= (1 << (i - 2 * 32));
      } else {
	OS2.ulUnicodeRange4 |= (1 << (i - 3 * 32));
      }
    }
    strncpy(OS2.achVendID, achVendID, 4);
#define establish3a(tname, sname, item)		\
    if (item.is_forced) {					\
      discrepancy(#item " in " tname " table", item);		\
      sname.item = item.forced_value;				\
    } else {							\
      sname.item = item.calculated_value;			\
    }
    OS2.fsSelection = fsSelection;
    establish3a("OS/2", OS2, usFirstCharIndex);
    establish3a("OS/2", OS2, usLastCharIndex);
    OS2.sTypoAscender = sTypoAscender;
    OS2.sTypoDescender = sTypoDescender;
    OS2.sTypoLineGap = sTypoLineGap;
    usWinAscent.calculated_value = head_yMax.calculated_value;
    usWinAscent.causer_type = head_yMax.causer_type;
    usWinAscent.causerID = head_yMax.causerID;
    establish3a("OS/2", OS2, usWinAscent);
    usWinDescent.calculated_value = -head_yMin.calculated_value;
    usWinDescent.causer_type = head_yMin.causer_type;
    usWinDescent.causerID = head_yMin.causerID;
    establish3a("OS/2", OS2, usWinDescent);
    OS2.ulCodePageRange1 = ulCodePageRange[0];
    OS2.ulCodePageRange2 = ulCodePageRange[1];
    sxHeight.causer_type = NO_CAUSER;
    sCapHeight.causer_type = NO_CAUSER;
    for (i = 0; i <= 120 && i < npt; i++) {
      int pos, junk;
      struct glyph_entry the_glyph;

      if ('H' == cpt[i]) {
	pos = glyph_spp[gid[i]].location;
	if (fseek(glyphs, pos, SEEK_SET)) {
	  perror("Seeking failed in glyph scratch file for 'H'.");
	  exit(1);
	}
	get_bine_glyph_entry(glyphs, &the_glyph, &junk, &pos);
	sCapHeight.causer_type = BY_GLYPH;
	sCapHeight.causerID = gid[i];
	sCapHeight.calculated_value = the_glyph.yMax;
	free_glyph_entry(&the_glyph);
      } else if ('x' == cpt[i]) {
	pos = glyph_spp[gid[i]].location;
	if (fseek(glyphs, pos, SEEK_SET)) {
	  perror("Seeking failed in glyph scratch file for 'x'.");
	  exit(1);
	}
	get_bine_glyph_entry(glyphs, &the_glyph, &junk, &pos);
	sxHeight.causer_type = BY_GLYPH;
	sxHeight.causerID = gid[i];
	sxHeight.calculated_value = the_glyph.yMax;
	free_glyph_entry(&the_glyph);
	break;
      }
    }
    establish3a("OS/2", OS2, sxHeight);
    establish3a("OS/2", OS2, sCapHeight);
    OS2.usDefaultChar = usDefaultChar;
    OS2.usBreakChar = usBreakChar;
    establish3a("OS/2", OS2, usMaxContext);
    put_bine_OS2_metrics(font, &OS2, &follow, &pos);
    OS2_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  fclose(glyphs);
  if (have_post) {
    int ig, idx;
    struct glyph_group *gg;

  /* Generate post table */
    post_entry.offset = old_pos = pos;
    post.version = 2 << 16; /* 2.0 */

    post.italicAngle = (int) myround(italicAngle * (1<<16));
    post.underlinePosition = underlinePosition;
    if (-1 == underlineThickness) underlineThickness = head.unitsPerEm / 20;
    post.underlineThickness = underlineThickness;
    isFixedPitch.calculated_value = !!isFixedPitch.calculated_value;
    establish2(isFixedPitch, post);
    post.minMemType42 = minMemType42;
    post.maxMemType42 = maxMemType42;
    post.minMemType1 = minMemType1;
    post.maxMemType1 = maxMemType1;
    post.numberOfGlyphs = maxp.numGlyphs;
    if (!(post.glyphNameIndex = malloc(maxp.numGlyphs * sizeof(short int)))) {
      fprintf(stderr, "Memory exhausted preparing post table.\n");
      exit(1);
    }
    memset(post.glyphNameIndex, 0, maxp.numGlyphs * sizeof(short int));
    for (ig = 3; ig < n_ordered_mac_name; ig++) {
      if (lookup_in_map(glyph_names, ordered_mac_name[ig], (void**)&gg) &&
	  SOLO == gg->grouping &&
	  !strcmp(postscript_name[gg->x.number], ordered_mac_name[ig])) {
	post.glyphNameIndex[gg->x.number] = ig;
      }
    }
    idx = 258;
    for (ig = 0; ig < maxp.numGlyphs; ig++) {
      if (post.glyphNameIndex[ig]) continue; /* Already done */
      if (!postscript_name[ig]) continue; /* No postscript name */
      if (!strcmp(postscript_name[ig], ".notdef")) continue;
      post.glyphNameIndex[ig] = idx++;
    }
    put_bine_postscript_information_nonames(font, &post, &follow, &pos);
    for (ig = 0; ig < maxp.numGlyphs; ig++) {
      int slen;
      if (post.glyphNameIndex[ig] < 258) continue; /* Standard name */
      fprintf(font, "%c", slen = strlen(postscript_name[ig])); pos++;
      fprintf(font, "%s", postscript_name[ig]); pos += slen;
    }
    post_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  if (have_fpgm) {
  /* Generate fpgm table */
    struct font_program fpgm;

    fpgm_entry.offset = old_pos = pos;
    fpgm.datum = fpgm_instr;
    FPGM_SIZE = n_fpgm_instr;
    put_bine_font_program(font, &fpgm, &follow, &pos);
    fpgm_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  if (have_prep) {
  /* Generate prep table */
    struct control_value_program prep;

    prep_entry.offset = old_pos = pos;
    prep.program = prep_instr;
    PREP_SIZE = n_prep_instr;
    put_bine_control_value_program(font, &prep, &follow, &pos);
    prep_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  if (have_cvt) {
/* Generate control value table */
    struct control_value_table cvt;
    cvt_entry.offset = old_pos = pos;
    cvt.datum = p_cvt;
    CVT_SIZE = n_cvt;
    put_bine_control_value_table(font, &cvt, &follow, &pos);
    cvt_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  if (have_GDEF) {
    struct gdef_table GDEF;
    struct class_definition_table cdt;
    struct class_range_record *crr;
    int igl, ir, nr, nval, oval, ngms;
    int tmpos;

    GDEF_entry.offset = old_pos = pos;
    memset(&GDEF, 0, sizeof GDEF);
    GDEF.version = (1 << 16) + 2; /* 1.2 */
    put_bine_gdef_table(font, &GDEF, &follow, &pos);

    nr = 0;
    oval = NO_CATEGORY;
    for (igl = 0; igl < N_GLYPHS; igl++) {
      nval = glyph_spp[igl].category;
      if (oval != nval && NO_CATEGORY != nval) nr++;
      oval = nval;
    }
    if (nr) {
      if (!(crr = malloc(nr * sizeof * crr))) {
	fprintf(stderr, "Memory exhausted.\n");
      }
      cdt.classFormat = 2;
      cdt.classRangeCount = nr;
      cdt.classRangeRecord = crr;
      for (ir = 0; ir < nr - 1; ir++) crr[ir]._next = &crr[ir+1];
      crr[ir]._next = 0;
      ir = 0;
      oval = NO_CATEGORY;
      for (igl = 0; igl < N_GLYPHS; igl++) {
	nval = glyph_spp[igl].category;
	if (oval == nval) {
	  /* Do nothing */
	} else {
	  if (NO_CATEGORY != oval) crr[ir++].end = igl - 1;
	  if (NO_CATEGORY != nval) {
	    crr[ir].start = igl;
	    crr[ir].class = nval;
	  }
	}
	oval = nval;
      }
      if (NO_CATEGORY != oval) crr[ir].end = N_GLYPHS - 1;
      GDEF.glyphClassDef = pos - old_pos;
      put_bine_class_definition_table(font, &cdt, &follow, &pos);
      free(crr); crr = 0;
    }

    add_location(GDEF_TGT, old_pos, 10 + old_pos, 2, &attachment_classes->lox);
    out_pos = &pos;
    outfp = font;
    GDEF.markAttachClassDef = pos - old_pos;
    store_classification(" -MAC- ", GDEF_TGT, attachment_classes);
    GDEF.markGlyphSetsDef = 0;
#if 1
    GDEF.markGlyphSetsDef = pos - old_pos;
    ngms = store_glyph_sets();
    if (0 == ngms) GDEF.markGlyphSetsDef = 0;
#endif
    out_pos = 0;
    outfp = 0;
    /* Rewrite updated table header */
    tmpos = old_pos;
    if (fseek(font, tmpos, SEEK_SET)) {
      perror("Writing GDEF table");
      exit(1);
    }
    put_bine_gdef_table(font, &GDEF, &follow, &tmpos);
    if (fseek(font, pos, SEEK_SET)) {
      perror("Writing GDEF table");
      exit(1);
    }
/* unsigned char attachment_class; /* As in GDEF */
/* #define NO_MAC 0xFF /* No mark attachment class to be recorded */
    GDEF_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
  if (have_GSUB || have_GPOS) {
    store_all_lookups();
    check_all_subtables();
    store_all_used_coverages();
    store_all_used_classifications();
  }
  if (have_GSUB) {
    int ngot;
/* Populate GSUB table */
    GSUB_entry.offset = old_pos = pos;
    if (!(whole_table = malloc(gsub_pos))) {
      fprintf(stderr, "Memory exhausted transferring GSUB table.\n");
      exit(1);
    }
    if (fseek(gsub, 0, SEEK_SET)) {
      perror("Seek failed on GSUB scratch file");
      exit(1);
    }
    if ((ngot = fread(whole_table, 1, gsub_pos, gsub)) != gsub_pos) {
      fprintf(stderr, "Got only %d of %d.\n", ngot, gsub_pos);
      perror("Copying from GSUB scratch file");
      exit(1);
    }
    if (fwrite(whole_table, 1, gsub_pos, font) != gsub_pos) {
      perror("Loading GSUB table");
      exit(1);
    }
    free(whole_table);
    pos += gsub_pos;
    GSUB_entry.length = pos - old_pos;
    
    align_file(font, &pos, 4);
  }

  if (have_GPOS) {
    int ngot;
/* Populate GPOS table */
    GPOS_entry.offset = old_pos = pos;
    if (!(whole_table = malloc(gpos_pos))) {
      fprintf(stderr, "Memory exhausted transferring GPOS table.\n");
      exit(1);
    }
    if (fseek(gpos, 0, SEEK_SET)) {
      perror("Seek failed on GPOS scratch file");
      exit(1);
    }
    if ((ngot = fread(whole_table, 1, gpos_pos, gpos)) != gpos_pos) {
      fprintf(stderr, "Got only %d of %d.\n", ngot, gpos_pos);
      perror("Copying from GPOS scratch file");
      exit(1);
    }
    if (fwrite(whole_table, 1, gpos_pos, font) != gpos_pos) {
      perror("Loading GPOS table");
      exit(1);
    }
    free(whole_table);
    pos += gpos_pos;
    GPOS_entry.length = pos - old_pos;
    
    align_file(font, &pos, 4);
  }

  if (have_CPAL) {
    struct cra cra;
    struct pta pta;
    struct pla pla;
    struct pela pela;
    int i;
    int tmpos;
    int dopta = 0, dopla = 0, dopela = 0;

    if (0) fprintf(stderr, "Writing CPAL table.\n");
    CPAL_entry.offset = old_pos = pos;
    palette.version = 0;
    palette.offsetPaletteTypeArray = 0;
    palette.offsetPaletteLabelArray = 0;
    palette.offsetPaletteEntryLabelArray = 0;

    for (i = 0; i < palette.numPalette; i++) {
      if (0)fprintf(stderr, "pal_type[%d] = %8.8x\n", i, pal_type[i]);
      if (pal_type[i]) {
        dopta = 1;
        pta.paletteType = pal_type;
        palette.version = 1;
        break;
      }
    }

    for (i = 0; i < palette.numPalette; i++) {
      if (0) fprintf(stderr, "pal_label[%d] = %4.4hx\n", i, pal_type[i]);
      if (0xffff != pal_label[i]) {
        dopla = 1;
        palette.version = 1;
        pla.paletteLabel = pal_label;
        break;
      }
    }

    if (pe_label) {
      for (i = 0; i < palette.numPalettesEntries; i++) {
        if (0)fprintf(stderr, "pe_label[%d] = %4.4hx\n", i, pe_label[i]);
        if (0xffff != pe_label[i]) {
          dopela = 1;
          pela.paletteEntryLabel = pe_label;
          pela._n_paletteEntryLabel = palette.numPalettesEntries;
          palette.version = 1;
          break;
        }
      }
    }

    put_bine_cpal_header(font, &palette, &follow, &pos);
    palette.offsetFirstColorRecord = pos - old_pos;
    cra.x = colval;
    for (i = 0; i < palette.numColorRecords; i++) {
      cra.x[i]._next = &cra.x[i+1];
    }
    cra.x[i]._next = 0;
    put_bine_cra(font, &cra, &follow, &pos);
    if (dopla) {
      palette.offsetPaletteLabelArray = pos - old_pos;
      put_bine_pla(font, &pla, &follow, &pos);
    }
    if (dopela) {
      palette.offsetPaletteEntryLabelArray = pos - old_pos;
      put_bine_pela(font, &pela, &follow, &pos);
    }
    if (dopta) {
      align_file(font, &pos, 4);
      palette.offsetPaletteTypeArray = pos - old_pos;
      put_bine_pta(font, &pta, &follow, &pos);
    }

    /* Rewrite updated table header */
    tmpos = old_pos;
    if (fseek(font, tmpos, SEEK_SET)) {
      perror("Rewriting CPAL header");
      exit(1);
    }
    put_bine_cpal_header(font, &palette, &follow, &tmpos);
    if (fseek(font, pos, SEEK_SET)) {
      perror("Updating CPAL table");
      exit(1);
    }

    CPAL_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }

  if (have_COLR) {
    int tmpos;
    int i;
    struct base_glyph_records bgr;
    struct layer_records layers;

    COLR_entry.offset = old_pos = pos;
    put_bine_colr_header(font, &colr, &follow, &pos);

    bgr.base = colr.baseGlyphRecord;
    for (i = 0; i < N_COLR_BGR; i++) {
      bgr.base[i]._next = &bgr.base[i+1];
#if 0
      fprintf(stdout, "%4hu %2hu %hu\n", bgr.base[i].GID,
              bgr.base[i].firstLayerIndex, bgr.base[i].numLayers);
#endif
    }
    bgr.base[N_COLR_BGR-1]._next = 0;
    colr.offsetBaseGlyphRecord = pos - old_pos;
    put_bine_base_glyph_records(font, &bgr, &follow, &pos);

    layers.layers = colr.layerRecord;
    for (i = 0; i < N_COLR_LAYER; i++) {
      layers.layers[i]._next = &layers.layers[i+1];
#if 0
      fprintf(stdout, "%4hu %hu\n",
              layers.layers[i].GID, layers.layers[i].paletteIndex);
#endif
    }
    layers.layers[N_COLR_LAYER-1]._next = 0;
    colr.offsetLayerRecord = pos - old_pos;
    put_bine_layer_records(font, &layers, &follow, &pos);

    tmpos = old_pos;
    if (fseek(font, tmpos, SEEK_SET)) {
      perror("Rewriting COLR header");
      exit(1);
    }
    put_bine_colr_header(font, &colr, &follow, &tmpos);
    if (fseek(font, pos, SEEK_SET)) {
      perror("Updating COLR table");
      exit(1);
    }
    COLR_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }

  if (have_gasp) {
/* Generate gash table */
    int tmpos, irge;
    unsigned short int max_gasp;
    struct gasp_header gasp;
    gasp_entry.offset = old_pos = pos;
    gasp.numRanges = n_gasp;
    gasp.gaspRanges = v_gasp;
    max_gasp = n_gasp > 0 ? v_gasp[0].behaviour : 0;
    for (irge = 1; irge < n_gasp; irge++) {
      v_gasp[irge-1]._next = &v_gasp[irge];
      if (v_gasp[irge].behaviour > max_gasp) max_gasp = v_gasp[irge].behaviour;
    }
    gasp.version = max_gasp > 2;
    if (n_gasp) v_gasp[n_gasp - 1]._next = 0;  
    put_bine_gasp_header(font, &gasp, &follow, &pos);
    gasp_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }

#if 0
  if (have_spec) {
  /* Generate specimen table */
    int tmpos;
    spec_entry.offset = old_pos = pos;
    spec.tableVersionNumber = 1 << 16; /* 1.0 */
    etc.;
    put_bine_whatever(font, &spec, &follow, &pos);
    Put other structures;
    tmpos = old_pos;
    if (fseek(font, tmpos, SEEK_SET)) {
      perror("Rewriting spec header");
      exit(1);
    }
    put_bine_whatever(font, &spec, &follow, &tmpos);
    if (fseek(font, pos, SEEK_SET)) {
      perror("Updating spec table");
      exit(1);
    }
    spec_entry.length = pos - old_pos;
    align_file(font, &pos, 4);
  }
#endif
/* Correct the directory entries - but still need to update checksums! */
  fseek(font, 0, SEEK_SET);
  put_bine_offset_table(font, &sfnt, &follow, &pos);
}

const char *get_glyph_name(unsigned short int gi) {
  static char buffer[2][20];
  static int which = 0;

  sprintf(buffer[!which], "glyph %d", gi);
  which = !which;
  if (gi > last_glyph) {
    return buffer[which];
  } else {
    if (postscript_name[gi]) {
      return postscript_name[gi];
    } else {
      struct map_entry **nl;
      int in, nn;
      char *res = 0;

      sort_map(glyph_names, &nl, &nn);
      for (in = 0; in < nn; in++) {
	struct glyph_group *gg;
	gg = nl[in]->datum;
	if (SOLO == gg->grouping && gi == gg->x.number) {
	  res = nl[in]->key;
	  break;
	}
      }
      free(nl);
      return res ? res : buffer[which];
    }
  }
}

