/*
Utility to examine and decompile OpenType fonts.
Copyright (C) 2004-2020      Richard Wordingham

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#define WHAT \
"-- OpenType Font Table (oft) examiner.  "      \
      "Copyright (C) 2020 Richard Wordingham\n"

#define WARRANTY \
"-- This program comes with ABSOLUTELY NO WARRANTY; for details\n" \
"-- run with option '-w'."

#define COPYING \
"This is free software, and you are welcome\n" \
"-- to redistribute it under certain conditions; for details \n" \
"-- run with option '-c'."

#define WELCOME WHAT WARRANTY "  " COPYING "\n"
#define WO WHAT WARRANTY "\n\n"
#define CO WHAT COPYING "\n\n"

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "bitcodes.h"
#include "dates1904.h"
#include "psplot.h"
#include "wordplot.h"
#include "opcodes.h"
#include "unidata.h"
#include "map.h"
#include "int_set.h"
int sum_UW(unsigned short int *arr, int n);
void* element(void *base, int index);
int advance_coords(int force_advance);
void load_unicode_names();        

#include "oft_io_globals.h"

#include "manip_structure.h"
#include "oft.h"
#include "manip_befile.h"
#include "manip_get.h"
#include "oft.h"
#include "manip_print.h"
#include "manip_display.h"
#include "oft.h"
#include "manip_free.h"
#include "oft.h"
#include "manip_emem.h"
#include "manip_get.h"
#include "oft.h"

#ifdef min /* Surrogate for VS compiler used */
#define strcasecmp strcmp
#define strncasecmp strncmp
#endif

int sum_UW(unsigned short int *arr, int n) {
  int total = 0;
  for (; n; n--) total += *arr++;
  return total;
}

#ifndef min
#define min(x, y) ((x)<(y)?(x):(y))
#endif
#ifndef max
#define max(x, y) ((x)>(y)?(x):(y))
#endif

extern space unicode_names;

#if 0 /* Also defined in oft_io_functions.c */
/* Array access function */
void* element(void *base, int index) {
  void *loc;
  for (loc = base; index; index--) loc = *(void**) loc;
  return loc;
}
#endif

void disassemble(FILE *fp,
                 unsigned char *code, int length, int as_data, int level){
  int i, j, count, datum;
  const char bits[8][4] = {"000", "001", "010", "011",
			   "100", "101", "110", "111"};
#define N_INDENT 7
  const char *indent[N_INDENT] = {" ", "     ", 
				  "         ",
				  "             ", 
				  "                 ",
				  "                     ",
				  "                         "};
  int indent_level = 0;
  const char *indent_ptr;
  unsigned char code_now;

  if (as_data) fprintf(fp, " %d disassembly:\n", level++);
  indent_ptr = indent[0];
  for (i = 0; i < length; i++) {
    if (as_data) {
      fprintf(fp, " %d [%2.2d]", level, i);
    } else {
      fprintf(fp, "     ");
    }
    code_now = code[i];
    if (code_now < 144) {
      switch (code_now) {
      case 0x1b: /* ELSE */
      case 0x59: /* EIF  */
      case 0x2d: /* ENDIF */
	indent_level--;
	if (indent_level >= N_INDENT) {
	  indent_ptr = indent[N_INDENT - 1];
	} else if (indent_level < 0) {
	  indent_level = 0;
	  indent_ptr = indent[0];
	} else {
	  indent_ptr = indent[indent_level];
	}
	break;
      }
      switch (code[i]) {
      case 0x40:
	count = (i+1<length) ? code[i+1] : 0;
	fprintf(fp, "%sNPUSHB %3d", indent_ptr, count);
	for (j = 1; j <= count && i+j+1 <= length; j++) {
	  if (j != 1 && j%8 == 1) fprintf(fp, "\n                  ");
	  fprintf(fp, " %3d", code[i+1+j]);
	};
	fprintf(fp, "\n");
	i += (count+1);
	break;
      case 0x41:
	count = (i+1<length) ? code[i+1] : 0;
	fprintf(fp, "%sNPUSHW %3d", indent_ptr, count);
	for (j = 1; j <= count && i+2*j+1 <= length; j++) {
	  if (j != 1 && j%8 == 1) fprintf(fp, "\n                  ");
	  datum = code[i+2*j]<<8 | code[i+2*j+1];
	  fprintf(fp, " %3d", (int)(short int)datum);
	};
	fprintf(fp, "\n");
	i += (2*count+1);
	break;
      default:
	fprintf(fp, "%s%-10s\n", indent_ptr, mnemonic[code[i]]);
	break;
      }
      switch (code_now) {
      case 0x58: /* IF   */
      case 0x1b: /* ELSE */
      case 0x2c: /* FDEF */
      case 0x89: /* IDEF */
	indent_level++;
	if (indent_level >= N_INDENT) {
	  indent_ptr = indent[N_INDENT-1];
	} else {
	  indent_ptr = indent[indent_level];
	}
	break;
      }
    } else {
      switch (code[i] / 8) {
      case 18:
      case 19:
      case 20:
      case 21:
	fprintf(fp, "%sundefined_%2.2X\n", indent_ptr, code[i]);
	break;
      case 22:
	count = (code[i] & 7) + 1;
	fprintf(fp, "%sPUSHB[%s] ", indent_ptr, bits[count-1]);
	for (j = 1; j <= count && i+j < length; j++) {
	  fprintf(fp, " %3d", code[i+j]);
	};
	fprintf(fp, "\n");
	i += count;
	break;
      case 23:
	count = (code[i] & 7) + 1;
	fprintf(fp, "%sPUSHW[%s] ", indent_ptr, bits[count-1]);
	for (j = 1; j <= count && i+2*j < length; j++) {
	  datum = code[i+2*j-1]<<8 | code[i+2*j];
	  fprintf(fp, " %3d", (int)(short int)datum);
	};
	fprintf(fp, "\n");
	i += 2*count;
	break;
      case 24:
	count = code[i] & 7;
	fprintf(fp, "%sMDRP[00%s]\n", indent_ptr, bits[count]);
	break;
      case 25:
	count = code[i] & 7;
	fprintf(fp, "%sMDRP[01%s]\n", indent_ptr, bits[count]);
	break;
      case 26:
	count = code[i] & 7;
	fprintf(fp, "%sMDRP[10%s]\n", indent_ptr, bits[count]);
	break;
      case 27:
	count = code[i] & 7;
	fprintf(fp, "%sMDRP[11%s]\n", indent_ptr, bits[count]);
	break;
      case 28:
	count = code[i] & 7;
	fprintf(fp, "%sMIRP[00%s]\n", indent_ptr, bits[count]);
	break;
      case 29:
	count = code[i] & 7;
	fprintf(fp, "%sMIRP[01%s]\n", indent_ptr, bits[count]);
	break;
      case 30:
	count = code[i] & 7;
	fprintf(fp, "%sMIRP[10%s]\n", indent_ptr, bits[count]);
	break;
      case 31:
	count = code[i] & 7;
	fprintf(fp, "%sMIRP[11%s]\n", indent_ptr, bits[count]);
	break;
      }
    }
  }
}

static struct postscript_information post_info;
/* Command codes */
/* General: */
#define ANALYSE     (1<<0)
#define DISPLAY_RAW (1<<1)  /* Flag r */
#define DISPLAY_RES (1<<2)  /* Flag m */
#define HELP        (1<<3)  /* Flag h */
#define COMPILER    (1<<4)  /* Flag c */
#define POSTNAME    (1<<5)  /* Flag --post:i, which affects many tables! */
/* Special: */
#define DRAW        HALF    /* glyf */ /* Flag d selects current pref of kjq */
#define KNOTTED     (1<<8)  /* glyf */ /* Flag k */
#define JOIN        (1<<9)  /* glyf */ /* Flag j */
#define HALF        (1<<10) /* glyf */ /* Flag q */
#define POSTSCRIPT  (1<<11) /* glyf */ /* Flag p */
#define ANY_DRAW (KNOTTED | JOIN | HALF)
#define EXPAND      (1<<12) /* CFF  */ /* Flag e */
#define COMPDAT     (1<<13) /* GDEF */ /* Just load data */
/* Now the real probing begins. */

/* Global data tables - can't be bothered to pass! */
static unsigned int *codepoint = 0; /* First character mapped to glyph */
#define NO_CHAR 0xfffe
static unsigned short int UnitsPerEm;
static struct map extra_charcodes;
static struct horizontal_metrics horizontalMetrics;

/* category[gid] is type of glyph gid and markClass[gid] it its mark
attachment class. Lack of assignment is recorded by NO_CATEGORY and 
NO_MAC, which are not the same as 0, though an application may treat them
the same.  Limiting markClass to 254 might cause a problem at some later
time. */
static unsigned char *category = 0;
static unsigned char *markClass = 0;
#define NO_CATEGORY 0xFF
#define NO_MAC 0xFF /* If range extended, make 0xFFFF. */

static int p0c0 = -1;
static struct base_glyph_records COLR_bgr;
static struct layer_records COLR_layers;

static void associate(unsigned int charno, unsigned short int glyphno) {
  char glyphno_text[6];
  int  *list1, *list2;

  if (glyphno < N_GLYPHS) {
    if (codepoint[glyphno] == charno) {
      /* No change */
    } else if (codepoint[glyphno] != NO_CHAR) {
      if (glyphno) {
	fprintf(stdout, "-- Glyph %d represents both U+%.4X and U+%.4X.\n", 
		glyphno, codepoint[glyphno], charno);
      }
      sprintf(glyphno_text, "%d", glyphno);
      if (lookup_in_map(extra_charcodes, glyphno_text, (void**)&list1)) {
	list2 = realloc(list1, (2 + *list1) * sizeof(int));
      } else {
	list2 = malloc(2 * sizeof(int));
	if (list2) list2[0] = 0;
	list1 = 0;
      }
      if (list2) {
	list2[++list2[0]] = charno;
	if (list2 != list1) {
/* Any appropriate deallocation has already been done by realloc()! */
	  replace_in_map(&extra_charcodes, glyphno_text, list2, 0);
	}
      } else {
	fprintf(stderr, "Memory exhausted in function associate().\n");
	exit(1);
      } 
    } else {
      codepoint[glyphno] = charno;
    }
  } else {
    fprintf(stderr, "Bad glyph ID %d derived for U+%.4X.\n", glyphno, charno);
  }
}

/* Get, display and explain cmap table. */
void display_cmap(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *cmap_table;
  int pos, junk, itab;
  struct cmap_header cmap_header;
  struct encoding_record *encodingRecord;
  char label[30];
  struct encoding f0e;
  int i;
  unsigned int charno;

  delete_map(&extra_charcodes, 1);
  create_map(&extra_charcodes);
  for (cmap_table = top.table_directory; cmap_table;
       cmap_table = cmap_table->_next) {
    if (!strncmp(cmap_table->tag, "cmap", 4)) break;
  }
  if (cmap_table) {
    pos = cmap_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking cmap table");
      exit(1);
    }
    get_bine_cmap_header(fp, &cmap_header, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_cmap_header(stdout, &cmap_header, &junk, "cmap", 0);
    }
    codepoint = malloc(N_GLYPHS * sizeof(unsigned int));
    if (!codepoint) {
      fprintf(stderr, "Malloc failed on glyph to character table.\n");
      exit(1);
    }
    for (i = 0; i < N_GLYPHS; i++) codepoint[i] = NO_CHAR;
    for (encodingRecord = cmap_header.encodingRecord, itab=0;
	 encodingRecord;
	 encodingRecord = encodingRecord->_next, itab++) {
      pos = cmap_table->offset + encodingRecord->offset;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking encoding table");
	exit(1);
      }
      if (instr & DISPLAY_RAW) sprintf(label, "Encoding %d", itab);
      get_bine_encoding(fp, &f0e, &junk, &pos);
      if (instr & DISPLAY_RAW) print_encoding(stdout, &f0e, &junk, label, 2);
      if (4 == f0e.format) {
	int seg, index;
	unsigned short int *start, *end, *offset;
	short int *delta;
	unsigned short int glyphno;
	
	if (instr & DISPLAY_RAW) fputs(" 3 explanation:\n", stdout);
	start  = f0e.startCount;
	end    = f0e.endCount;
	offset = f0e.idRangeOffset;
	delta  = f0e.idDelta;
	for (seg = 0; seg < f0e.segCountX2/2;
	     seg++, start++, end++, offset++, delta++) {
	  if (instr & DISPLAY_RAW) {
	    fprintf(stdout, "%2d [%d]: %4.4X to %4.4X use glyphs ",
		    4, seg, *start, *end);
	  }
	  if (*offset) {
	    index = seg + (*offset - f0e.segCountX2)/2;
	    if (instr & DISPLAY_RAW) {
	      fprintf(stdout, "glyphIdArray[%d to %d]",
		      index, index + (*end - *start));
	      if (*delta) fprintf(stdout, "%+d", *delta);
	      fprintf(stdout, "\n");
	    }
	    if (1 == encodingRecord->encodingID ||
		10 == encodingRecord->encodingID) {
	      for (charno = *start; charno <= *end; charno++) {
                int id_index;

		id_index = index + charno - *start;
		if (id_index < 0 || id_index >= f0e._n_glyphIdArray) {
		  fprintf(stdout, "Bad index %d into glyph ID array for "
			  "U+%.4X.\n", id_index, charno);
		} else {
		  glyphno = f0e.glyphIdArray[id_index];
		  if (glyphno) {
		    glyphno += *delta;
		  } else {
		    fprintf(stdout, "-- Missing glyph used for U+%.4X.\n",
			    charno);
		  }
		  associate(charno, glyphno);
		}
	      }
	    }
	  } else {
	    if (instr & DISPLAY_RAW) {
	      fprintf(stdout, "%hu to %hu\n",
		      *start + *delta, *end + *delta);
	    }
	    if (1 == encodingRecord->encodingID ||
		10 == encodingRecord->encodingID) {
	      for (charno = *start; charno <= *end; charno++) {
		associate(charno, charno + *delta);
	      }
	    }
	  }
	}
      } else if (12 == f0e.format) {
	int seg, lglyphno;
	struct format_8_group *group;

	for (seg = 0; seg < f0e.nGroups; seg++) {
	  group = element(f0e.groups, seg);
	  if (1 == encodingRecord->encodingID ||
	      10 == encodingRecord->encodingID) {
	    for (charno = group->startCharCode; charno <= group->endCharCode;
		 charno++) {
	      lglyphno = charno - group->startCharCode + group->startGlyphID;
	      associate(charno, lglyphno);
	    }
	  }
	}
      } else if (14 == f0e.format) {
        int ivs;
        for (ivs = 0; ivs < f0e.numVarSelectorRecords; ivs++) {
          struct default_UVS_table default_table;
          struct non_default_UVS_table nd_table;
          struct variation_selector *vsd = element(f0e.varSelector, ivs);
          if (vsd->defaultUVSOffset) {
            pos = cmap_table->offset + encodingRecord->offset +
              vsd->defaultUVSOffset;
            if (fseek(fp, pos, SEEK_SET)) {
              perror("Seeking default UVS table");
              exit(1);
            }
            if (instr & DISPLAY_RAW) sprintf(label, "DefaultUVS %d", ivs);
            get_bine_default_UVS_table(fp, &default_table, &junk, &pos);
            if (instr & DISPLAY_RAW) {
              print_default_UVS_table(stdout, &default_table, &junk, label, 3);
            }
            free_default_UVS_table(&default_table);
          }
          if (vsd->nonDefaultUVSOffset) {
            pos = cmap_table->offset + encodingRecord->offset +
              vsd->nonDefaultUVSOffset;
            if (fseek(fp, pos, SEEK_SET)) {
              perror("Seeking non-default UVS table");
              exit(1);
            }
            if (instr & DISPLAY_RAW) sprintf(label, "NonDefaultUVS %d", ivs);
            get_bine_non_default_UVS_table(fp, &nd_table, &junk, &pos);
            if (instr & DISPLAY_RAW) {
              print_non_default_UVS_table(stdout, &nd_table, &junk, label, 3);
            }
            free_non_default_UVS_table(&nd_table);
          }
        }
      }
      free_encoding(&f0e);
    }
    free_cmap_header(&cmap_header);
    if (instr & DISPLAY_RES) {
      for (i = 0; i <= N_GLYPHS; i++) {
	if (NO_CHAR != codepoint[i]) {
	  fprintf(stdout, "Glyph %5d represents U+%.4X.\n",
		  i, codepoint[i]);
	}
      }
    }
  } else {
    printf("cmap table was not found!\n");
  }
}

/* Get and display head table */
void display_gasp(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *gasp_table;
  int pos, junk;
  struct gasp_header header;

  for (gasp_table = top.table_directory; gasp_table;
       gasp_table = gasp_table->_next) {
    if (!strncmp(gasp_table->tag, "gasp", 4)) break;
  }
  if (gasp_table) {
    pos = gasp_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking gasp table");
      exit(1);
    }
    get_bine_gasp_header(fp, &header, &junk, &pos);
    if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
      print_gasp_header(stdout, &header, &junk, "gasp", 0);
    }
    if (instr & (DISPLAY_RES|COMPILER)) {
      int io, i, k, none;
      char *sep;
      const char * bitname[] = {"gridfit", "dogray", "sym_gridfit", "smooth"};
      const int cmd[2] = {DISPLAY_RES, COMPILER};
      struct gasp_range *rge;
      for (io = 0; io < 2; io++) {
        if (instr & cmd[io]) {
          if (COMPILER == cmd[io]) {
            fputs("gasp\n", stdout);
          } else {
            fputs(" 1 gaspRanges_dec\n", stdout);
          }
          for (i = 0, rge = header.gaspRanges; i < header.numRanges;
               i++, rge = rge->_next) {
            if (DISPLAY_RES == cmd[io]) {
              fprintf(stdout, " 2 [%d]\n 3 maxPPEM:", i);
            }
            fprintf(stdout, " %5d", rge->maxPPEM);
            none = 1;
            sep = " ";
            if (DISPLAY_RES == cmd[io]) {
              fputs("\n 3 behaviour: ", stdout);
              sep = "";
            }
            for (k = 0; k < 4; k++) {
              if ((int) rge->behaviour & (1 << k)) {
                fprintf(stdout, "%s %s", sep, bitname[k]);
                none = 0;
                sep = ",";
              }
            }
            for (; k < 16; k++) {
              if ((int) rge->behaviour & (1 << k)) {
                fprintf(stdout, "%s %d", sep, k);
                none = 0;
                sep = ",";
              }
            }
            if (none) fprintf(stdout, "%s none", sep);
            fputc('\n', stdout);
          }
          if (COMPILER == cmd[io]) fputs("end gasp\n", stdout);
        }
      }
    }
    free_gasp_header(&header);
  }
}

/* Get and display head table */
void display_head(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *head_table;
  int pos, junk;
  struct font_header fontHeader;

  for (head_table = top.table_directory; head_table;
       head_table = head_table->_next) {
    if (!strncmp(head_table->tag, "head", 4)) break;
  }
  if (head_table) {
    pos = head_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking font header table");
      exit(1);
    }
    get_bine_font_header(fp, &fontHeader, &junk, &pos);
    if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
      print_font_header(stdout, &fontHeader, &junk, "head", 0);
    }
    if (instr & (DISPLAY_RES|COMPILER)) {
      const char * const mname[12] = {"January", "February", "March",
                                      "April",   "May",      "June",
                                      "July",    "August",   "September",
                                      "October", "November", "December"};
      double version;
      int year, month, day, hour, min, sec, ntodo;
      long long unsigned int idate;
      const char *field;

      version = (double) fontHeader.fontRevision / 65536;
      if (instr & DISPLAY_RES) {
        fprintf(stdout, " 1 fontRevision_dec: %.3f\n", version);
      }
      if (instr & COMPILER) fprintf(stdout, "fontRevision %.3f\n", version);
      for (ntodo = 2, idate = fontHeader.created, field = "created";
           ntodo > 0; ntodo--) {
        if (seconds_to_date1904(idate,
                                &year, &month, &day, &hour, &min, &sec)) {
          fprintf(stderr,
                  "Could not convert date stamp %llu.\n", idate);
        } else {
          if (instr & DISPLAY_RES) {
          fprintf(stdout, " 1 %s_t: %d %s %d %d:%2.2d:%2.2d\n",
                  field, day, mname[month], year, hour, min, sec);
          }
          if (instr & COMPILER) {
            fprintf(stdout, "%s %d-%s-%d %d:%2.2d:%2.2d\n",
                    field, day, mname[month], year, hour, min, sec);
          }
        }
        idate = fontHeader.modified;
        field = "modified";
      }
    }
    if (instr & COMPILER) {
      int bit;
      const char * const hflag[16] = {
        "horizontal_baseline_0", "lsb_is_xmin", "ins_use_PPEM", "integer_PPEM",
        "aw_by_PPEM",            "vertical_baseline_0", 0,       "needs_layout",
        "metamporphosis",        "right2left",          "Indic", "Agfa",
        "converted",             "ClearType",           0,       0};
      const char * const style[16] = {
        "bold",   "italic",    "underline", "outline",
        "shadow", "condensed", "extended",  0,
        0,        0,           0,           0,
        0,        0,           0,           0};
      fputs("head_flags none", stdout);
      for (bit = 0; bit <= 15; bit++) {
        if (fontHeader.flags & (1 << bit)) {
          if (hflag[bit]) {
            fprintf(stdout, ", %s", hflag[bit]);
          } else {
            fprintf(stdout, ", %d", bit);
          }
        }
      }
      fputc('\n', stdout);
      fprintf(stdout, "unitsPerEm %d\n",        fontHeader.unitsPerEm);
      fprintf(stdout, "xMin %d\n",              fontHeader.xMin);
      fprintf(stdout, "yMin %d\n",              fontHeader.yMin);
      fprintf(stdout, "xMax %d\n",              fontHeader.xMax);
      fprintf(stdout, "yMax %d\n",              fontHeader.yMax);
      fputs("macStyle none", stdout);
      for (bit = 0; bit <= 15; bit++) {
        if (fontHeader.macStyle & (1 << bit)) {
          if (style[bit]) {
            fprintf(stdout, ", %s", style[bit]);
          } else {
            fprintf(stdout, ", %d", bit);
          }
        }
      }
      fputc('\n', stdout);
      fprintf(stdout, "lowestRecPPEM %d\n",     fontHeader.lowestRecPPEM);
      fprintf(stdout, "fontDirectionHint %d\n", fontHeader.fontDirectionHint);
      fprintf(stdout, "indexToLocFormat %d\n",  fontHeader.indexToLocFormat);
      fprintf(stdout, "-- glyphDataFormat %d\n", fontHeader.glyphDataFormat);
    }
    UnitsPerEm = fontHeader.unitsPerEm;
  }
}

static void encompass(int val, int *lower, int *upper) {
  if (val < *lower) {
    *lower = val;
  } else if (val > *upper) {
    *upper = val;
  }
}

static void get_cff_name(int gid, const char **pname, int *plen);

static void get_postname(int glyphno, const char **postname, int *plen) {
  int pindex;
  struct ASCIC *pascic;

  switch (post_info.version) {
  case 0x00010000:
    pindex = (glyphno < 258) ? glyphno : 0;
    break;
  case 0x00020000:
    if (0 <= glyphno && glyphno < post_info.numberOfGlyphs) {
      pindex = post_info.glyphNameIndex[glyphno];
    } else {
      pindex = 0; /* Rogue glyph number */
    }
    break;
  case 0x00025000:
    if (0 <= glyphno && glyphno < post_info.numberOfGlyphs) {
      pindex = glyphno + post_info.offset[glyphno];
    } else {
      pindex = 0; /* Rogue glyph number */
    }
    if (pindex < 0 || pindex >= 258) pindex = 0;
  default:
    pindex = 0;
  }
  if (0 == pindex) {
    *postname = 0;
  } else if (pindex < 258) {
    *postname = ordered_mac_name[pindex];
    *plen = strlen(*postname);
  } else {
    pascic = element(post_info.names, pindex - 258);
    *postname = pascic->body;
    *plen = pascic->count;
  }
  if (!*postname) {
    get_cff_name(glyphno, postname, plen);
  }
}

char *glyphname_r(int instr, int gid, char *buffer, int buflen) {
  const char *postname;
  int plen, do_post;
  if (instr & POSTNAME) {
    get_postname(gid, &postname, &plen);
    do_post = postname && plen && plen < buflen;
  } else {
    do_post = 0;
  }
  if (do_post) {
    sprintf(buffer, "%.*s", plen, postname);
  } else {
    sprintf(buffer, "glyph_%d", gid);
  }
  return buffer;
}

/* Separate functions glyphname1() and glyphname2() allow the concurrent use
   of two different glyph names. */ 
char *glyphname1(int instr, int gid) {
  static char buffer[256];
  return glyphname_r(instr, gid, buffer, 256);
}

char *glyphname2(int instr, int gid) {
  static char buffer[256];
  return glyphname_r(instr, gid, buffer, 256);
}

#define CODLEN 5
void unpack_coords(int npts,
                   struct coordinate *raw_x,
                   struct coordinate *raw_y,
                   struct repeatable_flag *raw_flags,
                   int *n_did, short int **x, short int **y,
                   unsigned char **flag, char **codes) {
  int ipt, nrpt;
  struct coordinate pre_x, pre_y;
  *x = malloc(2*npts);
  *y = malloc(2*npts);
  *flag = malloc(npts);
  *codes = malloc(CODLEN * npts);
  if (!*x || !*y || !*flag || !*codes) {
    fprintf(stderr, "Out of memory in unpack_coords().\n");
    exit(1);
  }
/* Necessary parentheses will become tedious. */
#define x (*x)
#define y (*y)
#define flag (*flag)
#define codes (*codes)
  flag[0] = raw_flags->val;
  if ((flag[0] & 0x12) == 0x10) {
    pre_x._next = raw_x;
    raw_x = &pre_x;
    x[0] = 0;
  } else {
    x[0] = raw_x->flipped ? (-raw_x->pos) : raw_x->pos;
  }
  if ((flag[0] & 0x24) == 0x20) {
    pre_y._next = raw_y;
    raw_y = &pre_y;
    y[0] = 0;
  } else {
    y[0] = raw_y->flipped ? (-raw_y->pos) : raw_y->pos;
  }
  if (flag[0] & 8) {
    raw_flags = raw_flags->_next;
    nrpt = raw_flags->val;
    codes[3] = 'r';
  } else {
    nrpt = 0;
    codes[3] = ' ';
  }
  *n_did = 1;
  for (ipt = 1; ipt < npts; ipt++) {
    if (nrpt-- > 0) {
      flag[ipt] = flag[ipt-1];
      codes[CODLEN * ipt + 3] = 'd';
    } else {
      raw_flags = raw_flags->_next;
      if (!raw_flags) break;
      flag[ipt] = raw_flags->val;
      if (flag[ipt] & 8) {
        raw_flags = raw_flags->_next;
        if (!raw_flags) break;
        nrpt = raw_flags->val;
        codes[CODLEN * ipt + 3] = 'r';
      } else {
        nrpt = 0;
        codes[CODLEN * ipt + 3] = ' ';
      }
    }
    if ((flag[ipt] & 0x12) == 0x10) {
      x[ipt] = x[ipt-1];
    } else {
      if (!raw_x->_next) break;
      raw_x = raw_x->_next;
      if (raw_x->flipped) {
        x[ipt] = x[ipt-1] - raw_x->pos;
      } else {
        x[ipt] = x[ipt-1] + raw_x->pos;
      }
    }
    if ((flag[ipt] & 0x24) == 0x20) {
      y[ipt] = y[ipt-1];
    } else {
      if (!raw_y->_next) break;
      raw_y = raw_y->_next;
      if (raw_y->flipped) {
        y[ipt] = y[ipt-1] - raw_y->pos;
      } else {
        y[ipt] = y[ipt-1] + raw_y->pos;
      }
    }
    *n_did = ipt + 1;
  }
  for (ipt = 0; ipt < *n_did; ipt++) {
    codes[CODLEN*ipt+0] = flag[ipt] & 1 ? 'c' : 's';
    codes[CODLEN*ipt+1] = flag[ipt] & 2 ? (flag[ipt] & 16 ? '+' : '-')
      : (flag[ipt] & 16 ? '=' : 'w');
    codes[CODLEN*ipt+2] = flag[ipt] & 4 ? (flag[ipt] & 32 ? '+' : '-')
      : (flag[ipt] & 32 ? '=' : 'w');
    codes[CODLEN*ipt+4] = 0;
  }
#undef x
#undef y
#undef flag
#undef codes
} /* End of unpack_coords() */

static void put_glyph_properties(int glyphno) {
  char glyphno_text[6];
  int *list, k;
  struct longHorMetric *el;
  int has_class;
        
  if (codepoint && codepoint[glyphno] != NO_CHAR) {
    fprintf(stdout, "  character 0x%4.4x\n", codepoint[glyphno]);
    sprintf(glyphno_text, "%d", glyphno);
    if (lookup_in_map(extra_charcodes, glyphno_text, (void**) &list)) {
      for (k = 1; k <= list[0]; k++) {
        fprintf(stdout, "  character 0x%4.4x\n", list[k]);
      }
    }
  }
#define CAT_DEFAULT   0
#define CAT_BASE      1
#define CAT_LIGATURE  2
#define CAT_MARK      3
#define CAT_COMPONENT 4

  has_class = markClass && markClass[glyphno] < NO_MAC;
  if (has_class) {
    if (!category || CAT_MARK != category[glyphno]) {
      fprintf(stdout, "  category pseudomark %d\n", markClass[glyphno]);
    }
  }
  if (category) {
    switch (category[glyphno]) {
    case CAT_DEFAULT:
      fprintf(stdout, "  category default\n");
      break;
    case CAT_BASE:
      fprintf(stdout, "  category base\n");
      break;
    case CAT_LIGATURE:
      fprintf(stdout, "  category ligature\n");
      break;
    case CAT_MARK:
      if (has_class) {
        fprintf(stdout, "  category mark %d\n", markClass[glyphno]);
      } else {
        fprintf(stdout, "  category defaultmark\n");
      }
      break;
    case CAT_COMPONENT:
      fprintf(stdout, "  category component\n");
      break;
    case NO_CATEGORY:
      /* Nothing to record */
      break;
    default:
      fprintf(stdout, "  category %d\n", category[glyphno]);
      break;
    }
  }
      /*
STRUCTURE(horizontal_metrics)
     ARRAY_VC(hMetrics, hhea_numberofHMetrics, longHorMetric)
     DATA_VR_SW(leftSideBearing, N_GLYPHS - hhea_numberofHMetrics)
END_STRUCTURE(horizontal_metrics)
      */
  if (glyphno < hhea_numberofHMetrics) {
    el = element(horizontalMetrics.hMetrics, glyphno);
    fprintf(stdout, "  advanceWidth %d", el->advanceWidth);
    if (glyphno == hhea_numberofHMetrics - 1) fprintf(stdout, ", default");
    fputc('\n', stdout);
    fprintf(stdout, "  leftSideBearing %d\n", el->lsb);
  } else {
    fprintf(stdout, "  leftSideBearing %d\n",
            horizontalMetrics.leftSideBearing
            [glyphno - hhea_numberofHMetrics]);
  }
}

extern void display_GDEF(FILE *fp, int instr, struct offset_table top);
/* Get, display and explain glyph table. */
void display_glyf(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *glyf_table, *loca_table;
  int pos, pos_next, junk, n;
  int glyphno;
  int def_size;
  int npts;
  short int *x, *y;
  unsigned char *flag;
  int con_start, icon;
  struct location_index location;
  char label[60];
  struct glyph_entry glyph_def;
  struct string_dump f0e;
  FILE *rtf = 0;
  int ipt1, ipt2, ipt3, ipt4, istep, n_dup, prev, next;
  float old_x, old_y, new_x, new_y, u, u2, uc2, ux, xs, ys, xe, ye;
  int glyphs_per_page = 10;
  char filename[80];
  char *uname;
  int xL, xU, yL, yU;
  float x_scale_req, y_scale_req, scale_req;
  float xbase, ybase;
  struct base_glyph_record *bgrptr = COLR_bgr.base;
  void (*pDrawSeg)(FILE *fp, float x1, float y1, float x2, float y2,
		   const char *options);
#define PAPER_WIDTH 8000.0 /* in twips - actually 8334 */
#define PAPER_HEIGHT 12500.0 /* in twips - actually 13946 */
  /* A4 %% BoundingBox: 0 0 595 841 in points */
#define hmargin   (72*20)
#define hsep      (15*20)
#define ncols        3
#define pswidth  (140*20)
#define nrows        4
#define vmargin   (72*20)
#define vsep      (30*20) /* One for each of *every* row */
#define psheight (144*20)

  if (1) fprintf(stderr, "Invoked display_glyf(%8.8X).\n", instr);
  if (instr & POSTSCRIPT) {
    pDrawSeg = DrawPSSeg;
    xbase = 160;
    ybase = PAPER_HEIGHT;
    glyphs_per_page = nrows * ncols;
  } else {
    pDrawSeg = DrawSeg;
    xbase =  100;
    ybase = -100;
    glyphs_per_page = 10;
  }
  memset(&glyph_def, 0, sizeof(glyph_def));
  for (loca_table = top.table_directory; loca_table;
       loca_table = loca_table->_next) {
    if (!strncmp(loca_table->tag, "loca", 4)) break;
  }
  if (loca_table) {
    pos = loca_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking glyph location table");
      exit(1);
    }
    get_bine_location_index(fp, &location, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_location_index(stdout, &location, &junk, "loca", 0);
    }
  }
  for (glyf_table = top.table_directory; glyf_table;
       glyf_table = glyf_table->_next) {
    if (!strncmp(glyf_table->tag, "glyf", 4)) break;
  }
  if (instr & (ANY_DRAW | COMPILER)) {
    (void) load_unicode_names();
  }
  if (instr & COMPILER) display_GDEF(fp, ANALYSE|COMPDAT, top);
  if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
    fprintf(stdout, " 0 glyph\n");
  }
  for (glyphno = 0; glyphno < N_GLYPHS; glyphno++) {
    const char *postname = 0;
    int plen = 0;

    if (instr & (ANY_DRAW | COMPILER)) {

      if (codepoint && NO_CHAR != codepoint[glyphno]) {
	if (!get_property(unicode_names, codepoint[glyphno],
			      (void**)&uname)) {
	  uname = 0;
	}
      } else {
	uname = 0;
      }
    }
    get_postname(glyphno, &postname, &plen);
    if (instr & COMPILER) {
      if (postname && plen && (instr & POSTNAME)) {
	fprintf(stdout, "glyph %.*s\n--  number %d\n",
		plen, postname, glyphno);
      } else {
	fprintf(stdout, "glyph glyph_%d", glyphno);
      }
      if (uname) fprintf(stdout, " -- %s", uname);
      fputc('\n', stdout);
      if (postname && plen) {
	fprintf(stdout, "  postname %.*s\n", plen, postname);
      }
      put_glyph_properties(glyphno);
    }
    if (glyphno % glyphs_per_page == 0) {
      if (!rtf) {
	/* Nothing to do */
      } else if (!(instr & POSTSCRIPT)) {
	EndWordFile(rtf);
      } else if (0) {
	EndPSFile(rtf);
      }
      if (instr & ANY_DRAW) {
	if (instr & POSTSCRIPT) {
	  if (glyphno == 0) {
	    strcpy(filename, "Opening outlines.ps");
	    rtf = StartPSFile(filename+8);
	  } else if (0) {
	    sprintf(filename, "Opening outlines_%d.ps", glyphno/glyphs_per_page);
	    rtf = StartPSFile(filename+8);
	  } else {
	    EndPSPage(rtf);
	  }
	} else {
	  sprintf(filename, "Opening outlines_%d.doc", glyphno/glyphs_per_page);
	  rtf = StartWordFile(filename+8);
	}
	if (!rtf) {
	  perror(filename);
	  exit(1);
	}
      } else {
	rtf = 0;
      }
    }
    sprintf(label, "[%2.2d]", glyphno);
    if (glyf_table) {
      if (LONG_OFFSETS) {
	pos      = glyf_table->offset + location.long_offsets[glyphno];
	pos_next = glyf_table->offset + location.long_offsets[glyphno+1];
      } else {
	pos      = glyf_table->offset + 2 * location.short_offsets[glyphno];
	pos_next = glyf_table->offset + 2 * location.short_offsets[glyphno+1];
      }
      def_size = (signed) pos_next - (signed) pos;
    } else {
      def_size = 0;
    }
    if (def_size <= 0) {
      if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
	fprintf(stdout, " 1 %s:\n 2 No outline.\n", label);
      }
    } else {
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking glyph definition");
	exit(1);
      }
      memset(&glyph_def, 0, sizeof(glyph_def));
      fflush(0);
      get_bine_glyph_entry(fp, &glyph_def, &junk, &pos);
      if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
	print_glyph_entry(stdout, &glyph_def, &junk, label, 1);
      }
      if (instr & (DISPLAY_RES|COMPILER)) {
	if (glyph_def.instructionLength) {
	  if (instr & COMPILER) fprintf(stdout, "  hints\n");
	  disassemble(stdout, glyph_def.instructions,
		      glyph_def.instructionLength, instr&DISPLAY_RES, 2);
	  if (instr & COMPILER) fprintf(stdout, "  end hints\n");
	}
	if (glyph_def.numInstr) {
	  if (instr & COMPILER) fprintf(stdout, "  hints\n");
	  disassemble(stdout, glyph_def.instr, glyph_def.numInstr,
		      instr & DISPLAY_RES, 2);
	  if (instr & COMPILER) fprintf(stdout, "  end hints\n");
	}
      }
      if (instr & COMPILER) {
	fprintf(stdout, "  xMin %6d\n", glyph_def.xMin);
	fprintf(stdout, "  xMax %6d\n", glyph_def.xMax);
	fprintf(stdout, "  yMin %6d\n", glyph_def.yMin);
	fprintf(stdout, "  yMax %6d\n", glyph_def.yMax);
      }
      if ((instr & (ANY_DRAW|DISPLAY_RES|COMPILER))
	  && glyph_def.numberOfContours > 0) {
	struct coordinate *raw_x, *raw_y;
	struct repeatable_flag *raw_flags;
        int ipt, n_did;
	char *codes;

	npts = glyph_def.endPtsOfContours[glyph_def.numberOfContours-1] + 1;
	raw_x = glyph_def.xCoordinates;
	raw_y = glyph_def.yCoordinates;
	raw_flags = glyph_def.flags;
        unpack_coords(npts, raw_x, raw_y, raw_flags,
                      &n_did, &x, &y, &flag, &codes);
	if (instr & DISPLAY_RES) {
	  fprintf(stdout, " 2 Points:\n");
	  for (ipt = 0; ipt < n_did; ipt++) {
	    fprintf(stdout,
		    " 3 [%2.2d] %6d %4hx   %6d %4hx   %3d %2x %s\n",
		    ipt, x[ipt], x[ipt],
		    y[ipt], y[ipt], flag[ipt], flag[ipt], codes + CODLEN*ipt);
	  }
	}
	if (instr & COMPILER) {
	  int icont;

	  fprintf(stdout, "  points\n");
	  icont = 0;
	  for (ipt = 0; ipt < n_did; ipt++) {
	    if (flag[ipt] & 2) {
	      fprintf(stdout, "    byte_to %6d", x[ipt]);
	    } else {
	      if (flag[ipt] & 16) {
		fprintf(stdout, "    same          ");
	      } else {
		fprintf(stdout, "    word_to %6d", x[ipt]);
	      }
	    } 
	    if (flag[ipt] & 4) {
	      fprintf(stdout, " byte_to %6d", y[ipt]);
	    } else {
	      if (flag[ipt] & 32) {
		fprintf(stdout, " same          ");
	      } else {
		fprintf(stdout, " word_to %6d", y[ipt]);
	      }
	    }
	    if (flag[ipt] & 1) {
	      fprintf(stdout, " on");
	    } else {
	      fprintf(stdout, " off");
	    }
	    if (ipt == glyph_def.endPtsOfContours[icont]) {
	      fprintf(stdout, ", end");
	      icont++;
	    }
/* Worry about flag control later! */
	    fputc('\n', stdout);
	  }
	  fprintf(stdout, "  end points\n");
	}
	if (instr & (DISPLAY_RES|COMPILER)) {
	  if (n_did != npts) {
	    fprintf(stderr, "Problem at point %d of glyph %d.\n", n_did + 1,
		    glyphno);
	  }
	}
/* Simple plotting routine */
	if (instr & ANY_DRAW) {
	  if (instr & POSTSCRIPT) {
	    char desc[2000];
	    int ir, ic;

            ic = glyphno % ncols;
            ir = (glyphno / ncols) % nrows;
            xbase = hmargin + ic * (pswidth + hsep);
            ybase = vmargin + (nrows - ir) * (psheight + vsep) - vsep;
	    sprintf(desc, "Contour sketch for Glyph %d", glyphno);
	    if (codepoint && NO_CHAR != codepoint[glyphno]) {
	      sprintf(desc + strlen(desc), " (Character U+%.4X",
		      codepoint[glyphno]);
	      if (uname) {
		sprintf(desc + strlen(desc), " %s)", uname);
	      } else {
		strcat(desc, ")");
	      }
	    }
	    if (postname) {
	      sprintf(desc + strlen(desc),
		      " (Postscript name %.*s)", plen, postname);
	    }
	    strcat(desc, ":");
/* For square characters, would be /160 - but not square! */
	    PSText(rtf, xbase, ybase + vsep, desc, pswidth/100);
	  } else {
	    fprintf(rtf, "Contour sketch for Glyph %d", glyphno);
	    if (codepoint && NO_CHAR != codepoint[glyphno]) {
	      fprintf(rtf, " (Character U+%.4X %s)", codepoint[glyphno],
		      uname ? uname : "");
	    }
	    if (postname) {
	      fprintf(rtf, " (Postscript name %.*s)", plen, postname);
	    }
	    fputc(':', rtf);
	  }
	  /* Determine scale of plot */
	  xL = yL = xU = yU = 0;
	  encompass(glyph_def.xMin, &xL, &xU);
	  encompass(glyph_def.xMax, &xL, &xU);
	  if (glyph_def.xMin < 0) {
	    encompass(UnitsPerEm/10, &xL, &xU);
	    encompass(glyph_def.xMin + UnitsPerEm, &xL, &xU);
	  } else {
	    encompass(UnitsPerEm, &xL, &xU);
	  }
	  encompass(glyph_def.yMin, &yL, &yU);
	  encompass(glyph_def.yMax, &yL, &yU);
	  if (glyph_def.yMin < 0) {
	    encompass(UnitsPerEm/10, &yL, &yU);
	    encompass(glyph_def.yMin + UnitsPerEm, &yL, &yU);
	  } else {
	    encompass(UnitsPerEm, &yL, &yU);
	  }
	  if (instr & POSTSCRIPT) {
	    x_scale_req = (double) pswidth  / (xU - xL);
	    y_scale_req = (double) psheight / (yU - yL);
	  } else {
	    x_scale_req = PAPER_WIDTH / (xU - xL);
	    y_scale_req = PAPER_HEIGHT / (yU - yL);
	  }
	  scale_req = min(x_scale_req, y_scale_req);
	  if (scale_req > 2.0) {
	    scale_req = (int) scale_req;
	  } else if (scale_req > 0.5) {
	    scale_req = 0.25 * (int) (4.0*scale_req);
	  }
#if 0
          fprintf(rtf, "xL=%d xU=%d yL=%d yU=%d \n", xL, xU, yL, yU);
	  fprintf(rtf, "x_scale_req=%.2f y_scale_req=%.2f \n",
		  x_scale_req, y_scale_req);
	  fprintf(rtf, "scale_req=%.2f\n", scale_req);
#endif
	  if (instr & POSTSCRIPT) {
	    NewPSGraph(rtf, scale_req, xbase - xL * scale_req,
		       ybase - yU * scale_req);
	  } else {
	    NewGraph(rtf, scale_req, xbase - xL * scale_req,
		     ybase - yU * scale_req);
	  }
	  /* Draw the co-ordinate axes */
#if 0
	  pDrawSeg(rtf, min(glyph_def.xMin,0),0, max(0,glyph_def.xMax),0, "P");
	  pDrawSeg(rtf, 0,min(glyph_def.yMin,0), 0,max(glyph_def.yMax,0), "P");
#endif
	  pDrawSeg(rtf, xL, 0, xU, 0, "P");
	  pDrawSeg(rtf, 0, yL, 0, yU, "P");
pDrawSeg(rtf,glyph_def.xMin,glyph_def.yMin,glyph_def.xMax,glyph_def.yMin, "P");
pDrawSeg(rtf,glyph_def.xMax,glyph_def.yMin,glyph_def.xMax,glyph_def.yMax, "P");
pDrawSeg(rtf,glyph_def.xMax,glyph_def.yMax,glyph_def.xMin,glyph_def.yMax, "P");
pDrawSeg(rtf,glyph_def.xMin,glyph_def.yMax,glyph_def.xMin,glyph_def.yMin, "P");
	}
	if (instr & JOIN) {
	  for (ipt = 0, con_start = 0, icon = 0; ipt < npts; ipt++) {
	    if (ipt == glyph_def.endPtsOfContours[icon]) {
	      if (flag[ipt] & flag[con_start] & 1) {
		pDrawSeg(rtf, x[ipt], y[ipt], x[con_start], y[con_start], "BA");
	      } else {
		pDrawSeg(rtf, x[ipt], y[ipt], x[con_start], y[con_start], "BDA");
	      }
	      con_start = ipt + 1;
	      icon++;
	    } else {
	      if (flag[ipt] & flag[ipt+1] & 1) {
		pDrawSeg(rtf, x[ipt], y[ipt], x[ipt+1], y[ipt+1], "A");
	      } else {
		pDrawSeg(rtf, x[ipt], y[ipt], x[ipt+1], y[ipt+1], "DgA");
	      }
	    }
	  }
	}
	if (instr & KNOTTED) {
	  /* Complex plotting - 4-pt quadratic splines */
	  /* On curve points are expanded to 3 consecutive points. */
	  con_start = 0;
	  for (icon = 0; icon < glyph_def.numberOfContours; icon++) {
	    ipt4 = con_start;
	    n_dup = (flag[ipt4] & 1) * 2;
	    ipt3 = glyph_def.endPtsOfContours[icon];
	    if (flag[ipt3] & 1 || ipt3 == ipt4) {
	      ipt1 = ipt2 = ipt3;
	    } else {
	      ipt2 = ipt3 - 1;
	      if (flag[ipt2] & 1) {
		ipt1 = ipt2;
	      } else if (ipt2 == ipt4) {
/* 2-point contour! */
		ipt1 = ipt3;
	      } else {
		ipt1 = ipt2 - 1;
	      }
	    }
	    old_x = 0.25 * (x[ipt1] + 2*x[ipt2] + x[ipt3]);
	    old_y = 0.25 * (y[ipt1] + 2*y[ipt2] + y[ipt3]);
#define NSTEP 4
	    while (1) {
	      for (istep = 1; istep <= NSTEP; istep++) {
		u = istep / (float) NSTEP;
		u2 = u * u;
		uc2 = (1 - u) * (1 - u);
		new_x = 0.25 *
		  (uc2*x[ipt1] + (2-u2)*x[ipt2] + (2-uc2)*x[ipt3] + u2*x[ipt4]);
		new_y = 0.25 *
		  (uc2*y[ipt1] + (2-u2)*y[ipt2] + (2-uc2)*y[ipt3] + u2*y[ipt4]);
		pDrawSeg(rtf, old_x, old_y, new_x, new_y, "r");
		old_x = new_x;
		old_y = new_y;
	      }
	      ipt1 = ipt2;
	      ipt2 = ipt3;
	      ipt3 = ipt4;
	      if (!n_dup--) {
		if (ipt4 >= glyph_def.endPtsOfContours[icon]) break;
		ipt4++;
		n_dup = (flag[ipt4] & 1) * 2;
	      }
	    }
	    con_start = ipt4 + 1;
	  }
	}
/* 3-point quadratic splines */
	if (instr & HALF) {
/* on  off on  - formula handles!
   on  off off - formula handles on off mid
   on  on  on  - all straight lines
   on  on  off - defer.
   off on  on  - 2nd bit straight
   off on  off - defer
   off off on  - formula handles mid off on
   off off off - formula handles mid off mid
*/
	  con_start = 0;
	  for (icon = 0; icon < glyph_def.numberOfContours; icon++) {
	    if (instr & POSTSCRIPT) {
	      StartPSContour(rtf);
	      for (ipt= con_start; ipt <= glyph_def.endPtsOfContours[icon]; ipt++){
		PSContourPoint(rtf, x[ipt], y[ipt], flag[ipt] & 1);
	      }
	      EndPSContour(rtf, "b");
	    } else {
	      prev = glyph_def.endPtsOfContours[icon];
	      for (ipt= con_start; ipt <= glyph_def.endPtsOfContours[icon]; ipt++){
		next = (ipt == glyph_def.endPtsOfContours[icon]) ? con_start
		  : (ipt + 1);
		if (flag[ipt] & 1) {
		  if (flag[next] & 1) {
		    pDrawSeg(rtf, x[ipt], y[ipt], x[next], y[next], "b");
		  }
		} else {
		  if (flag[next] & 1) {
		    xe = x[next];
		    ye = y[next];
		  } else {
		    xe = 0.5 * (x[ipt] + x[next]);
		    ye = 0.5 * (y[ipt] + y[next]);
		  }
		  if (flag[prev] & 1) {
		    xs = x[prev];
		    ys = y[prev];
		  } else {
		    xs = 0.5 * (x[prev] + x[ipt]);
		    ys = 0.5 * (y[prev] + y[ipt]);
		  }
		  old_x = xs;
		  old_y = ys;
#define NSTEP3 8
		  for (istep = 1; istep <= NSTEP3; istep++) {
		    u = istep / (float) NSTEP3;
		    u2 = u * u;
		    ux = 2 * u * (1 - u);
		    uc2 = (1 - u) * (1 - u);
		    new_x = uc2 * xs + ux * x[ipt] + u2 * xe;
		    new_y = uc2 * ys + ux * y[ipt] + u2 * ye;
		    pDrawSeg(rtf, old_x, old_y, new_x, new_y, "b");
		    old_x = new_x;
		    old_y = new_y;
		  }
		}
		prev = ipt;
	      }
	    }
	    con_start = ipt;
	  }
	}
	if (instr & ANY_DRAW) {
	  if (instr & POSTSCRIPT) {
	    EndPSGraph(rtf);
	  } else {
	    EndGraph(rtf);
	  }
	  free(x); free(y); free(flag); free(codes);
	}
      } else if (instr & COMPILER) {
	struct component_entry *comp;
	const char *pscompname;
	int pscomplen;

	comp = glyph_def.components;
	while (comp) {
	  fprintf(stdout, "  component\n");

	  if (instr & POSTNAME) {
	    get_postname(comp->the_glyphIndex, &pscompname, &pscomplen);
	  } else {
	    pscompname = 0;
	  }
	  if (pscompname && pscomplen) {
	    fprintf(stdout, "    use %.*s -- #%d\n",
		    pscomplen, pscompname, comp->the_glyphIndex);
	  } else {
	    fprintf(stdout, "    use glyph_%d\n", comp->the_glyphIndex);
	  }
	  if (ARGS_ARE_XY_VALUES & comp->flags) {
	    if (ARG_1_AND_2_ARE_WORDS & comp->flags) {
	      fprintf(stdout, "    word_xy %d %d\n",
		      comp->argument1, comp->argument2);
	    } else {
	      fprintf(stdout, "    byte_xy %d %d\n",
		      comp->argument1, comp->argument2);
	    }
	  } else {
	    if (ARG_1_AND_2_ARE_WORDS & comp->flags) {
	      fprintf(stdout, "    word_points %d %d\n",
		      comp->argument1, comp->argument2);
	    } else {
	      fprintf(stdout, "    points %d %d\n",
		      comp->argument1, comp->argument2);
	    }
	  }
	  if (ROUND_XY_TO_GRID & comp->flags) fprintf(stdout, "    round\n");
	  if (USE_MY_METRICS & comp->flags) {
	    fprintf(stdout, "    use_my_metrics\n");
	  }
	  if (OVERLAP_COMPOUND & comp->flags) {
	    fprintf(stdout, "    overlap_compound\n");
	  }
	  if (SCALED_COMPONENT_OFFSET & comp->flags) {
	    fprintf(stdout, "    scaled_offset\n");
	  }
	  if (UNSCALED_COMPONENT_OFFSET & comp->flags) {
	    fprintf(stdout, "    unscaled_offset\n");
	  }
	  /* Should add the flag_bit facility for other flags. */
	  switch ((WE_HAVE_A_SCALE | WE_HAVE_AN_X_AND_Y_SCALE |
		   WE_HAVE_A_TWO_BY_TWO)
		  & comp->flags) {
	  case 0: /* No scalings */
	    break;
	  case WE_HAVE_A_SCALE:
	    fprintf(stdout, "    scale %d\n", comp->scale);
	    break;
	  case WE_HAVE_AN_X_AND_Y_SCALE:
	    fprintf(stdout, "    xscale %d\n", comp->xscale);
	    fprintf(stdout, "    yscale %d\n", comp->yscale);
	    break;
	  case WE_HAVE_A_TWO_BY_TWO:
	    fprintf(stdout, "    xscale  %d\n", comp->xscale);
	    fprintf(stdout, "    scale01 %d\n", comp->scale01);
	    fprintf(stdout, "    scale10 %d\n", comp->scale10);
	    fprintf(stdout, "    yscale  %d\n", comp->yscale);
	    break;
	  default:
	    fprintf(stderr, "Inconsistent scalings for glyph %d - flag 0x%2x\n",
		    glyphno, comp->flags);
	    fprintf(stdout, "    scalings corrupt!  Flag = 0x%2x\n",
		    comp->flags);
	  }
	  fprintf(stdout, "  end component\n");
	  comp = comp->_next;
	}
      }
      free_glyph_entry(&glyph_def);
    }
    if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
      if (LONG_OFFSETS) {
	fprintf(stdout, " 2 long_offset: %u\n",
		location.long_offsets[glyphno]);
      } else {
	fprintf(stdout, " 2 short_offset: %hu\n",
		location.short_offsets[glyphno]);
      }
      if (pos_next >= pos) {
	fprintf(stdout, " 2 unused_bytes: %d bytes\n", pos_next - pos);
      } else {
	fprintf(stdout, " 2 OVERRUN: %d bytes\n", pos - pos_next);
      }
    }
    if (instr & COMPILER) {
      while (bgrptr && bgrptr->GID < glyphno) bgrptr = bgrptr->_next;
      if (bgrptr && bgrptr->GID == glyphno) {
        struct layer_record *layptr;
        int ntogo;
        fprintf(stdout, "  colour\n");
        for (ntogo = bgrptr->numLayers,
               layptr = element(COLR_layers.layers, bgrptr->firstLayerIndex);
             ntogo > 0;
             ntogo--, layptr = layptr->_next) {
          if (0xFFFF == layptr->paletteIndex) {
            fprintf(stdout, "    %s foreground\n",
                    glyphname1(instr, layptr->GID));
          } else {
            fprintf(stdout, "    %s col%d\n",
                    glyphname1(instr, layptr->GID),
                    p0c0 + layptr->paletteIndex);
          }
        }
        fprintf(stdout, "  end colour\n");
      }
      fprintf(stdout, "end glyph\n");
    }
#if 0
    if (glyphno > 10) break;
#endif
  }
  if (!rtf) {
    /* Nothing to do */
  } else if (instr & POSTSCRIPT) {
    EndPSPage(rtf);
    EndPSFile(rtf);
  } else {
    EndWordFile(rtf);
  }
  if (instr & (DISPLAY_RAW|DISPLAY_RES)) {
    fprintf(stdout, " 1 [%2.2d]:\n", glyphno);
    if (LONG_OFFSETS) {
      fprintf(stdout, " 2 long_offset: %u\n", location.long_offsets[glyphno]);
    } else {
      fprintf(stdout, " 2 short_offset: %hu\n",
	      location.short_offsets[glyphno]);
    }
  }
}

/* Get, display and explain maximum profile. */
void display_maxp(FILE *fp, int instr, struct offset_table top){
  struct directory_entry *maxp_table;
  struct maximum_profile mp;
  int pos, junk;

  for (maxp_table = top.table_directory; maxp_table;
       maxp_table = maxp_table->_next) {
    if (!strncmp(maxp_table->tag, "maxp", 4)) break;
  }
  if (maxp_table) {
    pos = maxp_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking maximum table");
      exit(1);
    }
    get_bine_maximum_profile(fp, &mp, &junk, &pos);
    if (instr & (DISPLAY_RES | DISPLAY_RAW)) {
      /* Display it. */
      print_maximum_profile(stdout, &mp, &junk, "maxp", 1);
    }
    if (instr & COMPILER) {
      fprintf(stdout, "\n-- numGlyphs %d\n", mp.numGlyphs);
      if (mp.tableVersionNumber != 0x00005000) {
	fprintf(stdout, "maxPoints %d\n",             mp.maxPoints);
	fprintf(stdout, "maxContours %d\n",           mp.maxContours);
	fprintf(stdout, "maxCompositePoints %d\n",    mp.maxCompositePoints);
	fprintf(stdout, "maxCompositeContours %d\n",  mp.maxCompositeContours);
	fprintf(stdout, "maxZones %d\n",              mp.maxZones);
	fprintf(stdout, "maxTwilightPoints %d\n",     mp.maxTwilightPoints);
	fprintf(stdout, "maxStorage %d\n",            mp.maxStorage);
	fprintf(stdout, "maxFunctionDefs %d\n",       mp.maxFunctionDefs);
	fprintf(stdout, "maxInstructionDefs %d\n",    mp.maxInstructionDefs);
	fprintf(stdout, "maxStackElements %d\n",      mp.maxStackElements);
	fprintf(stdout, "maxSizeOfInstructions %d\n", mp.maxSizeOfInstructions);
	fprintf(stdout, "maxComponentElements %d\n",  mp.maxComponentElements);
	fprintf(stdout, "maxComponentDepth %d\n",     mp.maxComponentDepth);
      }
    }
  }
}

/* Get, display and explain name table. */
void display_name(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *name_table;
  int pos, junk, n;
  struct name_table name_header;
  struct name_record *nameRecord;
  char label[60];
  struct string_dump f0e;
  int wide, foreign;
  char *low_part;
  if (1) fprintf(stderr, "Invoked display_name(%8.8X).\n", instr);
  for (name_table = top.table_directory; name_table;
       name_table = name_table->_next) {
    if (!strncmp(name_table->tag, "name", 4)) break;
  }
  if (name_table) {
    pos = name_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking name table");
      exit(1);
    }
    get_bine_name_table(fp, &name_header, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_name_table(stdout, &name_header, &junk, "name", 0);
    }
    for (n = 0, nameRecord = name_header.nameRecord;
	 n < name_header.count && nameRecord;
	 nameRecord = nameRecord->_next, n++) {
      pos = name_table->offset + name_header.stringOffset + nameRecord->offset;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking string");
	exit(1);
      }
      byte_dump_count = nameRecord->length;
      get_bine_string_dump(fp, &f0e, &junk, &pos);
      sprintf(label, "n=%d p=%d e=%d l=%d name=%d len=%d off=%d",
	      n, nameRecord->platformID, nameRecord->encodingID,
	      nameRecord->languageID, nameRecord->nameID, nameRecord->length,
	      nameRecord->offset);
      low_part = 0;
      switch (nameRecord->platformID) {
      case 0: /* Unicode platform */
	/* Assume all in BMP for now. */
	wide = 1;
	foreign = 0;
	break;
      case 3: /* Microsoft */
	switch (nameRecord->encodingID) {
	case 0:
	  wide = 0;
	  foreign = 1;
	  break;
	case 1:
	case 10:
	  wide = 1;
	  foreign = 0;
	  break;
	default:
	  wide = 0;
	  foreign = 1;
	};
	break;
      case 1: /* Macintosh */
	switch (nameRecord->encodingID) {
	case 0:
	  wide = 0;
	  foreign = 0;
	  break;
	default:
	  wide = 0;
	  foreign = 1;
	}
	break;
      case 2: /* ISO */
	switch (nameRecord->encodingID) {
	case 0: /* ASCII */
	  wide = 0;
	  foreign = 0;
	  break;
	case 1: /* Unicode */
	  wide = 1;
	  foreign = 0;
	  break;
	case 2: /* Latin-1 */
	  wide = 0;
	  foreign = 0;
	  break;
	default: /* Undefined! */
	  wide = 2;
	  foreign = 1;
	  break;
	}
	break;
      default: /* No definition available */
	wide = 2;
	foreign = 1;
      }
/*Z   fprintf(stdout, "wide=%d  foreign=%d\n", wide, foreign); Z*/
      if (1 == wide) {
	char *lp, onch;
	/* See if we can use alternative letters. */
	if (!(lp = low_part = (char *)malloc((f0e._n_chars+3)/2))) {
	  fprintf(stderr, "Insufficient memory displaying name table.\n");
	  foreign = 1;
	} else {
	  int ipos = 0;
	  for (; ipos < f0e._n_chars; ) {
	    if (!f0e.chars[ipos++]) {
	      onch = *lp++ = f0e.chars[ipos++];
/* Test for copyright symbol is complex because of signed char */
	      if (0xa9 == ((int) onch & 0x0ff)) {
		/* Allow copyright symbol through */
	      } else if (0xae == ((int) onch & 0x0ff)) {
		/* Allow registered mark symbol through */
	      } else if (onch <= 31 || onch >= 127) {
		foreign = 1;
		break;
	      }
	    } else {
	      foreign = 1;
/*Z	      fprintf(stdout, "Hoped for 0, got %d.\n", f0e.chars[ipos-1]); Z*/
	      break;
	    }
	  }
	  *lp = 0;
	}
      }
      if ((instr & DISPLAY_RAW) || (foreign && instr & DISPLAY_RES)) {
      /* Plain byte dump */
	print_string_dump(stdout, &f0e, &junk, label, 2);
      }
      if ((instr & DISPLAY_RES) && !foreign) {
      /* Print as simple string. */
	if (!(instr & DISPLAY_RAW)) {
	  fprintf(stdout, "%s\n", label);
	}
	if (1 == wide) {
	  fprintf(stdout, "%s\n", low_part);
	} else {
	  fprintf(stdout, "%.*s\n", f0e._n_chars, f0e.chars);
	}
      }
      if ((instr & COMPILER)) {
	int ipos, code, code2, high;
	static const char * const nameID[21] = {
	  "copyright",        "family",   "subfamily",       "identifier",
	  "full_name",        "version",  "PostScript_name", "trademark",
          "manufacturer",     "designer", "description",     "vendor_URL",
          "designer_URL",     "licence",  "licence_URL",     "15",
          "preferred_family", "preferred_subfamily",
             "compatible_full_name", "sample",
          "PostScript_CID_name"};

	fprintf(stdout, "name ");
	if (nameRecord->nameID <= 20) {
	  fprintf(stdout, "%s ", nameID[nameRecord->nameID]);
	} else {
	  fprintf(stdout, "%d ", nameRecord->nameID);
	}
	switch (nameRecord->platformID) {
	case 0:
	  fprintf(stdout, "Unicode ");
	  switch (nameRecord->encodingID) {
	  case 0:
	    fprintf(stdout, "v1.0 ");
	    break;
	  case 1:
	    fprintf(stdout, "v1.1 ");
	    break;
	  case 2:
	    fprintf(stdout, "ISO10646 ");
	    break;
	  case 3:
	    fprintf(stdout, "BMP ");
	    break;
	  case 4:
	    fprintf(stdout, "UCS-4");
	    break;
	  default:
	    fprintf(stdout, "%d ", nameRecord->encodingID);
	  }
#if 0
	  switch(nameRecord->languageID) {
	  case 0x0409:
	    fprintf(stdout, "en_US ");
	    break;
	  case 0x0809:
	    fprintf(stdout, "en_UK ");
	    break;
	  case 0x041e:
	    fprintf(stdout, "Thai ");
	    break;
	  default:
	    fprintf(stdout, "0x%4.4x ", nameRecord->languageID);
	  }
#endif
	  fprintf(stdout, "%d ", nameRecord->languageID); /* All undefined! */
	  fputc('"', stdout);
	  for (ipos = 0; ipos < f0e._n_chars; ipos+=2) {
	    code = ((unsigned char) f0e.chars[ipos] << 8) +
	      (unsigned char) f0e.chars[ipos+1];
	    if (32 <= code && code <= 127) {
	      switch (code) {
	      case '\\':
		fprintf(stdout, "\\\\");
		break;
	      case '\"':
		fprintf(stdout, "\\\"");
		break;
	      default:
		fputc(code, stdout);
	      }
	    } else if (code <= 255) {
	      fprintf(stdout, "\\x%2.2x", code);
	    } else if (code < 0xd800 || 0xe000 <= code) {
	      fprintf(stdout, "\\u%4.4x", code);
	    } else if (code < 0xdc00) { /* High surrogate */
	      high = code & 0x3ff;
	      if (ipos + 4 <= f0e._n_chars) {
		code2 = ((unsigned char) f0e.chars[ipos+2] << 8) +
		  (unsigned char) f0e.chars[ipos+3];
		if (0xdc00 <= code2 && code2 < 0xe000) {
		  code = (high << 10) + (1<<16) + (code2 & 0x3ff);
		  fprintf(stdout, "\\U%6.6x", code);
		  ipos += 2;
		} else { /* Unmatched! */
		  fprintf(stdout, "\\N%4.4x", code);
		}
	      } else { /* String truncated in midst of a value! */
		fprintf(stdout, "\\N%4.4x", code);
	      }
	    } else { /* Unmatched low surrogate */
	      fprintf(stdout, "\\N%4.4x", code);
	    }
	  }
	  fprintf(stdout, "\"\n");
	  break;
	case 1:
	  fprintf(stdout, "Macintosh ");
	  switch (nameRecord->encodingID) {
	  case 0:
	    fprintf(stdout, "Roman ");
	    break;
	  case 1:
	    fprintf(stdout, "Japanese ");
	    break;
	  case 2:
	    fprintf(stdout, "Chinese_Traditional ");
	    break;
	  case 3:
	    fprintf(stdout, "Korean ");
	    break;
	  case 4:
	    fprintf(stdout, "Arabic ");
	    break;
	  case 5:
	    fprintf(stdout, "Hebrew ");
	    break;
	  case 6:
	    fprintf(stdout, "Greek ");
	    break;
	  case 7:
	    fprintf(stdout, "Russian ");
	    break;
	  case 8:
	    fprintf(stdout, "RSymbol ");
	    break;
	  case 9:
	    fprintf(stdout, "Devanagari ");
	    break;
	  case 10:
	    fprintf(stdout, "Gurmukhi ");
	    break;
	  case 11:
	    fprintf(stdout, "Gujarati ");
	    break;
	  case 12:
	    fprintf(stdout, "Oriya ");
	    break;
	  case 13:
	    fprintf(stdout, "Bengali ");
	    break;
	  case 14:
	    fprintf(stdout, "Tamil ");
	    break;
	  case 15:
	    fprintf(stdout, "Telugu ");
	    break;
	  case 16:
	    fprintf(stdout, "Kannada ");
	    break;
	  case 17:
	    fprintf(stdout, "Malayalam ");
	    break;
	  case 18:
	    fprintf(stdout, "Sinhalese ");
	    break;
	  case 19:
	    fprintf(stdout, "Burmese ");
	    break;
	  case 20:
	    fprintf(stdout, "Khmer ");
	    break;
	  case 21:
	    fprintf(stdout, "Thai ");
	    break;
	  case 22:
	    fprintf(stdout, "Laotian ");
	    break;
	  case 23:
	    fprintf(stdout, "Georgian ");
	    break;
	  case 24:
	    fprintf(stdout, "Armenian ");
	    break;
	  case 25:
	    fprintf(stdout, "Chinese_Simplified ");
	    break;
	  case 26:
	    fprintf(stdout, "Tibetan ");
	    break;
	  case 27:
	    fprintf(stdout, "Mongolian ");
	    break;
	  case 28:
	    fprintf(stdout, "Geez ");
	    break;
	  case 29:
	    fprintf(stdout, "Slavic ");
	    break;
	  case 30:
	    fprintf(stdout, "Vietnamese ");
	    break;
	  case 31:
	    fprintf(stdout, "Sindhi ");
	    break;
	  case 32:
	    fprintf(stdout, "Uninterpreted ");
	    break;
	  default:
	    fprintf(stdout, "%d ", nameRecord->encodingID);
	  }
	  switch(nameRecord->languageID) {
	  case 0xffff:
	    fprintf(stdout, "None ");
	    break;
	  case 0:
	    fprintf(stdout, "English ");
	    break;
	  case 1:
	    fprintf(stdout, "French ");
	    break;
	  case 22:
	    fprintf(stdout, "Thai ");
	    break;
	  case 78:
	    fprintf(stdout, "Khmer ");
	    break;
	  case 79:
	    fprintf(stdout, "Lao ");
	    break;
	  default:
	    fprintf(stdout, "%d ", nameRecord->languageID);
	  }
	  /* Assume ASCII superset */
	  fputc('"', stdout);
	  for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	    code = (unsigned char) f0e.chars[ipos];
	    if ('"' == code) {
	      fprintf(stdout, "\\\"");
	    } else if ('\\' == code) {
	      fprintf(stdout, "\\x%2.2x", code);
	    } else if (32 <= code && code < 127) {
	      fputc(code, stdout);
	    } else {
	      fprintf(stdout, "\\x%2.2x", code);
	    }
	  }
	  fprintf(stdout, "\"\n");
	  break;
	case 3:
	  fprintf(stdout, "Microsoft ");
	  switch(nameRecord->encodingID) {
	  case 0:
	    fprintf(stdout, "symbol ");
	    break;
	  case 1:
	    fprintf(stdout, "BMP ");
	    break;
	  case 2:
	    fprintf(stdout, "ShiftJIS ");
	    break;
	  case 3:
	    fprintf(stdout, "PRC ");
	    break;
	  case 4:
	    fprintf(stdout, "Big5 ");
	    break;
	  case 5:
	    fprintf(stdout, "Wansung ");
	    break;
	  case 6:
	    fprintf(stdout, "Johab ");
	    break;
	  case 10:
	    fprintf(stdout, "UCS-4 ");
	    break;
	  default:
	    fprintf(stdout, "%d ", nameRecord->encodingID);
	  }
	  switch(nameRecord->languageID) {
	  case 0x0409:
	    fprintf(stdout, "en_US ");
	    break;
	  case 0x0809:
	    fprintf(stdout, "en_UK");
	    break;
	  case 0x041e:
	    fprintf(stdout, "Thai ");
	    break;
	  default:
	    fprintf(stdout, "%d ", nameRecord->languageID);
	  }
	  fputc('"', stdout);
	  switch(nameRecord->encodingID) {
	  case 0: /* Symbol */
	    for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	      fprintf(stdout, "\\x%2.2x", (unsigned char) f0e.chars[ipos]);
	    }
	    break;
	  case 1:  /* BMP */
	  case 10: /* UCS-4 */
	    for (ipos = 0; ipos < f0e._n_chars; ipos+=2) {
	      code = ((unsigned char) f0e.chars[ipos] << 8) +
		(unsigned char) f0e.chars[ipos+1];
	      if (32 <= code && code <= 127) {
		switch (code) {
		case '\\':
		  fprintf(stdout, "\\\\");
		  break;
		case '\"':
		  fprintf(stdout, "\\\"");
		  break;
		default:
		  fputc(code, stdout);
		}
	      } else if (code <= 255) {
		fprintf(stdout, "\\x%2.2x", code);
	      } else {
		fprintf(stdout, "\\u%4.4x", code);
	      }
	    }
	    break;
	  case 2: /* ShiftJIS */
	    for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	      code = (unsigned char) f0e.chars[ipos];
	      if (0x81 <= code && code <= 0x9f ||
		  0xe0 <= code && code <= 0xef) {
		code = (code << 8) + (unsigned char) f0e.chars[++ipos];
		fprintf(stdout, "\\N%4.4x", code);
	      } else if ('\\' == code) {
		fprintf(stdout, "\\x%2.2x", code);
	      } else if (32 <= code && code < 127) {
		fputc(code, stdout);
	      } else {
		fprintf(stdout, "\\x%2.2x", code);
	      }
	    }
	    break;
	  case 4: /* Big5 */
	    for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	      code = (unsigned char) f0e.chars[ipos];
	      if (0xa1 <= code && code <= 0xfb) {
		code = (code << 8) + (unsigned char) f0e.chars[++ipos];
		fprintf(stdout, "\\N%4.4x", code);
	      } else if ('\\' == code) {
		fprintf(stdout, "\\x%2.2x", code);
	      } else if (32 <= code && code < 127) {
		fputc(code, stdout);
	      } else {
		fprintf(stdout, "\\x%2.2x", code);
	      }
	    }
	    break;
	  case 3: /* PRC */
	  case 5: /* Wansung */
	  case 6: /* Johab */
	  default:
	    /* Assume ASCII superset */
	    for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	      code = (unsigned char) f0e.chars[ipos];
	      if ('"' == code) {
		fprintf(stdout, "\\\"");
	      } else if ('\\' == code) {
		fprintf(stdout, "\\%2.2x", code);
	      } else if (32 <= code && code < 127) {
		fputc(code, stdout);
	      } else {
		fprintf(stdout, "\\%2.2x", code);
	      }
	    }
	  }
	  fprintf(stdout, "\"\n");
	  break;
	case 4:
	default:
	  if (4 == nameRecord->platformID) {
	    fprintf(stdout, "Custom ");
	  } else {
	    fprintf(stdout, "%d ", nameRecord->platformID);
	  }
	  fprintf(stdout, "%d ", nameRecord->encodingID);
	  fprintf(stdout, "%d ", nameRecord->languageID);
	  /* Assume ASCII superset */
	  for (ipos = 0; ipos < f0e._n_chars; ipos++) {
	    code = (unsigned char) f0e.chars[ipos];
	    if ('"' == code) {
	      fprintf(stdout, "\\\"");
	    } else if ('\\' == code) {
	      fprintf(stdout, "\\x%2.2x", code);
	    } else if (32 <= code && code < 127) {
	      fputc(code, stdout);
	    } else {
	      fprintf(stdout, "\\x%2.2x", code);
	    }
	  }
	}
      }
      /* Clean up */
      free_string_dump(&f0e);
      if (low_part) free(low_part);
      }
    if (n < name_header.count) {
      fprintf(stdout, "Output terminated prematurely.\n");
    }
    free_name_table(&name_header);
  }
}

extern void display_coverage(FILE *fp, int pos, const char *label, int level);
extern void decompile_coverage(FILE *src, int pos,
                               FILE *dst, int instr, const char *name);
/* Get, display and explain GDEF table. */
#define MGS "mgs%d"
void display_GDEF(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *GDEF_table;
  struct gdef_table GDEF_header;
  struct attachment_list_table att_list_table;
  struct class_definition_table class_def_table;
  struct lig_caret_list_table caret_list_table;
  int pos, junk;
  int_set ligs;
  int nlig = 0;
  int dumpit;


  if (1) fprintf(stderr, "Invoked display_GDEF(%8.8X).\n", instr);
  for (GDEF_table = top.table_directory; GDEF_table;
       GDEF_table = GDEF_table->_next) {
    if (!strncmp((char*)GDEF_table->tag, "GDEF", 4)) break;
  }
  if (GDEF_table) {
    pos = GDEF_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking GDEF table");
      exit(1);
    }
    get_bine_gdef_table(fp, &GDEF_header, &junk, &pos);
    dumpit = instr & (DISPLAY_RAW | DISPLAY_RES);
    if (dumpit) print_gdef_table(stdout, &GDEF_header, &junk, "GDEF", 0);
    ligs = create_int_set();
    if (dumpit && GDEF_header.attachList) {
      pos = GDEF_table->offset + GDEF_header.attachList;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking attachment list");
	exit(1);
      }
      get_bine_attachment_list_table(fp, &att_list_table, &junk, &pos);
      print_attachment_list_table(stdout, &att_list_table, &junk,
				  "attachList->", 1);
      free_attachment_list_table(&att_list_table);
    }
    if (GDEF_header.glyphClassDef) {
      int i, cat, catmax = 0;

      pos = GDEF_table->offset + GDEF_header.glyphClassDef;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking glyph class definition");
	exit(1);
      }
      if (instr & COMPDAT) {
	category =      malloc(N_GLYPHS * sizeof * category);
	memset(category, 0xFF, N_GLYPHS * sizeof * category);
      }
      get_bine_class_definition_table(fp, &class_def_table, &junk, &pos);
      if (dumpit) {
	print_class_definition_table(stdout, &class_def_table, &junk,
				     "glyphClassDef->", 1);
      }
      if (1 == class_def_table.classFormat) {
	int gid;

	for (i = 0; i < class_def_table.glyphCount; i++) {
	  gid = class_def_table.startGlyph + i;
	  cat = class_def_table.classValueArray[i];
	  if (2 == cat && (instr & DISPLAY_RES)) {
	    (void) add_range(gid, gid, &ligs);
	    nlig++;
	  }
#define limit_category \
	  if (cat >= NO_CATEGORY) { \
	    if (cat > catmax) { \
	      fprintf(stderr, "Glyph class definition has type code %d for " \
		      "glyph %d - mistreated as %d.\n", \
		      cat, gid, NO_CATEGORY - 1); \
	      catmax = cat; \
	      cat = NO_CATEGORY - 1; \
	    } \
	  }
	  if (instr & COMPDAT) {
	    limit_category;
	    category[gid] = cat;
	  }
	}
      } else if (2 == class_def_table.classFormat) {
	struct class_range_record *rec;
	int gid;

	for (rec = class_def_table.classRangeRecord; rec; rec = rec->_next) {
	  gid = rec->start;
	  cat = rec->class;
	  if (2 == cat && (instr & DISPLAY_RES)) {
	    (void) add_range(rec->start, rec->end, &ligs);
	    nlig += (rec->end - rec->start + 1);
	  }
	  if (instr & COMPDAT) {
	    limit_category;
	    for (i = rec->start; i <= rec->end; i++) category[i] = cat;
	  }
	}
      }
      free_class_definition_table(&class_def_table);
    }
    if (GDEF_header.ligCaretList && dumpit) {
/* Seen in /usr/share/fonts/truetype/freefont/FreeSans.ttf 

   However, the table formats are all zeroes and the numbers of entries
   don't stack up! */
      int lclpos = GDEF_table->offset + GDEF_header.ligCaretList;
      int icv, covpos;

      pos = lclpos;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking ligature boundary definition");
	exit(1);
      }
      get_bine_lig_caret_list_table(fp, &caret_list_table, &junk, &pos);
      print_lig_caret_list_table(stdout, &caret_list_table, &junk,
				 "ligCaretList->", 1);
      if (caret_list_table.coverage) {
	int covpos = lclpos + caret_list_table.coverage;
	struct coverage_table coverage;

	pos = covpos;
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking ligature boundary coverage");
	  exit(1);
	}
	get_bine_coverage_table(stdout, &coverage, &junk, &pos);
	print_coverage_table(stdout, &coverage, &junk, "coverage->", 2);
/* Disabled because data does not match */
	if (0 && 0 == coverage.coverageFormat && (instr & DISPLAY_RES) &&
	    caret_list_table.ligGlyphCount > 0) {
	  int j, ilig = 0;
	  struct int_range *irge;

	  fprintf(stdout, " 3 nlig: %d %4.4x\n 3 glyphs:\n", nlig, nlig);
	  for (irge = ligs; irge; irge = irge->next) {
	    for (j = irge->start; j <=  irge->end; j++) {
	      fprintf(stdout, " 4 [%d]: %d %4.4x\n", ilig++, j, j);
	    }
	  }
	}
	free_coverage_table(&coverage);
      }
      for (icv = 0; icv < caret_list_table.ligGlyphCount; icv++) {
	int cvpos;
	struct caret_value_table cvt;
	char index[20];
	int headed = 0;

	if (!caret_list_table.ligGlyph[icv]) continue;
	if (!headed) {
	  fprintf(stdout, " 2 ligGlyph->:\n");
	  headed = 1;
	}
	sprintf(index, "[%d]", icv);
	pos = cvpos = lclpos + caret_list_table.ligGlyph[icv];
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking ligature boundary");
	  exit(1);
	}
	get_bine_caret_value_table(stdout, &cvt, &junk, &pos);
	print_caret_value_table(stdout, &cvt, &junk, index, 3);
	free_caret_value_table(&cvt);
      }
      free_lig_caret_list_table(&caret_list_table);
    }
    if (GDEF_header.markAttachClassDef) {
      int macpos = GDEF_table->offset + GDEF_header.markAttachClassDef;
      int i, cat, catmax = 0;

      pos = macpos;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking mark attachment class definition table");
	exit(1);
      }
      if (instr & COMPDAT) {
	markClass =      malloc(N_GLYPHS * sizeof * markClass);
	memset(markClass, 0xFF, N_GLYPHS * sizeof * markClass);
      }
      get_bine_class_definition_table(fp, &class_def_table, &junk, &pos);
      if (dumpit) {
	print_class_definition_table(stdout, &class_def_table, &junk,
				     "markAttachClassDef->", 1);
      }
      if (instr & COMPDAT) {
	int gid;

	if (1 == class_def_table.classFormat) {
	  for (i = 0; i < class_def_table.glyphCount; i++) {
	    gid = class_def_table.startGlyph + i;
	    cat = class_def_table.classValueArray[i];
#define limit_class							\
	    if (cat >= NO_MAC) {					\
	      if (cat > catmax) {					\
		fprintf(stderr, "Glyph class definition has type code %d for " \
			"glyph %d - mistreated as %d.\n",		\
			cat, gid, NO_MAC - 1);				\
		catmax = cat;						\
		cat = NO_MAC - 1;					\
	      }								\
	    }
	    limit_class;
	    markClass[gid] = cat;
	  }
	} else if (2 == class_def_table.classFormat) {
	  struct class_range_record *rec;

	  for (rec = class_def_table.classRangeRecord; rec; rec = rec->_next) {
	    gid = rec->start;
	    cat = rec->class;
	    limit_class;
	    for (i = rec->start; i <= rec->end; i++) markClass[i] = cat;
	  }
	}
      }
      free_class_definition_table(&class_def_table);
    }
    if (GDEF_header.markGlyphSetsDef) {
      int mgspos = GDEF_table->offset + GDEF_header.markGlyphSetsDef;
      struct mark_glyph_sets_table mgsi;
      int is;

      pos = mgspos;
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking mark glyph sets definition table");
	exit(1);
      }
      get_bine_mark_glyph_sets_table(fp, &mgsi, &junk, &pos);
      if (dumpit) {
	print_mark_glyph_sets_table(stdout, &mgsi, &junk,
				     "markGlyphSetsDef->", 1);
      }
      if (dumpit) fputs(" 2 offsets->\n", stdout);
      for (is = 0; is < mgsi.count; is++) {
        char label[20];
        pos = mgspos + mgsi.offsets[is];
        if (fseek(fp, pos, SEEK_SET)) {
          perror("Seeking a mark glyph set defintion");
          exit(1);
        }
        if (dumpit) {
          sprintf(label, "[%d]", is);
          display_coverage(fp, pos, label, 3);
        }
        if (instr & COMPILER) {
          sprintf(label, MGS, is);
          decompile_coverage(fp, pos, stdout, instr, label);
        }
      }
      free_mark_glyph_sets_table(&mgsi);
    }
    delete_int_set(&ligs);
    free_gdef_table(&GDEF_header);
  }
}

/* Utilities to display 'coverage' for GDEF, GPOS and GSUB (& others if need
   be).*/
/* Simple display: */
void display_coverage(FILE *fp, int pos, const char *label, int level) {
  struct coverage_table coverage;
  int junk;

  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking GDEF/GPOS/GSUB coverage table");
    exit(1);
  }
  get_bine_coverage_table(fp, &coverage, &junk, &pos);
  print_coverage_table(stdout, &coverage, &junk, label, level);
  free_coverage_table(&coverage);
}
/* Extraction for manual reporting */
void get_coverage(FILE *fp, int pos, struct coverage_table *coverage, int *ngl){
  int junk;
  struct range_record *rr;

  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking GPOS/GSUB coverage table");
    exit(1);
  }
  get_bine_coverage_table(fp, coverage, &junk, &pos);
  *ngl = 0;
  switch (coverage->coverageFormat) {
  case 1:
    *ngl = coverage->glyphCount;
    break;
  case 2:
    for (rr = coverage->rangeRecord; rr; rr = rr->_next) {
      *ngl += rr->end - rr->start + 1;
    }
    break;
  default:
    fprintf(stderr, "Unrecognised coverage format %d encountered at line %d "
	    "of %s.\n", coverage->coverageFormat, __LINE__, __FILE__);
    exit(1);
  }
}

struct coverage_iteration {
  struct coverage_table *coverage;
  struct range_record *rr;
  int pos;
};

struct coverage_iteration make_coverage_iteration(
  struct coverage_table *coverage) {
  struct coverage_iteration retval;
  retval.coverage = coverage;
  retval.pos = 0;
  if (2 == coverage->coverageFormat) {
    retval.rr = coverage->rangeRecord;
  }
  return retval;
}

int get_from_coverage(struct coverage_iteration *ci) {
  int gret;
  switch (ci->coverage->coverageFormat) {
  case 1:
    if (ci->pos >= ci->coverage->glyphCount) {
      return -1;
    } else {
      return ci->coverage->glyphArray[ci->pos++];
    }
    break;
  case 2:
    if (!ci->rr) return -1;
    gret = ci->rr->start + ci->pos++;
    if (gret >= ci->rr->end) {
      ci->rr = ci->rr->_next;
      ci->pos = 0;
    }
    return gret;
  default:
    fprintf(stderr, "Unrecognised coverage format %d encountered at line %d "
	    "of %s.\n", ci->coverage->coverageFormat, __LINE__, __FILE__);
    exit(1);
  }
}  

void decompile_coverage(FILE *src, int pos,
			FILE *dst, int instr, const char *name) {
  struct coverage_table coverage;
  int junk, igl, nput;
  struct range_record *rr;
  const char *mode;

  if (fseek(src, pos, SEEK_SET)) {
    perror("Seeking GDEF/GPOS/GSUB coverage table");
    exit(1);
  }
  get_bine_coverage_table(src, &coverage, &junk, &pos);
  mode = "is";
  switch (coverage.coverageFormat) {
  case 1:
    if (0 == coverage.glyphCount) {
      fprintf(dst, "coverage %s is very_empty\n", name);
    }
    for (igl = 0; igl < coverage.glyphCount; igl++) {
      fprintf(dst, "coverage %s %s %s\n", name, mode,
	      glyphname1(instr, coverage.glyphArray[igl]));
      mode = "also";
    }
    break;
  case 2:
    if (!coverage.rangeRecord) {
      fprintf(dst, "coverage %s is very_empty\n", name);
    }
    for (rr = coverage.rangeRecord; rr; rr = rr->_next) {
      if (rr->start == rr->end) {
	fprintf(dst, "coverage %s %s %s\n", name, mode,
		glyphname1(instr, rr->start));
      } else {
	fprintf(dst, "coverage %s %s %s to %s\n", name, mode,
		glyphname1(instr, rr->start), glyphname2(instr, rr->end));
      }
      mode = "also";
    }
    break;
  default:
    fprintf(dst, "-- Format %d for coverage %s not understood.\n",
	    coverage.coverageFormat, name);
  }
  free_coverage_table(&coverage);
}

/* Utilities to display 'class' for GPOS and GSUB (& others if need be).*/
/* Simple display: */
void display_class(FILE *fp, int pos, const char *label, int level) {
  struct class_definition_table class_def_table;
  int junk;

  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking GPOS/GSUB class definition table");
    exit(1);
  }
  get_bine_class_definition_table(fp, &class_def_table, &junk, &pos);
  print_class_definition_table(stdout, &class_def_table, &junk, label, level);
  free_class_definition_table(&class_def_table);
}

void new_class(FILE *dst, const char *name, int this_class, int *max_class) {
  int icl;

  if (this_class > *max_class) {
    for (icl = 1 + *max_class; icl < this_class; icl++) {
      fprintf(dst, "  empty %s_%d\n", name, icl);
    }
    *max_class = this_class;
  }
}

void decompile_class(FILE *src, int pos,
		     FILE *dst, int instr, const char *name) {
  struct class_definition_table class_def_table;
  int junk, igl, nput, this_class, max_class;
  struct class_range_record *crr;
  const char *sep;

  if (fseek(src, pos, SEEK_SET)) {
    perror("Seeking GPOS/GSUB class definition table");
    exit(1);
  }
  get_bine_class_definition_table(src, &class_def_table, &junk, &pos);
  fprintf(dst, "classification %s\n", name);
  max_class = -1;
  switch (class_def_table.classFormat) {
  case 1:
    for (igl = 0; igl < class_def_table.glyphCount; igl++) {
      this_class = class_def_table.classValueArray[igl];
      new_class(dst, name, this_class, &max_class);
      fprintf(dst, "  class %s_%d %s\n", name, this_class,
	      glyphname1(instr, class_def_table.startGlyph + igl));
    }
    break;
  case 2:
    for (crr = class_def_table.classRangeRecord; crr; crr = crr->_next) {
      new_class(dst, name, crr->class, &max_class);
      if (crr->start == crr->end) {
	fprintf(dst, "  class %s_%d %s\n", name, crr->class,
		glyphname1(instr, crr->start));
      } else {
	fprintf(dst, "  class %s_%d %s to %s\n", name, crr->class,
		glyphname1(instr, crr->start), glyphname2(instr, crr->end));
      }
    }
    break;
  default:
    fprintf(dst, "-- Format %d not understood.\n", class_def_table.classFormat);
  }
  fprintf(dst, "end classification\n");
  free_class_definition_table(&class_def_table);
}

/* Get, display and explain GSUB table. */
const char *feat_name(struct feature_list flt, int ife) {
  static char buffer[20];
  if (ife < flt.featureCount) {
    struct feature_record *ptr_fr;

    ptr_fr = element(flt.featureRecord, ife);
    sprintf(buffer, "%.4s_%d", ptr_fr->tag, ife);
  } else {
    sprintf(buffer, "feat_%d", ife);
  }
  return buffer;
}

void display_GSUBPOS_header(FILE *fp, int instr, struct offset_table top,
			    char *tag, const char *namefmt,
			    const char *types[], int ntypes,
			    struct directory_entry **table,
			    struct lookup_list *llt,
			    struct gsub_header *gshout) {
  struct directory_entry *GSUB_table;
  struct gsub_header GSUB_header;
  struct script_list script_list_table;
  struct script_table the_script_table;
  struct feature_list feature_list_table;
  struct feature_table the_feature_table;
  struct lang_sys lang_sys_table;
  struct lookup_list lookup_list_table;
  struct lookup_table the_lookup_table;
  int  pos, junk, is, ilang, ife, il, ist;
  char buffer[100];

  for (GSUB_table = top.table_directory; GSUB_table;
       GSUB_table = GSUB_table->_next) {
    if (!strncmp((char*)GSUB_table->tag, tag, 4)) break;
  }
  if (!GSUB_table) return;
  pos = GSUB_table->offset;
  if (fseek(fp, pos, SEEK_SET)) {
    sprintf(buffer, "Seeking %s table", tag);
    perror(buffer);
    exit(1);
  }
  get_bine_gsub_header(fp, &GSUB_header, &junk, &pos);
  if (instr & COMPILER) fprintf(stdout, "%s\n", tag);
  if (instr & DISPLAY_RAW) {
    print_gsub_header(stdout, &GSUB_header, &junk, tag, 0);
  }
/* Out of order reading to support compiler-style output. */
  if (GSUB_header.featureList) {
    pos = GSUB_table->offset + GSUB_header.featureList;
    if (fseek(fp, pos, SEEK_SET)) {
      sprintf(buffer, "Seeking %s feature list table", tag);
      perror(buffer);
      exit(1);
    }
    get_bine_feature_list(fp, &feature_list_table, &junk, &pos);
  }
  if (GSUB_header.scriptList) {
    pos = GSUB_table->offset + GSUB_header.scriptList;
    if (fseek(fp, pos, SEEK_SET)) {
      sprintf(buffer, "Seeking %s script list table", tag);
      perror(buffer);
      exit(1);
    }
    get_bine_script_list(fp, &script_list_table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_script_list(stdout, &script_list_table, &junk, "scriptList->", 1);
    }
    for (is = 0; is < script_list_table.scriptCount; is++) {
      int pos0, offset;
      struct script_record* recptr;

      recptr = element(script_list_table.scriptRecord, is);
      if (instr & COMPILER) fprintf(stdout, "  script %.4s\n", recptr->tag);
      offset = recptr->	script;
      if (!offset) {
	if (instr & COMPILER) fprintf(stdout, "  end script\n");
	continue;
      }
      pos0 = pos = GSUB_table->offset + GSUB_header.scriptList + offset;
      if (fseek(fp, pos, SEEK_SET)) {
	sprintf(buffer, "Seeking a %s script table", tag);
	perror(buffer);
	exit(1);
      }
      get_bine_script_table(fp, &the_script_table, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	fprintf(stdout, " 3 [%d]:\n", is);
	print_script_table(stdout, &the_script_table, &junk, "script->", 4);
      }
      for (ilang = 0; ilang < the_script_table.langSysCount; ilang++) {
	struct lang_sys_record* lanptr;
	int offset2;

	lanptr = element(the_script_table.langSysRecord, ilang);
	if (instr & COMPILER) {
	  fprintf(stdout, "    language %.4s\n", lanptr->tag);
	}
	offset2 = lanptr->langSys;
	if (!offset2) {
	  if (instr & COMPILER) fprintf(stdout, "    end language\n");
	  continue;
	}
	pos = pos0 + offset2;
	if (fseek(fp, pos, SEEK_SET)) {
	  sprintf(buffer, "Seeking a %s language system table", tag);
	  perror(buffer);
	  exit(1);
	}
	get_bine_lang_sys(fp, &lang_sys_table, &junk, &pos);
	if (instr & DISPLAY_RAW) {
	  fprintf(stdout, " 6 [%d]->\n", ilang);
	  print_lang_sys(stdout, &lang_sys_table, &junk, "langSys->", 7);
	}
	if (instr & COMPILER) {
	  int ifi;
	  int nput;
	  if (lang_sys_table.lookupOrder) {
	    fprintf(stdout, "-- Non-standard lookup order not supported.\n");
	  }
	  if (lang_sys_table.reqFeatureIndex != 0xFFFF) {
	    fprintf(stdout,
		    "-- Existence of required feature not supported.\n");
	  }
	  if (lang_sys_table.featureCount) fprintf(stdout, "     ");
	  nput = 0;
	  for (ifi = 0; ifi < lang_sys_table.featureCount; ifi++) {
	    int ife = lang_sys_table.featureIndex[ifi];
	    if (nput >= 8) {
	      fprintf(stdout, "\n     ");
	      nput = 0;
	    }
	    if (GSUB_header.featureList) {
	      fprintf(stdout, " %s", feat_name(feature_list_table, ife));
	    } else {
	      fprintf(stdout, " feat_%d", ife);
	    }
	    nput++;
	  }
	  if (lang_sys_table.featureCount) fputc('\n', stdout);
	}
	if (instr & COMPILER) fprintf(stdout, "    end language\n");
	free_lang_sys(&lang_sys_table);
      }
      if (the_script_table.defaultLangSys) {
	pos = pos0 + the_script_table.defaultLangSys;
	if (fseek(fp, pos, SEEK_SET)) {
	  sprintf(buffer, "Seeking %s default language system table", tag);
	  perror(buffer);
	  exit(1);
	}
	get_bine_lang_sys(fp, &lang_sys_table, &junk, &pos);
	if (instr & DISPLAY_RAW) {
	  print_lang_sys(stdout, &lang_sys_table, &junk, "defaultLangSys->", 5);
	}
	if (instr & COMPILER) {
	  int ifi;
	  int nput = 0;
	  fprintf(stdout, "    language default\n");
	  if (lang_sys_table.lookupOrder) {
	    fprintf(stdout, "-- Non-standard lookup order not supported.\n");
	  }
	  if (lang_sys_table.reqFeatureIndex != 0xFFFF) {
	    fprintf(stdout,
		    "-- Existence of required feature not supported.\n");
	  }
	  if (lang_sys_table.featureCount) fprintf(stdout, "     ");
	  for (ifi = 0; ifi < lang_sys_table.featureCount; ifi++) {
	    int ife = lang_sys_table.featureIndex[ifi];
	    if (nput >= 8) {
	      fprintf(stdout, "\n     ");
	      nput = 0;
	    }
	    if (GSUB_header.featureList) {
	      fprintf(stdout, " %s", feat_name(feature_list_table, ife));
	    } else {
	      fprintf(stdout, " feat_%d", ife);
	    }
	    nput++;
	  }
	  if (lang_sys_table.featureCount) fputc('\n', stdout);
	  fprintf(stdout, "    end language\n");
	}
	free_lang_sys(&lang_sys_table);
      }
      free_script_table(&the_script_table);
      if (instr & COMPILER) fprintf(stdout, "  end script\n");
    }
    free_script_list(&script_list_table);
  }
  if (GSUB_header.featureList) {
    if (instr & DISPLAY_RAW) {
      print_feature_list(stdout, &feature_list_table, &junk,
			 "featureList->", 1);
    }
    for (ife = 0; ife < feature_list_table.featureCount; ife++) {
      struct feature_record *ptr_fr;

      ptr_fr = element(feature_list_table.featureRecord, ife);
      if (instr & COMPILER) {
	fprintf(stdout, "  feature %.4s_%d\n", ptr_fr->tag, ife);
	if (!ptr_fr->feature) {
	  fprintf(stdout, "  end feature\n");
	  continue;
	}
      }
      pos = GSUB_table->offset + GSUB_header.featureList + ptr_fr->feature;
      if (fseek(fp, pos, SEEK_SET)) {
	sprintf(buffer, "Seeking %s feature table", tag);
	perror(buffer);
	exit(1);
      }
      get_bine_feature_table(fp, &the_feature_table, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	fprintf(stdout, " 3 [%d]:\n", ife);
	print_feature_table(stdout, &the_feature_table, &junk, "feature->", 4);
      }
      if (instr & COMPILER) {
	int ili;
	int nput = 0;
	if (the_feature_table.featureParams) {
	  fprintf(stdout, "-- Feature parameters not supported.\n");
	}
	if (the_feature_table.lookupCount) fprintf(stdout, "   ");
	for (ili = 0; ili < the_feature_table.lookupCount; ili++) {
	  if (nput >= 7) {
	    fprintf(stdout, "\n   ");
	    nput = 0;
	  }
	  fputc(' ', stdout);
	  fprintf(stdout, namefmt, the_feature_table.lookupListIndex[ili]);
	  nput++;
	}
	if (the_feature_table.lookupCount) fputc('\n', stdout);
	fprintf(stdout, "  end feature\n");
      }
      free_feature_table(&the_feature_table);
    }
    free_feature_list(&feature_list_table);
  }
/* General purpose reading in */
  if (GSUB_header.lookupList) {
    pos = GSUB_table->offset + GSUB_header.lookupList;
    if (fseek(fp, pos, SEEK_SET)) {
      sprintf(buffer, "Seeking %s lookup list table", tag);
      perror(buffer);
      exit(1);
    }
    get_bine_lookup_list(fp, &lookup_list_table, &junk, &pos);
    if (instr & COMPILER) {
      for (il = 0; il < lookup_list_table.lookupCount; il++) {
        int pos_lt;
	pos_lt = pos = GSUB_table->offset + GSUB_header.lookupList +
	  lookup_list_table.lookup[il];
	if (fseek(fp, pos, SEEK_SET)) {
	  sprintf(buffer, "Seeking %s lookup list table", tag);
	  perror(buffer);
	  exit(1);
	}
	get_bine_lookup_table(fp, &the_lookup_table, &junk, &pos);
	if (instr & COMPILER) {
	  int k, igo, lkp_type;

	  fprintf(stdout, "  lookup ");
	  fprintf(stdout, namefmt, il);
	  fputc('\n', stdout);
          lkp_type = the_lookup_table.lookupType;
          for (igo = 0; igo < 2; igo++) {
            if (0 < lkp_type && lkp_type <= ntypes) {
              if ('-' == *types[lkp_type]) {
                if (1 == igo) {
                  fputs("-- Invalidly recursively defined as extension.\n",
                        stdout);
                } else if (the_lookup_table.subTableCount) {
                  int pos2 = pos_lt + the_lookup_table.subTable[0];
                  struct extension_pos_table subtable;

                  if (fseek(fp, pos2, SEEK_SET)) {
                    perror("Seeking GSUB extension subtable");
                    exit(1);
                  }
                  get_bine_extension_pos_table(fp, &subtable, &junk, &pos2);
                  lkp_type = subtable.extensionLookupType;
                  free_extension_pos_table(&subtable);
                } else {
                  fputs("    type extension unknown -- Undefined!\n", stdout);
                  break;
                }
              } else if (0 == igo) {
                fprintf(stdout, "%s\n", types[lkp_type]);
                break;
              } else {
                fprintf(stdout, "    type extension %s\n", types[lkp_type]+9);
              }
            } else {
              fprintf(stdout, "-- Unrecognised lookup type %d.\n", lkp_type);
              break;
            }
          }

	  if (the_lookup_table.lookupFlag & 1) {
	    fprintf(stdout, "    anchor last\n");
	  }
	  if (the_lookup_table.lookupFlag & 2) {
	    fprintf(stdout, "    skip base\n");
	  }
	  if (the_lookup_table.lookupFlag & 4) {
	    fprintf(stdout, "    skip ligature\n");
	  }
	  if (the_lookup_table.lookupFlag & 8) {
	    fprintf(stdout, "    skip mark\n");
	  }
          if (the_lookup_table.lookupFlag & 16) {
            fprintf(stdout, "    skipexcept " MGS "\n",
                    the_lookup_table.markFilteringSet);
          }
	  for (k = 5; k <= 7; k++) {
	    if (the_lookup_table.lookupFlag & (1<<k)) {
	      fprintf(stdout, "    setbit %d\n", k);
	    }
	  }
	  if (the_lookup_table.lookupFlag & 0xFF00) {
	    fprintf(stdout, "    skipexcept %d\n", 
		    (the_lookup_table.lookupFlag & 0xFF00) >> 8);
	  }
	  for (ist = 0; ist < the_lookup_table.subTableCount; ist++) {
	    fprintf(stdout, "    subtable ");
	    fprintf(stdout, namefmt, il);
	    fprintf(stdout, "_%d", ist);
	    fputc('\n', stdout);
	  }
	}
	fprintf(stdout, "  end lookup\n");
      }
      fprintf(stdout, "end %s\n", tag);
    }
  }
  *table = GSUB_table;
  *gshout = GSUB_header;
  *llt = lookup_list_table;
}

/* Lookup type codes defined as part of OpenType */
#define GSUB_SINGLE 1
#define GSUB_MULTIPLE 2
#define GSUB_ALTERNATE 3
#define GSUB_LIGATURE 4
#define GSUB_CONTEXT 5
#define GSUB_CHAINING 6
#define GSUB_EXTENSION 7
#define GSUB_REVERSE 8

#define GPOS_SINGLE 1
#define GPOS_PAIR 2
#define CURSIVE 3
#define MARK2BASE 4
#define MARK2LIGATURE 5
#define MARK2MARK 6
#define GPOS_CONTEXT 7
#define GPOS_CHAINED 8
#define GPOS_EXTENSION 9

/* Private, unified code */
#define UF_GSUB 0
#define UF_GPOS 10

#define UF_GSUB_SINGLE    (UF_GSUB + GSUB_SINGLE)
#define UF_GSUB_MULTIPLE  (UF_GSUB + GSUB_MULTIPLE)
#define UF_GSUB_ALTERNATE (UF_GSUB + GSUB_ALTERNATE)
#define UF_GSUB_LIGATURE  (UF_GSUB + GSUB_LIGATURE)
#define UF_GSUB_CONTEXT   (UF_GSUB + GSUB_CONTEXT)
#define UF_GSUB_CHAINING  (UF_GSUB + GSUB_CHAINING)
#define UF_GSUB_EXTENSION (UF_GSUB + GSUB_EXTENSION)
#define UF_GSUB_REVERSE   (UF_GSUB + GSUB_REVERSE)

#define UF_GPOS_SINGLE    (UF_GPOS + GPOS_SINGLE)
#define UF_GPOS_PAIR      (UF_GPOS + GPOS_PAIR)
#define UF_CURSIVE        (UF_GPOS + CURSIVE)
#define UF_MARK2BASE      (UF_GPOS + MARK2BASE)
#define UF_MARK2LIGATURE  (UF_GPOS + MARK2LIGATURE)
#define UF_MARK2MARK      (UF_GPOS + MARK2MARK)
#define UF_GPOS_CONTEXT   (UF_GPOS + GPOS_CONTEXT)
#define UF_GPOS_CHAINED   (UF_GPOS + GPOS_CHAINED)
#define UF_GPOS_EXTENSION (UF_GPOS + GPOS_EXTENSION)

void display_chain_subtable(FILE *fp, int stloc, int il, int ist, int stype,
			    int instr, const char *namefmt, int ie) { 
  struct chain_context_subst subtable;
  int ic, iset, irule;
  /* Variables for compiler format output */
  int n_classifications = 0;
  char gpname_c[100], gpname_b[100], gpname_i[100], gpname_a[100];
  char stname[100];
  struct coverage_table fmt1cov;
  struct coverage_iteration ci;
  int gi0, ng;
  int pos, junk;
  char label[20];

#define unsupported_case { \
  fprintf(stderr, "Internal error: Process for unified lookup type %d " \
	  "is missing at line %d of %s.\n", stype, __LINE__, __FILE__); \
  exit(1); }

  sprintf(stname, namefmt, il);
  sprintf(stname + strlen(stname), "_%d", ist);
  sprintf(label, "[%d]->", ist);
  pos = stloc;
  switch (stype) {
  case UF_GSUB_CHAINING:
    get_bine_chain_context_subst(fp, &subtable, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_chain_context_subst(stdout, &subtable, &junk, label, 5+ie);
    }
    break;
  case UF_GPOS_CHAINED:
    {
      struct chain_context_pos_table trusub;
      get_bine_chain_context_pos_table(fp, &trusub, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	print_chain_context_pos_table(stdout, &trusub, &junk, label, 5+ie);
      }
      assert(sizeof trusub == sizeof subtable);
      memcpy(&subtable, &trusub, sizeof subtable);
    }
    break;
  case UF_GSUB_CONTEXT:
    {
      struct context_subst trusub;
      get_bine_context_subst(fp, &trusub, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	print_context_subst(stdout, &trusub, &junk, label, 5+ie);
      }
      memset(&subtable, 0, sizeof(subtable));
      subtable.substFormat = trusub.substFormat;
      switch (trusub.substFormat) {
      case 1:
	subtable.coverage =             trusub.coverage;
	subtable.chainSubRuleSetCount = trusub.subRuleSetCount;
	subtable.chainSubRuleSet =      trusub.subRuleSet;
	break;
      case 2:
	subtable.coverage =            trusub.coverage;
	subtable.backtrackClassDef = 0;
	subtable.inputClassDef =       trusub.classDef;
	subtable.lookaheadClassDef = 0;
	subtable.chainSubClassSetCnt = trusub.subClassSetCnt;
	subtable.chainSubClassSet =    trusub.subClassSet;
	break;
      case 3:
	subtable.backtrackGlyphCount = 0;
	subtable.coverageb = 0;
	subtable.inputGlyphCount =   trusub.glyphCount;
	subtable.coveragei =         trusub.coverage3;
	subtable.lookaheadGlyphCount = 0;
	subtable.coveragea = 0;
	subtable.substCount =        trusub.substCount;
	subtable.substLookupRecord = trusub.substLookupRecord;
	break;
      default:
	if (instr & COMPILER) {
	  fprintf(stdout, "-- Format %d is not supported.\n",
		  trusub.substFormat);
	} else {
	  fprintf(stderr, "Format %d of context substitution table %s is not "
		  "supported.\n", trusub.substFormat, stname);
	}
	free_context_subst(&trusub);
	return;
      }
    }
    break;
  default:
    unsupported_case;
  }
  switch (subtable.substFormat) {
  case 1:
    if (instr & DISPLAY_RAW) {
      display_coverage(fp, stloc + subtable.coverage, "coverage->", 6+ie);
    }
    get_coverage(fp, stloc + subtable.coverage, &fmt1cov, &ng);
    ci = make_coverage_iteration(&fmt1cov);
    if (instr & COMPILER) {
      sprintf(gpname_c, "%s_cov", stname);
      decompile_coverage(fp, stloc + subtable.coverage,
			 stdout, instr, gpname_c);
      fprintf(stdout, "lookup %s\n", stname);
      fprintf(stdout, "-- coverage %s\n", gpname_c);
      if ((instr & COMPILER) && ng != subtable.chainSubRuleSetCount) {
	fprintf(stdout, "-- Coverage size and chainSubRuleSetCount "
		"do not match!\n");
      }
    }
    for (iset = 0; iset < subtable.chainSubRuleSetCount; iset++) {
      struct chain_sub_rule_set set;
      gi0 = get_from_coverage(&ci);
      if ((instr & COMPILER) && !subtable.chainSubRuleSet[iset]) {
	fprintf(stdout, "-- No context starting with %s\n",
		glyphname1(instr, gi0));
	continue;
      }
      pos = stloc + subtable.chainSubRuleSet[iset];
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking GSUB chain rule set");
	exit(1);
      }
      get_bine_chain_sub_rule_set(fp, &set, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	sprintf(label, "[%d]->:", iset);
	switch (stype) {
	case UF_GSUB_CHAINING:
	  print_chain_sub_rule_set(stdout, &set, &junk, label, 7+ie);
	  break;
	case UF_GPOS_CHAINED:
	  {
	    assert(sizeof(struct chain_sub_rule_set) ==
		   sizeof(struct chain_pos_rule_set));
	    print_chain_pos_rule_set(stdout, (struct chain_pos_rule_set *)&set,
				     &junk, label, 7+ie);
	  }
	  break;
	case UF_GSUB_CONTEXT:
	  {
	    assert(sizeof(struct chain_sub_rule_set) ==
		   sizeof(struct sub_rule_set));
	    print_sub_rule_set(stdout, (struct sub_rule_set *)&set,
			       &junk, label, 7+ie);
	  }
	  break;
	default:
	  unsupported_case;
	}
      }
      for (irule = 0; irule < set.chainSubRuleCount; irule++) {
	struct chain_sub_rule rule;
	if (!set.chainSubRule[irule]) continue;
	pos = stloc + subtable.chainSubRuleSet[iset] +
	  set.chainSubRule[irule];
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking GSUB chain rule");
	  exit(1);
	}
	switch (stype) {
	case UF_GSUB_CHAINING:
	  get_bine_chain_sub_rule(fp, &rule, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    sprintf(label, "[%d]->:", irule);
	    print_chain_sub_rule(stdout, &rule, &junk, label, 9+ie);
	  }
	  break;
	case UF_GPOS_CHAINED:
	  assert(sizeof(struct chain_sub_rule) ==
		 sizeof(struct chain_pos_rule));
	  get_bine_chain_sub_rule(fp, &rule, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    sprintf(label, "[%d]->:", irule);
	    print_chain_pos_rule(stdout, (struct chain_pos_rule *)&rule, &junk,
				 label, 9+ie);
	  }
	  break;
	case UF_GSUB_CONTEXT:
	  {
	    struct sub_rule trusub;
	    get_bine_sub_rule(fp, &trusub, &junk, &pos);
	    if (instr & DISPLAY_RAW) {
	      sprintf(label, "[%d]->:", irule);
	      print_sub_rule(stdout, &trusub, &junk, label, 9+ie);
	    }
	    rule.backtrackGlyphCount = 0;
	    rule.backtrack =           0;
	    rule.inputGlyphCount =     trusub.glyphCount;
	    rule.input =               trusub.input;
	    rule.lookaheadGlyphCount = 0;
	    rule.lookahead =           0;
	    rule.substCount =          trusub.substCount;
	    rule.substLookupRecord =   trusub.substLookupRecord;
	  }
	  break;
	default:
	  unsupported_case;
	}
	if (instr & COMPILER) {
	  int igl;
	  struct subst_lookup_record *slr;
	  int putbar;

	  switch (stype) {
	  case UF_GSUB_CHAINING:
	    putbar = 1;
	    break;
	  case UF_GPOS_CHAINED:
	    putbar = 1;
	    break;
	  case UF_GSUB_CONTEXT:
	    putbar = 0;
	    break;
	  default:
	    unsupported_case;
	  }
	  fprintf(stdout, "  context");
	  for (igl = rule.backtrackGlyphCount - 1; igl >= 0; igl--) {
	    fprintf(stdout, " %s",
		    glyphname1(instr, rule.backtrack[igl]));
	  }
	  if (putbar) fprintf(stdout, " |");
	  fprintf(stdout, " %s", glyphname1(instr, gi0));
	  for (igl = 0; igl < rule.inputGlyphCount - 1; igl++) {
	    fprintf(stdout, " %s",
		    glyphname1(instr, rule.input[igl]));
	  }
	  if (putbar) fprintf(stdout, " |");
	  for (igl = 0; igl < rule.lookaheadGlyphCount; igl++) {
	    fprintf(stdout, " %s",
		    glyphname1(instr, rule.lookahead[igl]));
	  }
	  if (rule.backtrackGlyphCount > 1) {
	    fprintf(stdout, " -- Backtrack reversed from "
		    "font order.\n");
	  } else {
	    fputc('\n', stdout);
	  }
	  for (slr = rule.substLookupRecord; slr; slr = slr->_next) {
	    fprintf(stdout, "  %d ", slr->sequenceIndex);
	    fprintf(stdout, namefmt, slr->lookupListIndex);
	    fputc('\n', stdout);
	  }
	}
      }
    }
    if (instr & COMPILER) free_coverage_table(&fmt1cov);
    break;
  case 2:
    if (instr & DISPLAY_RAW) {
      display_coverage(fp, stloc + subtable.coverage, "coverage->", 6+ie);
      if (subtable.backtrackClassDef) {
	display_class(fp, stloc + subtable.backtrackClassDef,
		      "backtrackClassDef->", 6+ie);
      }
      display_class(fp, stloc + subtable.inputClassDef,
		    "inputClassDef->", 6+ie);
      if (subtable.lookaheadClassDef) {
	display_class(fp, stloc + subtable.lookaheadClassDef,
		      "lookaheadClassDef->", 6+ie);
      }
      switch (stype) {
      case UF_GSUB_CHAINING:
	fprintf(stdout, " %d chainSubClassSet:\n", 6+ie);
	break;
      case UF_GPOS_CHAINED:
	fprintf(stdout, " %d chainPosClassSet:\n", 6+ie);
	break;
      case UF_GSUB_CONTEXT:
	fprintf(stdout, " %d subClassSet:\n", 6+ie);
	break;
      default:
	unsupported_case;
      }
    }
    if (instr & COMPILER) {
      sprintf(gpname_c, "%s_cov", stname);
      decompile_coverage(fp, stloc + subtable.coverage,
			 stdout, instr, gpname_c);
      /* Some of these classifications are clearly identical, and this should be
	 addressed. */
      sprintf(gpname_i, "%s_i", stname);
      if (subtable.backtrackClassDef == subtable.inputClassDef) {
	strcpy(gpname_b, gpname_i);
      } else if (0 == subtable.backtrackClassDef) {
	/* Do not display - does not exist. */
      } else {
	sprintf(gpname_b, "%s_b", stname);
	decompile_class(fp, stloc + subtable.backtrackClassDef,
			stdout, instr, gpname_b);
      }
      decompile_class(fp, stloc + subtable.inputClassDef,
		      stdout, instr, gpname_i);
      if (subtable.lookaheadClassDef == subtable.inputClassDef) {
	strcpy(gpname_a, gpname_i);
      } else if (subtable.backtrackClassDef ==
		 subtable.inputClassDef) {
	strcpy(gpname_a, gpname_b);
      } else if (0 == subtable.lookaheadClassDef) {
	/* Do not display - does not exist */
      } else {
	sprintf(gpname_a, "%s_a", stname);
	decompile_class(fp, stloc + subtable.lookaheadClassDef,
			stdout, instr, gpname_a);
      }
      fprintf(stdout, "lookup %s\n", stname);
      fprintf(stdout, "-- coverage %s\n", gpname_c);
    }
    for (iset = 0; iset < subtable.chainSubClassSetCnt; iset++) {
      struct chain_sub_class_set set;
      if (!subtable.chainSubClassSet[iset]) continue;
      pos = stloc + subtable.chainSubClassSet[iset];
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking GSUB chain class set");
	exit(1);
      }
      get_bine_chain_sub_class_set(fp, &set, &junk, &pos);
      switch (stype) {
      case UF_GSUB_CHAINING:
	if (instr & DISPLAY_RAW) {
	  sprintf(label, "[%d]->:", iset);
	  print_chain_sub_class_set(stdout, &set, &junk, label, 7+ie);
	}
	break;
      case UF_GPOS_CHAINED:
	assert(sizeof(struct chain_sub_class_set) ==
	       sizeof(struct chain_pos_class_set));
	if (instr & DISPLAY_RAW) {
	  sprintf(label, "[%d]->:", iset);
	  print_chain_pos_class_set(stdout, (struct chain_pos_class_set *)&set,
				    &junk, label, 7+ie);
	}
	break;
      case UF_GSUB_CONTEXT:
	assert(sizeof(struct chain_sub_class_set) ==
	       sizeof(struct sub_class_set));
	if (instr & DISPLAY_RAW) {
	  sprintf(label, "[%d]->:", iset);
	  print_sub_class_set(stdout, (struct sub_class_set *)&set,
			      &junk, label, 7+ie);
	}
	break;
      default:
	unsupported_case;
      }
      for (irule = 0; irule < set.chainSubClassRuleCnt; irule++) {
	struct chain_sub_class_rule rule;
	if (!set.chainSubClassRule[irule]) continue;
	pos = stloc + subtable.chainSubClassSet[iset] +
	  set.chainSubClassRule[irule];
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking GSUB chain class rule");
	  exit(1);
	}
	switch (stype) {
	case UF_GSUB_CHAINING:
	  get_bine_chain_sub_class_rule(fp, &rule, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    sprintf(label, "[%d]->:", irule);
	    print_chain_sub_class_rule(stdout, &rule, &junk, label, 9+ie);
	  }
	  break;
	case UF_GPOS_CHAINED:
	  assert(sizeof(struct chain_sub_class_rule) ==
		 sizeof(struct chain_pos_rule));
	  get_bine_chain_sub_class_rule(fp, &rule, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    sprintf(label, "[%d]->:", irule);
	    print_chain_pos_rule(stdout, (struct chain_pos_rule *) &rule, &junk,
				 label, 9+ie);
	  }
	  break;
	case UF_GSUB_CONTEXT:
	  {
	    struct sub_class_rule trusub;

	    get_bine_sub_class_rule(fp, &trusub, &junk, &pos);
	    if (instr & DISPLAY_RAW) {
	      sprintf(label, "[%d]->:", irule);
	      print_sub_class_rule(stdout, &trusub, &junk, label, 9+ie);
	    }
	    rule.backtrackGlyphCount = 0;
	    rule.backtrack =           0;
	    rule.inputGlyphCount =     trusub.glyphCount;
            rule.input =               trusub.class;
	    rule.lookaheadGlyphCount = 0;
	    rule.lookahead =           0;
	    rule.substCount =          trusub.substCount;
	    rule.substLookupRecord =   trusub.substLookupRecord;
	  }
	  break;
	default:
	  unsupported_case;
	}
	if (instr & COMPILER) {
	  int igl;
	  struct subst_lookup_record *slr;
	  int putbar;

	  switch (stype) {
	  case UF_GSUB_CHAINING: putbar = 1; break;
	  case UF_GPOS_CHAINED:  putbar = 1; break;
	  case UF_GSUB_CONTEXT:  putbar = 0; break;
	  default: unsupported_case;
	  }

	  fprintf(stdout, "  context");
	  for (igl = rule.backtrackGlyphCount - 1; igl >= 0; igl--) {
	    fprintf(stdout, " %s_%d", gpname_b, rule.backtrack[igl]);
	  }
	  if (putbar) fprintf(stdout, " |");
	  fprintf(stdout, " %s_%d", gpname_i, iset);
	  for (igl = 0; igl < rule.inputGlyphCount - 1; igl++) {
	    fprintf(stdout, " %s_%d", gpname_i, rule.input[igl]);
	  }
	  if (putbar) fprintf(stdout, " |");
	  for (igl = 0; igl < rule.lookaheadGlyphCount; igl++) {
	    fprintf(stdout, " %s_%d", gpname_a, rule.lookahead[igl]);
	  }
	  if (rule.backtrackGlyphCount > 1) {
	    fprintf(stdout, " -- Backtrack reversed from "
		    "font order.\n");
	  } else {
	    fputc('\n', stdout);
	  }
	  for (slr = rule.substLookupRecord; slr; slr = slr->_next) {
	    fprintf(stdout, "  %d ", slr->sequenceIndex);
	    fprintf(stdout, namefmt, slr->lookupListIndex);
	    fputc('\n', stdout);
	  }
	}
      }
    }
    break;
  case 3:
    if (instr & DISPLAY_RAW) {
      if (subtable.backtrackGlyphCount) {
	fprintf(stdout, "%2d coverageb:\n", 6+ie);
	for (ic = 0; ic < subtable.backtrackGlyphCount; ic++) {
	  sprintf(label, "[%d]->", ic);
	  pos = stloc + subtable.coverageb[ic];
	  display_coverage(fp, pos, label, 7+ie);
	}
      }		  
      if (subtable.inputGlyphCount) {
	fprintf(stdout, "%2d coveragei:\n", 6+ie);
	for (ic = 0; ic < subtable.inputGlyphCount; ic++) {
	  sprintf(label, "[%d]->", ic);
	  pos = stloc + subtable.coveragei[ic];
	  display_coverage(fp, pos, label, 7+ie);
	}
      }		  
      if (subtable.lookaheadGlyphCount) {
	fprintf(stdout, "%2d coveragea:\n", 6+ie);
	for (ic = 0; ic < subtable.lookaheadGlyphCount; ic++) {
	  sprintf(label, "[%d]->", ic);
	  pos = stloc + subtable.coveragea[ic];
	  display_coverage(fp, pos, label, 7+ie);
	}
      }		  
    }
#define MAXCOVNAM 64
    if (instr & COMPILER) {
      int i, j, ncov, off, base;
      struct subst_lookup_record *slr;
      char *fmt3buf;
      char **fmt3ptr;
      unsigned short int *fmt3cov;
      int putbar;

      ncov = subtable.backtrackGlyphCount +
	subtable.inputGlyphCount + subtable.lookaheadGlyphCount;
      if (!(fmt3ptr = malloc(ncov * sizeof * fmt3ptr)) ||
	  !(fmt3buf = malloc(ncov * MAXCOVNAM * sizeof * fmt3buf)) ||
	  !(fmt3cov = malloc(ncov * sizeof * fmt3cov))) {
	fprintf(stderr, "Memory exhausted.\n");
      }
      for (i = 0; i < ncov; i++) fmt3ptr[i] = fmt3buf + i * MAXCOVNAM;
      for (i = 0, off = 0; i < subtable.inputGlyphCount; i++, off++) {
	sprintf(fmt3ptr[off], "%s_i%d", stname, i);
	fmt3cov[off] = subtable.coveragei[i];
      }
      for (i = 0; i < subtable.lookaheadGlyphCount; i++, off++) {
	sprintf(fmt3ptr[off], "%s_a%d", stname, i);
	fmt3cov[off] = subtable.coveragea[i];
      }
      for (i = 0; i < subtable.backtrackGlyphCount; i++, off++) {
	sprintf(fmt3ptr[off], "%s_b%d", stname, i);
	fmt3cov[off] = subtable.coverageb[i];
      }
      for (j = 0; j < ncov; j++) {
	int found = 0;
	for (i = 0; i < j; i++) {
	  if (fmt3cov[i] == fmt3cov[j]) {
	    found = 1;
	    strcpy(fmt3ptr[j], fmt3ptr[i]);
	    break;
	  }
	}
	if (!found) {
	  decompile_coverage(
			     fp, stloc + fmt3cov[j], stdout, instr, fmt3ptr[j]);
	}
      }
      fprintf(stdout, "lookup %s\n", stname);
      fprintf(stdout, "  context");
      switch (stype) {
        case UF_GSUB_CHAINING: putbar = 1; break;
        case UF_GPOS_CHAINED:  putbar = 1; break;
        case UF_GSUB_CONTEXT:  putbar = 0; break;
        default: unsupported_case;
      }
      base = subtable.inputGlyphCount + subtable.lookaheadGlyphCount;
      for (i = subtable.backtrackGlyphCount - 1; i >= 0; i--) {
	fprintf(stdout, " %s", fmt3ptr[base+i]);
      }
      if (putbar) fprintf(stdout, " |");
      base = 0;
      for (i = 0; i < subtable.inputGlyphCount; i++) {
	fprintf(stdout, " %s", fmt3ptr[base+i]);
      }
      if (putbar) fprintf(stdout, " |");
      base = subtable.inputGlyphCount;
      for (i = 0; i < subtable.lookaheadGlyphCount; i++) {
	fprintf(stdout, " %s", fmt3ptr[base+i]);
      }
      if (subtable.backtrackGlyphCount > 1) {
	fprintf(stdout, " -- Backtrack reversed from "
		"font order.\n");
      } else {
	fputc('\n', stdout);
      }
      for (slr = subtable.substLookupRecord; slr; slr = slr->_next) {
	fprintf(stdout, "  %d ", slr->sequenceIndex);
	fprintf(stdout, namefmt, slr->lookupListIndex);
	fputc('\n', stdout);
      }
      free(fmt3ptr);
      free(fmt3buf);
      free(fmt3cov);
    }
    break;
  default:
    if (instr & COMPILER) fprintf(stdout, "lookup %s\n", stname);
    fprintf(stderr, "Unknown GSUB lookup-type 6 format %d "
	    "encountered.\n",
	    subtable.substFormat);
    fprintf(stdout, "%2d [%d]: Unknown format\n", 6+ie, ic);
  }
  free_chain_context_subst(&subtable);
#undef unsupported_case
}

#define SLKP "slkp%d"
void display_GSUB(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *GSUB_table;
  struct gsub_header GSUB_header;
  struct lookup_list lookup_list_table;
  struct lookup_table the_lookup_table;
  int pos, junk, il, ist, ie;
  char label[20];
  const char *types[] = {
    "",
    "    type single",
    "    type multiple",
    "    type alternate",
    "    type ligature",
    "    type context",
    "    type chained",
    "-- extension look-up not yet supported.",
    "    type reverse"};
  
  if (1) fprintf(stderr, "Invoked display_GSUB(%8.8X).\n", instr);
  display_GSUBPOS_header(fp, instr, top, "GSUB", SLKP, types, 8,
			 &GSUB_table, &lookup_list_table, &GSUB_header);
/*-------------*/
  if (GSUB_header.lookupList) {
    if (instr & DISPLAY_RAW) {
      print_lookup_list(stdout, &lookup_list_table, &junk, "lookupList->", 1);
    }
    for (il = 0; il < lookup_list_table.lookupCount; il++) {
      pos = GSUB_table->offset + GSUB_header.lookupList +
	lookup_list_table.lookup[il];
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking GSUB lookup list table");
	exit(1);
      }
      get_bine_lookup_table(fp, &the_lookup_table, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	sprintf(label, "[%d]", il);
	print_lookup_table(stdout, &the_lookup_table, &junk, label, 3);
      }
/*-------------*/	
      for (ist = 0; ist < the_lookup_table.subTableCount; ist++) {
	int lkp_type;

	if (0 == the_lookup_table.subTable[ist]) continue;
	pos = GSUB_table->offset + GSUB_header.lookupList +
	  lookup_list_table.lookup[il] + the_lookup_table.subTable[ist];
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking GSUB lookup subtable");
	  exit(1);
	}
	sprintf(label, "[%d]->", ist);

	lkp_type = the_lookup_table.lookupType;
	if (GSUB_EXTENSION == lkp_type) {
	  struct extension_pos_table subtable;
	  int tabpos = pos;

	  get_bine_extension_pos_table(fp, &subtable, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    print_extension_pos_table(stdout, &subtable, &junk, label, 5);
	  }
	  lkp_type = subtable.extensionLookupType;
	  pos = tabpos + subtable.extensionOffset;
	  if (fseek(fp, pos, SEEK_SET)) {
	    perror("Seeking GSUB lookup subtable");
	    exit(1);
	  }
          strcpy(label, "extension->");
          ie = 1;
	} else {
          ie = 0;
        }
	switch (lkp_type) {
	case GSUB_SINGLE:
	  {
	    struct single_subst subtable;
	    int stloc = pos;

	    get_bine_single_subst(fp, &subtable, &junk, &pos);
	    pos = GSUB_table->offset + GSUB_header.lookupList +
	      lookup_list_table.lookup[il] + the_lookup_table.subTable[ist] +
	      subtable.coverage;
	    pos = stloc + subtable.coverage;
	    if (instr & DISPLAY_RAW) {
	      print_single_subst(stdout, &subtable, &junk, label, 5+ie);
	      display_coverage(fp, pos, "coverage->", 6+ie);
	    }
	    if (instr & COMPILER) {
	      int igl, ngl;
	      struct coverage_table coverage;
	      int gin, gout;
	      struct coverage_iteration ci;

	      fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	      fprintf(stdout, "-- Original uses format %d.\n",
		      subtable.substFormat);
	      get_coverage(fp, pos, &coverage, &ngl);
	      if (2 == subtable.substFormat && ngl != subtable.glyphCount) {
		fprintf(stdout, "-- Mismatch in numbers of input (%d) and "
			"output (%d) glyphs!\n", ngl, subtable.glyphCount);
		fflush(stdout);
	      }
	      ci = make_coverage_iteration(&coverage);
	      igl = 0;
	      while ((gin = get_from_coverage(&ci)) >= 0) {
		switch (subtable.substFormat) {
		case 1:
		  gout = gin + subtable.deltaGlyphID; 
		  break;
		case 2:
		  gout = subtable.substitute[igl];
		  break;
		default:
		  fprintf(stdout, "-- Format not understood!\n");
		  break;
		}
		igl++;
		fprintf(stdout, "  %s > %s\n",
			glyphname1(instr, gin), glyphname2(instr, gout));
	      }
	      free_coverage_table(&coverage);
	    }
	    free_single_subst(&subtable);
	  }
	  break;
	case GSUB_LIGATURE:
	  {
	    struct ligature_subst subtable;
	    struct ligature_set the_set;
	    struct subst_ligature_table the_lig;
	    int is, ilig;
	    int stloc;
	    /* Items for compiler format */
	    struct coverage_table coverage;
	    int ngl;
	    struct coverage_iteration ci;
	    int gl1;

	    stloc = pos;
	    get_bine_ligature_subst(fp, &subtable, &junk, &pos);
	    pos = GSUB_table->offset + GSUB_header.lookupList +
	    lookup_list_table.lookup[il] + the_lookup_table.subTable[ist] +
	    subtable.coverage;
	    pos = stloc + subtable.coverage;
	    if (instr & DISPLAY_RAW) {
	      print_ligature_subst(stdout, &subtable, &junk, label, 5+ie);
	      display_coverage(fp, pos, "coverage->", 6+ie);
	    }
	    if (instr & COMPILER) {
	      fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	      get_coverage(fp, pos, &coverage, &ngl);
	      if (ngl != subtable.ligSetCount) {
		fprintf(stdout, "-- Mismatch between coverage (%d glyphs) and "
			"number of ligature sets (%d).\n", ngl,
			subtable.ligSetCount);
	      }
	      ci = make_coverage_iteration(&coverage);
	    }
            if (instr & DISPLAY_RAW) {
              fprintf(stdout, "%2d ligatureSet->:\n", 6+ie);
            }
	    for (is = 0; is < subtable.ligSetCount; is++) {
	      pos = GSUB_table->offset + GSUB_header.lookupList +
		lookup_list_table.lookup[il] + the_lookup_table.subTable[ist] +
		subtable.ligatureSet[is];
	      pos = stloc + subtable.ligatureSet[is];
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking ligature set");
		exit(1);
	      }
	      get_bine_ligature_set(fp, &the_set, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		sprintf(label, "[%d]->", is);
		print_ligature_set(stdout, &the_set, &junk, label, 7+ie);
	      }
	      if (instr & COMPILER) gl1 = get_from_coverage(&ci);
	      for (ilig = 0; ilig < the_set.ligatureCount; ilig++) {
		pos = GSUB_table->offset + GSUB_header.lookupList +
		  lookup_list_table.lookup[il] +
		  the_lookup_table.subTable[ist] +
		  subtable.ligatureSet[is] + the_set.ligature[ilig];
		pos = stloc + subtable.ligatureSet[is] + the_set.ligature[ilig];
		if (fseek(fp, pos, SEEK_SET)) {
		  perror("Seeking ligature");
		  exit(1);
		}
		get_bine_subst_ligature_table(fp, &the_lig, &junk, &pos);
		if (instr & DISPLAY_RAW) {
		  sprintf(label, "[%d]->", ilig);
		  print_subst_ligature_table(stdout, &the_lig, &junk, label,
                                             9+ie);
		}
		if ((instr & COMPILER) && gl1 >= 0) {
		  int igl;
		  fprintf(stdout, "  %s", glyphname1(instr, gl1));
		  for (igl = 1; igl < the_lig.compCount; igl++) {
		    fprintf(stdout, " %s",
			    glyphname1(instr, the_lig.component[igl-1]));
		  }
		  fprintf(stdout, " > %s\n",
			  glyphname1(instr, the_lig.ligGlyph));
		}
		free_subst_ligature_table(&the_lig);
	      }
	      free_ligature_set(&the_set);
	    }
	    if (instr & COMPILER) free_coverage_table(&coverage);
	    free_ligature_subst(&subtable);
	  }
	  break;
	case GSUB_CONTEXT:
	  display_chain_subtable(fp, pos, il, ist, UF_GSUB_CONTEXT,
				 instr, SLKP, ie);
	  break;
	case GSUB_CHAINING:
	  display_chain_subtable(fp, pos, il, ist, UF_GSUB_CHAINING,
				 instr, SLKP, ie);
	  break;
	case GSUB_ALTERNATE:
	  {
	    struct alternate_subst subtable;
	    int stloc;
	    struct coverage_table coverage;
	    int ngl, is;
	    struct coverage_iteration ci;

	    stloc = pos;
	    get_bine_alternate_subst(fp, &subtable, &junk, &pos);
	    pos = stloc + subtable.coverage;
	    if (instr & DISPLAY_RAW) {
	      print_alternate_subst(stdout, &subtable, &junk, label, 5);
	      display_coverage(fp, pos, "coverage->:", 6);
	    }
	    if (instr & COMPILER) {
	      fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	      get_coverage(fp, pos, &coverage, &ngl);
	      if (ngl != subtable.alternateSetCount) {
		fprintf(stdout, "-- Mismatch between coverage (%d glyphs) and "
			"number of alternate sets (%d).\n", ngl,
			subtable.alternateSetCount);
	      }
	      ci = make_coverage_iteration(&coverage);
	    }
	    for (is = 0; is < subtable.alternateSetCount; is++) {
	      struct alternate_set_table altset;

	      pos = stloc + subtable.alternateSet[is];
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking alternate set");
		exit(1);
	      }
	      get_bine_alternate_set_table(fp, &altset, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		fprintf(stdout, " 6 alternateSet:\n");
		sprintf(label, "[%d]->", is);
		print_alternate_set_table(stdout, &altset, &junk, label, 7);
	      }
	      if (instr & COMPILER) {
		int gno;
		int ialt;

		gno = get_from_coverage(&ci);
		fprintf(stdout, "  %s >", glyphname1(instr, gno));
		for (ialt = 0; ialt < altset.glyphCount; ialt++) {
		  fprintf(stdout, " %s",
			  glyphname1(instr, altset.alternate[ialt]));
		}
		fputc('\n', stdout);
	      }
	      free_alternate_set_table(&altset);
	    }
	    if (instr & COMPILER) free_coverage_table(&coverage);
	    free_alternate_subst(&subtable);
	  }
	  break;
	case GSUB_MULTIPLE:
	  {
	    struct multiple_subst subtable;
	    int stloc;
	    struct coverage_table coverage;
	    int ngl, is;
	    struct coverage_iteration ci;

	    stloc = pos;
	    get_bine_multiple_subst(fp, &subtable, &junk, &pos);
	    pos = stloc + subtable.coverage;
	    if (instr & DISPLAY_RAW) {
	      print_multiple_subst(stdout, &subtable, &junk, label, 5);
	      display_coverage(fp, pos, "coverage->:", 6);
	    }
	    if (instr & COMPILER) {
	      fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	      get_coverage(fp, pos, &coverage, &ngl);
	      if (ngl != subtable.sequenceCount) {
		fprintf(stdout, "-- Mismatch between coverage (%d glyphs) and "
			"number of sequences (%d).\n", ngl,
			subtable.sequenceCount);
	      }
	      ci = make_coverage_iteration(&coverage);
	    }
	    for (is = 0; is < subtable.sequenceCount; is++) {
	      struct sequence_table expansion;

	      pos = stloc + subtable.sequence[is];
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking alternate set");
		exit(1);
	      }
	      get_bine_sequence_table(fp, &expansion, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		fprintf(stdout, " 6 sequence:\n");
		sprintf(label, "[%d]->", is);
		print_sequence_table(stdout, &expansion, &junk, label, 7);
	      }
	      if (instr & COMPILER) {
		int gno;
		int ialt;

		gno = get_from_coverage(&ci);
		fprintf(stdout, "  %s >", glyphname1(instr, gno));
		for (ialt = 0; ialt < expansion.glyphCount; ialt++) {
		  fprintf(stdout, " %s",
			  glyphname1(instr, expansion.substitute[ialt]));
		}
		fputc('\n', stdout);
	      }
	      free_sequence_table(&expansion);
	    }
	    if (instr & COMPILER) free_coverage_table(&coverage);
	    free_multiple_subst(&subtable);
	  }
	  break;
	case GSUB_EXTENSION:
	case GSUB_REVERSE:
	  if (instr & COMPILER) {
	    fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	  }
	  fprintf(stderr, "Unsupported GSUB lookup-type %d encountered.\n",
		  the_lookup_table.lookupType);
	  fprintf(stdout, " 5 [%d]: Unsupported lookup type\n", ist);
	  break;
	default:
	  if (instr & COMPILER) {
	    fprintf(stdout, "lookup slkp%d_%d\n", il, ist);
	  }
	  fprintf(stderr, "Unknown GSUB lookup-type %d encountered.\n",
		  the_lookup_table.lookupType);
	  fprintf(stdout, " 5 [%d]: Unknown lookup type\n", ist);
	  break;
	}
	if (instr & COMPILER) fprintf(stdout, "end lookup\n");
      }
      free_lookup_table(&the_lookup_table);
    }
    free_lookup_list(&lookup_list_table);
  }
  free_gsub_header(&GSUB_header);
}

void decompile_value(struct ValueRecord vr, unsigned short int vf, FILE *fp,
                     const char *prefix, const char *suffix) {
  if (vf & 1) {
    fprintf(fp, "%sdx%s %d", prefix, suffix, vr.xPlacement);
  }
  if (vf & 2) {
    fprintf(fp, "%sdy%s %d", prefix, suffix, vr.yPlacement);
  }
  if (vf & 4) {
    fprintf(fp, "%sax%s %d", prefix, suffix, vr.xAdvance);
  }
  if (vf & 8) {
    fprintf(fp, "%say%s %d", prefix, suffix, vr.yAdvance);
  }
/* Device tables are currently unsupported */
  if ((vf & 16) && vr.xPlaDevice) {
    fprintf(fp, "-- x placement device table not supported.\n");
  }
  if ((vf & 32) && vr.yPlaDevice) {
    fprintf(fp, "-- y placement device table not supported.\n");
  }
  if ((vf & 64) && vr.xAdvDevice) {
    fprintf(fp, "-- x advance device table not supported.\n");
  }
  if ((vf & 128) && vr.yAdvDevice) {
    fprintf(fp, "-- y advance device table not supported.\n");
  }
}

void decompile_anchor(const struct anchor_table anchor, FILE *op) {
  fprintf(op, "  x %d", anchor.xCoordinate);
  fprintf(op, "  y %d", anchor.yCoordinate);
  switch (anchor.anchorFormat) {
  case 1:
    break;
  case 2:
    fprintf(op, " pt %d", anchor.anchorPoint);
    break;
  case 3:
    fprintf(op, "\n-- Device tables are not supported.");
    break;
  default:
    fprintf(op, "\n-- Anchor table format %d is not supported.",
	    anchor.anchorFormat);
  }
  fputc('\n', op);
}

#define PLKP "plkp%d"
void display_GPOS(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *GPOS_table;
  struct gsub_header GPOS_header;
  struct lookup_list lookup_list_table;
  struct lookup_table the_lookup_table;
  int pos, junk, il, ie = 0;
  char label[20];
  const char *types[] = {
    "",
    "    type single",
    "    type pair",
    "    type cursive",
    "    type mark2base",
    "    type mark2ligature",
    "    type mark2mark",
    "    type context",
    "    type chained",
    "-- extension look-up not yet supported."};

  display_GSUBPOS_header(fp, instr, top, "GPOS", PLKP, types, 9,
			 &GPOS_table, &lookup_list_table, &GPOS_header);
/*-------------*/
  if (GPOS_header.lookupList) {
    if (instr & DISPLAY_RAW) {
      print_lookup_list(stdout, &lookup_list_table, &junk, "lookupList->", 1);
    }
    for (il = 0; il < lookup_list_table.lookupCount; il++) {
      int ist;

      pos = GPOS_table->offset + GPOS_header.lookupList +
	lookup_list_table.lookup[il];
      if (fseek(fp, pos, SEEK_SET)) {
	perror("Seeking GSUB lookup list table");
	exit(1);
      }
      get_bine_lookup_table(fp, &the_lookup_table, &junk, &pos);
      if (instr & DISPLAY_RAW) {
	sprintf(label, "[%d]", il);
	print_lookup_table(stdout, &the_lookup_table, &junk, label, 3);
      }
/*-------------*/	
      for (ist = 0; ist < the_lookup_table.subTableCount; ist++) {
	int stloc, lkp_type;

	if (0 == the_lookup_table.subTable[ist]) continue;
	stloc = GPOS_table->offset + GPOS_header.lookupList +
	  lookup_list_table.lookup[il] + the_lookup_table.subTable[ist];
	pos = stloc;
	if (fseek(fp, pos, SEEK_SET)) {
	  perror("Seeking GPOS lookup subtable");
	  exit(1);
	}
	sprintf(label, "[%d]->", ist);
	lkp_type = the_lookup_table.lookupType;
	if (GPOS_EXTENSION == lkp_type) {
	  struct extension_pos_table subtable;

	  get_bine_extension_pos_table(fp, &subtable, &junk, &pos);
	  if (instr & DISPLAY_RAW) {
	    print_extension_pos_table(stdout, &subtable, &junk, label, 5);
	  }
	  lkp_type = subtable.extensionLookupType;
	  pos = stloc = stloc + subtable.extensionOffset;
	  if (fseek(fp, pos, SEEK_SET)) {
	    perror("Seeking GSUB lookup subtable");
	    exit(1);
	  }
          strcpy(label, "extension->");
          ie = 1;
	} else {
          ie = 0;
        }
	switch (lkp_type) {
	case GPOS_SINGLE:
	  {
	    struct sap subtable;
            char gpname[100];

	    get_bine_sap(fp, &subtable, &junk, &pos);
            if (instr & COMPILER) {
              sprintf(gpname, "plkp%d_%d_cov", il, ist);
              decompile_coverage(fp, stloc + subtable.coverage,
                                 stdout, instr, gpname);
              fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
              fprintf(stdout, "%s", 1 ? "  " : "-- ");
              fprintf(stdout, "coverage %s\n", gpname);
	      switch (subtable.posFormat) {
              case 1:
                {
                  decompile_value(subtable.value, subtable.valueFormat,
                                  stdout, "  ", "");
                  fputc('\n', stdout);
                }
                break;
              case 2: /* Currently untested */
                {
		  struct coverage_table cov;
                  struct coverage_iteration ci;
                  int ig, glyphno, ng;
                  struct ValueRecord *vr;

                  get_coverage(fp, stloc + subtable.coverage, &cov, &ng);
                  if (ng != subtable.valueCount) {
                    fprintf(stdout, "-- Coverage size and valueCount do "
                            "not match!\n");
                  }
                  ci = make_coverage_iteration(&cov);
                  vr = subtable.valueArr; 
                  for (ig = 0; ig < subtable.valueCount; ig++) {
                    int glyphno;

                    if (0 && ig) fputc('\n', stdout);
                    glyphno = get_from_coverage(&ci);
                    fprintf(stdout, "  glyph %s ", glyphname1(instr, glyphno));
                    decompile_value(*vr, subtable.valueFormat,
                                    stdout, "  ", "");
                    fputc('\n', stdout);
                    vr = vr->_next;
                  }
                  free_coverage_table(&cov);
                }
                break;
              default:
                fprintf(stdout, "-- Unsupported format %d\n",
                        subtable.posFormat);
              }
            }
	    if (instr & DISPLAY_RAW) {
	      print_sap(stdout, &subtable, &junk, label, 5);
              display_coverage(fp, stloc + subtable.coverage, "coverage->", 6);
            }
	    free_sap(&subtable);
	  }
	  break;
	case GPOS_PAIR:
	  {
	    struct pairpos_table subtable;
	    char gpname_c[100], gpname_b[100], gpname_a[100];

	    get_bine_pairpos_table(fp, &subtable, &junk, &pos);
	    if (instr & COMPILER) {
	      sprintf(gpname_c, "plkp%d_%d_cov", il, ist);
	      decompile_coverage(fp, stloc + subtable.coverage,
				 stdout, instr, gpname_c);
	      if (2 == subtable.posFormat) {
		sprintf(gpname_b, "plkp%d_%d_b", il, ist);
		sprintf(gpname_a, "plkp%d_%d_a", il, ist);
		decompile_class(fp, stloc + subtable.classDef1,
				stdout, instr, gpname_b);
		decompile_class(fp, stloc + subtable.classDef2,
				stdout, instr, gpname_a);
	      }
	      fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
	      fprintf(stdout, "-- coverage %s\n", gpname_c);
	    }
	    if (instr & DISPLAY_RAW) {
	      print_pairpos_table(stdout, &subtable, &junk, label, 5);
	    }
	    switch (subtable.posFormat) {
	    case 1:
	      {
		struct coverage_table cov;
		struct coverage_iteration ci;
		int ig;

		if (instr & COMPILER) {
		  int ng;

                  get_coverage(fp, stloc + subtable.coverage, &cov, &ng);
                  if (ng != subtable.pairSetCount) {
                    fprintf(stdout, "-- Coverage size and pairSetCount do "
                            "not match!\n");
                  }
                  ci = make_coverage_iteration(&cov);
		}
		for (ig = 0; ig < subtable.pairSetCount; ig++) {
		  struct pairset_table pst;
		  struct pair_value_record *pvr;
		  int ip;

		  if (instr & COMPILER) {
		    int g1 = get_from_coverage(&ci);
		    if (0 && ig) fputc('\n', stdout);
		    fprintf(stdout, "  glyph1 %s\n", glyphname1(instr, g1));
		  }
		  pos = stloc + subtable.pairSetOffset[ig];
		  if (fseek(fp, pos, SEEK_SET)) {
		    perror("Seeking GPOS pair set table");
		    exit(1);
		  }
		  get_bine_pairset_table(fp, &pst, &junk, &pos);
		  if (instr & DISPLAY_RAW) {
		    char sublab[20];
		    sprintf(sublab, "[%d]->:", ig);
		    print_pairset_table(stdout, &pst, &junk, sublab, 7);
		  }
		  if (instr & COMPILER) {
		    pvr = pst.pairValueRecord;
		    for (ip = 0; ip < pst.pairValueCount; ip++) {
		      fprintf(stdout, "    glyph2 %s ",
			      glyphname1(instr, pvr->secondGlyph));
		      decompile_value(pvr->value1, subtable.valueFormat1,
				      stdout, "  ", "1");
		      decompile_value(pvr->value2, subtable.valueFormat2,
				      stdout, "  ", "2");
                      fputc('\n', stdout);
		      pvr = pvr->_next;
		    }
		  }
		  free_pairset_table(&pst);
		}
		if (instr & COMPILER) { 
		  free_coverage_table(&cov);
		}
	      }
	      break;
	    case 2:
	      if (instr & COMPILER) {
		struct Class1Record *c1r;
		struct Class2Record *c2r;
		int ig1, ig2;

		c1r = subtable.class1Record;
		for (ig1 = 0; ig1 < subtable.class1Count; ig1++) {
		  if (0 && ig1) fputc('\n', stdout);
		  fprintf(stdout, "  glyph1 %s_%d\n", gpname_b, ig1);
		  c2r = c1r->class2Record;
		  for (ig2 = 0; ig2 < subtable.class2Count; ig2++) {
		    if (0 && ig2) fputc('\n', stdout);
		    fprintf(stdout, "    glyph2 %s_%d ", gpname_a, ig2);
		    decompile_value(c2r->value1, subtable.valueFormat1,
				    stdout, "  ", "1");
		    decompile_value(c2r->value2, subtable.valueFormat2,
				    stdout, "  ", "2");
                    fputc('\n', stdout);
		    c2r = c2r->_next;
		  }
		  c1r = c1r->_next;
		}
	      }
	      break;
	    default:
	      if (instr & COMPILER) {
		fprintf(stdout, "-- Unsupported format %d\n",
			subtable.posFormat);
	      }
	    }
	    if (instr & DISPLAY_RAW) {
	      display_coverage(fp, stloc + subtable.coverage, "coverage->", 6);
	      display_class(fp, stloc + subtable.classDef1, "classDef1->", 6);
	      display_class(fp, stloc + subtable.classDef2, "classDef2->", 6);
	    }
	  }
	  break;
#if 1
	case MARK2BASE:
	case MARK2LIGATURE:
	case MARK2MARK:
	  {
	    struct mark_basepos_table subtable;
	    char gpname_b[100], gpname_m[100];

	    assert(sizeof(struct mark_lig_pos_table) ==
		   sizeof(struct mark_basepos_table));
	    assert(sizeof(struct mark_mark_pos_table) ==
		   sizeof(struct mark_basepos_table));
	    get_bine_mark_basepos_table(fp, &subtable, &junk, &pos);
	    if (instr & COMPILER) {
	      if (1 == subtable.posFormat) {
		sprintf(gpname_b, "plkp%d_%d_b", il, ist);
		decompile_coverage(fp, stloc + subtable.baseCoverage,
				   stdout, instr, gpname_b);
		sprintf(gpname_m, "plkp%d_%d_m", il, ist);
		decompile_coverage(fp, stloc + subtable.markCoverage,
				 stdout, instr, gpname_m);
	      }
	      fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
	      if (1 == subtable.posFormat) {
		fprintf(stdout, "-- markCoverage %s\n", gpname_m);
		fprintf(stdout, "-- baseCoverage %s\n", gpname_b);
	      } else {
		fprintf(stdout, "-- Unsupported format %d\n",
			subtable.posFormat);
	      }
	    }
	    if (instr & DISPLAY_RAW) {
	      switch (lkp_type) {
	      case MARK2BASE:
		print_mark_basepos_table(stdout, &subtable, &junk, label, 5);
		break;
	      case MARK2LIGATURE:
		print_mark_lig_pos_table
		  (stdout, (struct mark_lig_pos_table *) &subtable,
		   &junk, label, 5);
		break;
	      case MARK2MARK:
		print_mark_mark_pos_table
		  (stdout, (struct mark_mark_pos_table *) &subtable,
		   &junk, label, 5);
		break;
	      }
	    }
	    if (1 == subtable.posFormat) {
	      struct base_array_table bat;
	      int ib, imc, im;
	      struct base_record *br;
	      int batloc, matloc;
	      struct mark_array_table mat;
	      struct mark_record *mr;
	      struct coverage_table bcov, mcov;
	      struct coverage_iteration ci;
	      int max_class = -1;
	      int forlig, ilig, n_bat;
	      int ligloc;
	      struct ligature_array_table lat;

/* Mark processing */
	      pos = matloc = stloc + subtable.markArray;
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking GPOS mark array table");
		exit(1);
	      }
	      get_bine_mark_array_table(fp, &mat, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		print_mark_array_table(stdout, &mat, &junk, "markArray->:", 7);
	      }
	      if (instr & COMPILER) {
		int ng;
		get_coverage(fp, stloc + subtable.markCoverage, &mcov, &ng);
		if (ng != mat.markCount) {
		  fprintf(stderr, "Number of glyphs (%d) in coverage of "
			  "subtable %d of lookup %d does not match number of "
			  "base records (%d).\n", ng, ist, il, bat.baseCount);
		}
		ci = make_coverage_iteration(&mcov);
	      }
	      mr = mat.markRecord;
	      for (im = 0; im < mat.markCount; im++) {
		struct anchor_table anchor;

		if (instr & COMPILER) {
		  int gno = get_from_coverage(&ci);
		  if (0 && im) fputc('\n', stdout);
		  if (mr->class > max_class + 1) {
		    int j;
		    for (j = max_class + 1; j < mr->class; j++) {
		      fprintf(stdout, "  empty %s_%d\n", gpname_m, j);
		    }
		    max_class = mr->class;
		  } else if (mr->class > max_class) {
		    max_class = mr->class;
		  }
		  fprintf(stdout, "  mark %s_%d %s ", gpname_m, mr->class,
			  glyphname1(instr, gno));
		}
		pos = matloc + mr->markAnchor;
		if (fseek(fp, pos, SEEK_SET)) {
		  perror("Seeking GPOS mark anchor.\n");
		  exit(1);
		}
		get_bine_anchor_table(fp, &anchor, &junk, &pos);
		if (instr & DISPLAY_RAW) {
		  fprintf(stdout, " 9 [%d]:\n", im);
		  print_anchor_table(stdout, &anchor, &junk,
				     "markAnchor->:", 10);
		}
		if (instr & COMPILER) {
		  decompile_anchor(anchor, stdout);
		}
		free_anchor_table(&anchor);
		mr = mr->_next;
	      }
	      if (max_class >= subtable.classCount) {
		fprintf(stderr, "Subtable %d of lookup %d says %d classes "
			"for attached mark, but mark array defines or "
			"implies %d classes.\n",
			ist, il, subtable.classCount, max_class + 1);
	      }
/* Base processing */
/* Ligatures are subtly different because each component has to be treated
   separately, so anchors are defined by the triplet (lig glyph, component,
   mark class).  The ligatures glyphs are handled by a separate layer of
   offsets, so components have to be treated parallel to base / mark2 glyphs.
 */
	      if (forlig = (MARK2LIGATURE == lkp_type)) {
		pos = stloc + subtable.baseArray;
		if (fseek(fp, pos, SEEK_SET)) {
		  perror("Seeking GPOS ligature array table");
		  exit(1);
		}
		ligloc = pos;
		get_bine_ligature_array_table(fp, &lat, &junk, &pos);
		if (instr & DISPLAY_RAW) {
		  print_ligature_array_table
		    (fp, &lat, &junk, "ligatureArray->:", 7);
		}
		n_bat = lat.ligatureCount;
	      } else {
		n_bat = 1;
	      }
	      for (ilig = 0; ilig < n_bat; ilig++) {
		const char *error_prefix;
		int level;

		if (forlig) {
		  batloc = ligloc + lat.ligatureAttach[ilig];
		  error_prefix = "Seeking GPOS ligature attach table";
		} else {
		  batloc = stloc + subtable.baseArray;
		  error_prefix = "Seeking GPOS base array table";
		}
		pos = batloc;
		if (fseek(fp, pos, SEEK_SET)) {
		  perror(error_prefix);
		  exit(1);
		}
		get_bine_base_array_table(fp, &bat, &junk, &pos);
		if (instr & DISPLAY_RAW) {
		  switch (lkp_type) {
		  case MARK2BASE:
		    print_base_array_table
		      (stdout, &bat, &junk, "baseArray->:", level = 7);
		    break;
		  case MARK2MARK:
		    assert(sizeof(struct base_array_table) ==
			   sizeof(struct mark2_array));
		    print_mark2_array
		      (stdout, (struct mark2_array *) &bat, &junk,
		       "mark2Array->:", level = 7);
		    break;
		  case MARK2LIGATURE:
		    assert(sizeof(struct base_array_table) ==
			   sizeof(struct ligature_attach));
		    sprintf(label, "[%d]->:", ilig);
		    print_ligature_attach
		      (stdout, (struct ligature_attach *) &bat, &junk,
		       label, level = 9);
		    break;
		  }		      
		}
		if (0 == ilig && instr & COMPILER) {
		  int ng;
		  get_coverage(fp, stloc + subtable.baseCoverage, &bcov, &ng);
		  if (forlig) {
		    if (ng != lat.ligatureCount) {
		      fprintf(stderr, "Number of base glyphs (%d) in coverage "
			      "of subtable %d of lookup %d does not match "
			      "number of base records (%d).\n",
			      ng, ist, il, lat.ligatureCount);
		    }
		  } else {
		    if (ng != bat.baseCount) {
		      fprintf(stderr, "Number of base glyphs (%d) in coverage "
			      "of subtable %d of lookup %d does not match "
			      "number of base records (%d).\n",
			      ng, ist, il, bat.baseCount);
		    }
		  }
		  ci = make_coverage_iteration(&bcov);
		}
		br = bat.baseRecord;
		for (ib = 0; ib < bat.baseCount; ib++) {
		  if (instr & COMPILER) {
		    if (forlig && ilig) fputc('\n', stdout);
		    if (!(ib && forlig)) {
		      int gno = get_from_coverage(&ci);
		      if (0 && ib) fputc('\n', stdout);
		      fprintf(stdout, "  base %s\n", glyphname1(instr, gno));
		    }
		    if (forlig) fprintf(stdout, "  component %d\n", ib);
		  }
		  if (instr & DISPLAY_RAW) {
		    fprintf(stdout, "%2d [%d]:\n", level + 2, ib);
		    fprintf(stdout, "%2d baseAnchor:\n", level + 3);
		  }
		  for (imc = 0; imc < subtable.classCount; imc++) {
		    struct anchor_table anchor;
		    char sublab[20];

		    if (!br->baseAnchor[imc]) continue;
		    pos = batloc + br->baseAnchor[imc];
		    if (fseek(fp, pos, SEEK_SET)) {
		      perror("Seeking GPOS base anchor");
		      exit(1);
		    }
		    get_bine_anchor_table(fp, &anchor, &junk, &pos);
		    if (instr & COMPILER) {
		      if (0 && imc) fputc('\n', stdout);
		      fprintf(stdout, "    attach %s_%d ", gpname_m, imc);
		      decompile_anchor(anchor, stdout);
		    }
		    if (instr & DISPLAY_RAW) {
		      sprintf(sublab, "[%d]->:", imc);
		      print_anchor_table(stdout, &anchor, &junk,
					 sublab, level + 4);
		    }
		    free_anchor_table(&anchor);
		  }
		  br = br->_next;
		}
		free_base_array_table(&bat);
	      }
	      if (instr & COMPILER) {
		free_coverage_table(&bcov);
		free_coverage_table(&mcov);
	      }
	      free_mark_array_table(&mat);
	    }
	    if (1 == subtable.posFormat && (instr & DISPLAY_RAW)) {
	      const char *mcname, *bcname;
	      
	      switch (lkp_type) {
	      case MARK2BASE:
		mcname = "markCoverage";
		bcname = "baseCoverage";
		break;
	      case MARK2MARK:
		mcname = "mark1Coverage";
		bcname = "mark2coverage";
		break;
	      case MARK2LIGATURE:
		mcname = "markCoverage";
		bcname = "ligCoverage";
		break;
	      }		      
	      display_coverage(fp, stloc + subtable.markCoverage, mcname, 6);
	      display_coverage(fp, stloc + subtable.baseCoverage, bcname, 6);
	    }
	  }
	  break;
#else /* Only MARK2BASE - less likely to be faulty */
	case MARK2BASE:
	  {
	    struct mark_basepos_table subtable;
	    char gpname_b[100], gpname_m[100];

	    get_bine_mark_basepos_table(fp, &subtable, &junk, &pos);
	    if (instr & COMPILER) {
	      if (1 == subtable.posFormat) {
		sprintf(gpname_b, "plkp%d_%d_b", il, ist);
		decompile_coverage(fp, stloc + subtable.baseCoverage,
				   stdout, instr, gpname_b);
		sprintf(gpname_m, "plkp%d_%d_m", il, ist);
		decompile_coverage(fp, stloc + subtable.markCoverage,
				 stdout, instr, gpname_m);
	      }
	      fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
	      if (1 == subtable.posFormat) {
		fprintf(stdout, "-- markCoverage %s\n", gpname_m);
		fprintf(stdout, "-- baseCoverage %s\n", gpname_b);
	      } else {
		fprintf(stdout, "-- Unsupported format %d\n",
			subtable.posFormat);
	      }
	    }
	    if (instr & DISPLAY_RAW) {
	      print_mark_basepos_table(stdout, &subtable, &junk, label, 5);
	    }
	    if (1 == subtable.posFormat) {
	      struct base_array_table bat;
	      int ib, imc, im;
	      struct base_record *br;
	      int batloc, matloc;
	      struct mark_array_table mat;
	      struct mark_record *mr;
	      struct coverage_table bcov, mcov;
	      struct coverage_iteration ci;
	      int max_class = -1;

/* Mark processing */
	      pos = matloc = stloc + subtable.markArray;
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking GPOS mark array table");
		exit(1);
	      }
	      get_bine_mark_array_table(fp, &mat, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		print_mark_array_table(stdout, &mat, &junk, "markArray->:", 7);
	      }
	      if (instr & COMPILER) {
		int ng;
		get_coverage(fp, stloc + subtable.markCoverage, &mcov, &ng);
		if (ng != mat.markCount) {
		  fprintf(stderr, "Number of glyphs (%d) in coverage of "
			  "subtable %d of lookup %d does not match number of "
			  "base records (%d).\n", ng, ist, il, bat.baseCount);
		}
		ci = make_coverage_iteration(&mcov);
	      }
	      mr = mat.markRecord;
	      for (im = 0; im < mat.markCount; im++) {
		struct anchor_table anchor;

		if (instr & COMPILER) {
		  int gno = get_from_coverage(&ci);
		  if (0 && im) fputc('\n', stdout);
		  if (mr->class > max_class + 1) {
		    int j;
		    for (j = max_class + 1; j < mr->class; j++) {
		      fprintf(stdout, "  empty %s_%d\n", gpname_m, j);
		    }
		    max_class = mr->class;
		  } else if (mr->class > max_class) {
		    max_class = mr->class;
		  }
		  fprintf(stdout, "  mark %s_%d %s ", gpname_m, mr->class,
			  glyphname1(instr, gno));
		}
		pos = matloc + mr->markAnchor;
		if (fseek(fp, pos, SEEK_SET)) {
		  perror("Seeking GPOS mark anchor.\n");
		  exit(1);
		}
		get_bine_anchor_table(fp, &anchor, &junk, &pos);
		if (instr & DISPLAY_RAW) {
		  fprintf(stdout, " 9 [%d]:\n", ib);
		  print_anchor_table(stdout, &anchor, &junk,
				     "markAnchor->:", 10);
		}
		if (instr & COMPILER) {
		  decompile_anchor(anchor, stdout);
		}
		free_anchor_table(&anchor);
		mr = mr->_next;
	      }
	      if (max_class >= subtable.classCount) {
		fprintf(stderr, "Subtable %d of lookup %d says %d classes "
			"for attached mark, but mark array defines or "
			"implies %d classes.\n",
			ist, il, subtable.classCount, max_class + 1);
	      }
/* Base processing */ 
	      pos = batloc = stloc + subtable.baseArray;
	      if (fseek(fp, pos, SEEK_SET)) {
		perror("Seeking GPOS base array table");
		exit(1);
	      }
	      get_bine_base_array_table(fp, &bat, &junk, &pos);
	      if (instr & DISPLAY_RAW) {
		print_base_array_table(stdout, &bat, &junk, "baseArray->:", 7);
	      }
	      if (instr & COMPILER) {
		int ng;
		get_coverage(fp, stloc + subtable.baseCoverage, &bcov, &ng);
		if (ng != bat.baseCount) {
		  fprintf(stderr, "Number of glyphs (%d) in coverage of "
			  "subtable %d of lookup %d does not match number of "
			  "base records (%d).\n", ng, ist, il, bat.baseCount);
		}
		ci = make_coverage_iteration(&bcov);
	      }
	      br = bat.baseRecord;
	      for (ib = 0; ib < bat.baseCount; ib++) {
		if (instr & COMPILER) {
		  int gno = get_from_coverage(&ci);
		  if (ib) fputc('\n', stdout);
		  fprintf(stdout, "  base %s\n", glyphname1(instr, gno));
		}
		if (instr & DISPLAY_RAW) {
		  fprintf(stdout, " 9 [%d]:\n", ib);
		  fprintf(stdout, "10 baseAnchor:\n", ib);
		}
		for (imc = 0; imc < subtable.classCount; imc++) {
		  struct anchor_table anchor;
		  char sublab[20];

		  pos = batloc + br->baseAnchor[imc];
		  if (fseek(fp, pos, SEEK_SET)) {
		    perror("Seeking GPOS base anchor");
		    exit(1);
		  }
		  get_bine_anchor_table(fp, &anchor, &junk, &pos);
		  if (instr & COMPILER) {
		    if (imc) fputc('\n', stdout);
		    fprintf(stdout, "  attach %s_%d ", gpname_m, imc);
		    decompile_anchor(anchor, stdout);
		  }
		  if (instr & DISPLAY_RAW) {
		    sprintf(sublab, "[%d]->:", imc);
		    print_anchor_table(stdout, &anchor, &junk, sublab, 11);
		  }
		  free_anchor_table(&anchor);
		}
		br = br->_next;
	      }
	      if (instr & COMPILER) {
		free_coverage_table(&bcov);
		free_coverage_table(&mcov);
	      }
	      free_base_array_table(&bat);
	      free_mark_array_table(&mat);
	    }
	    if (1 == subtable.posFormat && (instr & DISPLAY_RAW)) {
	      display_coverage(fp, stloc + subtable.markCoverage,
			       "markCoverage", 6);
	      display_coverage(fp, stloc + subtable.baseCoverage,
			       "baseCoverage", 6);
	    }
	  }
	  break;
	case MARK2LIGATURE:
	case MARK2MARK:
#endif
	case GPOS_CHAINED:
	  display_chain_subtable(fp, pos, il, ist, UF_GPOS_CHAINED,
				 instr, PLKP, ie);
	  break;
	case CURSIVE:
	case GPOS_CONTEXT:
	case GPOS_EXTENSION:
          if (instr & COMPILER) {
            fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
          }
	  fprintf(stderr, "Unsupported GPOS lookup-type %d encountered.\n",
		  the_lookup_table.lookupType);
	  fprintf(stdout, " 5 [%d]: Unsupported lookup type\n", ist);
	  break;
	default:
          if (instr & COMPILER) {
            fprintf(stdout, "lookup plkp%d_%d\n", il, ist);
          }
	  fprintf(stderr, "Unknown GPOS lookup-type %d encountered.\n",
		  lkp_type);
	  if (instr & DISPLAY_RAW) {
	    fprintf(stdout, " 5 [%d]: Unknown lookup type\n", ist);
	  }
	  break;
	}
	if (instr & COMPILER) fprintf(stdout, "end lookup\n");
      }
      free_lookup_table(&the_lookup_table);
    }
    free_lookup_list(&lookup_list_table);
  }
  free_gsub_header(&GPOS_header);
}

/* Get and display hhea table */
void display_hhea(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *hhea_table;
  int pos, junk;
  struct horizontal_header horizontalHeader;

  for (hhea_table = top.table_directory; hhea_table;
       hhea_table = hhea_table->_next) {
    if (!strncmp(hhea_table->tag, "hhea", 4)) break;
  }
  if (hhea_table) {
    pos = hhea_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking horizontal header table");
      exit(1);
    }
    get_bine_horizontal_header(fp, &horizontalHeader, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_horizontal_header(stdout, &horizontalHeader, &junk, "hhea", 0);
    }
    if (instr & COMPILER) {
      fprintf(stdout, "ascender  %4d\n", horizontalHeader.ascender);
      fprintf(stdout, "descender %4d\n", horizontalHeader.descender);
      fprintf(stdout, "lineGap   %4d\n", horizontalHeader.lineGap);
      fprintf(stdout, "advanceWidthMax     %4d\n",
	      horizontalHeader.advanceWidthMax);
      fprintf(stdout, "minLeftSideBearing  %4d\n",
	      horizontalHeader.minLeftSideBearing);
      fprintf(stdout, "minRightSideBearing %4d\n",
	      horizontalHeader.minRightSideBearing);
      fprintf(stdout, "xMaxExtent          %4d\n",
	      horizontalHeader.xMaxExtent);
      fprintf(stdout, "caretSlopeRise %d\n",  horizontalHeader.caretSlopeRise);
      fprintf(stdout, "caretSlopeRun  %d\n",  horizontalHeader.caretSlopeRun);
      fprintf(stdout, "caretOffset %d\n",     horizontalHeader.caretOffset);
    }
  }
}

/* Get and display hmtx table */
void display_hmtx(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *hmtx_table;
  int pos, junk;

  for (hmtx_table = top.table_directory; hmtx_table;
       hmtx_table = hmtx_table->_next) {
    if (!strncmp(hmtx_table->tag, "hmtx", 4)) break;
  }
  if (hmtx_table) {
    pos = hmtx_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking horizontal metrics table");
      exit(1);
    }
    get_bine_horizontal_metrics(fp, &horizontalMetrics, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_horizontal_metrics(stdout, &horizontalMetrics, &junk, "hmtx", 0);
    }
  }
}

static void put_bitcode(int max_items, int indent, const char *tag,
			int *nonline) {
  if (*nonline) fputc(',', stdout);
  if (*nonline >= max_items) {
    fprintf(stdout, "\n%*s", indent, " ");
    *nonline = 0;
  }
  fprintf(stdout, " %s", tag);
  (*nonline)++;
}

static void put_bitnum(int max_items, int indent, int bitno,
		       int *nonline) {
  char image[20];

  sprintf(image, "%d", bitno);
  put_bitcode(max_items, indent, image, nonline);
}

/* Get and display OS/2 table */
void display_OS_2(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *OS_2_table;
  int pos, junk, i, j, nput;
  struct OS2_metrics OS2Metrics;
  unsigned int const *ure[4];
  unsigned int const *upg[2];

  ure[0] = &OS2Metrics.ulUnicodeRange1;
  ure[1] = &OS2Metrics.ulUnicodeRange2;
  ure[2] = &OS2Metrics.ulUnicodeRange3;
  ure[3] = &OS2Metrics.ulUnicodeRange4;

  upg[0] = &OS2Metrics.ulCodePageRange1;
  upg[1] = &OS2Metrics.ulCodePageRange2;

  for (OS_2_table = top.table_directory; OS_2_table;
       OS_2_table = OS_2_table->_next) {
    if (!strncmp(OS_2_table->tag, "OS/2", 4)) break;
  }
  if (OS_2_table) {
    pos = OS_2_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking OS/2 and Windows metrics");
      exit(1);
    }
    get_bine_OS2_metrics(fp, &OS2Metrics, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_OS2_metrics(stdout, &OS2Metrics, &junk, "OS/2", 0);
    }
    if (instr & COMPILER) {
      fprintf(stdout, "avgCharWidth %d\n", OS2Metrics.xAvgCharWidth);

      fprintf(stdout, "weightClass ");
      switch (OS2Metrics.usWeightClass) {
      case 100: fprintf(stdout, "thin"); break;
      case 200: fprintf(stdout, "extra_light"); break;
      case 300: fprintf(stdout, "light"); break;
      case 400: fprintf(stdout, "normal"); break;
      case 500: fprintf(stdout, "medium"); break;
      case 600: fprintf(stdout, "semibold"); break;
      case 700: fprintf(stdout, "bold"); break;
      case 800: fprintf(stdout, "extrabold"); break;
      case 900: fprintf(stdout, "black"); break;
      default: fprintf(stdout, "%u", OS2Metrics.usWeightClass);
      }
      fputc('\n', stdout);

      fprintf(stdout, "widthClass ");
      switch (OS2Metrics.usWidthClass) {
      case 1: fprintf(stdout, "ultracondensed"); break;
      case 2: fprintf(stdout, "extracondensed"); break;
      case 3: fprintf(stdout, "condensed"); break;
      case 4: fprintf(stdout, "semicondensed"); break;
      case 5: fprintf(stdout, "normal"); break;
      case 6: fprintf(stdout, "semiexpanded"); break;
      case 7: fprintf(stdout, "expanded"); break;
      case 8: fprintf(stdout, "extraexpanded"); break;
      case 9: fprintf(stdout, "ultraexpanded"); break;
      default :fprintf(stdout, "%u\n", OS2Metrics.usWidthClass);
      }
      fputc('\n', stdout);

      if (OS2Metrics.fsType) {
#define val OS2Metrics.fsType
	nput = 0;
	fprintf(stdout, "embedding");
	if (1 & val) put_bitcode(5, 9, "bit0", &nput);
	if (2 & val) put_bitcode(5, 9, "banned", &nput);
	if (4 & val) put_bitcode(5, 9, "print", &nput);
	if (8 & val) put_bitcode(5, 9, "edit", &nput);
	if (16 & val) put_bitcode(5, 9, "4", &nput);
	if (32 & val) put_bitcode(5, 9, "5", &nput);
	if (64 & val) put_bitcode(5, 9, "6", &nput);
	if (128 & val) put_bitcode(5, 9, "7", &nput);
	if (256 & val) put_bitcode(5, 9, "no_subsetting", &nput);
	if (512 & val) put_bitcode(5, 9, "bitmap", &nput);
	for (i = 10; i < 16; i++) {
	  if ((1<<i) & val) put_bitnum(5, 9, i, &nput);
	}
	fputc('\n', stdout);
#undef val
      } else {
	fprintf(stdout, "embedding free\n");
      }

      fprintf(stdout, "subscriptXSize %d\n", OS2Metrics.ySubscriptXSize);
      fprintf(stdout, "subscriptYSize %d\n", OS2Metrics.ySubscriptYSize);
      fprintf(stdout, "subscriptXOffset %d\n", OS2Metrics.ySubscriptXOffset);
      fprintf(stdout, "subscriptYOffset %d\n", OS2Metrics.ySubscriptYOffset);
      fprintf(stdout, "superscriptXSize %d\n", OS2Metrics.ySuperscriptXSize);
      fprintf(stdout, "superscriptYSize %d\n", OS2Metrics.ySuperscriptYSize);
      fprintf(stdout, "superscriptXOffset %d\n",
	      OS2Metrics.ySuperscriptXOffset);
      fprintf(stdout, "superscriptYOffset %d\n",
	      OS2Metrics.ySuperscriptYOffset);
      fprintf(stdout, "strikeoutSize %d\n", OS2Metrics.yStrikeoutSize);
      fprintf(stdout, "strikeoutPosition %d\n", OS2Metrics.yStrikeoutPosition);
      fprintf(stdout, "familyClass 0x%X\n", OS2Metrics.sFamilyClass);
      fprintf(stdout, "panose all");
      for (i = 0; i < 10; i++) fprintf(stdout, " %d", OS2Metrics.panose[i]);
      fputc('\n', stdout);
      fprintf(stdout, "unicodeRange none -- augmentation follows\n");
      nput = 0;
#if 0
      for (j = 0; j < 4; j++) {
	for (i = 0; i < 32; i++) {
	  if (*ure[j] & (1 << i)) {
	    if (!nput) fprintf(stdout, "unicodeRange");
	    fprintf(stdout, " %d", j * 32 + i); nput++;
	    if (nput >= 10) {
	      fputc('\n', stdout);
	      nput = 0;
	    }
	  }
	}
      }
#else
      {
	int putit = 0, iel = 0;
	const char *sp;

	for (j = 0; j < 4; j++) {
	  i = 0;
	  while (i < 32) {
	    const int ij =  j * 32 + i;
	    if ((*ure[j] & (1 << i)) == 0) {
	      i++;
	    } else if (iel >= n_script_table) {
	      putit = 1;
	    } else if (script_table[iel].bit == ij) {
	      putit = 2;
	    } else if (script_table[iel].bit > ij) {
	      putit = 1;
	    } else {
	      iel++;
	    }
	    if (putit) {
	      if (nput) {
		fputc(',', stdout);
	      } else {
		fprintf(stdout, "unicodeRange");
	      }
	      if (1 == putit) {
		fprintf(stdout, " %d", ij);
	      } else {
		sp = script_table[iel].sn;
		if (strchr(sp, '-')) {
		  fprintf(stdout, " \"%s\"", sp);
		} else {
		  fprintf(stdout, " %s", sp);
		}
	      }
	      putit = 0;
	      i++;
	      nput++;
	      if (nput >= 4) {
		fputc('\n', stdout);
		nput = 0;
	      }
	    }
	  }
	}
      }
#endif
      if (nput) fputc('\n', stdout);
      fprintf(stdout, "vendID \"%4.4s\"\n", OS2Metrics.achVendID);
      if (OS2Metrics.fsSelection) {
#define val OS2Metrics.fsSelection
	nput = 0;
	fprintf(stdout, "fsSelection");
	if (1 & val) put_bitcode(5, 11, "italic", &nput);
	if (2 & val) put_bitcode(5, 11, "underscore", &nput);
	if (4 & val) put_bitcode(5, 11, "negative", &nput);
	if (8 & val) put_bitcode(5, 11, "outlined", &nput);
	if (16 & val) put_bitcode(5, 11, "strikeout", &nput);
	if (32 & val) put_bitcode(5, 11, "bold", &nput);
	if (64 & val) put_bitcode(5, 11, "regular", &nput);
	if (128 & val) put_bitcode(5, 11, "typo", &nput);
	if (256 & val) put_bitcode(5, 11, "wws", &nput);
	if (512 & val) put_bitcode(5, 11, "oblique", &nput);
	for (i = 10; i < 16; i++) {
	  if ((1<<i) & val) put_bitnum(5, 11, i, &nput);
	}
	fputc('\n', stdout);
#undef val
      } else {
	fprintf(stdout, "fsSelection none\n");
      }
      fprintf(stdout, "firstCharIndex 0x%4.4X\n", OS2Metrics.usFirstCharIndex);
      fprintf(stdout, "lastCharIndex 0x%4.4X\n", OS2Metrics.usLastCharIndex);
      fprintf(stdout, "typoAscender %d\n", OS2Metrics.sTypoAscender);
      fprintf(stdout, "typoDescender %d\n", OS2Metrics.sTypoDescender);
      fprintf(stdout, "typoLineGap %d\n", OS2Metrics.sTypoLineGap);
      fprintf(stdout, "winAscent %u\n", OS2Metrics.usWinAscent);
      fprintf(stdout, "winDescent %u\n", OS2Metrics.usWinDescent);
      if (OS2Metrics.version >= 1) {
	fprintf(stdout, "codePage none -- augmentation follows\n");
	nput = 0;
#if 0
	for (j = 0; j < 2; j++) {
	  for (i = 0; i < 32; i++) {
	    if (*upg[j] & (1 << i)) {
	      if (!nput) fprintf(stdout, "codePage extend");
	      fprintf(stdout, " %d", j * 32 + i); nput++;
	      if (nput >= 10) {
		fputc('\n', stdout);
		nput = 0;
	      }
	    }
	  }
	}
#else
	{
	  int putit = 0, iel = 0;
	  const char *sp;

	  for (j = 0; j < 2; j++) {
	    i = 0;
	    while (i < 32) {
	      const int ij =  j * 32 + i;
	      if ((*upg[j] & (1 << i)) == 0) {
		i++;
	      } else if (iel >= n_code_page_table) {
		putit = 1;
	      } else if (code_page_table[iel].bit == ij) {
		putit = 2;
	      } else if (code_page_table[iel].bit > ij) {
		putit = 1;
	      } else {
		iel++;
	      }
	      if (putit) {
		if (nput) {
		  fputc(',', stdout);
		} else {
		  fprintf(stdout, "codePage extend");
		}
		if (1 == putit) {
		  fprintf(stdout, " %d", ij);
		} else {
		  fprintf(stdout, " %s", code_page_table[iel].n1);
		}
		putit = 0;
		i++;
		nput++;
		if (nput >= 4) {
		  fputc('\n', stdout);
		  nput = 0;
		}
	      }
	    }
	  }
	}
#endif
	if (nput) fputc('\n', stdout);
      }
      if (OS2Metrics.version >= 2) {
	fprintf(stdout, "xHeight %d\n", OS2Metrics.sxHeight);
	fprintf(stdout, "capHeight %d\n", OS2Metrics.sCapHeight);
	fprintf(stdout, "defaultChar 0x%4.4X\n", OS2Metrics.usDefaultChar);
	fprintf(stdout, "breakChar 0x%4.4X\n", OS2Metrics.usBreakChar);
	fprintf(stdout, "maxContext %u\n", OS2Metrics.usMaxContext);
      }
    }
  }
}

/* Get and display post table */
void display_post(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *post_table;
  int pos, junk;

  if (1) fprintf(stderr, "Invoked display_post(%8.8X).\n", instr);
  for (post_table = top.table_directory; post_table;
       post_table = post_table->_next) {
    if (!strncmp(post_table->tag, "post", 4)) break;
  }
  if (post_table) {
    pos = post_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking PostScript information");
      exit(1);
    }
    get_bine_postscript_information(fp, &post_info, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_postscript_information(stdout, &post_info, &junk, "post", 0);
    }
    if (instr & DISPLAY_RES) {
      fprintf(stdout, " 1 italicAngle_deg: %.6f\n",
	      (double) post_info.italicAngle / (1<<16));
    }
    if (instr & COMPILER) {
      fprintf(stdout, "italicAngle %.6f\n",
	      (double) post_info.italicAngle / (1<<16));
      fprintf(stdout, "underlinePosition %d\n", post_info.underlinePosition); 
      fprintf(stdout, "underlineThickness %d\n", post_info.underlineThickness);
      fprintf(stdout, "minMemType42 %u\n", post_info.minMemType42); 
      fprintf(stdout, "maxMemType42 %u\n", post_info.maxMemType42); 
      fprintf(stdout, "minMemType1 %u\n", post_info.minMemType1); 
      fprintf(stdout, "maxMemType1 %u\n", post_info.maxMemType1);
    }
  }
}

/* Get and display font program */
void display_fpgm(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *font_program_table;
  int pos, junk;
  struct font_program program;

  for (font_program_table = top.table_directory; font_program_table;
       font_program_table = font_program_table->_next) {
    if (!strncmp(font_program_table->tag, "fpgm", 4)) break;
  }
  if (font_program_table) {
    pos = font_program_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking font program");
      exit(1);
    }
    get_bine_font_program(fp, &program, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_font_program(stdout, &program, &junk, "font_program", 0);
    }
    if (instr & COMPILER) fprintf(stdout, "font_program\n");
    if (instr & (DISPLAY_RES | COMPILER)) {
      disassemble(stdout, program.datum, program._n_datum,
		  instr & DISPLAY_RES, 1);
    }
    if (instr & COMPILER) fprintf(stdout, "end font_program\n");
  }
}

/* Get and display font program */
void display_prep(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *prep_table;
  int pos, junk;
  struct control_value_program program;

  for (prep_table = top.table_directory; prep_table;
       prep_table = prep_table->_next) {
    if (!strncmp(prep_table->tag, "prep", 4)) break;
  }
  if (prep_table) {
    pos = prep_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking font program");
      exit(1);
    }
    get_bine_control_value_program(fp, &program, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_control_value_program(stdout, &program, &junk, "prep", 0);
    }
    if (instr & COMPILER) fprintf(stdout, "cvt_program\n");
    if (instr & (DISPLAY_RES | COMPILER)) {
      disassemble(stdout, program.program, program._n_program,
		  instr & DISPLAY_RES, 1);
    }
    if (instr & COMPILER) fprintf(stdout, "end cvt_program\n");
  }
}

/* Get and display font program */
void display_PCLT(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *PCL_table;
  int pos, junk;
  struct PCL_table table;

  for (PCL_table = top.table_directory; PCL_table;
       PCL_table = PCL_table->_next) {
    if (!strncmp(PCL_table->tag, "PCLT", 4)) break;
  }
  if (PCL_table) {
    pos = PCL_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking PCLT table");
      exit(1);
    }
    get_bine_PCL_table(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_PCL_table(stdout, &table, &junk, "PCLT", 0);
    }
  }
}

#if 0
simple_table_dump(CPAL, cpal_header)
#else
void display_CPAL(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *CPAL_table;
  int pos,junk;
  struct cpal_header table;
  struct cra cra;
  struct pta pta;
  struct pla pla;
  struct pela pela;

  for (CPAL_table = top.table_directory; CPAL_table;
       CPAL_table = CPAL_table->_next) {
    if (!strncmp(CPAL_table->tag, "CPAL", 4)) break;
  }
  memset(&table, 0, sizeof table);
  if (CPAL_table) {
    pos = CPAL_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CPAL table");
      exit(1);
    }
    get_bine_cpal_header(fp, &table, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_cpal_header(stdout, &table, &junk, "CPAL", 0);
    }
    p0c0 = table.colorRecordIndices[0];
  } else if (ANALYSE == instr) {
    /* Only needed if table is present */
    return;
  } else {
    return; /* No error message, as yet. */
  }
  if (table.offsetFirstColorRecord) {
    pos = CPAL_table->offset + table.offsetFirstColorRecord;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CPAL colour record");
      exit(1);
    }
    get_bine_cra(fp, &cra, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_cra(stdout, &cra, &junk, "colorRecord", 1);
    }
  }
  if (/* 1 == table.version && */ table.offsetPaletteTypeArray) {
    pos = CPAL_table->offset + table.offsetPaletteTypeArray;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CPAL palette types");
      exit(1);
    }
    get_bine_pta(fp, &pta, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_pta(stdout, &pta, &junk, "paletteType", 1);
    }
  }
  if (/* 1 == table.version && */ table.offsetPaletteLabelArray) {
    pos = CPAL_table->offset + table.offsetPaletteLabelArray;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CPAL labels");
      exit(1);
    }
    get_bine_pla(fp, &pla, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_pla(stdout, &pla, &junk, "paletteLabel", 1);
    }
  }
  if (/* 1 == table.version && */ table.offsetPaletteEntryLabelArray) {
    pos = CPAL_table->offset + table.offsetPaletteEntryLabelArray;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CPAL labels");
      exit(1);
    }
    get_bine_pela(fp, &pela, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_pela(stdout, &pela, &junk, "paletteEntryLabel", 1);
    }
  }
  if (instr & COMPILER) {
    int ip;
    fprintf(stdout, "colours -- ARGB\n");
    if (table.offsetFirstColorRecord) {
      int ic;
      struct color_record *p;
      int ie0, ie9;
      if (table.numPalette) {
        ie0 = table.colorRecordIndices[0];
        ie9 = ie0 + table.numPalettesEntries - 1;
      } else {
        ie0 = -1;
        ie9 = -2;
      }
      for (ic = 0, p = cra.x; ic < table.numColorRecords; ic++, p = p->_next) {
        fprintf(stdout, "  col%d: %2.2X%2.2X%2.2X%2.2X", ic,
                p->alpha, p->red, p->green, p->blue);
        if (ie0 == ic) {
          fprintf(stdout, ", first");
        } else if (ie9 == ic) {
          fprintf(stdout, ", last");
        }
        if (ie0 <= ic && ic <= ie9) {
          int label = 0xffff;
          if (table.offsetPaletteEntryLabelArray) {
            label = pela.paletteEntryLabel[ic-ie0];
          }
          if (label < 0xffff) fprintf(stdout, ", label %d", label);
        }
        fputc('\n', stdout);
      }
    } else if (table.numColorRecords) {
      fprintf(stdout, "ERROR! Nominally %d colour records, but no values.\n",
              table.numColorRecords);
    } else {
      fprintf(stdout, "ERROR! No colour records.\n");
    }
    fprintf(stdout, "end colours\n");
    fprintf(stdout, "palettes\n");
    for (ip = 0; ip < table.numPalette; ip++) {
      fprintf(stdout, "  col%d ", table.colorRecordIndices[ip]);
      if (table.offsetPaletteLabelArray) {
        fprintf(stdout, "%d ", pla.paletteLabel[ip]);
      } else {
        fputs("0xFFFF ", stdout);
      }
      if (!table.offsetPaletteTypeArray) {
        fputs("none", stdout);
      } else if (pta.paletteType[ip]) {
        int flags = pta.paletteType[ip];
        char *sep = "";
        int io;
        if (1 & flags) {
          fprintf(stdout, "%slight", sep);
          sep = ",";
        }
        if (2 & flags) {
          fprintf(stdout, "%sdark", sep);
          sep = ",";
        }
        for (io = 2, flags >>= 2; flags && io < 32; io++) {
          if (flags & (1 << io)) {
            fprintf(stdout, "%s%d", sep, io);
            sep = ",";
          }
        }
      } else {
        fputs("none", stdout);
      }
      fputc('\n', stdout);
    }
    fprintf(stdout, "end palettes\n");
  }
}
#endif

void display_COLR(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *COLR_table;
  int pos,junk;
  struct colr_header table;
  struct base_glyph_records bgr;
  struct layer_records layers;

  memset(&bgr,    0, sizeof bgr);
  memset(&layers, 0, sizeof layers);
  for (COLR_table = top.table_directory; COLR_table;
       COLR_table = COLR_table->_next) {
    if (!strncmp(COLR_table->tag, "COLR", 4)) break;
  }
  if (COLR_table) {
    /* No problem */
  } else if (ANALYSE == instr) {
    /* Only needed if table is present */
    return;
  } else {
    return; /* No error message, as yet. */
  }
  memset(&table,  0, sizeof table);
  pos = COLR_table->offset;
  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking COLR table");
    exit(1);
  }
  get_bine_colr_header(fp, &table, &junk, &pos);
  if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
    print_colr_header(stdout, &table, &junk, "COLR", 0);
  }

  pos = COLR_table->offset + table.offsetBaseGlyphRecord;
  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking COLR base glyph records");
    exit(1);
  }
  get_bine_base_glyph_records(fp, &bgr, &junk, &pos);
  if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
    print_base_glyph_records(stdout, &bgr, &junk, "baseGlyphRecord", 1);
  }

  pos = COLR_table->offset + table.offsetLayerRecord;
  if (fseek(fp, pos, SEEK_SET)) {
    perror("Seeking COLR layer records");
    exit(1);
  }
  get_bine_layer_records(fp, &layers, &junk, &pos);
  if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
    print_layer_records(stdout, &layers, &junk, "layerRecord", 1);
  }

  if (instr & COMPILER) {
    COLR_bgr = bgr;
    COLR_layers = layers;
  }
}

#define simple_table_dump(name, type)					\
  void display_ ## name(FILE *fp, int instr, struct offset_table top) {	\
    struct directory_entry *name ## _table;				\
    int pos, junk;							\
    struct type table;							\
									\
    for (name ## _table = top.table_directory; name ## _table;		\
	 name ## _table = name ## _table->_next) {			\
      if (!strncmp(name ## _table->tag, #name "   ", 4)) break;		\
    }									\
    if (name ## _table) {						\
      pos = name ## _table->offset;					\
      if (fseek(fp, pos, SEEK_SET)) {					\
	perror("Seeking " #name " table");				\
	exit(1);							\
      }									\
      get_bine_ ## type(fp, &table, &junk, &pos);			\
      if (instr & DISPLAY_RAW) {					\
	print_ ## type(stdout, &table, &junk, #name, 0);		\
      }									\
    }									\
}

#if 0
simple_table_dump(Silf, silf)
#else
/* Get and display basic SIL Graphite table */
unsigned int *pscodepoint;
unsigned int Silf_maxGlyphID;
static const char *Silf_glyph_name(int gid) {
  int cpt;
  static char buffer[40];
  if (gid < N_GLYPHS) {
    cpt = codepoint[gid];
    if (NO_CHAR == cpt) {
      sprintf(buffer, "%5d (deriv)  ", gid);
    } else {
      sprintf(buffer, "%5d <--U+%4.4X", gid, cpt);
    }
  } else if (gid < Silf_maxGlyphID) {
    cpt = pscodepoint[gid - N_GLYPHS];
    if (NO_CHAR == cpt) {
      sprintf(buffer, "%5d (pseudo) ", gid);
    } else {
      sprintf(buffer, "%5d <--U+%4.4X", gid, cpt);
    }
  } else {
    sprintf(buffer,   "%5d          ", gid);
  }
  return buffer;
}
void display_Silf(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *Silf_table;
  int pos, junk;
  struct silf table;

  for (Silf_table = top.table_directory; Silf_table;
       Silf_table = Silf_table->_next) {
    if (!strncmp(Silf_table->tag, "Silf", 4)) break;
  }
  if (Silf_table) {
    pos = Silf_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking " "Silf" " table");
      exit(1);
    }
    get_bine_silf(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_silf(stdout, &table, &junk, "Silf", 0);
    }
    if (instr & DISPLAY_RES) {
      int it;
      struct sil_sub *tt;
      fprintf(stdout, " 1 exp_table\n");
      for (it = 0, tt = table.tables; it < table.num; it++, tt = tt->_next) {
	char index[20];
	int ips, ilin, ilu, ipass;
        struct pseudo_map *pps;
	int offset0, gid;
        struct lookup_class *clu;
        struct sil_pass *pass;

        Silf_maxGlyphID = tt->maxGlyphID; 
	sprintf(index, " 2 [%d]", it);
	print_sil_sub(stdout, tt, &junk, index, 2);
        fprintf(stdout, " 3 numPseudo: %d\n", tt->numPseudo);
        pscodepoint = malloc((tt->maxGlyphID - N_GLYPHS) * sizeof(*pscodepoint));
	if (pscodepoint) {
	  int i;
	  for (i = 0; i < (tt->maxGlyphID - N_GLYPHS); i++) {
	    pscodepoint[i] = NO_CHAR;
	  }
	} else {
	  fprintf(stderr, "Malloc failed on pseudoglyph to character table.\n");
	  exit(1);
	}
        for (ips = 0, pps = tt->pMaps; pps; ips++, pps = pps->_next) {
	  fprintf(stdout, " 4 [%d] pseudoglyph %d represents U+%4.4X\n", ips,
		  pps->nPseudo, pps->uniCode);
	  pscodepoint[pps->nPseudo - N_GLYPHS] = pps->uniCode;
	}
        fprintf(stdout, " 3 numClass: %d\n",  tt->classes.numClass);
        fprintf(stdout, " 3 numLinear: %d\n", tt->classes.numLinear);
        fprintf(stdout, " 3 classes:\n");
	offset0 = (3 + tt->classes.numClass);
        for (ilin = 0; ilin < tt->classes.numLinear; ilin++) {
	  int el1 = tt->classes.oClass[ilin]/2   - offset0;
	  int nel = (tt->classes.oClass[ilin+1] - tt->classes.oClass[ilin])/2;
	  int iel, cpt;
	  #define N_ONLINE 5
          fprintf(stdout, " 4 [%d]:", ilin);
	  for (iel = 0; iel < nel; iel++) {
	    if (0 == iel % N_ONLINE) {
	      fprintf(stdout, "\n 5 [%2.2d-", iel);
	    }
	    gid = tt->classes.glyphs[el1+iel];
	    fputc(' ', stdout);
	    fputs(Silf_glyph_name(gid), stdout);
	  }
	  fputc('\n', stdout);
	}
	for (ilu = tt->classes.numLinear, clu=tt->classes.lookups;
	     ilu < tt->classes.numClass; ilu++, clu = clu->_next) {
          int iel;
          struct lookup_pair *ptel;
	  fprintf(stdout, " 4 [%d]:\n", ilu);
	  fprintf(stdout, " 5 numIDs: %d\n", clu->numIDs);
	  for (iel = 0, ptel = clu->lookups; iel < clu->numIDs;
	       iel++, ptel = ptel->_next) {
	    fprintf(stdout, " 6 [%d]: At position %d, glyph ",
		    iel, ptel->index);
	    fputs(Silf_glyph_name(ptel->glyphid), stdout);
	    fputc('\n', stdout);
	  }
	}
	fprintf(stdout, " 3 passes:\n");
	for (ipass = 0, pass = tt->passes; ipass < tt->numPasses;
	     ipass++, pass = pass->_next) {
          int ir, is, it, ic;
          int nco; /* Number of non-success transitional states */
          struct pass_range *pr;
	  /* For now, just analyse pass 0 */
	  if (0 == ipass) {
	  fprintf(stdout, " 4 [%d]:\n", ipass);
          fprintf(stdout, " 5 numRows: %d\n", pass->numRows);
          fprintf(stdout, " 5 numTransitional: %d\n", pass->numTransitional);
          fprintf(stdout, " 5 numSuccess: %d\n", pass->numSuccess);
          fprintf(stdout, " 5 numColumns: %d\n", pass->numColumns);
          fprintf(stdout, " 5 numRange: %d\n", pass->numRange);
          fprintf(stdout, " 5 range:\n");
	  for (ir = 0, pr = pass->ranges; ir < pass->numRange;
	       ir++, pr = pr->_next) {
	    int k;
	    char sep = ' ';
#if 0
	    fprintf(stdout, " 6 [%d] %d - %d (%d) => col %d\n", ir, pr->firstId, pr->lastId, pr->lastId - pr->firstId + 1, pr->colId);
#endif
	    fprintf(stdout, " 6 [%d] To col %d for (%d:)", ir, pr->colId,
		    pr->lastId - pr->firstId + 1);
	    for (k = pr->firstId; k <= pr->lastId; k++) {
	      fputc(sep, stdout);
	      fputs(Silf_glyph_name(k), stdout);
	    }
	    fputc('\n', stdout);
	  }
	  nco = pass->numRows - pass->numSuccess;
	  fprintf(stdout, " 5 Rules enabled by entry to state:\n");
          for (is = 0; is < pass->numSuccess; is++) {
	    fprintf(stdout, " 6 [%d]: ", nco + is);
	    if (pass->oRuleMap[is] >= pass->oRuleMap[is+1]) {
	      fprintf(stdout, "No rules\n");
	    } else {
	      int ino;
	      const char *sep = "";
	      for (ino = pass->oRuleMap[is]; ino < pass->oRuleMap[is+1]; ino++){
		fprintf(stdout, "%s%d", sep, pass->ruleMap[ino]);
		sep = ", ";
	      }
	      fputc('\n', stdout);
	    }  
	  }
          fprintf(stdout, " 5 State transition diagram:\n");
	  fprintf(stdout, " 6 col ");
	  for (ic = 0; ic < pass->numColumns; ic++) {
	    fprintf(stdout, " %2d", ic);
	  }
	  fputc('\n', stdout);
	  for (it = 0; it < pass->numTransitional; it++) {
	    fprintf(stdout, " 6 [%2.2d]", it);
	    for (ic = 0; ic < pass->numColumns; ic++) {
	      fprintf(stdout, " %2d",
		      pass->stateTrans[it*pass->numColumns + ic]);
	    }
	    fputc('\n', stdout);
          }
	  }
	}
      }
    }
  }
}
#endif

#define GRAPHITE_ADVANCED 1
#if GRAPHITE_ADVANCED
/* Get and display SIL Graphite language to feature table */
#if 0
simple_table_dump(Sill, sill)
#else
void display_Sill(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *Sill_table;
  int pos, junk;
  struct sill table;
  int ient;
  struct LanguageEntry *p_ent;
  struct LangFeatureSetting set; 
  char cent[20];

  for (Sill_table = top.table_directory; Sill_table;
       Sill_table = Sill_table->_next) {
    if (!strncmp(Sill_table->tag, "Sill", 4)) break;
  }
  if (Sill_table) {
    pos = Sill_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking Sill table");
      exit(1);
    }
    get_bine_sill(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_sill(stdout, &table, &junk, "Sill", 0);
      fprintf(stdout, " 1 settings:\n");
      for (ient = 0, p_ent = table.entries->_next; ient < table.numLangs;
	   ient++, p_ent = p_ent->_next) {
	while (pos < Sill_table->offset + p_ent->offset) {
	  sprintf(cent, "[%d]", ient);
	  get_bine_LangFeatureSetting(fp, &set, &junk, &pos);
	  print_LangFeatureSetting(stdout, &set, &junk, cent, 2);
	}
      }
    }
  }
}
#endif

/* Get and display SIL external file specification */
simple_table_dump(Sile, sile)

/* Get and display graphite glyph attribute index  */
struct gloc gloc_table;
void display_Gloc(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *p_Gloc_table;
  int pos, junk;
  
  for (p_Gloc_table = top.table_directory; p_Gloc_table;
       p_Gloc_table = p_Gloc_table->_next) {
    if (!strncmp(p_Gloc_table->tag, "Gloc", 4)) break;
  }
  if (p_Gloc_table) {
    pos = p_Gloc_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking Gloc table");
      exit(1);
    }
    get_bine_gloc(fp, &gloc_table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_gloc(stdout, &gloc_table, &junk, "Gloc", 0);
    }
  }
}

/* Get and display graphite glyph attributes  */
void display_Glat(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *p_Glat_table;
  int pos, junk, endpos;
  struct glat_header table;
  struct glat_entry entry;
  int iglyf, iset;
  char tag[20];
  
  for (p_Glat_table = top.table_directory; p_Glat_table;
       p_Glat_table = p_Glat_table->_next) {
    if (!strncmp(p_Glat_table->tag, "Glat", 4)) break;
  }
  if (p_Glat_table) {
    pos = p_Glat_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking Glat table");
      exit(1);
    }
    get_bine_glat_header(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_glat_header(stdout, &table, &junk, "Glat", 0);
      fprintf(stdout, " 1 entry_set:\n");
      for (iglyf = 0; iglyf <= max_graphite_glyph_ID; iglyf++) {
	fprintf(stdout, " 2 [%d]:\n", iglyf);
	if (1 & gloc_table.flags) {
	  endpos = gloc_table.locations4[iglyf+1];
	} else {
	  endpos = gloc_table.locations2[iglyf+1];
	}
	iset = 0;
	while (pos < p_Glat_table->offset + endpos) {
	  get_bine_glat_entry(fp, &entry, &junk, &pos);
	  sprintf(tag, "[%d]", iset++);
	  print_glat_entry(stdout, &entry, &junk, tag, 3); 
	}
      }
    }
#if 0
    fprintf(stdout, "Next data at %d, next table at %d.\n", pos,
	    p_Glat_table->offset + p_Glat_table->length);
#endif
  }
}

/* Get and display Graphite Feat table */
void display_Feat(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *Feat_table;
  int pos, junk;
  struct Feat table;
  const int fsd_size = 4;
  struct FeatureSettingDefn fsd;
  int nfsd, ifsd;
  char cfsd[20];

  for (Feat_table = top.table_directory; Feat_table;
       Feat_table = Feat_table->_next) {
    if (!strncmp(Feat_table->tag, "Feat", 4)) break;
  }
  if (Feat_table) {
    pos = Feat_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking Feat table");
      exit(1);
    }
    memset(&table, 0, sizeof(table));
    get_bine_Feat(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_Feat(stdout, &table, &junk, "Feat", 0);
      nfsd = (Feat_table->length - (pos - Feat_table->offset)) / fsd_size;
      for (ifsd = 0; ifsd < nfsd; ifsd++) {
	get_bine_FeatureSettingDefn(fp, &fsd, &junk, &pos);
	sprintf(cfsd, "[%d]", ifsd);
	print_FeatureSettingDefn(stdout, &fsd, &junk, cfsd, 2);
      }
    }
  }
}
#else
void display_Feat(FILE *fp, int instr, struct offset_table top) {}
void display_Glat(FILE *fp, int instr, struct offset_table top) {}
void display_Gloc(FILE *fp, int instr, struct offset_table top) {}
void display_Sile(FILE *fp, int instr, struct offset_table top) {}
void display_Sill(FILE *fp, int instr, struct offset_table top) {}
#endif

/* Get and display control value table. */
void display_cvt(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *cvt_table;
  int pos, i, junk;
  struct control_value_table table;

  for (cvt_table = top.table_directory; cvt_table;
       cvt_table = cvt_table->_next) {
    if (!strncmp(cvt_table->tag, "cvt ", 4)) break;
  }
  if (cvt_table) {
    pos = cvt_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking cvt table");
      exit(1);
    }
    get_bine_control_value_table(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_control_value_table(stdout, &table, &junk, "cvt", 0);
    }
    if (instr & COMPILER) {
      fprintf(stdout, "control_values\n");
      for (i = 0; i < CVT_SIZE; i++) {
	if (0 == i % 8) fprintf(stdout, "  ");
	fprintf(stdout, " %6d", table.datum[i]);
	if (7 == i % 8) fputc('\n', stdout);
      }
      fputc('\n', stdout);
      fprintf(stdout, "end control_values\n");
    }
  }
}

/* Action flags */
#define MORX_LIGATURE_PUSH    0x8000
#define MORX_LIGATURE_LINGER  0x4000
#define MORX_LIGATURE_PERFORM 0x2000
/* Ligature action flags */
#define MORX_LIGATURE_LAST  0x80000000
#define MORX_LIGATURE_STORE 0x40000000
#define MORX_LIGATURE_SIGN  0x20000000
static void put_action(struct action *action, int ia, const char *subnam,
		       char *adecl, char *sdecl) {
  int verb;
  const char *sep;
  const char* indic_verb[] = {
    "nop",         "Ax_xA",       "xD_Dx",       "AxD_DxA",
    "ABx_xAB",     "ABx_xBA",     "xCD_CDx",     "xCD_DCx",
    "AxCD_CDxA",   "AxCD_DCxA",   "ABxD_DxAB",   "ABxD_DxBA",
    "ABxCD_CDxAB", "ABxCD_CDxBA", "ABxCD_DCxAB", "ABxCD_DCxAB"};

  fprintf(stdout, "action a%d ", ia);
  switch (action->newState) {
  case 0:
    fprintf(stdout, "state SOT");
    break;
  case 1:
    fprintf(stdout, "state SOL");
    break;
  default:
    if (!sdecl[action->newState]) fputs("new", stdout);
    sdecl[action->newState] = 'Y';
    fprintf(stdout, "state s%d", action->newState);
  }
  if (!action->flags) fprintf(stdout, " zero");
  sep = "";
  switch (AAT_TABLE) {
  case MORX_INDIC:
    if (0x8000 & action->flags) {
      fprintf(stdout, "%s first", sep);
      sep = ",";
    }
    if (0x4000 & action->flags) {
      fprintf(stdout, "%s linger", sep);
      sep = ",";
    }
    if (0x2000 & action->flags) {
      fprintf(stdout, "%s last", sep);
      sep = ",";
    }
    if (0x1FF0 & action->flags) {
      fprintf(stdout, "%s 0x%4.4X", sep, 0x1FF0 & action->flags);
      sep = ",";
    }
    verb = 0x0F & action->flags;
    if (action->flags) {
      fprintf(stdout, "%s %s", sep, indic_verb[verb]);
    }
    break;
  case MORX_CONTEXTUAL:
    if (0x8000 & action->flags) {
      fprintf(stdout, "%s mark", sep);
      sep = ",";
    }
    if (0x4000 & action->flags) {
      fprintf(stdout, "%s linger", sep);
      sep = ",";
    }
    if (0x3FFF & action->flags) {
      fprintf(stdout, "%s 0x%4.4X", sep, 0x3FFF & action->flags);
      sep = ",";
    }
    if (0xFFFF == action->markIndex) {
      fprintf(stdout, " none");
    } else {
      fprintf(stdout, " %s_t%d", subnam, action->markIndex);
    }
    if (0xFFFF == action->currentIndex) {
      fprintf(stdout, " none");
    } else {
      fprintf(stdout, " %s_t%d", subnam, action->currentIndex);
    }
    break;
  case MORX_LIGATURE:
    if (MORX_LIGATURE_PUSH & action->flags) {
      fprintf(stdout, "%s push", sep);
      sep=",";
    }
    if (MORX_LIGATURE_LINGER & action->flags) {
      fprintf(stdout, "%s linger", sep);
      sep=",";
    }
/* The page, as on 19 February 2015,
https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6morx.html
is wrong to say that 0x3FFF are reserved! */
    if (0x1FFF & action->flags) {
      fprintf(stdout, "%s 0x%4.4X", sep, 0x3FFF & action->flags);
      sep = ",";
    }
    if (MORX_LIGATURE_PERFORM & action->flags) {
      fprintf(stdout, "%s perform lai%d", sep, action->ligActionIndex);
      sep=",";
    }
    break;
  default:
    if (action->flags) fprintf(stdout, " 0x%4.4X", action->flags);
    fprintf(stdout, " ?" "?");
  }
  fputc('\n', stdout);
  adecl[ia] = 'Y';
}

/* Get and display AAT morx table */
void display_morx(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *morx_table;
  int pos, junk;
  struct morx table;
  struct morx2_chain *chain;
  struct morx3_chain *ch3;
  struct metamorphosis_table *mt; /* The subtable */
  int ic, it;
  const char *chname;

  for (morx_table = top.table_directory; morx_table;
       morx_table = morx_table->_next) {
    if (!strncmp(morx_table->tag, "morx", 4)) break;
  }
  if (morx_table) {
    pos = morx_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking morx table");
      exit(1);
    }
    memset(&table, 0, sizeof(table));
    get_bine_morx(fp, &table, &junk, &pos);
    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
      print_morx(stdout, &table, &junk, "morx", 0);
    }
    if (2 == table.version || 3 == table.version) {
      switch (table.version) {
      case 2:
	chname = "chains2";
	chain = table.chains2;
	break;
      case 3:
	chname = "chains3";
	ch3 = table.chains3;
	chain = &ch3->c;
	break;
      default:;
      }
      for (ic = 0; ic < table.nChains; ic++) {
	if (table.nChains > 1) {
	  fprintf(stdout, " 2 [%d] again:\n 3 metamorphosis again:\n", ic);
	}
	for (it = 0, mt = chain->metamorphosis; mt; it++, mt=mt->_next) {
#define mtu(n) (mt->data + (n - mt->header_size)) /* Relative to header */
	  const char *stname;
          struct STXHeader *stable;
	  char *mp;
	  int mpos = 0;
          int trust_actions;

	  AAT_TABLE = mt->coverage & 0xFF;
	  switch (AAT_TABLE) {
	  case MORX_INDIC:
	    stname = "indic";
	    stable = &mt->indic;
            trust_actions = 1;
	    break;
	  case MORX_CONTEXTUAL:
	    stname = "context";
	    stable = &mt->context;
            trust_actions = 1;
	    break;
	  case MORX_LIGATURE:
	    stname = "ligature";
	    stable = &mt->ligature;
            trust_actions = 1;
	    break;
	  default:
	    stname = 0;
	    stable = 0;
	  }
	  if (stname) {
	    struct lookup_2 classes;
	    struct state_array transitions;
	    int max_state, max_action;
            int n_states, n_actions;
	    struct actions actions;
	    char subnam[16];
/* Whether compiler form has declared states and actions */
	    char *sdecl, *adecl;
            int maxsub = -1;     /* Max substitution table index */
	    int ligactmax  = -1; /* Max index for ligature actions in tables */
	    int ligactmax2 = -1; /* Max index allowing for run on */
	    char *ligisttgt = NULL;
            int caplig = 0;
	    struct UL_items ligature_actions;

	    memset(&ligature_actions, 0, sizeof ligature_actions);
	    N_AAT_CLASSES = stable->nClasses;
	    fprintf(stdout, " 4 [%d]:\n", it);
#if 0
	    fprintf(stdout, " 5 %s:\n", stname);
#else
            print_STXHeader(stdout, stable, &junk, stname, 5); 
#endif
	    memset(&classes, 0, sizeof classes);
	    mp = mtu(MORX_COMLEN + stable->classTableOffset);
	    get_meme_lookup_2(&mp, &classes, &junk, &mpos);
	    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
	      print_lookup_2(stdout, &classes, &junk, "classTable", 6);
	    }
	    if (instr & COMPILER) {
	      int cnext = 1;

	      sprintf(subnam, "c%ds%d", ic, it);
	      fprintf(stdout, "classification cls_%s\n", subnam);
	      fprintf(stdout, "    empty %s_0\n", subnam);
/* OTL class numbers are 3 less than AAT user class numbers. */
	      if (8 == classes.format) {
		int ig, glyph, aclass, wclass, icl;

		for (ig = 0; ig < classes.glyphCount; ig++) {
		  glyph = ig + classes.firstGlyph;
		  aclass = classes.valueArray[ig];
		  switch (aclass) {
		  case 1: /* 'out of bounds' - default class */
		    break;
		  case 0: /* Non-glyph classes */
		  case 2:
		  case 3:
		    fprintf(stdout, "Glyph %d (%s) erroneously assigned to "
			    "Apple class %d.\n",
			    glyph, glyphname1(instr, glyph), aclass);
		    break;
		  default:
		    wclass = aclass - 3;
		    if (cnext <= wclass) {
		      for (icl = cnext; icl < wclass; icl++) {
			fprintf(stdout, "    empty %s_%d\n", subnam, icl);
		      }
		      cnext = wclass + 1;
		    }
		    if (instr & POSTNAME) {
		      fprintf(stdout, "    class %s_%d %s -- %d\n", subnam,
			      wclass, glyphname1(instr, glyph), glyph);
		    } else {
		      fprintf(stdout, "    class %s_%d %s\n", subnam,
			      wclass, glyphname1(instr, glyph));
		    }
		  }
		}
	      } else {
		fprintf(stderr, "Only lookup type 8 is supported!\n");
	      }
	      fprintf(stdout, "end classification\n");
	    }
            if (0) {
	      fprintf(stdout, "header_size = %d, classTableOffset = %d "
		      "rest = %2.2x %2.2x %2.2x %2.2x...\n",
		      mt->header_size, stable->classTableOffset,
		      mt->data[0], mt->data[1], mt->data[2], mt->data[3]);
	    }
            N_AAT_STATES = n_states = 0;
            max_state = 1; /* Edge of text and not edge of text */
            N_AAT_ACTIONS = n_actions = 0;
            max_action = -1;
	    memset(&transitions, 0, sizeof transitions);
	    memset(&actions, 0, sizeof actions);

            while (max_state >= n_states | max_action >= n_actions) {
	      if (max_state >= n_states) {
		struct response_array *action;
		int ia;

	    /* Ploddingly reload from start each time */
		free_state_array(&transitions);
                N_AAT_STATES = max_state + 1;
		mp = mtu(MORX_COMLEN + stable->stateArrayOffset);
		get_meme_state_array(&mp, &transitions, &junk, &mpos);
		for (action = element(transitions.responses, n_states);
		     action; action = action->_next) {
		  for (ia = 0; ia < N_AAT_CLASSES; ia++) {
		    if (action->action[ia] > max_action) {
		      max_action = action->action[ia];
		    }
		  }
		}
		n_states = N_AAT_STATES;
	      }
	      if (max_action >= n_actions) {
		struct action *action;

	    /* Ploddingly reload from start each time */
		free_actions(&actions);
		N_AAT_ACTIONS = max_action + 1;
		mp = mtu(MORX_COMLEN + stable->entryTableOffset);
		get_meme_actions(&mp, &actions, &junk, &mpos);
		for (action = element(actions.action, n_actions);
		     action; action = action->_next) {
		  if (trust_actions) {
		    if (action->newState > max_state) {
		      max_state = action->newState;
		    }
		  }
		  switch (AAT_TABLE) {
		  case MORX_INDIC:
		    break;
		  case MORX_CONTEXTUAL:
		    if (action->markIndex != 0xFFFF) {
		      if (action->markIndex > maxsub) {
			maxsub = action->markIndex;
		      }
		    }
		    if (action->currentIndex != 0xFFFF) {
		      if (action->currentIndex > maxsub) {
			maxsub = action->currentIndex;
		      }
		    }
		    break;
		  case MORX_LIGATURE:
		    if (action->flags & MORX_LIGATURE_PERFORM) {
		      if (action->ligActionIndex > ligactmax) {
			ligactmax = action->ligActionIndex;
			if (ligactmax >= caplig) {
			  int newcap, k;
			  newcap = ligactmax + 4 + caplig / 2;
			  ligisttgt = realloc(ligisttgt, newcap);
			  for (k = caplig; k < newcap; k++) ligisttgt[k] = 0;
			  caplig = newcap;
			}
		      }
		      ligisttgt[action->ligActionIndex] = 1;
		    }
		    break; 
		  default:;
		  }
		}
		n_actions = N_AAT_ACTIONS;
	      }
	    }
	    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
	      print_state_array(stdout, &transitions, &junk, "stateArray", 6);
	    }
	    if (MORX_LIGATURE == AAT_TABLE) {
	      int k; /* Stack size is reported to be 16, so use that as limit */
	      mp = mtu(MORX_COMLEN + mt->ligActionOffset);
	      for (k = 0; k < 16; k++) {
		free_UL_items(&ligature_actions);
		ligactmax2 = ligactmax + k;
                N_ITEMS = ligactmax2 + 1;
		mp = mtu(MORX_COMLEN + mt->ligActionOffset);
		get_meme_UL_items(&mp, &ligature_actions, &junk, &mpos);
		if (ligature_actions.val[ligactmax2] & MORX_LIGATURE_LAST) {
		  break;
		}
	      }
	      if (instr & COMPILER) {
		int k, n;
		struct UW_items fish;

		memset(&fish, 0, sizeof fish);
/* Desperate attempts to examine the data */
		fprintf(stdout, "summands\n");
		N_ITEMS = (mt->ligatureOffset - mt->componentOffset) / 2;
		if (N_ITEMS > 0) {
		  mp = mtu(MORX_COMLEN + mt->componentOffset);
		  get_meme_UW_items(&mp, &fish, &junk, &mpos);
		  fprintf(stdout, "    comp0: %3d\n", fish.val[0]);
		  for (k = 1; k < N_ITEMS; k++) {
		    fprintf(stdout, "           %3d\n", fish.val[k]);
		  }
		  free_UW_items(&fish);
		  fprintf(stdout, "end summands\n");
		}
		N_ITEMS = (mt->length - MORX_COMLEN - mt->ligatureOffset) / 2;
		mp = mtu(MORX_COMLEN + mt->ligatureOffset);
		get_meme_UW_items(&mp, &fish, &junk, &mpos);
		fprintf(stdout, "ligatures\n");
		for (k = 0; k < N_ITEMS; k++) {
		  fprintf(stdout, "    %s -- at %d.\n",
			  glyphname1(instr,fish.val[k]), k);
		}
		fprintf(stdout, "end ligatures\n");
		free_UW_items(&fish);
		fprintf(stdout, "ligature actions\n");
		for (k = 0; k <= ligactmax2; k++) {
		  int sval;
		  unsigned int uval;

		  if (k <= ligactmax && ligisttgt[k]) {
		    fprintf(stdout, "lai%d: ", k);
		  } else {
		    fprintf(stdout, "      ");
		  }
		  if (MORX_LIGATURE_LAST & ligature_actions.val[k]) {
		    fprintf(stdout, "last, ");
		  }
		  if (MORX_LIGATURE_STORE & ligature_actions.val[k]) {
		    fprintf(stdout, "store, ");
		  }
		  uval = ligature_actions.val[k];
		  uval &=~ MORX_LIGATURE_LAST;
		  uval &=~ MORX_LIGATURE_STORE;
		  sval = ((int)(uval << 2)) >> 2;
		  if (sval > 0) {
		    fprintf(stdout, "\"%s\" at comp%d 0x%x\n",
			    glyphname1(instr, 0), sval,
			    ligature_actions.val[k]);
		  } else {
		    fprintf(stdout, "%s at comp0 -- %d\n",
			    glyphname1(instr, -sval), -sval);
		  }
		}
		fprintf(stdout, "end actions\n");
	      }
	    }
	    if (instr & COMPILER) {
	      const char *fixclass[4] =
		{"EOT    ", "OOB    ", "DEL    ", "EOL    "};
	      int ist, icl, iadef, actno;
              const char *clnam;
              char clbuf[32];
	      struct response_array *resp, *resp0, *resp1;

	      sdecl = calloc(1, N_AAT_STATES);
	      adecl = calloc(1, N_AAT_ACTIONS);
	      assert(N_AAT_STATES >= 2);
	      if (AAT_TABLE == MORX_LIGATURE) {
		iadef = 1;
	      } else {
		iadef = 0;
	      }
	      actno = iadef;
	      if (!adecl[actno]) {
		put_action(element(actions.action, actno), actno, subnam,
			   adecl, sdecl);
	      }
	      fprintf(stdout, "default a%d\n", iadef);
	      resp0 = transitions.responses;
              resp1 = resp0->_next;
	      for (icl = 0; icl < N_AAT_CLASSES; icl++) {
		if (icl < 4) {
		  clnam = fixclass[icl];
		} else {
		  sprintf(clbuf, "%s_%d", subnam, icl - 3);
		  clnam = clbuf;
		}
		if ((actno = resp0->action[icl]) == resp1->action[icl]) {
		  if (actno != iadef) {
		    if (!adecl[actno]) {
		      put_action(element(actions.action, actno), actno, subnam,
				 adecl, sdecl);
		    }
		    fprintf(stdout, "state SOTL class %s > a%d\n",
			    clnam, actno);
		  }
		} else {
		  if (actno != iadef) {
		    if (!adecl[actno]) {
		      put_action(element(actions.action, actno), actno, subnam,
				 adecl, sdecl);
		    }
		    fprintf(stdout, "state SOT  class %s > a%d\n",
			    clnam, actno);
		  }
		  actno = resp1->action[icl];
		  if (actno != iadef) {
		    if (!adecl[actno]) {
		      put_action(element(actions.action, actno), actno, subnam,
				 adecl, sdecl);
		    }
		    fprintf(stdout, "state SOL  class %s > a%d\n",
			    clnam, actno);
		  }
		}
	      }
	      for (resp = resp1->_next, ist = 2; resp;
		   resp = resp->_next, ist++) {
		for (icl = 0; icl < N_AAT_CLASSES; icl++) {
		  if (icl < 4) {
		    clnam = fixclass[icl];
		  } else {
		    sprintf(clbuf, "%s_%d", subnam, icl - 3);
		    clnam = clbuf;
		  }
		  if ((actno = resp->action[icl]) != iadef) {
		    fputs(sdecl[ist] ? "   " : "new", stdout);
		    sdecl[ist] = 'Y';
		    if (0 || adecl[actno]) {
		      fprintf(stdout, "state s%-3.1d class %s > a%d\n",
			      ist, clnam, actno);
		    } else {
		      fprintf(stdout, "state s%-3.1d class %s > ", ist, clnam);
		      put_action(element(actions.action, actno), actno, subnam,
				 adecl, sdecl);
		    }
		  }
		}
	      }
	    }
	    if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
	      print_actions(stdout, &actions, &junk, "entryTable", 6);
	    }
	    if (instr & COMPILER) {
	      struct action *action;
	      int ia;

	      for (action = actions.action, ia = 0; action;
		   action = action->_next, ia++) {
		if (!adecl[ia]) put_action(action, ia, subnam, adecl, sdecl);
		adecl[ia] = 'Y';
	      }
	      free(sdecl);
	      free(adecl);
	    }
/* Read in and report table specific supplements */
	    switch (AAT_TABLE) {
	    case MORX_INDIC:
	      break;
	    case MORX_CONTEXTUAL:
	      { int isub, subpos, posoff;
		struct UL_item uli;
		char label[16];
		struct lookup_2 substab;

		memset(&substab, 0, sizeof substab);
		if (maxsub >= 0) fprintf(stdout, " 5 substitutionTables:\n");
		for (isub = 0; isub <= maxsub; isub++) {
		  sprintf(label, "[%d]", isub);
		  mp = mtu(MORX_COMLEN + mt->substitutionTable + 4 * isub);
		  get_meme_UL_item(&mp, &uli, &junk, &mpos);
		  if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
		    print_UL_item(stdout, &uli, &junk, label, 6);
		  }
		  mp = mtu(MORX_COMLEN + mt->substitutionTable + uli.val);
		  get_meme_lookup_2(&mp, &substab, &junk, &mpos);
		  if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
		    print_lookup_2(stdout, &substab, &junk, "table", 7);
		  }
		  if (instr & COMPILER) {
		    fprintf(stdout, "substitution %s_t%d\n", subnam, isub);
		    if (8 == substab.format) {
		      int ig, glyph;

		      for (ig = 0; ig < substab.glyphCount; ig++) {
			/* Apparent changes to zero are ignored! */
			if (!substab.valueArray[ig]) continue;
			glyph = ig + substab.firstGlyph;
			fprintf(stdout, "    %s > %s\n",
				glyphname1(instr, glyph),
				glyphname2(instr, substab.valueArray[ig]));
		      }
		    } else {
		      fprintf(stderr, "Only lookup type 8 is supported!\n");
		    }
		    fprintf(stdout, "end substitution\n");
		  }
		  free_lookup_2(&substab);
		}
	      }
	      break;
	    case MORX_LIGATURE:
	      if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
		struct UW_items fish;
		print_UL_items(stdout, &ligature_actions, &junk,
			       "ligatureActions", 6);
/* Desperate attempts to examine the data */
		memset(&fish, 0, sizeof fish);

		N_ITEMS = (mt->ligatureOffset - mt->componentOffset) / 2;
		mp = mtu(MORX_COMLEN + mt->componentOffset);
		get_meme_UW_items(&mp, &fish, &junk, &mpos);
		print_UW_items(stdout, &fish, &junk, "components", 6);
		free_UW_items(&fish);

		N_ITEMS = (mt->length - MORX_COMLEN - mt->ligatureOffset) / 2;
		mp = mtu(MORX_COMLEN + mt->ligatureOffset);
		get_meme_UW_items(&mp, &fish, &junk, &mpos);
		print_UW_items(stdout, &fish, &junk, "ligatures", 6);
		free_UW_items(&fish);
	      }
	      break;
	    default:;
	    }
	    free(ligisttgt);
	    free_UL_items(&ligature_actions);
	  }
	}
	switch (table.version) {
	case 2:
	  chain = chain->_next;
	  break;
	case 3:
	  ch3 = ch3->_next;
	  chain = ch3 ? &ch3->c : 0;
	  break;
	default:;
	}
      }
    }
  }
}

/* Get and display AAT feat table */
void display_feat(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *feat_table;
  int pos, junk;
  struct feat table;
  const int fsd_size = 4;
  struct setting_name fsd;
  int nfsd, ifsd;
  char cfsd[20];

  for (feat_table = top.table_directory; feat_table;
       feat_table = feat_table->_next) {
    if (!strncmp(feat_table->tag, "feat", 4)) break;
  }
  if (feat_table) {
    pos = feat_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking feat table");
      exit(1);
    }
    memset(&table, 0, sizeof(table));
    get_bine_feat(fp, &table, &junk, &pos);
    if (instr & DISPLAY_RAW) {
      print_feat(stdout, &table, &junk, "feat", 0);
      nfsd = (feat_table->length - (pos - feat_table->offset)) / fsd_size;
      for (ifsd = 0; ifsd < nfsd; ifsd++) {
	get_bine_setting_name(fp, &fsd, &junk, &pos);
	sprintf(cfsd, "[%d]", ifsd);
	print_setting_name(stdout, &fsd, &junk, cfsd, 2);
      }
    }
  }
}

#define NFIX 391
static const char *standard_string[NFIX] = {
  ".notdef",     "space",      "exclam",       "quotedbl",    "numbersign",
  "dollar",      "percent",    "ampersand",    "quoteright",  "parenleft",
  "parenright",  "asterisk",   "plus",         "comma",       "hyphen",
  "period",      "slash",      "zero",         "one",         "two",
  "three",       "four",       "five",         "six",         "seven",
  "eight",       "nine",       "colon",        "semicolon",   "less",
  "equal",       "greater",    "question",     "at",          "A",
  "B",           "C",          "D",            "E",           "F",
  "G",           "H",          "I",            "J",           "K",
  "L",           "M",          "N",            "O",           "P",
  "G",           "R",          "S",            "T",           "U",
  "V",           "W",          "X",            "Y",           "Z",
  "bracketleft", "backslash",  "bracketright", "asciicircum", "underscore",
  "quoteleft",   "a",          "b",            "c",           "d",
  "e",           "f",          "g",            "h",           "i",
  "j",           "k",          "l",            "m",           "n",
  "o",           "p",          "q",            "r",           "s",
  "t",           "u",          "v",            "w",           "x",
  "y",           "z",          "braceleft",    "bar",         "braceright",
  "asciitilde",  "exclamdown", "cent",         "sterling",    "fraction",
  "yen",         "florin",     "section",      "currency",    "quotesingle",
  "quotedblleft", "guillemotleft", "guilsinglleft", "guilsinglright", "fi",
  "fl",           "endash",    "dagger",       "daggerdbl",   "periodcentered",
  "paragraph",   "bullet",  "quotesinglbase", "quotedblbase", "quotedblright",
  "guillemotright", "ellipsis", "perthousand", "questiondown", "grave",
  "acute",       "circumflex", "tilde",        "macron",      "breve",
  "dotaccent",   "dieresis",   "ring",         "cedilla",     "hungarumlaut",
  "ogonek",      "caron",      "emdash",       "AE",          "ordfeminine",
  "Lslash",      "Oslash",     "OE",           "ordmasculine", "ae",
  "dotlessi",    "lslash",     "oslash",       "oe",          "germandbls",
  "onesuperior", "logicalnot", "mu",           "trademark",   "Eth",
  "onehalf",     "plusminus",  "Thorn",        "onequarter",  "divide",
  "brokenbar",   "degree",     "thorn",        "threequarters", "twosuperior",
  "registered",  "minus",      "eth",          "multiply",    "threesuperior",
  "copyright",   "Aacute",     "Acircumflex",  "Adieresis",   "Agrave",
  "Aring",       "Atilde",     "Ccedilla",     "Eacute",      "Ecircumflex",
  "Edieresis",   "Egrave",     "Iacute",       "Icicumflex",  "Idieresis",
  "Igrave",      "Ntilde",     "Oacute",       "Ocircumflex", "Odieresis",
  "Ograve",      "Otilde",     "Scaron",       "Uacute",      "Ucircumflex",
  "Udieresis",   "Ugrave",     "Yacute",       "Ydieresis",   "Zcaron",
  "aacute",      "acircumflex", "adieresis",   "agrave",      "aring",
  "atilde",      "ccedilla",   "eacute",       "ecircumflex", "edieresis",
  "egrave",      "iacute",     "icircumflex",  "idieresis",   "igrave",
  "ntilde",      "oacute",     "ocircumflex",  "odieresis",   "ograve",
  "otilde",      "scaron",     "uacute",       "ucircumflex", "udieresis",
  "ugrave",      "yacute",     "ydieresis",    "zcaron",      "exclamsmall",
  "Hungarumlautsmall", "dollaroldstyle", "dollarsuperior", "ampersandsmall",
                                                           "Acutesmall",
  "parenleftsuperior", "parenrightsuperior", "twodotenleader",
                                       "onedotenleader", "zerooldstyle",
  "oneoldstyle", "twooldstyle", "threeoldstyle", "fouroldstyle", "fiveoldstyle",
  "sixoldstyle", "sevenoldstyle", "eightoldstyle", "nineoldstyle",
                                                             "commasuperior",
  "threequartersemdash", "periodsuperior", "questionsmall",
                                                 "asuperior", "bsuperior",
  "centsuperior", "dsuperior", "esuperior",    "isuperior",   "Isuperior",
  "msuperior",    "nsuperior", "osuperior",    "rsuperior",   "ssuperior",
  "tsuperior",    "ff",        "ffi",          "ffl",      "parenleftinferior",
  "parenrightinferior", "Circumflexsmall", "hyphensuperior", "Gravesmall",
                                                              "Asmall",
  "Bsmall",       "Csmall",    "Dsmall",       "Esmall",      "Fsmall",
  "Gsmall",       "Hsmall",    "Ismall",       "Jsmall",      "Ksmall",
  "Lsmall",       "Msmall",    "Nsmall",       "Osmall",      "Psmall",
  "Qsmall",       "Rsmall",    "Ssmall",       "Tsmall",      "Usmall",
  "Vsmall",       "Wsmall",    "Xsmall",       "Ysmall",      "Zsmall",
  "colonmonetary", "onefitted", "rupiah",      "Tildesmall", "exclamdownsmall",
  "centoldstyle", "Lslashsmall", "Scaronsmall", "Zcaronsmall", "Dieresissmall",
  "Brevesmall",   "Caronsmall", "Dotaccentsmall", "Macronsmall", "figuredash",
  "hypheninferior", "Ogoneksmall", "Ringsmall", "Cedillasmall",
                                                        "questindownsmall",
  "oneeighth",    "threeeights", "fiveeighths", "seveneighths", "onethird",
  "twothirds",   "zerosuperior", "foursuperior", "fivesuperior", "sixsuperior",
  "sevensuperior", "eightsuperior", "ninesuperior", "zeroinferior",
                                                                "oneinferior",
  "twoinferior", "threeinferior", "fourinferior", "fiveinferior", "sixinferior",
  "seveninferior", "eightinferior", "nineinferior", "centinferior",
                                                             "dollarinferior",
  "periodinferior", "commainferior", "Agravesmall", "Aacutesmall",
                                                          "Acircumflexsmall",
  "Atildesmall", "Adieresissmall", "Aringsmall", "AEsmall", "Ccedillasmall",
  "Egravesmall", "Eacutesmall", "Ecircumflexsmall", "Edieresissmall",
                                                            "Igravesmall",
  "Iacutesmall", "Icircumflexsmall", "Idieresissmall", "Ethsmall",
                                                             "Ntildesmall",
  "Ogravesmall", "Oacutesmall", "Ocircumflexsmall", "Otildesmall",
                                                          "Odieresissmall",
  "OEsmall", "Oslashsmall", "Ugravesmall", "Uacutesmall", "Ucircumflexsmall",
  "Udieresissmall", "Yacutesmall", "Thornsmall", "Ydieresissmall", "001.000",
  "001.001",      "001.002",   "001.003",      "Black",       "Bold",
  "Book",         "Light",     "Medium",       "Regular",     "Roman",
  "Semibold"};

struct map top_dict, private_dict;
static struct charset charset;
static int have_charset;
static struct cff_index strings;

static void get_cff_name(int gid, const char **pname, int *plen) {
  int strno;

  *plen = 0;
  if (!have_charset) {
    *pname = 0; /* No CFF data! */
  } else if (gid < 0) {
    *pname = 0;
  } else if (0 == gid) {
    *pname = ".notdef";
    *plen  = 7;
  } else if (gid >= N_GLYPHS) {
    *pname = 0;
  } else {
    /* Lookup in charset */
    if (0 == charset.format) {
      strno = charset.glyph[gid - 1];
    } else { /* Unsupported format */
      strno = -1;
    }
    if (-1 == strno) {
      static int said = 0;
      if (!said) {
        fprintf(stderr, "Charset format %d is not yet supported.\n",
                charset.format);
      said = 1;
      }
      *pname = 0;
    } else if (strno < NFIX) {
      *pname = standard_string[strno];
      *plen = strlen(*pname);
    } else {
      strno -= NFIX;
      if (strno >= strings.count) {
      fprintf(stderr, "Charset contains invalid string number %d "
              "for glyph %d.\n", strno + NFIX, gid);
      *pname = 0;
      } else {
    /* Now get from appropriate string list */
      struct cff_offset *start_el, *beyond_el;
      int start, beyond;

      start_el = element(strings.offset, strno);
      beyond_el = start_el->_next;

      start =  (start_el->x_hi  << 16) | start_el->x_lo;
      beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
      *pname = (char *) &strings.data[start - 1];
      *plen  = beyond - start;
      }
    }
  }
}

static void ps_put(FILE *psf, double x) {
  int ival = (int) x;
  if (x == ival) {
    fprintf(psf, " %d", ival);
  } else {
    fprintf(psf, " %.5f", x);
  }
}

/* Roughly *t*abular form of above */
static void ps_putt(FILE *psf, double x) {
  int ival = (int) x;
  if (x == ival) {
    fprintf(psf, " %4d", ival);
  } else {
    fprintf(psf, " %.5f", x);
  }
}

void do_dict(unsigned char *data, int length, int is_top, int cmd) {
  int k, value, kk;
  unsigned char obi;
#define MAX_STACK 100
  double stack[MAX_STACK], *val;
  int n_on_stack, show;
  struct map *dict;

#define push(x)								\
  if (n_on_stack < MAX_STACK) {						\
    stack[n_on_stack++] = x;						\
  } else {								\
    fprintf(stderr, "Stack overflow at offset %d in dictionary.\n",	\
	    n_on_stack);						\
    abort();								\
  }

#if 0
  fprintf(stdout, "-- Processing dictionary\n-- ");
  for (k = 0; k < length; k++) {
    fprintf(stdout, " %2.2x", data[k]);
  }
  fputc('\n', stdout);
#endif

  dict = is_top ? &top_dict : &private_dict;
  show = cmd & (DISPLAY_RAW | DISPLAY_RES);
  n_on_stack = 0;
  for (k = 0; k < length; k++) {
    obi = data[k];
    if (32 <= obi && obi <= 246) {
      value = obi - 139;
      if (show) fprintf(stdout, " %d", value);
      push(value);
    } else if (247 <= obi && obi <= 250) {
      value = (obi - 247) * 256 + 108;
      value += data[++k];
      if (show) fprintf(stdout, " %d", value);
      push(value);
    } else if (251 <= obi && obi <= 254) {
      value = (obi - 251) * 256 + 108;
      value += data[++k];
      if (show) fprintf(stdout, " -%d", value);
      push(-value);
    } else if (28 == obi) {
      value = 256 * data[++k];
      value += data[++k];
      value = (signed short int) value;
      if (show) fprintf(stdout, " %d", value);
      push(value);
    } else if (29 == obi) {
      value = data[++k] << 24;
      value += data[++k] << 16;
      value += data[++k] << 8;
      value += data[++k];
      if (show) fprintf(stdout, " %d", value);
      push(value);
    } else if (30 == obi) {
#define MAX_FLEN 20
      char flit[MAX_FLEN + 1], onch;
      int nchar;

      flit[0] = 0;
      nchar = 0;
      if (show) fputc(' ', stdout);
      do {
	int j;
	int nib[2];

	obi = data[++k];
	nib[0] = (obi >> 4) & 0x0f;
	nib[1] = obi & 0x0f;
	for (j = 0; j < 2; j++) {
	  if (nib[j] <= 9) {
	    if (show) fprintf(stdout, "%d", nib[j]);
	    onch = 0x30 | nib[j];
	  } else {
	    onch = 0;
	    switch (nib[j]) {
	    case 10:
	      onch = '.';
	      if (show) fputc(onch, stdout);
	      break;
	    case 11:
	      onch = 'E';
	      if (show) fputc(onch, stdout);
	      break;
	    case 12:
	      if (show) fputs("E-", stdout);
	      break;
	    case 13:
	      onch = '?';
	      if (show) fputc(onch, stdout);
	      break;
	    case 14:
	      onch = '-';
	      if (show) fputc(onch, stdout);
	      break;
	    case 15:
	      /* Do nothing */
	      break;
	    }
	  }
	  if (12 == nib[j]) {
	    if (nchar < MAX_FLEN - 1) {
	      flit[nchar++] = 'E';
	      flit[nchar++] = '-';
	    } else {
	      fprintf(stderr, "Real number string overflows at offset %d "
		      "in dictionary - %s,\n", k, flit);
	      abort();
	    }
	  } else if (onch) {
	    if (nchar < MAX_FLEN) {
	      flit[nchar++] = onch;
	    } else {
	      fprintf(stderr, "Real number string overflows at offset %d "
		      "in dictionary - %s,\n", k, flit);
	      abort();
	    }
	  }
	}
      } while ((obi & 0x0f) != 0x0f);
      push(atof(flit)); /* Should use strtod() and check */
    } else if (0 <= obi && obi <= 21) {
      enum {SID, NUMBER, NUMBERS, INT2, A4, A6, BOOLEAN, DELTA, FAIL} vtype;
      const char *name;
      if (is_top) { /* Top dict */
	switch (obi) {
#define dict_op " /%s exch def\n"
	case 0:
	  vtype = SID;
	  name = "version";
	  break;
	case 1:
	  name = "Notice";
	  vtype = SID;
	  break;
	case 2:
	  name = "FullName";
	  vtype = SID;
	  break;
	case 3:
	  name = "FamilyName";
	  vtype = SID;
	  break;
	case 4:
	  name = "Weight";
	  vtype = SID;
	  break;
	case 5:
	  name = "FontBBox";
	  vtype = A4;
	  break;
	case 12:
	  obi = data[++k];
	  switch (obi) {
	  case 0:
	    name = "Copyright";
	    vtype = SID;
	    break;
	  case 1:
	    name = "isFixedPitch";
	    vtype = BOOLEAN;
	    break;
	  case 2:
	    name = "ItalicAngle";
	    vtype = NUMBER;
	    break;
	  case 3:
	    name = "UnderlinePosition";
	    vtype = NUMBER;
	    break;
	  case 4:
	    name = "UnderlineThickness";
	    vtype = NUMBER;
	    break;
	  case 5:
	    name = "PaintType";
	    vtype = NUMBER;
	    break;
	  case 6:
	    name = "CharstringType";
	    vtype = NUMBER;
	    break;
	  case 7:
	    name = "Font Matrix";
	    vtype = A6;
	    break;
	  case 8:
	    name = "StrokeWidth";
	    vtype = NUMBER;
	    break;
	  case 20:
	    name = "SyntheticBase";
	    vtype = NUMBER;
	    break;
	  case 21:
	    name = "Postscript";
	    vtype = SID;
	    break;
	  case 22:
	    name = "BaseFontName";
	    vtype = SID;
	    break;
	  case 23:
	    name = "BaseFontBlend";
	    vtype = DELTA;
	    break;
	  default:
	    fprintf(stderr, "-- Dictionary operator 12, %d not recognised.\n",
		    obi);
	    vtype = FAIL;
	    break;
	  }
	  break;
	case 13:
	  name = "UniqueID";
	  vtype = NUMBER;
	  break;
	case 14:
	  name = "XUID";
	  vtype = NUMBERS;
	  break;
	case 15:
	  name = "charset";
	  vtype = NUMBER;
	  break;
	case 16:
	  name = "Encoding";
	  vtype = NUMBER;
	  break;
	case 17:
	  name = "CharStrings";
	  vtype = NUMBER;
	  break;
	case 18:
	  name = "Private";
	  vtype = INT2;
	  break;
	default:
	  fprintf(stderr, "-- Dictionary operator %d not recognised.\n", obi);
	  vtype = FAIL;
	  break;
	}
      } else {
	/* Private dict goes here */
	switch (obi) {
        case 6:
          name = "BlueValues";
          vtype = DELTA;
          break;
        case 7:
          name = "OtherBlues";
          vtype = DELTA;
          break;
        case 8:
          name = "FamilyBlues";
          vtype = DELTA;
          break;
        case 9:
          name = "FamilyOtherBlues";
          vtype = DELTA;
          break;
	case 10:
	  name = "StdHW";
	  vtype = NUMBER;
	  break;
	case 11:
	  name = "StdVW";
	  vtype = NUMBER;
	  break;
        case 12:
	  obi = data[++k];
	  switch (obi) {
          case 9:
            name = "BlueScale";
            vtype = NUMBER;
            break;
          case 10:
            name = "BlueShift";
            vtype = NUMBER;
            break;
          case 11:
            name = "BlueFuzz";
            vtype = NUMBER;
            break;
          case 12:
            name = "StemSnapH";
            vtype = DELTA;
            break;
          case 13:
            name = "StemSnapV";
            vtype = DELTA;
            break;
          case 14:
            name = "ForceBold";
            vtype = BOOLEAN;
            break;
          case 17:
            name = "LanguageGroup";
            vtype = NUMBER;
            break;
          case 18:
            name = "ExpansionFactor";
            vtype = NUMBER;
            break;
          case 19:
            name = "InitialRandomSeed";
            vtype = NUMBER;
            break;
	  default:
	    fprintf(stderr, "-- Dictionary operator 12, %d not recognised.\n",
		    obi);
	    vtype = FAIL;
	    break;
	  }
          break;
        case 19:
          name = "Subrs";
          vtype = NUMBER;
          break;
        case 20:
          name = "defaultWidthX";
          vtype = NUMBER;
          break;
        case 21:
          name = "nominalWidthX";
          vtype = NUMBER;
          break;
	default:
	  fprintf(stderr, "-- Dictionary operator %d not recognised.\n", obi);
	  vtype = FAIL;
	  break;
	}
      }
	if (show && FAIL != vtype) fprintf(stdout, dict_op, name);
	/*      enum {SID, NUMBER, NUMBERS, INT2, A4, A6, BOOLEAN, DELTA, FAIL} vtype; */
	switch (vtype) {
	case SID:
	case NUMBER:
	  if (1 != n_on_stack) {
	    fprintf(stderr, "-- Expected one number on stack for %s "
		    "at offset %d, but had %d numbers.  Definition ignored.\n",
		    name, k, n_on_stack);
	  } else {
	    if (!(val = malloc(sizeof(double)))) abort();
	    *val = stack[0]; 
	    replace_in_map(dict, name, val, 1);
            if (cmd & COMPILER) {
              if (!strcmp("charset", name)) {
                /* Will be calculated automatically */
              } else if (!strcmp("Encoding", name)) {
                switch ((int) val) {
                case 0:
                  fputs("-- Encoding standard\n", stdout);
                  break;
                case 1:
                  fputs("-- Encoding expert\n", stdout);
                default:
                  fputs("-- Encoding custom -- Not implemented!\n", stdout);
                }
              } else if (!strcmp("CharStrings", name)) {
                /* Will be calculated automatically */
              } else if (!strcmp("Private", name)) {
                /* Will be calculated automatically */
              } else if (!strcmp("Subrs", name)) {
                /* Will be calculated automatically */
              } else if (SID == vtype) {
                struct cff_offset *start_el, *beyond_el;
                int start, beyond;
                int strno = (int) stack[0];

                if (strno < NFIX) {
                  fprintf(stdout, "%s \"%s\"\n", name, standard_string[strno]);
                } else {
                  strno -= NFIX;
                  start_el = element(strings.offset, strno);
                  beyond_el = start_el->_next;

                  start =  (start_el->x_hi  << 16) | start_el->x_lo;
                  beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
                  fprintf(stdout, "%s \"%.*s\"\n", name, beyond - start,
                          (char *) &strings.data[start - 1]);
                }
              } else {
                fputs(name, stdout);
                ps_put(stdout, stack[0]);
                fputc('\n', stdout);
              }
            }
	  }
	  break;
	case BOOLEAN:
	  if (1 != n_on_stack) {
	    fprintf(stderr, "Expected one number on stack for %s "
		    "at offset %d, but had %d numbers.  Definition ignored.\n",
		    name, k, n_on_stack);
	  } else if (0 != stack[0] && 1 != stack[0]) {
	    fprintf(stderr, "Expected 0 or 1 for %s at offset %d, "
		    "but got %e.  Definition ignored.\n", name, k, val[0]);
	  } else {
	    if (!(val = malloc(sizeof(double)))) abort();
	    *val = stack[0]; 
	    replace_in_map(dict, name, val, 1);
	  }
	  break;
	case INT2:
	  if (2 != n_on_stack) {
	    fprintf(stderr, "-- Expected two numbers on stack for %s "
		    "at offset %d, but had %d numbers.  Definition ignored.\n",
		    name, k, n_on_stack);
	  } else {
	    if (!(val = malloc(2 * sizeof(double)))) abort();
	    val[0] = stack[0]; 
	    val[1] = stack[1]; 
	    replace_in_map(dict, name, val, 1);
	  }
	  break;
	case A4:
	  if (4 != n_on_stack) {
	    fprintf(stderr, "-- Expected four numbers on stack for %s "
		    "at offset %d, but had %d numbers.  Definition ignored.\n",
		    name, k, n_on_stack);
	  } else {
	    if (!(val = malloc(4 * sizeof(double)))) abort();
	    val[0] = stack[0]; 
	    val[1] = stack[1]; 
	    val[2] = stack[2]; 
	    val[3] = stack[3]; 
	    replace_in_map(dict, name, val, 1);
	  }
	  break;
	case A6:
	  if (6 != n_on_stack) {
	    fprintf(stderr, "-- Expected six numbers on stack for %s "
		    "at offset %d, but had %d numbers.  Definition ignored.\n",
		    name, k, n_on_stack);
	  } else {
	    if (!(val = malloc(6 * sizeof(double)))) abort();
	    val[0] = stack[0]; 
	    val[1] = stack[1]; 
	    val[2] = stack[2]; 
	    val[3] = stack[3]; 
	    val[4] = stack[4]; 
	    val[5] = stack[5]; 
	    replace_in_map(dict, name, val, 1);
	  }
	  break;
	case NUMBERS:
	case DELTA:
	  if (!(val = malloc((n_on_stack + 1) * sizeof(double)))) abort();
	  for (kk = 0; kk < n_on_stack; kk++) val[kk+1] = stack[kk];
	  val[0] = n_on_stack;
	  replace_in_map(dict, name, val, 1);
	  break;
	case FAIL:
	  /* Do nothing */
	  break;
	default:
	  fprintf(stderr, "-- Unhandled switch value %d at line %d of %s.\n",
		  vtype, __LINE__, __FILE__);
	  break;
	}
      n_on_stack = 0;
    } else {
      fprintf(stderr, "-- Operand/operator code %d not recognised.\n", obi);
    }
  }
}

static struct cff_index subrs, gsubrs;
#define MAX_AUX 32 /* From Appendix B of Type 2 Charstring definition */
static double auxarr[MAX_AUX];
enum cs_state {INIT, WANT_H, WANT_V, NORMAL};
enum cs_type {CS_GLYPH, CS_LOCAL_SUBR, CS_GLOBAL_SUBR};
struct cs_id {
  unsigned short int serial;
  enum cs_type type;
};
struct cs_loc {
  double x, y;
  struct cs_id id, root;
  enum cs_state state;
};

static void put_cs(FILE *dst, FILE *psf, int instr,
                   struct cs_loc *state, int *nstem,
                   double *stack, int *nons,
                   const unsigned char *data, int length) {
  int k, m;
  int ival;
  unsigned char obi;
  char *oper;
  double rval;
  int horiz;
  const struct cs_id my_id = state->id;
  int decompile_cs = 0;

  if (INIT == state->state) {
    *nstem = 0;
    state->state = WANT_H;
    *nons = 0;
  }
#define n_on_stack (*nons)
  if (instr & COMPILER) {
    if (state->id.type == state->root.type &&
        state->id.serial == state->root.serial) {
      decompile_cs = 1;
    } else if (instr & EXPAND) { /* Decompilation is to expand inline. */
      decompile_cs = CS_GLYPH == state->root.type;
    }
  }
  for (k = 0; k < length; k++) {
    obi = data[k];
    if (32 <= obi && obi <= 246) {
      ival = obi - 139;
      if (instr & DISPLAY_RES) fprintf(dst, " %d", ival);
      push(ival);
    } else if (247 <= obi && obi <= 250) {
      ival = (obi - 247) * (int) 256 + data[++k] + 108;
      if (instr & DISPLAY_RES) fprintf(dst, " %d", ival);
      push(ival);
    } else if (251 <= obi && obi <= 254) {
      ival = (obi - 251) * (int) 256 + data[++k] + 108;
      ival = -ival;
      if (instr & DISPLAY_RES) fprintf(dst, " %d", ival);
      push(ival);
    } else {
      switch(obi) {
      case 1:
      case 18:
        oper = (1 == obi) ? "hstem" : "hstemhm";
        if (instr & DISPLAY_RES) {
          fputc(' ', dst);
          fputs(oper, dst);
        }
        switch (state->state) {
        case WANT_H:
          *nstem += (*nons)/2;
          if (0 && (instr & DISPLAY_RES)) {
            fprintf(dst, " (%d horizontal stems.)", *nstem);
          }
          if (decompile_cs) {
            int j;
#define setoff if (instr & DISPLAY_RES) fputc('\n', dst)
            setoff;
            fputs("  ", dst);
            for (j = *nons & 1; j < *nons; j += 2) {
              fputc(' ', dst);
              ps_put(dst, stack[j]);
              ps_put(dst, stack[j+1]); /* Consider translating */
            }
            fprintf(dst, " %s\n", oper);
          }
          state->state = WANT_V;
          *nons = 0;
          break;
        default:
          fprintf(stderr, "\n%s out of place.\n", oper);
          fflush(0);
          abort();
        }
        break;
      case 3:
      case 23:
        oper = (3 == obi) ? "vstem" : "vstemhm";
        if (instr & DISPLAY_RES) {
          fputc(' ', dst);
          fputs(oper, dst);
        }
        switch (state->state) {
        case WANT_H:
        case WANT_V:
          *nstem += (*nons)/2;
          state->state = NORMAL;
          if (0 && (instr & DISPLAY_RES)) {
            fprintf(dst, " (Now %d stems.)", *nstem);
          }
          if (decompile_cs) {
            int j;
            setoff;
            fputs("  ", dst);
            for (j = *nons & 1; j < *nons; j += 2) {
              fputc(' ', dst);
              ps_put(dst, stack[j]);
              ps_put(dst, stack[j+1]); /* Consider translating */
            }
            fprintf(dst, " %s\n", oper);
          }
          *nons = 0;
          break;
        default:
          fprintf(stderr, "\n%s out of place.\n", oper);
          fflush(0);
          abort();
        }
        break;
      case 4:
      case 21:
      case 22:
        {
          int nexp;
          double x, y;
          switch (obi) {
          case 4:
            oper = "vmoveto";
            nexp = 1;
            if (WANT_H == state->state && *nons > nexp) nexp++;
            x = 0;
            y = stack[nexp-1];
            break;
          case 21:
            oper = "rmoveto";
            nexp = 2;
            if (WANT_H == state->state && *nons > nexp) nexp++;
            x = stack[nexp-2];
            y = stack[nexp-1];
            break;
          case 22:
            oper = "hmoveto";
            nexp = 1;
            if (WANT_H == state->state && *nons > nexp) nexp++;
            x = stack[nexp-1];
            y = 0;
            break;
          default:
            fprintf(stderr, "Coding error at line %d of %s.\n",
                    __LINE__, __FILE__);
            exit(1);
          }
          if (instr & DRAW) {
            fputs(" currentpoint closepath moveto\n", psf); 
            ps_put(psf, x); ps_put(psf, y); fputs(" rmoveto\n", psf);
            if (*nons != nexp) {
              fprintf(stderr, "Expected %d points on stack, "
                      "but had %d points.\n", nexp, *nons);
              fprintf(psf, " %% Expected %d points on stack, "
                      "but had %d points.\n", nexp, *nons);
            }
          }
          if (instr & DISPLAY_RES) fprintf(dst, " %s", oper);
          state->x += x;
          state->y += y;
          if (0 && (instr & COMPILER)) {
              fprintf(stderr, "CS Types: %d %d\n",
                      state->root.type, state->id.type);
          }
          if (decompile_cs) {
            setoff;
            fputs("   ", dst);
            ps_putt(dst, state->x); ps_putt(dst, state->y);
            fputs(" m\n", dst);
          }
          *nons = 0;
          state->state = NORMAL;
        }
        break;
      case 5:
        if (instr & DISPLAY_RES) fputs(" rlineto", dst);
        {
          int i;
          for (i = 0; i <= *nons - 2; i += 2) {
            if (instr & DRAW) {
              ps_put(psf, stack[i]); ps_put(psf, stack[i+1]);
              fputs(" rlineto\n", psf);
            }
            state->x += stack[i];
            state->y += stack[i+1];
            if (decompile_cs) {
              if (0 == i) setoff;
              fputs("   ", dst);
              ps_putt(dst, state->x); ps_putt(dst, state->y);
              fputs(" l\n", dst);
            }
          }
        }
        *nons = 0;
        break;
      case 6:
      case 7:
        if (6 == obi) {
          oper = "hlineto";
          horiz = 1;
        } else {
          oper = "vlineto";
          horiz = 0;
        }
        if (instr & DISPLAY_RES) fprintf(dst, " %s", oper);
        {
          int i;
          for (i = 0; i < *nons; i++) {
            if (horiz) {
              if (instr & DRAW) {ps_putt(psf, stack[i]); ps_putt(psf, 0);}
              state->x += stack[i];
            } else {
              if (instr & DRAW) {ps_put(psf, 0); ps_put(psf, stack[i]);}
              state->y += stack[i];
            }
            if (instr & DRAW) fputs(" rlineto\n", psf);
            if (decompile_cs) {
              if (0 == i) setoff;
              fputs("   ", dst);
              ps_putt(dst, state->x); ps_putt(dst, state->y);
              fputs(" l\n", dst);
            }
            horiz = ! horiz;
          }
        }
        *nons = 0;
        break;
      case 8:
        if (instr & DISPLAY_RES) fputs(" rrcurveto", dst);
        {
          int i;
          double pdx, pdy;
          for (i = 0; i + 5 < *nons; i += 6) {
            if (decompile_cs) fputs("   ", dst);
            pdx = stack[i+0];
            pdy = stack[i+1];
#define dodxdy                                                          \
            if (instr & DRAW) {ps_put(psf, pdx); ps_put(psf, pdy);}     \
            state->x += pdx;                                            \
            state->y += pdy;                                            \
            if (decompile_cs) {ps_putt(dst, state->x); ps_putt(dst, state->y);}

            if (0 == i) setoff;
            dodxdy;
            pdx = stack[i+2];
            pdy = stack[i+3];
            dodxdy;
            pdx = stack[i+4];
            pdy = stack[i+5];
            dodxdy;
            if (instr & DRAW) fputs(" rrto\n", psf);
            if (decompile_cs) fputs(" c\n", dst);
          }
          if (*nons % 6) {
            fprintf(stderr, "Expected 6n values on stack for rrcurveto, "
                    "but had %d values.\n", *nons);
            fprintf(psf, "%% Expected 6n values on stack for rrcurveto, "
                    "but had %d values.\n", *nons);
          }
        }
        *nons = 0;
        break;
      case 10:
        {
          int bsno, usno; /* Biased and unbiased subroutine numbers */
          double rbsno;
          if (instr & DISPLAY_RES) fputs(" callsubr", dst);
          rbsno = stack[--(*nons)];
          bsno = (int) rbsno;
          if (subrs.count <= 0) {
            fprintf(stderr, "Attempt to invoke subroutine even though there "
                    "are %d local subroutines.\n", subrs.count);
            fflush(0);
            exit(1);
          } else if (subrs.count < 1240) {
            usno = bsno + 107;
          } else if (subrs.count < 33900) {
            usno = bsno + 1131;
          } else {
            usno = bsno + 32768;
          }
          if (decompile_cs) {
            setoff;
            if (instr & EXPAND) fputs(" -- ", dst);
            fprintf(dst, " invoke lsub%5.5d\n", usno);
          }
          if (usno < 0 || subrs.count <= usno) {
            fprintf(stderr, "Invocation of non-existent local subroutine "
                    "number %d (unbiased number %d) from %d available "
                    "local subroutines.\n", usno, bsno, subrs.count);
            if (instr & DISPLAY_RES) fputs(" (invalid subroutine number)", dst);
          } else {
            struct cff_offset *start_el, *beyond_el;
            int start, beyond;
            int task;

            start_el = element(subrs.offset, usno);
            beyond_el = start_el->_next;
            start =  (start_el->x_hi  << 16) | start_el->x_lo;
            beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
            task = instr & (ANALYSE | DRAW | COMPILER | EXPAND);
            state->id.serial = usno;
            state->id.type = CS_LOCAL_SUBR;
            put_cs(dst, psf, task, state, nstem, stack, nons,
                   &subrs.data[start-1], beyond - start);
            state->id = my_id;
          }
        }
        break;
      case 11:
        if (instr & DISPLAY_RES) fputs(" return", dst);
        if (instr & COMPILER) {
          if (state->id.type == state->root.type &&
              state->id.serial == state->root.serial) {
            fputs("    return\n", dst);
          } else if (instr & EXPAND) { /* Decompilation is expanded inline. */
            if (CS_GLYPH == state->root.type) {
              fputs(" -- return\n", dst);
            } else if (1) {
              fputs(" -- uninformative_return_here\n", dst);
            }
          }
        }
        break;
      case 12:
        switch (obi = data[++k]) {
        case 3:
          if (instr & DISPLAY_RES) fputs(" and", dst);
          stack[*nons - 2] = stack[*nons - 2] && stack[*nons - 1];
          (*nons)--;
          break;
        case 4:
          if (instr & DISPLAY_RES) fputs(" or", dst);
          stack[*nons - 2] = stack[*nons - 2] || stack[*nons - 1];
          (*nons)--;
          break;
        case 5:
          if (instr & DISPLAY_RES) fputs(" not", dst);
          stack[*nons - 1] = ! stack[*nons - 1];
          break;
        case 9:
          if (instr & DISPLAY_RES) fputs(" abs", dst);
          stack[*nons - 1] = fabs(stack[*nons - 1]);
          break;
        case 10:
          if (instr & DISPLAY_RES) fputs(" add", dst);
          stack[*nons - 2] = stack[*nons - 2] + stack[*nons - 1];
          (*nons)--;
          break;
        case 11:
          if (instr & DISPLAY_RES) fputs(" sub", dst);
          stack[*nons - 2] = stack[*nons - 2] - stack[*nons - 1];
          (*nons)--;
          break;
        case 12:
          if (instr & DISPLAY_RES) fputs(" div", dst);
          stack[*nons - 2] = stack[*nons - 2] / stack[*nons - 1];
          (*nons)--;
          break;
        case 14:
          if (instr & DISPLAY_RES) fputs(" neg", dst);
          stack[*nons - 1] = - stack[*nons - 1];
          break;
        case 15:
          if (instr & DISPLAY_RES) fputs(" eq", dst);
          stack[*nons - 2] = stack[*nons - 2] == stack[*nons - 1];
          (*nons)--;
          break;
        case 18:
          if (instr & DISPLAY_RES) fputs(" drop", dst);
          (*nons)--;
          break;
        case 20:
          if (instr & DISPLAY_RES) fputs(" put", dst);
          ival = (int) stack[*nons - 1];
          if (0 <= ival && ival < MAX_AUX) {
            auxarr[ival] = stack[*nons - 2];
          } else {
            fprintf(stderr, "Put index in charstring is invalid!\n");
          }
          (*nons)-=2;
          break;
        case 21:
          if (instr & DISPLAY_RES) fputs(" get", dst);
          ival = (int) stack[*nons - 1];
          if (0 <= ival && ival < MAX_AUX) {
            stack[*nons - 1] = auxarr[ival];
          } else {
            fprintf(stderr, "Put index in charstring is invalid!\n");
          }
          break;
        case 22:
          if (instr & DISPLAY_RES) fputs(" ifelse", dst);
          if (stack[*nons-2] > stack[*nons-1]) stack[*nons-4] = stack[*nons-3];
          (*nons)-=3;
          break;
        case 23:
          if (instr & DISPLAY_RES) fputs(" random", dst);
          push(0);
          break;
        case 24:
          if (instr & DISPLAY_RES) fputs(" mul", dst);
          stack[*nons-2] *= stack[*nons-1];
          (*nons)--;
          break;
        case 26:
          if (instr & DISPLAY_RES) fputs(" sqrt", dst);
          if (stack[*nons-1] >= 0) stack[*nons-1] = sqrt(stack[*nons-1]);
          break;
        case 27:
          if (instr & DISPLAY_RES) fputs(" dup", dst);
          rval = stack[*nons-1];
          push(rval);
          break;
        case 28:
          if (instr & DISPLAY_RES) fputs(" exch", dst);
          rval = stack[*nons - 2];
          stack[*nons - 2] = stack[*nons - 1];
          stack[*nons - 1] = rval;
          break;
        case 29:
          if (instr & DISPLAY_RES) fputs(" index", dst);
          ival = (int) stack[*nons - 1];
          if (stack[*nons - 1] <= 0) {
            stack[*nons - 1] = stack[*nons - 2];
          } else if (ival < *nons - 1) {
            stack[*nons - 1] = stack[*nons - 2 - ival];
          }
          break;
        case 30:
          if (instr & DISPLAY_RES) fputs(" roll", dst);
          {
            int i, j, k, n;
            double temp;
            j = stack[*nons - 1];
            n = stack[*nons - 2];
            (*nons)-=2;
            if (n != stack[*nons-2] || n < 1) {
              /* Ill-defined - so do nothing */
            } else if (j > 0) {
              for (i = 0; i < j; i++) {
                temp = stack[*nons-1];
                for (k = 1; k < n; k++) stack[*nons-k] = stack[*nons-k-1];
                stack[*nons - n] = temp;
              }
            } else if (j < 0) {
              for (i = 0; i < -j; i++) {
                temp = stack[*nons - n];
                for (k = n; k > 1; k--) stack[*nons - k] = stack[*nons - k - 1];
                stack[*nons - 1] = temp;
              }
            }
          }
          break;
        case 34:
          if (instr & DISPLAY_RES) fputs(" hflex", dst);
          if (instr & (DRAW | COMPILER)) {
            fputs("Operator hflex not yet translated.\n", stderr);
          }
          *nons = 0;
          break;
        case 35:
          if (instr & DISPLAY_RES) fputs(" flex", dst);
          if (13 == *nons) {
            double pdx, pdy;

            if (decompile_cs) {
              fputs("   ", dst);
              ps_put(dst, stack[12]);
              fputs(" flex\n", dst);
            }
            if (decompile_cs) {
              setoff;
              fputs("   ", dst);
            }
            pdx = stack[0];
            pdy = stack[1];
            dodxdy;
            pdx = stack[2];
            pdy = stack[3];
            dodxdy;
            pdx = stack[4];
            pdy = stack[5];
            dodxdy;
            if (instr & DRAW) fputs(" rrto\n", psf);
            if (decompile_cs) fputs(" c\n   ", dst);
            pdx = stack[6];
            pdy = stack[7];
            dodxdy;
            pdx = stack[8];
            pdy = stack[9];
            dodxdy;
            pdx = stack[10];
            pdy = stack[11];
            dodxdy;
            if (instr & DRAW) fputs(" rrto\n", psf);
            if (decompile_cs) fputs(" c\n", dst);
          } else {
            fprintf(stderr, "Expected 13 values on stack for flex, "
                    "but had %d values.\n", *nons);
            fprintf(psf, "%% Expected 13 values on stack for flex, "
                    "but had %d values.\n", *nons);
          }
          *nons = 0;
          break;
        case 36:
          if (instr & DISPLAY_RES) fputs(" hflex1", dst);
          if (instr & (DRAW|COMPILER)) {
            fputs("Operator hflex1 not yet translated.\n", stderr);
          }
          *nons = 0;
          break;
        case 37:
          if (instr & DISPLAY_RES) fputs(" flex1", dst);
          if (11 == *nons) {
            double pdx, pdy;
            double dx = stack[0] + stack[2] + stack[4] + stack[6] + stack[8];
            double dy = stack[1] + stack[3] + stack[5] + stack[7] + stack[9];

            if (decompile_cs) {
              setoff;
              fputs(" flex1\n   ", dst);
            }
            pdx = stack[0];
            pdy = stack[1];
            dodxdy;
            pdx = stack[2];
            pdy = stack[3];
            dodxdy;
            pdx = stack[4];
            pdy = stack[5];
            dodxdy;
            if (instr & DRAW) fputs(" rrto\n", psf);
            if (decompile_cs) fputs(" c\n   ", dst);
            pdx = stack[6];
            pdy = stack[7];
            dodxdy;
            pdx = stack[8];
            pdy = stack[9];
            dodxdy;
            if (fabs(dx) > fabs(dy)) {
              pdx = stack[10];
              pdy = -dy;
            } else {
              pdx = -dx;
              pdy = stack[10];
            }
            dodxdy;
            if (instr & DRAW) fputs(" rrto\n", psf);
            if (decompile_cs) fputs(" c\n", dst);
          } else {
            fprintf(stderr, "Expected 11 values on stack for flex1, "
                    "but had %d values.\n", *nons);
            fprintf(psf, "%% Expected 11 values on stack for flex1, "
                    "but had %d values.\n", *nons);
          }
          *nons = 0;
          break;
        default:
          fprintf(stderr, "Bad byte %d (0x%2.2X) in charstring after escape "
                  "code 12 (0x0C)\n", obi, obi);
          fflush(0);
          abort();
        }
        break;
      case 14:
        if (instr & DISPLAY_RES) fputs(" endchar", dst);
        if (instr & DRAW) fputs(" closepath stroke\n", psf);
        *nons = 0;
        return;
      case 19:
      case 20:
        oper = (19 == obi) ? "hintmask" : "cntrmask";
        if (instr & DISPLAY_RES) fprintf(dst, " %s", oper);
        if (WANT_V == state->state) {
          *nstem += (*nons)/2;
          if (1 && (instr & DISPLAY_RES)) {
            fprintf(dst, " (Now %d stems.)", *nstem);
          }
        }
        for (m = 0; m < (*nstem + 7) / 8; m++) {
          if (instr & DISPLAY_RES) fprintf(dst, " %2.2X", data[k+m+1]);
        }
        if (decompile_cs) {
          setoff;
          if (WANT_V == state->state) {
            int j;
            fputs("    ", dst);
            for (j = *nons & 1; j < *nons; j += 2) {
              ps_put(dst, stack[j]);
              ps_put(dst, stack[j+1]); /* Consider translating */
            }
            if (*nons) fprintf(dst, " vstemhm\n");
          }
          fprintf(dst, "    %s", oper);
          for (m = 0; m < (*nstem + 7) / 8; m++) {
            fprintf(dst, " %2.2X", data[k+m+1]);
          }
          fputc('\n', dst);
        }
        state->state = NORMAL;
        k += m;
        *nons = 0;
        break;
      case 24:
        if (instr & DISPLAY_RES) fputs(" rcurveline", dst);
        if (instr & (DRAW|COMPILER)) {
          fputs("Operator rcurveline not yet translated.\n", stderr);
        }
        *nons = 0;
        break;
      case 25:
        if (instr & DISPLAY_RES) fputs(" rlinecurve", dst);
        if (instr & (DRAW|COMPILER)) {
          fputs("Operator rlinecurve not yet translated.\n", stderr);
        }
        *nons = 0;
        break;
      case 26:
        if (instr & DISPLAY_RES) fputs(" vvcurveto", dst);
        if (instr & (DRAW|COMPILER)) {
          fputs("Operator vvcurveto not yet translated.\n", stderr);
        }
        *nons = 0;
        break;
      case 27:
        if (instr & DISPLAY_RES) fputs(" hhcurveto", dst);
        if (instr & (DRAW|COMPILER)) {
          fputs("Operator hhcurveto not yet translated.\n", stderr);
        }
        *nons = 0;
        break;
      case 28:
        ival = (signed char) data[++k];
        ival = 256 * ival + data[++k];
        if (instr & DISPLAY_RES) fprintf(dst, " %d", ival);
        push(ival);
        break;
      case 29:
        if (instr & DISPLAY_RES) fputs(" callgsubr", dst);
        if (instr & (DRAW|COMPILER)) {
          fputs("Operator callgsubr not yet translated.\n", stderr);
        }
        (*nons)--;
        break;
      case 30:
      case 31:
        {
          int i, rem = *nons % 4;
          if (30 == obi) {
            horiz = 0;
            oper  = "vhcurveto";
          } else {
            horiz = 1;
            oper = "hvcurveto";
          }
          if (instr & DISPLAY_RES) fprintf(dst, " %s", oper);
          if (*nons < 4) {
            fprintf(stderr, "Need at least 4 values on stack for %s, "
                    "but there were only %d values.\n", oper, *nons);
            fprintf(psf, "%% Need at least 4 values on stack for %s, "
                    "but there were only %d values.\n", oper, *nons);
          } else if (rem <= 1) {
            double pdx, pdy;
            if (decompile_cs) setoff;
            for (i = 0; i + 4 <= *nons; i += 4) {
              if (decompile_cs) fputs("   ", dst);
              if (horiz) {
                pdx = stack[i+0]; pdy = 0;
                dodxdy;
                pdx = stack[i+1]; pdy = stack[i+2];
                dodxdy;
                if (rem && i+8 > *nons) {
                  pdx = stack[i+4];
                } else {
                  pdx = 0;
                }
                pdy = stack[i+3];
                dodxdy;
              } else {
                pdx = 0; pdy = stack[i+0];
                dodxdy;
                pdx = stack[i+1]; pdy = stack[i+2];
                dodxdy;
                pdx = stack[i+3];
                if (rem && i+8 > *nons) {
                  pdy = stack[i+4];
                } else {
                  pdy = 0;
                }
                dodxdy;
              }
              if (instr & DRAW) fputs(" rrto\n", psf);
              if (decompile_cs) fputs(" c\n", dst);
              horiz = !horiz;
            }
          } else {
            fprintf(stderr, "Need 4n or 4n+1 values on stack for %s, "
                    "but there %d values.\n", oper, *nons);
            fprintf(psf, "%% Need 4n or 4n+1 values on stack for %s, "
                    "but there %d values.\n", oper, *nons);
          }
          *nons = 0;
        }
        break;
      case 255:
        ival = (signed char) data[++k];
        ival = 256 * ival + data[++k];
        ival = 256 * ival + data[++k];
        ival = 256 * ival + data[++k];
        rval = ((double) ival) / 256 / 256;
        if (instr & DISPLAY_RES) fprintf(dst, " %.5f", rval);
        push(rval);
        break;
      default:
        fprintf(stderr, "Bad operator code in charstring.  "
                "Byte %d has value %d (0x%2.2X) in charstring.\n",
                k, obi, obi);
        fflush(0);
        abort();
      }
    }
  }
#undef n_on_stack
}

static char *source;

/* Get and display Postscript font */
void display_CFF(FILE *fp, int instr, struct offset_table top) {
  struct directory_entry *CFF_table;
  int pos, junk;
  struct cff_base comp;
  struct cff_encoding encoding;
  double *enc_loc, *set_loc, *cstr_loc, *private_loc;
  struct cff_index glyphs;
  int start, size, beyond;
  struct cff_offset *start_el, *beyond_el;
  FILE *psf;

  if (1) fprintf(stderr, "Invoked display_CFF(%8.8X).\n", instr);
  memset(&charset,  0, sizeof(charset));
  create_map(&top_dict);
  create_map(&private_dict);
  memset(&glyphs, 0, sizeof(glyphs));
  memset(&gsubrs, 0, sizeof(gsubrs));
  memset(&subrs, 0, sizeof(subrs));
  for (CFF_table = top.table_directory; CFF_table;
       CFF_table = CFF_table->_next) {
    if (!strncmp(CFF_table->tag, "CFF ", 4)) break;
  }
  if (CFF_table) {
    if (instr & COMPILER) display_GDEF(fp, ANALYSE|COMPDAT, top);
    pos = CFF_table->offset;
    if (fseek(fp, pos, SEEK_SET)) {
      perror("Seeking CFF table");
      exit(1);
    }
   get_bine_cff_base(fp, &comp, &junk, &pos);
   if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
     print_cff_base(stdout, &comp, &junk, "CFF", 1);
   }
   if (instr & (DRAW | COMPILER)) (void) load_unicode_names();
   strings = comp.strings;
   memset(&comp.strings, 0, sizeof(comp.strings)); /* Protect against freeing */
   start_el = element(comp.top_dict.offset, 0);
   beyond_el = start_el->_next;
   start =  (start_el->x_hi  << 16) | start_el->x_lo;
   beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
   size = beyond - start;
   do_dict(&comp.top_dict.data[start-1], size, 1, instr);
   if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
     if (lookup_in_map(top_dict, "Encoding", (void*) &enc_loc)) {
       fprintf(stdout, "-- Encoding is at offset %d.\n", (int) *enc_loc);
       if (*enc_loc > 10 && (instr & (DISPLAY_RAW | DISPLAY_RES))) {
	 pos = CFF_table->offset + (int) *enc_loc;
	 if (fseek(fp, pos, SEEK_SET)) {
	   perror("Seeking CFF encoding");
	   exit(1);
	 }
	 get_bine_cff_encoding(fp, &encoding, &junk, &pos);
	 print_cff_encoding(stdout, &encoding, &junk, "Encoding", 1);
       }
     }
   }
   if (lookup_in_map(top_dict, "charset", (void*) &set_loc)) {
     fprintf(stdout, "-- Charset is at offset %d.\n", (int) *set_loc);
     pos = CFF_table->offset + (int) *set_loc;
     if (fseek(fp, pos, SEEK_SET)) {
       perror("Seeking CFF encoding");
       exit(1);
     }
     get_bine_charset(fp, &charset, &junk, &pos);
     have_charset = 1;
     if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
       print_charset(stdout, &charset, &junk, "charset", 2);
     }
     if (instr & DRAW) {
       if (!(psf = fopen("psglyph.ps", "w"))) {
	 perror("psglyph.ps");
	 exit(1);
       }
       fprintf(psf, "%%!\n");
       fprintf(psf, "/TR /Times-Roman findfont 8 scalefont def\n");
     }
   }
   if ((instr & (DISPLAY_RAW | DISPLAY_RES | DRAW | COMPILER)) &&
       lookup_in_map(top_dict, "Private", (void*) &private_loc)) {
     struct string_dump raw_dict;
     double *subr_loc;

     memset(&raw_dict, 0, sizeof raw_dict);
     fprintf(stdout, "-- Private dictionary is at offset %d and has size %d.\n",
             (int) private_loc[1], (int) private_loc[0]);
     pos = CFF_table->offset + (int) private_loc[1];
     if (fseek(fp, pos, SEEK_SET)) {
       perror("Seeking CFF private dictionary");
       exit(1);
     }
     byte_dump_count = (int) private_loc[0];
     get_bine_string_dump(fp, &raw_dict, &junk, &pos);
     if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
       print_string_dump(stdout, &raw_dict, &junk, "Private", 2);
     }
     do_dict(raw_dict.chars, byte_dump_count, 0, instr);
     if (lookup_in_map(private_dict, "Subrs", (void*) &subr_loc)) {
       if (*subr_loc > 0) {
         pos = CFF_table->offset + (int) private_loc[1] + (int) *subr_loc;
         if (fseek(fp, pos, SEEK_SET)) {
           perror("Seeking CFF encoding");
           exit(1);
         }
         get_bine_cff_index(fp, &subrs, &junk, &pos);
         if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
           print_cff_index(stdout, &subrs, &junk, "Subrs", 2);
         }
         if (instr & (DISPLAY_RES|COMPILER)) {
           int isub;
           struct cff_offset *start_el, *beyond_el;
           int start, beyond;
           struct string_dump raw_text;
           if (instr & DISPLAY_RES) fputs(" 2 subr:\n", stdout);
           beyond_el = subrs.offset;
           for (isub = 0; isub < subrs.count; isub++) {
             int nstem, ns;
             double stack[MAX_STACK];
             struct cs_loc state;

             if (instr & DISPLAY_RES) fprintf(stdout, " 3 [%d]:\n", isub);
             start_el = beyond_el;
             beyond_el = start_el->_next;
             start =  (start_el->x_hi  << 16) | start_el->x_lo;
             beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
             raw_text.chars = (char *) &subrs.data[start -1];
             raw_text._n_chars = byte_dump_count = beyond - start;
             if (instr & DISPLAY_RES) {
               print_string_dump(stdout, &raw_text, &junk, "rawtext", 4);
             }
             if (instr & COMPILER) fprintf(stdout, "lsub lsub%5.5d\n", isub);
             if (instr & DISPLAY_RES) fprintf(stdout, " 4 text: ");
             state.x = state.y = 0.0;
             state.id.type = CS_LOCAL_SUBR;
             state.id.serial = isub;
             state.root = state.id;
             state.state = INIT;
             put_cs(stdout, psf, instr &~ DRAW,
                    &state, &nstem, stack, &ns, raw_text.chars, beyond - start);
             if (instr & COMPILER) fputs("end lsub\n", stdout);
             fputc('\n', stdout);
           }
         }
       }
     }
   }
   if (lookup_in_map(top_dict, "CharStrings", (void*) &cstr_loc)) {
     int n_glyphs;

     fprintf(stdout, "-- CharStrings are at offset %d.\n", (int) *cstr_loc);
     pos = CFF_table->offset + (int) *cstr_loc;
     if (fseek(fp, pos, SEEK_SET)) {
       perror("Seeking CFF encoding");
       exit(1);
     }
     get_bine_cff_index(fp, &glyphs, &junk, &pos);
     if (instr & (DISPLAY_RAW | DISPLAY_RES)) {
       print_cff_index(stdout, &glyphs, &junk, "CharStrings", 1);
     }
     n_glyphs = glyphs.count;
     if (N_GLYPHS && n_glyphs != N_GLYPHS) {
       fprintf(stderr, "-- Reading CFF changes glyph number from %d to %d!\n",
	       N_GLYPHS, n_glyphs);
       N_GLYPHS = n_glyphs;
     }
     if (instr & (DISPLAY_RES|DRAW|COMPILER)) {
       int igl, plen;
       const char *postname;
       struct cff_offset *start_el, *beyond_el;
       int start, beyond;
       struct string_dump raw_text;
       if (instr & DISPLAY_RES) fputs(" 2 gdef:\n", stdout);
       if (instr & DRAW) {
         fputs("/rrto {         % xa ya xb    yb    xc       yc\n",       psf);
         fputs("  4 3 roll      % xa ya yb    xc    yc       xb\n",       psf);
         fputs("  5 index add   % xa ya yb    xc    yc       xa+xb\n",    psf);
         fputs("  4 1 roll      % xa ya xa+xb yb    xc       yc\n",       psf);
         fputs("  3 2 roll      % xa ya xa+xb xc    yc       yb\n",       psf);
         fputs("  4 index add   % xa ya xa+xb xc    yc       ya+yb\n",    psf);
         fputs("  3 1 roll exch % xa ya xa+xb ya+yb yc       xc\n",       psf);
         fputs("  3 index add   % xa ya xa+xb ya+yb yc       xa+xb+xc\n", psf);
         fputs("  exch          % xa ya xa+xb ya+yb xa+xb+xc yc\n",       psf);
         fputs("  2 index add   % xa ya xa+xb ya+yb xa+xb+xc ya+yb+yc\n", psf);
         fputs("  rcurveto\n", psf);
         fputs("} def\n",       psf);
         fputs("/unipem 1000 def /em 50 def\n", psf);
         fputs("/box {\n", psf);
         fputs("newpath\n", psf);
         fputs("em neg 0 moveto\n", psf);
         fputs("em 1.5 mul 0 lineto\n", psf);
         fputs("[2] 0 setdash stroke\n", psf);
         fputs("0 em neg moveto\n", psf);
         fputs("0 em 1.5 mul lineto\n", psf);
         fputs("stroke\n", psf);
         fputs("0  em moveto\n", psf);
         fputs("em em lineto\n", psf);
         fputs("em  0 lineto\n", psf);
         fputs("stroke\n", psf);
         fputs("em unipem div dup dup scale\n", psf);
         fputs("1 exch div setlinewidth\n", psf);
         fputs("0 0 moveto\n", psf);
         fputs("} def\n", psf);

       }
       for (igl = 0; igl < n_glyphs; igl++) {
         int nstem, ns;
         double stack[MAX_STACK];
         struct cs_loc state;
         char *uname;
         
         get_cff_name(igl, &postname, &plen);
         if (instr & DISPLAY_RES) {
           fprintf(stdout, " 3 [%d]:\n", igl);
           fprintf(stdout, " 4 name: %.*s\n", plen, postname);
         }
         if (codepoint && NO_CHAR != codepoint[igl]) {
           if (!get_property(unicode_names, codepoint[igl],
                             (void**)&uname)) {
             uname = 0;
           }
         } else {
           uname = 0;
         }
         if (instr & DRAW) {
#define nrow 4
#define ncol 2
           if (igl && 0 == (igl % (nrow * ncol))) {
             fprintf(psf, "showpage\n");
           }
           fprintf(psf, "gsave\nTR setfont\n");
           fprintf(psf, "%d ", 150 + 200 * (igl % ncol));
           fprintf(psf, "%d translate\n",
                   150 + 170 * (nrow - 1 - (igl/ncol) % nrow));
           fprintf(psf, "0 92 moveto (Glyph %d) show\n", igl);
           if (codepoint && NO_CHAR != codepoint[igl]) {
             fprintf(psf, "0 84 moveto (Character U+%.4X) show\n",
                     codepoint[igl]);
             if (uname) fprintf(psf, "0 76 moveto (%s) show\n", uname);
           }
           if (postname) {
             fprintf(psf, "0 68 moveto (Postscript name %.*s) show\n",
                     plen, postname);
           }
#if 1
           fputs("box\n", psf);
#else
           fputs("0 0 moveto 50 1000 div dup scale\n", psf);
#endif
         }

         if (instr & COMPILER) {
           if (uname) {
             fprintf(stdout, "psglyph %.*s -- %s\n", plen, postname, uname);
           } else {
             fprintf(stdout, "psglyph %.*s\n", plen, postname);
           }
           fprintf(stdout, "-- number %d\n", igl);
           put_glyph_properties(igl);
           fputs("  shape\n", stdout);
         }
         start_el = element(glyphs.offset, igl);
         beyond_el = start_el->_next;
         start =  (start_el->x_hi  << 16) | start_el->x_lo;
         beyond = (beyond_el->x_hi << 16) | beyond_el->x_lo;
         raw_text.chars = (char *) &glyphs.data[start -1];
         raw_text._n_chars = byte_dump_count = beyond - start;
         if (instr & DISPLAY_RES) {
           print_string_dump(stdout, &raw_text, &junk, "rawtext", 4);
           fprintf(stdout, " 4 text: ");
         }
         state.x = state.y = 0.0;
         state.id.serial = igl;
         state.id.type = CS_GLYPH;
         state.root = state.id;
         state.state = INIT;
         put_cs(stdout, psf, instr,
                &state, &nstem, stack, &ns,
                raw_text.chars, beyond - start);
         if (instr & DISPLAY_RES) fputc('\n', stdout);
         if (instr & DRAW) fputs("grestore\n", psf);
         if (instr & COMPILER) {
           fputs("  end shape\n", stdout);
           fputs("end psglyph\n", stdout);
         }
       }
       if (instr & DRAW) fclose(psf);
     }
     free_cff_index(&glyphs);
   } else {
     fprintf(stdout, "-- Font defines no glyphs!");
   }
   free_cff_base(&comp);
  }
}

simple_table_dump(kern, kern_table)

int main(int argc, char *argv[]) {
#define SOURCE "trebuc.ttf"
  FILE *fp;
  int pos, junk, itab, have_font, iarg, badargs;
  char *flagptr;
  struct offset_table_header top2;
  struct offset_table top;
  struct maximum_profile maximumProfile;
  int help_only, suppress;
  int table_cmd;
  /* Use macro tables() to perform actions systematically. Some jiggery pokery
     may be needed for awkward tags such as "OS/2" and "cvt ".  The order of
     examination of the tables is occasionally significant. This might get
     fixed later. For compiler format, there is serious interweaving, handled
     by internal flag COMPDAT and redundant reading.*/
/* glyf needs CFF and GDEF */
#define tables(f) tables_req(f), tables_ps(f), \
    tables_outline(f), tables_bit(f), tables_OTL(f), tables_other(f)
  /* Required tables: */
#define tables_req(f) f(maxp), f(cmap), f(head), f(post), f(hhea), f(hmtx), \
    f(name), f(OS_2), f(CPAL), f(COLR)
  /* True Type Outline Data: */
#define tables_outline(f) f(cvt), f(fpgm), f(glyf), f(loca), f(prep)
  /* Postscript Outline Data: */
#define tables_ps(f) f(CFF), f(VORG)
  /* Bitmap data: */
#define tables_bit(f) f(EBDT), f(EBLC), f(EBSC)
  /* Advanced typographic tables: */
#define tables_OTL(f) f(BASE), f(GDEF), f(GPOS), f(GSUB), f(JSTF)
  /* Other tables: */
#define tables_other(f) f(DSIG), f(gasp), f(hdmx), f(kern), f(LTSH), \
    f(PCLT), f(VDMX), f(vhea), f(vmtx),            \
    f(Silf), f(Sill), f(Sile), f(Feat), f(Gloc), f(Glat), \
    f(morx), f(feat)

#define cmd_name(name) name ## _cmd
  int tables(cmd_name);

  source = 0;
  table_cmd = 0;
#define zero_cmd(name) cmd_name(name) = 0
  tables(zero_cmd);
#undef zero_cmd
  badargs = 0;
  help_only = 0;
  suppress  = 0;

  /* Case insensitivity to accommodate Windows etc. */
  for (iarg = 1; iarg < argc; iarg++) {
    int colpos, x1 = -1, x2 = -1;

    have_font = 0;
    help_only = 0;
    if (!strncasecmp(argv[iarg], "--cmap", 6)) {
      if (strchr(":", argv[iarg][6])) {
	cmap_cmd |= ANALYSE | DISPLAY_RAW;
	maxp_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--head", 6)) {
      if (strchr(":", argv[iarg][6])) {
	head_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'c':
	    case 'C':
	      head_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--head:crh\n"
		      "  c  Display head table in compiler format\n"
		      "  m  Display meaning of head table\n"
		      "  r  Display raw contents of head table\n"
		      "  h  Display this help.\n\n");
	    break;
	    case 'm':
	    case 'M':
	      head_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      head_cmd |= DISPLAY_RAW;
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  head_cmd |= DISPLAY_RAW;
        }
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--hhea", 6)) {
      if (strchr(":", argv[iarg][6])) {
	hhea_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'c':
	    case 'C':
	      hhea_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--hhea:crh\n"
		      "  c  Display hhea table in compiler format\n"
		      "  m  Display meaning of hhea table\n"
		      "  r  Display raw contents of hhea table\n"
		      "  h  Display this help.\n\n");
	    break;
	    case 'm':
	    case 'M':
	      hhea_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      hhea_cmd |= DISPLAY_RAW;
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--hmtx", 6)) {
      if (strchr(":", argv[iarg][6])) {
	hmtx_cmd |= ANALYSE | DISPLAY_RAW;
	hhea_cmd |= ANALYSE;
	maxp_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--maxp", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'c':
	    case 'C':
	      maxp_cmd |= COMPILER;
	    break;
	    case 'm':
	    case 'M':
	      maxp_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      maxp_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--maxp:cmrh\n"
		      "  c  Display maxp table in compiler format\n"
		      "  m  Display meaning of maxp table\n"
		      "  r  Display raw contents of maxp table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  maxp_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--name", 6)) {
      if (strchr(":", argv[iarg][6])) {
	name_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'c':
	    case 'C':
	      name_cmd |= COMPILER;
	      break;
	    case 'm':
	    case 'M':
	      name_cmd |= DISPLAY_RES;
	      break;
	    case 'r':
	    case 'R':
	      name_cmd |= DISPLAY_RAW;
	      break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--name:cmrh\n"
		      "  c  Display 'names' in compiler format\n"
		      "  m  Display meaning of name table\n"
		      "  r  Display raw contents of name table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  name_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--OS/2", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'c':
	    case 'C':
	      OS_2_cmd |= COMPILER;
	      break;
	    case 'm':
	    case 'M':
	      OS_2_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      OS_2_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--OS/2:c" /* "m" */ "rh\n"
		      "  c  Display OS/2 table in compiler format\n"
#if 0
		      "  m  Display meaning of OS/2 table\n"
#endif
		      "  r  Display raw contents of OS/2 table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  OS_2_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--post", 6)) {
      if (strchr(":", argv[iarg][6])) {
	post_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      post_cmd |= DISPLAY_RES;
	      break;
	    case 'r':
	    case 'R':
	      post_cmd |= DISPLAY_RAW;
	      break;
	    case 'c':
	    case 'C':
	      post_cmd |= COMPILER;
	      break;
	    case 'i': case 'I':
#define seti(name) name ## _cmd |= POSTNAME
	      tables(seti);
#undef seti
              CFF_cmd  |= ANALYSE;
              maxp_cmd |= ANALYSE;
	      break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--post:mrcih\n"
		      "  m  Display meaning of post table\n"
		      "  r  Display raw contents of post table\n"
		      "  c  Display glyph definition in compiler format\n"
		      "  i  Use postscript names rather than artificial names "
		            "to identify glyphs\n"
		      "     in compiler format\n"
		      "  h  Display this help.\n\n"
		      "  For meaning of postscript names, use --glyf:c.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  post_cmd |= DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--cvt", 5)) {
      if (strchr(":", argv[iarg][5])) {
	if (':' == argv[iarg][5]) {
	  for (flagptr = argv[iarg]+6; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'r':
	    case 'R':
	      cvt_cmd |= DISPLAY_RAW;
	      break;
	    case 'c':
	    case 'C':
	      cvt_cmd |= COMPILER;
	      break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--post:mrch\n"
		      "  r  Display raw contents of control value table\n"
		      "  c  Display cvt definition in compiler format\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.5s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  cvt_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--fpgm", 6)) {
      if (strchr(":", argv[iarg][6])) {
	fpgm_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      fpgm_cmd |= DISPLAY_RES;
	      break;
	    case 'r':
	    case 'R':
	      fpgm_cmd |= DISPLAY_RAW;
	      break;
	    case 'c':
	    case 'C':
	      fpgm_cmd |= COMPILER;
	      break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--fpgm:mrh\n"
		      "  m  Display meaning of fpgm table\n"
		      "  r  Display raw contents of fpgm table\n"
		      "  c  Display fpgm definition in compiler format\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  fpgm_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--glyf", 6)) {
      if (strchr(":", argv[iarg][6])) {
	glyf_cmd |= ANALYSE;
	maxp_cmd |= ANALYSE;
	head_cmd |= ANALYSE;
	cmap_cmd |= ANALYSE;
	post_cmd |= ANALYSE;
	hhea_cmd |= ANALYSE;
	hmtx_cmd |= ANALYSE;
	CFF_cmd  |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'd':
	    case 'D':
	      glyf_cmd |= DRAW;
	    break;
	    case 'q':
	    case 'Q':
	      glyf_cmd |= HALF;
	    break;
	    case 'j':
	    case 'J':
	      glyf_cmd |= JOIN;
	    break;
	    case 'k':
	    case 'K':
	      glyf_cmd |= KNOTTED;
	    break;
	    case 'm':
	    case 'M':
	      glyf_cmd |= DISPLAY_RES;
	    head_cmd |= DISPLAY_RAW;
	    break;
	    case 'r':
	    case 'R':
	      glyf_cmd |= DISPLAY_RAW;
	    head_cmd |= DISPLAY_RAW;
	    break;
	    case 'p':
	    case 'P':
	      glyf_cmd |= POSTSCRIPT;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--glyf:cdmrhjkq\n"
		      "  c  Display glyph definition in compiler format\n"
		      "  d  Display unhinted glyphs\n"
		      "  h  Display this help.\n"
		      "  j  Display unhinted glyphs by joining the dots\n"
		      "  k  Display unhinted glyphs as quadratic "
		      "splines\n"
		      "  m  Display meaning of glyf table\n"
		      "  p  Use PostScript, not Word, for display\n"
		      "  q  Display unhinted glyphs as conic sections,\n"
		      "     with on-curve points inserted between\n"
		      "     consecutive off-curve points\n"
		      "  r  Display raw contents of glyf table\n\n");
	    break;
	    case 'c':
	    case 'C':
	      glyf_cmd |= COMPILER;
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  glyf_cmd |= DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--loca", 6)) {
      if (strchr(":", argv[iarg][6])) {
	loca_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--prep", 6)) {
      if (strchr(":", argv[iarg][6])) {
	prep_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      prep_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      prep_cmd |= DISPLAY_RAW;
	    break;
	    case 'c':
	    case 'C':
	      prep_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--prep:mrh\n"
		      "  m  Display meaning of prep table\n"
		      "  r  Display raw contents of prep table\n"
		      "  c  Display prep definition in compiler format\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  prep_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--CFF", 5)) {
      if (strchr(":", argv[iarg][5])) {
	maxp_cmd |= ANALYSE;
	head_cmd |= ANALYSE;
	cmap_cmd |= ANALYSE;
	post_cmd |= ANALYSE;
	hhea_cmd |= ANALYSE;
	hmtx_cmd |= ANALYSE;
	CFF_cmd  |= ANALYSE;
	if (':' == argv[iarg][5]) {
	  for (flagptr = argv[iarg]+6; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'd':
	    case 'D':
	      CFF_cmd  |= DRAW;
	      cmap_cmd |= ANALYSE;
	    break;
	    case 'm':
	    case 'M':
	      CFF_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      CFF_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--CFF:mrdh\n"
		      "  m  Display meaning of CFF table\n"
		      "  r  Display raw contents of CFF table\n"
		      "  c  Display CFF definition in compiler format\n"
                      "  e  Expand subroutine calls in compiler format dump\n"
		      "  h  Display this help.\n"
		      "  d  Display glyphs\n\n");
	    break;
            case 'c':
            case 'C':
              CFF_cmd |= COMPILER;
              break;
            case 'e':
            case 'E':
              CFF_cmd |= EXPAND;
              break;
	    default:
	      fprintf(stderr, "Flag %c on option %.5s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  CFF_cmd |= DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--VORG", 6)) {
      if (strchr(":", argv[iarg][6])) {
	VORG_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--EBDT", 6)) {
      if (strchr(":", argv[iarg][6])) {
	EBDT_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--EBLC", 6)) {
      if (strchr(":", argv[iarg][6])) {
	EBLC_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--EBSC", 6)) {
      if (strchr(":", argv[iarg][6])) {
	EBSC_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--BASE", 6)) {
      if (strchr(":", argv[iarg][6])) {
	BASE_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	      default:
		fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
			*flagptr, argv[iarg]);
		badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--GDEF", 6)) {
      if (strchr(":", argv[iarg][6])) {
	GDEF_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      GDEF_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      GDEF_cmd |= DISPLAY_RAW;
	    break;
	    case 'c':
	    case 'C':
	      GDEF_cmd |= COMPILER;
	      GDEF_cmd |= COMPDAT;
	      maxp_cmd |= ANALYSE;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--GDEF:mrch\n"
		      "  m  Display meaning of GDEF table\n"
		      "  r  Display raw contents of GDEF table\n"
		      "  c  Display GDEF definition in compiler format "
		            "(also needs --glyf:c)\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  GDEF_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--GPOS", 6)) {
      if (strchr(":", argv[iarg][6])) {
	GPOS_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	      /*
	    case 'm':
	    case 'M':
	      GPOS_cmd |= DISPLAY_RES;
	      */
	    break;
	    case 'r':
	    case 'R':
	      GPOS_cmd |= DISPLAY_RAW;
	    break;
	    case 'c':
	    case 'C':
	      GPOS_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--GPOS:mch\n"
		      "  m  Display meaning of GPOS table\n"
/*		      "  r  Display raw contents of GPOS table\n" */
		      "  c  Display GPOS definition in compiler format "
		            "(also needs --glyf:c)\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  GPOS_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--GSUB", 6)) {
      if (strchr(":", argv[iarg][6])) {
	GSUB_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	      /*
	    case 'm':
	    case 'M':
	      GSUB_cmd |= DISPLAY_RES;
	      */
	    break;
	    case 'r':
	    case 'R':
	      GSUB_cmd |= DISPLAY_RAW;
	    break;
	    case 'c':
	    case 'C':
	      GSUB_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--GSUB:mch\n"
/*		      "  m  Display meaning of GSUB table\n" */
		      "  r  Display raw contents of GSUB table\n"
		      "  c  Display GSUB definition in compiler format "
		            "(also needs --glyf:c)\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  GSUB_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--JSTF", 6)) {
      if (strchr(":", argv[iarg][6])) {
	JSTF_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--DSIG", 6)) {
      if (strchr(":", argv[iarg][6])) {
	DSIG_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--gasp", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      gasp_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      gasp_cmd |= DISPLAY_RAW;
	    break;
	    case 'c':
	    case 'C':
	      gasp_cmd |= COMPILER;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--gasp:mrch\n"
		      "  m  Display meaning of gasp table\n"
		      "  r  Display raw contents of gasp table\n"
		      "  c  Display gasp definition in compiler format\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
          }
        } else {
          gasp_cmd |= DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--hdmx", 6)) {
      if (strchr(":", argv[iarg][6])) {
	hdmx_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--kern", 6)) {
      if (strchr(":", argv[iarg][6])) {
	kern_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--CPAL", 6)) {
      if (strchr(":", argv[iarg][6])) {
	CPAL_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      CPAL_cmd |= DISPLAY_RES;
              break;
	    case 'r':
	    case 'R':
	      CPAL_cmd |= DISPLAY_RAW;
              break;
	    case 'c':
	    case 'C':
	      CPAL_cmd |= COMPILER;
/*	      maxp_cmd |= ANALYSE; */
              break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--CPAL:mrch\n"
		      "  m  Display meaning of CPAL table\n"
		      "  r  Display raw contents of CPAL table\n"
		      "  c  Display CPAL definition in compiler format"
		      "  h  Display this help.\n\n");
              break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  CPAL_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--COLR", 6)) {
      if (strchr(":", argv[iarg][6])) {
	COLR_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      COLR_cmd |= DISPLAY_RES;
              break;
	    case 'r':
	    case 'R':
	      COLR_cmd |= DISPLAY_RAW;
              break;
	    case 'c':
	    case 'C':
	      COLR_cmd |= COMPILER;
	      CPAL_cmd |= ANALYSE;
              break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--COLR:mrch\n"
		      "  m  Display meaning of COLR table\n"
		      "  r  Display raw contents of COLR table\n"
		      "  c  Display COLR definition in compiler format "
		            "(also needs --glyf:c)\n"
		      "  h  Display this help.\n\n");
              break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  COLR_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--LTSH", 6)) {
      if (strchr(":", argv[iarg][6])) {
	LTSH_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--PCLT", 6)) {
      if (strchr(":", argv[iarg][6])) {
	PCLT_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--VDMX", 6)) {
      if (strchr(":", argv[iarg][6])) {
	VDMX_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--vhea", 6)) {
      if (strchr(":", argv[iarg][6])) {
	vhea_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--vmtx", 6)) {
      if (strchr(":", argv[iarg][6])) {
	vmtx_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
/* --feat denotes the AAT feat table. */
/* -- This comparison must precede comparison of argument with --Silf. */
    } else if (!strncmp(    argv[iarg],    "--Feat", colpos = 6) ||
               !strncasecmp(argv[iarg], "--SILFeat", colpos = 9)) {
      if (strchr(":", argv[iarg][colpos])) {
	if (':' == argv[iarg][colpos]) {
	  for (flagptr = argv[iarg]+colpos+1; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm': case 'M':
	      Feat_cmd |= ANALYSE | DISPLAY_RES;
	      break;
	    case 'r': case 'R':
	      Feat_cmd |= ANALYSE | DISPLAY_RAW;
	      break;
	    case 'h': case 'H':
	      fprintf(stdout,
		      "--Feat:mrh\n"
		      "  m  Display meaning of Feat table\n"
		      "  r  Display raw contents of Feat table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.*s not recognised.\n",
		      *flagptr, colpos, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Feat_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not at all recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--Silf", 6)) {
      if (strchr(":", argv[iarg][6])) {
	Silf_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      Silf_cmd |= DISPLAY_RES;
	      cmap_cmd |= ANALYSE;
	    break;
	    case 'r':
	    case 'R':
	      Silf_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--Silf:mrh\n"
		      "  m  Display meaning of Silf table\n"
		      "  r  Display raw contents of Silf table\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Silf_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--Sill", 6)) {
      if (strchr(":", argv[iarg][6])) {
	Sill_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      Sill_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      Sill_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--Sill:mrh\n"
		      "  m  Display meaning of Sill table\n"
		      "  r  Display raw contents of Sill table\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Sill_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--Sile", 6)) {
      if (strchr(":", argv[iarg][6])) {
	Sile_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      Sile_cmd |= DISPLAY_RES;
	    break;
	    case 'r':
	    case 'R':
	      Sile_cmd |= DISPLAY_RAW;
	    break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--Sile:mrh\n"
		      "  m  Display meaning of Sile table\n"
		      "  r  Display raw contents of Sile table\n"
		      "  h  Display this help.\n\n");
	    break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Sile_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--Gloc", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm': case 'M':
	      Gloc_cmd |= ANALYSE | DISPLAY_RES;
	      Silf_cmd |= ANALYSE;
	      break;
	    case 'r': case 'R':
	      Gloc_cmd |= ANALYSE | DISPLAY_RAW;
	      Silf_cmd |= ANALYSE;
	      break;
	      case 'h': case 'H':
		fprintf(stdout,
			"--Gloc:mrh\n"
			"  m  Display meaning of Gloc table\n"
			"  r  Display raw contents of Gloc table\n"
			"  h  Display this help.\n\n");
		break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Gloc_cmd |= ANALYSE | DISPLAY_RAW;
	  Silf_cmd |= ANALYSE;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--Glat", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm': case 'M':
	      Glat_cmd |= ANALYSE | DISPLAY_RES;
	      Gloc_cmd |= ANALYSE;
	      Silf_cmd |= ANALYSE;
	      break;
	    case 'r': case 'R':
	      Glat_cmd |= ANALYSE | DISPLAY_RAW;
	      Gloc_cmd |= ANALYSE;
	      Silf_cmd |= ANALYSE;
	      break;
	    case 'h': case 'H':
	      fprintf(stdout,
		      "--Glat:mrh\n"
		      "  m  Display meaning of Glat table\n"
		      "  r  Display raw contents of Glat table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  Glat_cmd |= ANALYSE | DISPLAY_RAW;
	  Gloc_cmd |= ANALYSE;
	  Silf_cmd |= ANALYSE;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--morx", 6)) {
      if (strchr(":", argv[iarg][6])) {
	morx_cmd |= ANALYSE;
        maxp_cmd |= ANALYSE;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm':
	    case 'M':
	      morx_cmd |= ANALYSE | DISPLAY_RES;
	      break;
	    case 'r':
	    case 'R':
	      morx_cmd |= ANALYSE | DISPLAY_RAW;
	      break;
	    case 'c':
	    case 'C':
	      morx_cmd |= ANALYSE | COMPILER;
	      break;
	    case 'h':
	    case 'H':
	      fprintf(stdout,
		      "--morx:mrch\n"
		      "  m  Display meaning of AAT morx table\n"
		      "  r  Display raw contents of AAT morx table\n"
		      "  c  Display morx definition in compiler format\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  morx_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
/* AAT feat logic must follow SIL Feat logic */
    } else if (!strncasecmp(argv[iarg], "--feat", 6)) {
      if (strchr(":", argv[iarg][6])) {
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm': case 'M':
	      feat_cmd |= ANALYSE | DISPLAY_RES;
	      break;
	    case 'r': case 'R':
	      feat_cmd |= ANALYSE | DISPLAY_RAW;
	      break;
	    case 'h': case 'H':
	      fprintf(stdout,
		      "--feat:mrh\n"
		      "  m  Display meaning of AAT feat table\n"
		      "  r  Display raw contents of AAT feat table\n"
		      "  h  Display this help.\n\n");
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  feat_cmd |= ANALYSE | DISPLAY_RAW;
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--table", 7)) {
      if (strchr(":", argv[iarg][7])) {
	table_cmd |= ANALYSE | DISPLAY_RAW;
	if (':' == argv[iarg][6]) {
	  for (flagptr = argv[iarg]+7; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	}
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!strncasecmp(argv[iarg], "--all", 5)) {
      if (strchr(":", argv[iarg][5])) {
	int all_opt = 0;
	if (':' == argv[iarg][5]) {
	  for (flagptr = argv[iarg]+6; *flagptr; flagptr++) {
	    switch (*flagptr) {
	    case 'm': case 'M':
	      all_opt |= DISPLAY_RES;
	      break;
	    case 'r': case 'R':
	      all_opt |= DISPLAY_RAW;
	      break;
	    case 'h': case 'H':
	      fprintf(stdout,
		      "--all:mrh\n"
		      "  m  Display meaning of all tables\n"
		      "  r  Display raw contents of all tables\n"
		      "  c  Display in compiler format\n"
		      "  h  Display this help.\n\n");
	    break;
	    case 'c': case 'C':
	      all_opt |= COMPILER;
	      glyf_cmd |= ANALYSE;
	      maxp_cmd |= ANALYSE;
	      head_cmd |= ANALYSE;
	      cmap_cmd |= ANALYSE;
	      post_cmd |= ANALYSE;
	      hhea_cmd |= ANALYSE;
	      hmtx_cmd |= ANALYSE;
	      CFF_cmd  |= ANALYSE;
	      break;
	    default:
	      fprintf(stderr, "Flag %c on option %.6s not recognised.\n",
		      *flagptr, argv[iarg]);
	      badargs++;
	    }
	  }
	} else {
	  all_opt = DISPLAY_RAW;
	}
#define propagate(name) cmd_name(name) |= all_opt
	tables(propagate);
#undef propagate
	table_cmd |= all_opt;
      } else {
	fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      }
    } else if (!*argv[iarg]) {
      continue;
    } else if ('-' == *argv[iarg] && argv[iarg][1] && '-' != argv[iarg][1]) {
      const char *optch;
      for (optch = 1 + argv[iarg]; *optch; optch++) {
        switch (*optch) {
        case 'c':
          help_only = 1;
          if (!suppress) {
            fputs(WO, stdout);
            suppress = 1; /* But not again */
          }
          fprintf(stdout,
              "This program is licensed under Version 3 or later of the\n"
              "GNU General Public Licence as published by the\n"
              "Free Software Foundation.\n\n"

              "You may convey verbatim copies of the Program's source code\n"
              "as you receive it, in any medium, provided that you\n"
              "conspicuously and appropriately publish on each copy\n"
              "an appropriate copyright notice; keep intact all notices\n"
              "stating that the License and any non-permissive terms added\n"
              "in accord with section 7 apply to the code; keep intact\n"
              "all notices of the absence of any warranty; and give all\n"
              "recipients a copy of this License along with the Program.\n"
                  );
          break;
        case 'h':
          help_only = 1;
          fprintf(stdout,
	      "oft displays the contents of an OpenType font.\n\n"
	      "Usage: oft --all --table --<table>:opt.. font..\n"
	      "or     oft -[chw]\n\n"
              "-c        Copying\n"
	      "-h        Help\n"
              "-s        Suppress identification banner\n"
              "-w        Warranty\n"
	      "--table   Display list of OpenType tables\n"
	      "--all     Report on all tables\n"
	      "--<table> Report on table <table>.  Recognised tables are:\n"
	      "          cmap, head, hhea, hmtx, maxp, name, OS/2, post,\n"
	      "          cvt, fpgm, glyf, loca, prep, CFF, VORG, EBDT,\n"
	      "          EBLC, EBSC, BASE, GDEF, GPOS, GSUB, JSTF, DSIG,\n"
	      "          gasp, hdmx, kern, LTSH, PCLT, VDMX, vhea, vmtx,\n"
	      "          CPAL, COLR, Silf, Sill, Sile, Gloc, Glat,\n"
              "          morx, feat.\n"
              "          SILFeat may be used to specify the SIL Feat table.\n"
	      "opt       List of letters encoding various display options:\n"
	      "          Common options (not always yet supported) are:\n"
	      "          r   Display Raw contents\n"
	      "          m   Display Meaning of table\n"
	      "          c   Display in compiler input format\n"
	      "          h   Display Help on options for table.\n"
	      "              This option is always available if any option\n"
	      "              besides these four is available.\n"
	      "font      OpenType (or TrueType) font definition file.\n");
          break;
        case 's':
          suppress = 1;
          break;
        case 'w':
          help_only = 1;
          if (!suppress) {
            fputs(CO, stdout);
            suppress = 1; /* But not again */
          }
          /* Clauses 15 & 16 of GNU GPL Version 3: */
          fprintf(stdout,
              "THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED\n"
              "BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING\n"
              "THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM\n"
              "“AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR\n"
              "IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES\n"
              "OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.\n"
              "THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE\n"
              "PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE,\n"
              "YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR\n"
              "OR CORRECTION.\n\n"
              
              "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO\n"
              "IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO\n"
              "MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE,\n"
              "BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,\n"
              "INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE\n"
              "OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n"
              "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES\n"
              "SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM\n"
              "TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER\n"
              "OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH\n"
              "DAMAGES.\n"
                  );
          break;
        default:
          fprintf(stderr, "Option -%c not recognised.\n", *optch);
          badargs = 1;
        }
      }
    } else if ('-' == argv[iarg][0]) {
      fprintf(stderr, "Option %s not recognised.\n", argv[iarg]);
      badargs = 1;
    } else {
      have_font = !help_only;
    }
    if (!have_font) continue;
    if (badargs) return 1;
    if (help_only) continue;
    if (!suppress) {
      fputs(WELCOME, stdout);
      suppress = 1; /* But not again */
    }
    if (argc == iarg) return 0;
    source = argv[iarg];
    fp = fopen(source, "rb");
    if (!fp) perror(source);
    if (!fp) return 1;
    printf("\n-- Reporting on %s:\n", source);
    swap_order = 0;
    get_bine_offset_table_header(fp, &top2, &junk, &pos);
#if 0
    print_offset_table_header(stdout, &top2, &junk, "Offsets", 0);
#endif
    if ((top2.entrySelector & 0xff) == 0 &&
	(top2.entrySelector & 0xff00) != 0) {
      /* Wrong way round! */
      swap_order = 1;
      fseek(fp, 0, SEEK_SET);
      printf("-- Fixing endianity and re-reading.\n");
    } else if ((top2.entrySelector & 0xff) != 0 &&
	       (top2.entrySelector & 0xff00) == 0) {
      printf("-- File is in machine's natural order!\n");
    } else {
      printf("Entry selector appears to be corrupt!\n");
      return 1;
    }
    fseek(fp, 0, SEEK_SET);
    get_bine_offset_table(fp, &top, &junk, &pos);
    if (0x4f54544f == top.sfnt_version) { /* OTTO */
      printf("-- CFF file - no truetype outlines.\n");
    } else if (top.sfnt_version != 0x00010000) {
      printf("Format is %8.8X, not 1.0\n", top.sfnt_version);
      return 1;
    }
    if (table_cmd & DISPLAY_RAW) {
      print_offset_table(stdout, &top, &junk, "Offsets", 0);
    }
/* Remove stale data */
    have_charset = 0;
    category = 0;
    markClass = 0;

/* Permanent do nothing display routines */
#define display_loca(x, y, z) 42 /* Job done by display_glyf() */
/* As yet unimplemented display routines */
#define display_VORG(x, y, z) 42
#define display_EBDT(x, y, z) 42
#define display_EBLC(x, y, z) 42
#define display_EBSC(x, y, z) 42
#define display_BASE(x, y, z) 42
#define display_JSTF(x, y, z) 42
#define display_DSIG(x, y, z) 42
#define display_hdmx(x, y, z) 42
#if 0
#define display_kern(x, y, z) 42
#endif
#define display_LTSH(x, y, z) 42
#define display_VDMX(x, y, z) 42
#define display_vhea(x, y, z) 42
#define display_vmtx(x, y, z) 42
/* tables() expands to a comma *expression*.  To make each call conditional,
   use short circuit property of '&&'.  As called functions return void, use
   another comma expression to replace the return value by an arbitrary
   integer. */
#define display(x) \
    (cmd_name(x) &~ POSTNAME) && (display_ ## x(fp, cmd_name(x), top), 1)
    tables(display);
#undef display
    fclose(fp);

/* Remove temporary data */
    if (category) free(category);
    if (markClass) free(markClass);
  }
  if (!suppress) {
    fputs(WELCOME, stdout);
  }
  return 0;
}
